<?php

namespace DXIM;

class Hook {

	protected static $instance = null;


	/**
	 *
	 * @return Hook
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}


	/**
	 *
	 */
	private function __construct() {}


    /**
     * @return void
     */
    public function registerHooks() {

        /**
         * Bereinigung nach Import
         */
        add_action(DXIM_CORE_AFTER_IMPORT,[Import::getInstance(),'cleanup']);

        /**
         * Bereinigung nach löschen eines Fahrzeuges
         */
        add_action('before_delete_post',[Vehicle::getInstance(),'_on_delete_post'],10,2);

        /**
         * Nach dem ein Fahrzeug Importiert wurde,
         * prüfen ob CSP Bilder vorliegen die dem Fahrzeug zugeordnet werden müssen
         */
        add_action(DXIM_CORE_VEHICLE_SAVED,[CarShotsPROImport::getInstance(),'assignImages'],10,1);

        /**
         *
         */
        add_action(DXIM_FILTER_VEHICLE_BEFORE_IMPORT, [VehicleCF::getInstance(),'modify_import_values'],10,1);

	    /**
	     * nach dem ein Fahrzeug importiert wurde,
	     * prüfen ob Bilder freigstellt werden sollen
	     * CSP Freistellung
	     */
        add_action(DXIM_CORE_VEHICLE_SAVED,function($post_id, $is_update) {
			if($is_update) return;
	        CarShotsPROApi::getInstance()->createJob($post_id);
        },10,2);

	    /**
	     * nach dem ein Fahrzeug importiert wurde
	     * aktualisieren Eintrag in Filtertabelle
	     */
        add_action(DXIM_CORE_VEHICLE_SAVED,[FilterTable::getInstance(),'setFiltersForVehicle'],10,1);

        /**
         *
         */
        add_action('wp_footer', function () {
            Template::getInstance()->enqueue_footer_scripts();
        }, 1000);

        /** -----------------------------
         * redirect http://www.domain.de/seller_inventory_key nach vehicle permalink
         */
        add_action('wp', function() {
            if(!is_admin() && is_404()) {
                $url = explode("/",untrailingslashit($_SERVER['REQUEST_URI']));
                $index = count($url) - 1;
                $seller_inventory_key = $url[$index] ?? null;
                if(!empty($seller_inventory_key)) {
                    $v = new Vehicle();
                    $vehicle = $v->getByAdKey($seller_inventory_key, false, "seller_inventory_key");
                    if(!empty($vehicle)) {
                        wp_redirect(get_permalink($vehicle->ID));
                        exit;
                    }
                }
            }

        });

    }


    /**
     * @return void
     */
    public function handleRequests() {

        // reset filter manually
        if (isset($_GET['resetFilter'])) {
            FilterTable::getInstance()->resetFilterTable();
        }

        // create Filter Table if not exists
        $ft = FilterTable::getInstance();
        if(!$ft->tableExists()) $ft->resetFilterTable();


        /** ------------------------------------------------------------------------------------------------------------
         * CarShotsPRO
         */
        // API initialisieren
        $api = CarShotsPROApi::getInstance();
        $api->initApi();

        if(isset($_GET['csp_create_job'])) {
            $response = CarShotsPROApi::getInstance()->createJob($_GET['csp_create_job'], true);
            wpcs_p($response);
            exit;
        }

        /** ------------------------------------------------------------------------------------------------------------
         * SYNC
         */
        // check & start sync
        Sync::getInstance()->_init();

        // aufruf der queue verarbeitung
        if(isset($_REQUEST['wpcs_process_queue'])) {
            Queue::getInstance()->_process();
            exit;
        }

        //
        if(isset($_GET['wpcs_clean_images'])) {
            Cleaner::getInstance()->cleanup_vehicle_dir();
            exit;
        }

        /**
         * INFO
         */
        if(isset($_GET['wpcs_info'])) {
            $info = [
                'vehicle_count' => VehicleFilter::getCount()
            ];
            print json_encode($info);
            exit;
        }

    }


}