<?php
/*
Plugin Name: WPCarSync
Plugin URI: https://www.wpcarsync.de
Description: Der Wordpress Standard im Bereich Automotive
Version: 3.5.3
Author: DXMedia
Author URI: https://www.dxmedia.de
*/

namespace DXIM;


use DXIM\theme\Theme;

class WPCarSync {

    protected static $instance = null;

    /**
     *
     * @return WPCarSync
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     *
     */
    public function __construct() {
        // lade plugins
        do_action(DXIM_CORE_LOAD_ADDONS);

        Admin::getInstance();

        Hook::getInstance()->registerHooks();
        Theme::getInstance()->init();

        VehicleFields::getInstance()->init();
        Filter::getInstance()->init();

        CarCloud::getInstance();
        Session::getInstance();
        PostType::getInstance();
        GlobalFilter::getInstance();
        VehicleData::getInstance();
        VehicleSearchRequest::getInstance();
        VehicleCF::getInstance();
        VehicleFilter::getInstance();
        VehicleHooks::getInstance();
        Shortcode::getInstance();
        Template::getInstance();
        Form::getInstance();
        Compare::getInstance();
        Park::getInstance();
        Customizer::getInstance();
        TextSearch::getInstance();
        Sync::getInstance();
        Queue::getInstance();
        Lead::getInstance()->init();
        StructuredData::getInstance()->init();
        Footnote::getInstance()->init();

        Hook::getInstance()->handleRequests();

        /**
         * Updater
         */
        if (Admin::getInstance()->_updates()) {
            new \Updater('WPCarSync', DXIM_PluginVersion);
        }

        /**
         * Paypal
         */
        $options = Admin::getInstance()->getOptions('paypal');
        $mode = $options['mode'] ?? "";
        if(!empty($mode) && $mode != "#") {
            require_once DXIM_PluginPath.'/classes/payment/paypal/Paypal.php';
            \DXIM\Paypal::getInstance();
            $token = $mode == "sandbox" ? $options['sandbox_client_id'] : $options['live_client_id'];
            define('WPCS_PAYPAL_CLIENT_ID', $token);
        }
    }
}

// ----------------------------------------------------------

define('DXIM_PluginVersion', '3.5.3');

define('DXIM_PluginPath', dirname(__FILE__));

define('DXIM_PluginUrl', plugins_url('', __FILE__));

define('DXIM_PluginWebAddress', 'https://www.wpcarsync.de');

define('DXIM_PluginWebAddressPro', 'https://www.wpcarsync.de/pro');

define('DXIM_PluginWebsiteName', 'wpcarsync.de');

define('DXIM_PluginDocumentation', DXIM_PluginWebAddress . '/dokumentation/');

define('DXIM_PluginContact', DXIM_PluginWebAddress . '/kontakt/');

define('DXIM_PluginName', 'WPCarSync');

define('DXIM_PluginLogo', DXIM_PluginUrl . '/img/logo_black.png');

define('WPCS_UPLOADS_DIR', ABSPATH . 'wp-content/uploads/wpcarsync');

define('WPCS_UPLOADS_URL', network_site_url('/wp-content/uploads') . '/wpcarsync');

define('DXIM_SELECT_NOTHING_SELECTED_TEXT', ' - - - ');

define('CARCLOUD', true);
define('CARSHOTSPRO', true);

require('functions.php');
require('hooks.php');
require('Hook.php');
require 'plugins/fpdf/fpdf.php';

require('classes/admin/Admin.php');
require('classes/admin/AdminSetting.php');
require('classes/admin/Conf.php');

require('classes/AddOn.php');
require('classes/data/Features.php');
require('classes/data/Category.php');
require('classes/data/Make.php');
require('classes/data/Model.php');
require('classes/data/VehicleClass.php');
require('classes/data/Select2Data.php');

require('classes/Mail.php');
require('classes/Updater.php');
require('classes/Html.php');
require('classes/Template.php');
require('classes/CustomTemplate.php');
require('classes/Account.php');
require('classes/PostType.php');
require('classes/Customizer.php');
require('classes/Compare.php');
require('classes/Park.php');
require('classes/TextSearch.php');
require('classes/Session.php');
require('classes/Location.php');
require('classes/Lead.php');
require('classes/StructuredData.php');
require('classes/Footnote.php');

require('classes/Vehicle/Vehicle.php');
require('classes/Vehicle/VehicleFields.php');
require('classes/Vehicle/VehicleData.php');
require('classes/Vehicle/VehicleEmission.php');
require('classes/Vehicle/VehicleCF.php');
require('classes/Vehicle/VehicleHtml.php');
require('classes/Vehicle/VehicleFilter.php');
require('classes/Vehicle/VehicleHooks.php');
require('classes/Vehicle/VehiclePDF.php');
require('classes/Vehicle/VehicleSearchRequest.php');
require('classes/Vehicle/VehicleSchema.php');

require('classes/filter/Filter.php');
require('classes/filter/GlobalFilter.php');
require('classes/filter/FilterTable.php');
require('classes/filter/FilterElementColor.php');
require('classes/filter/FilterElementMultipleCheckbox.php');
require('classes/filter/FilterElementRadio.php');
require('classes/filter/FilterElementSlider.php');
require('classes/filter/FilterElementRange.php');
require('classes/filter/FilterElementSelect.php');

require('classes/contactform/Form.php');
require('classes/contactform/FormContext.php');

require('classes/XML.php');
require('classes/Shortcode.php');
require('classes/FilterForm.php');
require('classes/WPCSString.php');

require('classes/import/MediaImport.php');
require('classes/import/FilesystemImport.php');
require('classes/import/mobile_de/MOBILEAdapter.php');
require('classes/import/csv/CSVAdapter.php');
require('classes/import/csv/CSVExtendedImportHooks.php');
require('classes/import/car_cloud/CarCloud.php');
require('classes/import/car_cloud/CarCloudAdapter.php');
require('classes/import/Delete.php');
require('classes/import/Queue.php');
require('classes/import/Import.php');
require('classes/import/Sync.php');
require('classes/import/Cleaner.php');

require('classes/carshotspro/CarShotsPRO.php');
require('classes/carshotspro/CarShotsPROApi.php');
require('classes/carshotspro/CarShotsPROImport.php');

/**
 * THEME
 */
require('classes/theme/Theme.php');
require('classes/theme/TaxCF.php');
require('classes/theme/UserCF.php');
require('classes/theme/PTCF.php');

require('classes/theme/division/Division.php');
require('classes/theme/division/DivisionData.php');
require('classes/theme/division/DivisionTemplate.php');
require('classes/theme/division/DivisionShortcode.php');

require('classes/theme/person/Person.php');
require('classes/theme/person/PersonTemplate.php');
require('classes/theme/person/PersonShortcode.php');
require('classes/theme/person/PersonData.php');

require('classes/theme/brand/Brand.php');
require('classes/theme/brand/BrandData.php');
require('classes/theme/brand/BrandTemplate.php');
require('classes/theme/brand/BrandShortcode.php');

require('classes/theme/model/Model.php');
require('classes/theme/model/ModelTemplate.php');
require('classes/theme/model/ModelShortcode.php');
require('classes/theme/model/ModelData.php');

require('classes/theme/location/Location.php');
require('classes/theme/location/LocationTemplate.php');
require('classes/theme/location/LocationShortcode.php');
require('classes/theme/location/LocationData.php');
require('classes/theme/location/LocationSchema.php');

require('classes/theme/tag/Tag.php');
require('classes/theme/tag/TagData.php');
require('classes/theme/tag/TagShortcode.php');

require('classes/theme/offer/Offer.php');
require('classes/theme/offer/OfferData.php');
require('classes/theme/offer/OfferShortcode.php');
require('classes/theme/offer/OfferTemplate.php');

require('classes/theme/offergroup/OfferGroup.php');
require('classes/theme/offergroup/OfferGroupData.php');
require('classes/theme/offergroup/OfferGroupShortcode.php');
require('classes/theme/offergroup/OfferGroupTemplate.php');

require('classes/theme/job/Job.php');
require('classes/theme/job/JobData.php');
require('classes/theme/job/JobShortcode.php');
require('classes/theme/job/JobTemplate.php');
require('classes/theme/job/JobSchema.php');

require('classes/theme/service/Service.php');
require('classes/theme/service/ServiceData.php');
require('classes/theme/service/ServiceShortcode.php');
require('classes/theme/service/ServiceTemplate.php');

// start Plugin!
add_action('init', function () {
    WPCarSync::getInstance();
});
