<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM;


class Account {

	protected static $instance = null;
	protected $fieldname = 'accounts';

	protected $data = [
		'name' => null, 'password' => null, 'location' => null
	];

	public $account_id = null;


	/**
	 *
	 * @return Account
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	public function __construct() {

	}


	/**
	 *
	 * @return string
	 */
	private function _get_basename() {
		return 'dxim['.$this->fieldname.']['.$this->account_id.']';
	}


	/**
	 * @param $delete_id
	 * @param $options
	 */
	public function delete_account($delete_id, &$options) {
		$accounts = $this->get_accounts();
		$new_accounts = [];

		foreach($accounts as $account_id => $account) {
			if($delete_id == $account_id) continue;
			$new_accounts[$account_id] = $account;
		}

		$options[$this->fieldname] = $new_accounts;
	}

    /**
     * @return array|mixed|void
     */
    private function _load_accounts() {
        $options = Admin::getInstance()->getOptions();
        $accounts = $options[$this->fieldname] ?? [];
        $accounts['delete'] = ['type' => 'delete', 'user' => 'delete', 'name' => 'Fahrzeugbereinigung', 'buildin' => true];

        return $accounts;
    }

	/**
	 * liefert alle Accounts
	 *
	 * @return string
	 */
	public function get_accounts($type = null) {
        $accounts = $this->_load_accounts();

		if($type != null) {
		    foreach($accounts as $account_id => $tmp) {
		        if($tmp['type'] != $type) unset($accounts[$account_id]);
            }
        }
		return !empty($accounts) ? $accounts : [];
	}

	/**
	 * liefert einen bestimmten Account
	 * @param $account_id
	 * @param array $accounts
	 */
	public function get_account($account_id, $accounts = []) {
		if(empty($accounts)) {
			$accounts = $this->_load_accounts();
		}
		return $accounts[$account_id] ?? ['type' => null];
	}


    /**
     * @param $account_id
     * @return bool
     */
    public function is_mobile_de($account_id) {
        $account = $this->get_account($account_id);
        return $account['type'] == "mobile.de";
    }

    /**
     * @param $account_id
     * @return bool
     */
    public function is_csv($account_id) {
        $account = $this->get_account($account_id);
        return $account['type'] == "csv";
    }

    /**
     * @param $account_id
     * @return bool
     */
    public function is_carcloud($account_id) {
        $account = $this->get_account($account_id);
        return $account['type'] == "carcloud";
    }


    /**
     * @param $account_id
     * @return mixed|string|void
     */
    public function getImageConfig($account_id) {
        $options = Admin::getInstance()->getOptions();
        if($this->is_mobile_de($account_id)) {
            $val = $options['api_options']['import_images'];
            if($val == "none") return "links";
            elseif($val == "all_filesystem") return "filesystem";
            elseif($val == "all_media") return "media";
        }
        elseif($this->is_csv($account_id)) {
            return "filesystem";
        }
        elseif($this->is_carcloud($account_id)) {
            return $options['carcloud']['images'];
        }
    }


}