<?php

namespace DXIM;

class Compare {


    protected static $instance = null;
    protected $comparisan_values = [];

    /**
     *
     * @return Compare
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Compare constructor.
     */
    public function __construct() {
        add_action('wp_ajax_wpcs_compare', array($this,'_ajax'));
        add_action('wp_ajax_nopriv_wpcs_compare', array($this,'_ajax'));
    }

    /**
     * @return int
     */
    public function getCount() {
        return count($this->getVehiclesToCompare());
        #return is_array(Session::val('compare')) ? count(Session::val('compare')) : 0;
    }

    /**
     * @return string
     */
    public function drawCount() {
        return '<span class="wpcs_count_vehicles_compared">'.$this->getCount().'</span>';
    }


    /**
     *
     */
    public function _ajax() {
        $response = [
            'action' => $_POST['action']
        ];

        // check vehicle id
        $vehicle_id = intval($_POST['vehicle_id']);
        if($vehicle_id < 1) {
            $response['error'] = 1;
            $response['msg'] = 'No vehicle id';
        }

        // remove from list
        elseif($_POST['todo'] == 'removeFromList') {
            $this->removeFromList($vehicle_id);
        }

        // add to list
        elseif($_POST['todo'] == 'addToList') {
            $this->addToList($vehicle_id);
        }

        $count = $this->getCount();
        $response['count'] = $count;

        // return JSON
        header('Content-Type: application/json');
        die(json_encode($response));
    }


    /**
     * @param $data
     * @param $ids
     * @return string
     */
    private function draw_data_table($data, $ids) {
        $count = count($ids);
        $colspan = $count + 1;

        $html = '<div id="wpcs_vehicle_compare">';
        $html .= '<table id="wpcs_compare_table" class="wpcs_table">'."\n";

        foreach($data as $section => $tmp) {

            $tmp['type'] = $tmp['type'] ?? null;
            $tmp['label'] = $tmp['label'] ?? null;

            // wenn alle cols einer Zeile leer => überspringen
            if($tmp['type'] !== "header") {
                $one_matched = false;
                $z = 0;
                foreach($tmp['cols'] as $tmpVal) {
                    if(!empty($tmpVal)) $one_matched = true;
                    else {
                        if(substr($section,0,8) == "feature_") $tmp['cols'][$z] = '<span class="dashicons dashicons-no"></span>';
                    }
                    $z++;
                }
                if(!$one_matched) continue;
            }

            $depends = $tmp['depends'] ?? null;
            $hidden = $depends !== null ? "wpcs_hidden" : null;
            $html .= '<tr class="'.$section.' '.$hidden.'" rel="'.$depends.'">'."\n";

            //
            if($tmp['type'] == "header") {
                $html .= '<td colspan="'.$colspan.'" class="header"><a href="#" class="section_link closed" rel="'.$section.'">'.$tmp['label']."</a></td>\n";
            } else {
                $html .= '<td>'.$tmp['label']."</td>\n";
            }


            if(isset($tmp['cols'])) {
                #wpcs_p($tmp['cols']);
                for($a = 0; $a < $count; $a++) {
                    $vehicleData = $tmp['cols'][$a] ?? 'leer';
                    $html .= '<td data-vehicle-id="'.$ids[$a].'">';
                    $html .= $vehicleData;
                    $html .= "</td>\n";
                }
            }
            $html .= '</tr>'."\n\n";
        }

        $html .= '</table>'."\n";
        $html .= '</div>'."\n";

        return $html;
    }

    /**
     * @return array
     */
    public function getVehiclesToCompare() {
        $compare_array = Session::val('compare');
        if(!is_array($compare_array)) $compare_array = [];
        $ids = array_keys($compare_array);
        if(empty($ids)) return [];

        $q = new \WP_Query([
            'post_type' => 'vehicle', 'post__in' => $ids, 'posts_per_page' => -1
        ]);

        if($q->post_count > 0) return $q->posts;
        else return [];
    }

    /**
     * @return string
     */

    public function drawVehicleComparison() {
        $vehicles = $this->getVehiclesToCompare();
        $data =[];
        $ids = [];

        if(empty($vehicles)) return "<h3>Keine Fahrzeuge zum Vergleichen ausgewählt.</h3>";

        $fields = Admin::getInstance()->getOptions('active_fields_single');
        $vcf = new VehicleCF();

        foreach($vehicles as $vehicle) {
            $vehicle = VehicleData::initDataObject($vehicle);
            $ids[] = $vehicle->ID;

            // images
            $url = get_permalink($vehicle->ID);
            $data['images']['cols'][] = '<a href="'.$url.'"><img src="'.VehicleData::getVehicleImages($vehicle)['large'][0].'" alt="'.$vehicle->post_title.'" /></a>';

            // buttons
            $data['button']['cols'][] = Html::drawButton(
                ['label' => '<span class="wpcs_icon dashicons dashicons-no-alt"></span>', 'url' => '#', 'classes' => 'wpcs_compare_table_remove_btn','ajax' => 1],
                ['data-vehicle-id' => $vehicle->ID]
            );

            // titles
            $data['title']['label'] = "Titel";
            $data['title']['cols'][] = VehicleHtml::drawVehicleArchiveTitle($vehicle);

            // price
            $price = VehicleData::getVehiclePrice($vehicle);
            $data['price']['label'] = "Preis";
            $data['price']['cols'][] = $price['display_price_brutto'];

            $data['price_netto']['label'] = "Preis netto";
            $data['price_netto']['cols'][] = $price['display_price_netto'];

            $data['vatable']['label'] = "MwSt. ausweisbar";
            $data['vatable']['cols'][] = $price['vatable'] ? "Ja (".($price['vat_rate']*100)."%)" : "Nein";


            // specifics
            $data['fields_header'] = ['label' => 'Fahrzeugdaten', 'type' => 'header'];
            $z = 0;
            foreach($fields as $fieldname) {
                $value = $vcf->getValue($vehicle, $fieldname);
                $label = $vcf->getFieldLabel($fieldname);

                $data['fields_'.$z]['label'] = $label;
                $data['fields_'.$z]['depends'] = "fields_header";
                $data['fields_'.$z]['cols'][] = $value;
                $z++;
            }

            // features
            $features = VehicleData::getVehicleFeatures($vehicle, ['layout'=>'grouped']);
            foreach(Features::getFeaturesGrouped() as $group_slug => $allFeatures) {

                $data['features_'.$group_slug] = ['label' => Features::$featureCategories[$group_slug], 'type' => 'header'];

                foreach($allFeatures as $key => $label) {

                    $data['feature_'.$key]['label'] = $label;
                    $data['feature_'.$key]['depends'] = "features_".$group_slug;
                    $data['feature_'.$key]['cols'][] = isset($features[$group_slug][strtolower($key)]) ? '<span class="dashicons dashicons-saved"></span>' : null;
                }
            }

            // Verbrauch uns Emission
            $emission = VehicleData::getVehicleEmission($vehicle);
            $data['emission_header'] = ['label' => "Verbrauch & Emission", 'type' => "header"];
            $data['emission']['depends'] = "emission_header";
            $data['emission']['cols'][] = VehicleHtml::drawVehicleEmissionData($vehicle);
            $data['emission_label']['depends'] = "emission_header";
            $data['emission_label']['cols'][] = VehicleHtml::drawVehicleEecImage($vehicle);

        }

        return $this->draw_data_table($data, $ids);
    }

    /**
     * @param $vehicle_id
     */
    public function addToList($vehicle_id) {
        $array = Session::val('compare');
        $array[$vehicle_id] = true;
        Session::set('compare',$array);
    }

    /**
     * @param $vehicle_id
     */
    public function removeFromList($vehicle_id) {
        $array = Session::val('compare');
        unset($array[$vehicle_id]);
        Session::set('compare',$array);
    }

    /**
     * @param $vehicle_id
     * @return bool
     */
    public function is_in_list($vehicle_id) {
        $list = Session::val('compare');
        return isset($list[$vehicle_id]) ? true : false;
    }

}