<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 12/09/16
 * Time: 12:30
 */

namespace DXIM;

class CustomTemplate {

    protected static $instance = null;

    /**
     *
     * @return CustomTemplate
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /** ------------------
     * get template directory
     * @return false|string
     */
    public function get_custom_template_dir() {
//        $dir = wp_upload_dir();
//        $templ_dir = $dir['basedir']."/wpcarsync/templates";
        $templ_dir = WPCS_UPLOADS_DIR."/templates";

        if(!is_dir($templ_dir)) {
            if(!mkdir($templ_dir,0777,true)) {
                return false;
            }
        }

        return $templ_dir;
    }


    public function get_template_file($string, $path = null) {
        $info = pathinfo($string);
        if(!isset($info['extension']) || empty($info['extension'])) $info['extension'] = "php";
        $filename = $info['filename'].".".$info['extension'];

        // path nicht angegeben => WPCS PAth verwenden
        if($path == null) $path = DXIM_PluginPath.'/tpl/';
        // Path angegeben => AddOn Path verwenden
        else $path = trailingslashit($path);

        if(!empty($info['dirname']) && $info['dirname'] != ".") $path .= $info['dirname']."/";

        $cpath = trailingslashit($this->get_custom_template_dir()).$info['dirname']."/";

        if(is_file($cpath.$filename)) return $cpath.$filename;
        elseif (is_file($path.$filename)) return $path.$filename;

        return "";
    }

    /**
     * @param $string
     * @return false|string
     */
    public function get_template_content($string, $data = null) {
        $file = $this->get_template_file($string);
        if(!is_file($file)) return "Template Datei ".$string." nicht gefunden";
        ob_start();
        try {
            if(is_file($file)) require($file);
        } catch (\Throwable $e) {
            print 'Fehler in WPCarSync Template Datei: <strong>'.$string.'</strong> in Zeile '.$e->getLine().': <span class="wpcs_error">'.$e->getMessage().'</span>';
        }
        return ob_get_clean();
    }


    /**
     * @param $string
     * @param $addon_path
     * @return false|string
     */
    public function get_addon_template_content($string, $addon_path, $data = null) {
        $addon_path .= '/tpl';
        $file = $this->get_template_file($string, $addon_path);
        ob_start();
        if(is_file($file)) require($file);
        return ob_get_clean();
    }


    /** ------------------------------------------------------------------------------
     * abwärtskompatibel zu alter template struktur
     * bewegt alte Templates unter neuem namen in die richtigen Ordner
     * function läuft nur beim speichern der admin settings
     * @return void
     */
    public function convert_old_templates() {
        $options = Admin::getInstance()->getOptions();
        $layout = $options['default_layout_archive'];
        $loop_layout = $options['archive_vehicle_layout'];
        $dir = $this->get_custom_template_dir();
        if(!is_dir($dir)) return false;

        $d = dir($dir);
        while (false !== ($entry = $d->read())) {
            if(substr($entry,-4) !== ".php") continue;
            if($entry == "archive.php") $this->_move_template($dir.'/'.$entry,$dir.'/archive','archive_'.str_replace("layout_",null, $layout).".php");
            elseif($entry == "single.php") $this->_move_template($dir.'/'.$entry,$dir.'/single','single.php');
            elseif(strpos($entry, 'on_archive_default') !== false) $this->_move_template($dir.'/'.$entry,$dir.'/loop',$loop_layout.'.php');
            elseif(strpos($entry, 'on_archive_') !== false) $this->_move_template($dir.'/'.$entry,$dir.'/loop',str_replace('on_archive_',null,$entry));
        }
        $d->close();

        delete_option('wpcs_templates');
    }

    private function _move_template($file, $new_dir, $new_name) {
        if(!is_dir($new_dir) && !mkdir($new_dir,0777)) return false;
        rename($file,$new_dir."/".$new_name);
    }

}