<?php

namespace DXIM;

class Footnote {

	protected static $instance = null;

    public static $post_type = "wpcs_footnote";

    protected $footnotes = [];
    protected $footnotes_info = [];

	/**
	 *
	 * @return Footnote
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() { }

    /**
     * @return void
     */
    public function init() {

        $this->_register_post_type();

        add_shortcode('WPCS_footnotes', [&$this, '_draw_footnotes_wrapper']);

        add_action('wp_footer', [&$this,'_generate_footnotes']);
    }

    public function register_footnote($uniq_key, $args = ['pt' => 'wpcs_footnote']) {
        if(!in_array($uniq_key, $this->footnotes)) {
            $this->footnotes[] = $uniq_key;
            $this->footnotes_info[$uniq_key] = $args;
        }
        $key = array_search($uniq_key, $this->footnotes);
        return $key;
    }

    public function _draw_footnotes_wrapper() {
        return '<div id="wpcs_footnotes"></div>';
    }


    /**
     * @return string
     */
    public function _generate_footnotes() {
        $html = '<div id="wpcs_generated_footnotes" class="wpcs_hidden">';

        foreach($this->footnotes as $index => $uniq_key) {
            $nr = ($index+1);
            $info = $this->footnotes_info[$uniq_key] ?? "";
            if(empty($info)) continue;

            if(isset($info['pt'])) {
                $q = new \WP_Query(['p' => $uniq_key, 'post_type' => $info['pt']]);
                $content = $q->posts[0]->post_content ?? "";
            }
            elseif(isset($info['meta_key'])) {
                $content = get_post_meta($info['post_id'], $info['meta_key'], true);
            }

            $html .= '<a name="footnote_'.$nr.'"></a>';
            $html .= '<p>'.$nr.' : '. $content.'</p>';
        }
        $html .= '</div>';

        print $html;
    }

    /**
     * @param $data
     * @return string
     */
    public function draw_footnote_links($data, $limit = []) {
        if(empty($data)) return "";

        $footnotes = [];
        foreach($data as $category => $tmp) {
            if(!empty($limit) && !in_array($category, $limit)) continue;
            foreach($tmp as $footnote) {
                if(!in_array($footnote, $footnotes)) $footnotes[] = $footnote;
            }
        }

        natsort($footnotes);
        if(empty($footnotes)) return "";

        $html = ' <sup class="wpcs_footnote_links">';
        $z = 0;
        foreach($footnotes as $index) {
            $nr = ($index + 1);
            if($z != 0) $html .= ",";
            $html .= '<a href="#footnote_'.$nr.'">'.$nr.'</a>';
            $z++;
        }
        $html .= '</sup>';

        return $html;
    }

    public function _register_post_type() {

        $labels = array(
            'name'                  => _x( 'Fußnote', 'Post Type General Name', 'wpcs' ),
            'singular_name'         => _x( 'Fußnote', 'Post Type Singular Name', 'wpcs' ),
            'menu_name'             => __( 'Fußnoten', 'wpcs' ),
            'name_admin_bar'        => __( 'Fußnoten', 'wpcs' ),
            'archives'              => __( 'Fußnoten', 'wpcs' ),
            'attributes'            => __( 'Fußnoten Attributes', 'wpcs' ),
            'parent_item_colon'     => __( 'Parent Fußnote', 'wpcs' ),
            'all_items'             => __( 'Alle Fußnoten', 'wpcs' ),
            'add_new_item'          => __( 'neue Fußnote', 'wpcs' ),
            'add_new'               => __( 'anlegen', 'wpcs' ),
            'new_item'              => __( 'anlegen', 'wpcs' ),
            'edit_item'             => __( 'editieren', 'wpcs' ),
            'update_item'           => __( 'update', 'wpcs' ),
            'view_item'             => __( 'anzeigen', 'wpcs' ),
            'view_items'            => __( 'anzeigen', 'wpcs' ),
            'search_items'          => __( 'suchen', 'wpcs' ),
            'not_found'             => __( 'nicht gefunden', 'wpcs' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'wpcs' ),
            'featured_image'        => __( 'Featured Image', 'wpcs' ),
            'set_featured_image'    => __( 'Set featured image', 'wpcs' ),
            'remove_featured_image' => __( 'Remove featured image', 'wpcs' ),
            'use_featured_image'    => __( 'Use as featured image', 'wpcs' ),
            'insert_into_item'      => __( 'Insert into item', 'wpcs' ),
            'uploaded_to_this_item' => __( 'Uploaded to this item', 'wpcs' ),
            'items_list'            => __( 'Items list', 'wpcs' ),
            'items_list_navigation' => __( 'Items list navigation', 'wpcs' ),
            'filter_items_list'     => __( 'Filter items list', 'wpcs' ),
        );


        $args = array(
            'label'                 => __( 'Fußnoten', 'wpcs' ),
            'description'           => __( 'Fußnoten', 'wpcs' ),
            'labels'                => $labels,
            'supports'              => array( 'title', 'editor' ),
            'taxonomies'            => array( ),
            'hierarchical'          => false,
            'public'                => false,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'menu_position'         => 5,
            'show_in_admin_bar'     => false,
            'show_in_nav_menus'     => false,
            'can_export'            => true,
            'has_archive'           => false,
            'exclude_from_search'   => true,
            'publicly_queryable'    => false,
            'rewrite'               => null,
            'capability_type'       => 'page',
        );

        register_post_type( self::$post_type, $args );

    }

}