<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 12/09/16
 * Time: 12:11
 */

namespace DXIM;

class Html {


    /**
     * -----------------------------------------------------------------------------------------------------------------
     * PAGINATION
     */
    public static function pagination() {
        $separator = apply_filters(DXIM_FILTER_PAGINATION_SEPARATOR,'<span class="sep">/</span>');
        $num_pages = VehicleFilter::getNumPages();

        $formData = \DXIM\VehicleFilter::getInstance()->getFormData();
        if(!isset($formData['wpcsp'])) $formData['wpcsp'] = null;
        $page = intval($formData['wpcsp']) > 0 ? $formData['wpcsp'] : 1;
        unset($formData['wpcsp']);


        $html = null;
        $html .= '<div class="dxim_pagination">';

        if($page > 1) {
            $html .= '<a href="#" data-wpcsp="'.($page-1).'" class="page-numbers prev"><span class="wpcs_icon dashicons dashicons-arrow-left-alt2"></span></a>';
        }

        $page_nr = $page - 2;
        $pages = 0;

        if($page_nr < 1) $page_nr = 1;

        if($page_nr > 1) {

            $html .= '<a href="" data-wpcsp="1" class="page-numbers">1</a>';
            $pages++;

            if($page_nr > 2) {
                $html .= $separator;
                $html .= '<span class="page-numbers dots">...</span>';
                $pages++;
            }
        }

        for($a = 1; $a <= 5; $a++, $page_nr++, $pages++) {
            if($num_pages < 2) break;

            if($page_nr > $num_pages) break;

            if($pages > 0) $html .= $separator;

            if($page_nr == $page) {
                $html .= '<span class="page-numbers current">'.$page_nr.'</span>';
            } else {
                $html .= '<a href="#" data-wpcsp="'.$page_nr.'" class="page-numbers">'.$page_nr.'</a>';
            }
        }

        if(($page_nr+1) < $num_pages) {
            $html .= $separator;
            $html .= '<span class="page-numbers dots">...</span>';
            $pages++;
        }

        if($page_nr < $num_pages) {
            $html .= $separator;
            $html .= '<a href="#" data-wpcsp="'.$num_pages.'" class="page-numbers">'.$num_pages.'</a>';
            $pages++;
        }


        if($page < $num_pages) {
            $html .= '<a href="#" data-wpcsp="'.($page+1).'" class="page-numbers next"><span class="wpcs_icon dashicons dashicons-arrow-right-alt2"></span></a>';
        }

        $html .= '</div>';

        if(!wp_doing_ajax()) $html .= VehicleFilter::getInstance()->drawFormHiddens();

        return $html;
    }

    /**
     * -----------------------------------------------------------------------------------------------------------------
     * PAGINATION
     */
    public static function load_more_button($args) {

        if(!is_array($args)) $args = [];
        if(!isset($args['button_label'])) $args['button_label'] = WPCSString::getString('labels_archive_pagination_button');

        $html =  Html::drawButton(
            ['url' => '#', 'label' => $args['button_label']],
            ['id' => 'wpcs_pagination_button']
        );

        if(!wp_doing_ajax()) $html .= VehicleFilter::getInstance()->drawFormHiddens();

        return $html;
    }


    /**
     * @param $label
     * @param $value
     * @param $key
     * @param $cf // all custom fields
     * @return null|string
     */
    public static function vehicle_fact($array) {
        $html = null;
        if(!empty($array['display_value'])) {
            if(is_array($array['display_value'])) {
                $display_value = null;
                foreach($array['display_value'] as $tmp) {
                    if($display_value !== null) $display_value .= ', ';
                    $display_value .= $tmp;
                }
            }
            else $display_value = $array['display_value'];
            $html .= '<div class="label '.str_replace('specific_','',$array['key']).'">'.apply_filters(DXIM_FILTER_SPECIFICS_LABEL,$array['label'],$array).'</div>';
            $html .= '<div class="fact '.str_replace('specific_','',$array['key']).'">'.$display_value.'</div>';
        }
        return $html;
    }


    /**
     * @param $letter
     * @return string
     */
    public static function getEnergieLabelUrl($letter) {
        $letter = strtolower($letter);
        $srcArray = ['a+'=>'aa','a++'=>'aaa','a+++'=>'aaaa'];
        $src = isset($srcArray[$letter]) ? $srcArray[$letter] : $letter;
        if(empty($src)) return null;
        return DXIM_PluginPath.'/img/labels/'.$src.'.jpg';
    }

    /**
     * @param $args
     * @param array $atts
     * @return string
     */
    public static function drawButton($args, $atts = []) {
        $atts_string = null;
        foreach($atts as $att => $val) { $atts_string .= $att.'="'.$val.'" '; }

        if(!isset($args['classes'])) $args['classes'] = null;

        // ajax classes
        if(isset($args['ajax'])) {
            $args['label'] = '<span class="wpcs_icon wpcs_icon_spin wpcs_icon_spinner dashicons dashicons-update"></span> '.$args['label'];
            $args['classes'] .= ' ajax_btn';
        }

        // default classes
	    $args['classes'] .= ' dxim_button';

	    // Styling
        if(isset($args['lightButton'])) {
            $args['classes'] .= " dxim_button_light";
        } else {
            $mods = Customizer::getInstance()->getMods();
            $args['classes'] .= !empty($mods['button_classes']) ? " ".$mods['button_classes'] : ' dxim_button_styled';
        }

	    // return
        return '<a href="'.$args['url'].'" class="'.$args['classes'].'" '.$atts_string.'>'.$args['label'].'</a>';
    }

    /**
     * @param $args
     * @param array $atts
     * @return string
     */
    public static function drawLightButton($args, $atts = []) {
        $args['lightButton'] = 1;
        return self::drawButton($args, $atts);
    }


    /**
     * @param $string
     * @return string|string[]|null
     */
    public static function sanitize_title($string) {
        $search = [' ','ä','ö','ü','ß'];
        $replace = ['-','ae','oe','ue','ss'];
        $string = str_replace($search,$replace,$string);
        $string = preg_replace('/[^-0-9a-zA-Z_\s]/', '', $string);
        return $string;
    }

    /**
     * @return string
     */
    public static function getBackButtonUrl() {
        $args = \DXIM\Session::val('last_vehicle_archive');
        return $args;
    }

    /**
     * @param $args
     * @return string
     */
    public static function linkToArchive($args) {
        $link = Conf::getInstance()->getVehicleArchiveUrl();
        $string = http_build_query($args);
        return $link."?".$string;
    }

    /**
     * @param $args
     * @return string
     * @return string
     */
    public static function select($args) {
        $values = $args['values'] ?? [];
        $class = $args['class'] ?? "custom";
        $keys = array_keys($values);
        $selected = $args['selected'] ?? $keys[0];
        $name = $args['name'] ?? null;
        $label = $values[$selected];

        $html = '<div class="wpcsselect '.$class.'" data-name="'.$name.'">';
            $html .= '<div class="wpcs_select_handle">'.$label.'</div>';
            $html .= '<div class="wpcs_select_loading"></div>';
            $html .= '<div class="wpcs_select_arrow"></div>';
            $html .= '<div class="data">';
            foreach ($values as $val => $tmp) {
                $html .= '<div data-value="' . $val . '">' . $tmp . '</div>';
            }
            $html .= '</div>';

            #id="wpcs_select_value_'.$name.'"
            $html .= '<input type="hidden" name="'.$name.'" value="'.$selected.'" class="wpcs_select_value" />';
        $html .= '</div>';

        return $html;
    }

    /**
     * @param array $labels
     * @param array $contents
     * @param array $args
     * @return string
     */
    public static function tabs(array $labels, array $contents, array $args = []) {
        $id = $args['id'] ?? uniqid();

        $html = '<div id="'.$id.'" class="wpcs_tabs">';

            // Header
            $html .= '<div class="wpcs_tabs_header">';
            $z = 0;
            foreach($labels as $label) {
                $active = ($z == 0) ? "active" : "";
                $html .= '<a href="#" class="wpcs_tab_control '.$active.'">'.$label.'</a>';
                $z++;
            }
            $html .= '</div>'; // controls

            // Content
            $html .= '<div class="wpcs_tabs_content">';
                $z = 0;
                foreach($contents as $content) {
                    $active = $z == 0 ? "active" : "";
                    $html .= '<div class="wpcs_tab_content '.$active.'">';
                        $html .= $content;
                    $html .= '</div>';
                    $z++;
                }
            $html .= '</div>'; // content

        $html .= '</div>'; // wpcs_tabs container

        return $html;
    }

    /**
     * @param array $data
     * @param array $args
     * @return string
     */
    public static function accordion(array $data, array $args = []) {
        $defaults = ['active' => 1, 'id' => ""];
        $args = array_merge($defaults, $args);
        $id = $args['id'];
        $active = $args['active'];

        $html = '<div class="wpcs_accordion" id="'.$id.'">';
        $z = 1;
        foreach($data as $tmp) {
            $active = $z == $active ? "active" : "";
            $html .= '<div class="wpcs_accordion_control '.$active.'"><h3>'.$tmp['title'].'</h3></div>';
            $html .= '<div class="wpcs_accordion_content '.$active.'">'.$tmp['content'].'</div>';
            $z++;
        }
        $html .= '</div>'; // wpcs_accordion

        return $html;
    }


    public static function slider(array $slides, array $args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['autoplay' => 0, 'speed' => 5000, 'slides' => 4, 'layout' => 'carousel'];
        $args = array_merge($defaults, $args);

        $items = [];
        if($args['layout'] == 'carousel') {

            foreach($args as $key => $value) {
                if(substr($key,0,11) == 'breakpoint_') {
                    $args['breakpoints'][intval(str_replace("breakpoint_","",$key))] = intval($value);
                    unset($args[$key]);
                }
            }

            $items = [
                'slides' => 4,
                'breakpoints' => [
                    '1200' => 3,
                    '998' => 2,
                    '768' => 1,
                ]
            ];
        }

        if($args['slides'] > 0) $items['slides'] = $args['slides'];
        if(isset($args['breakpoints']) && is_array($args['breakpoints'])) $items['breakpoints'] = $args['breakpoints'];
        if(!isset($items['breakpoints'])) $items['breakpoints'] = [];

        $slide_count = count($slides);
        $per_slide = $items['slides'];
        $show_nav = ($slide_count > $per_slide);

        $html = null;
        $slider_id = uniqid();

        $html .= '<div class="dxim_vehicle_carousel_wrapper '.$args['layout'].'" data-slides="'.$args['slides'].'" id="dxim_carousel_wrapper_'.$slider_id.'" style="display:block; width:100%;">';

            $html .= '<div class="dxim_vehicle_carousel" id="slider_'.$slider_id.'" >';

            // Slides
            foreach($slides as $slide) {
                $html .= '<div class="'.$slider_id.'_slide dxim_carousel_slide ">';
                    $html .= '<div class="inner">';
                    $html .= $slide;
                    $html .= '</div>'; // inner
                $html .= '</div>'; // Slide
            }

            // close slider
            $html .= '</div>';

            // Nav Buttons
            if($show_nav) {
                $html .= '<div class="dxim_control_buttons">';
                $html .= Html::drawButton(
                    ['classes' => 'dxim_control_prev_btn dxim_button','label' => '<span class="wpcs_icon dashicons dashicons-arrow-left-alt2"></span>','url' => '#'],
                    ['id' => 'dxim_carousel_prev_btn_'.$slider_id]
                );
                $html .= Html::drawButton(
                    ['classes' => 'dxim_control_next_btn dxim_button','label' => '<span class="wpcs_icon dashicons dashicons-arrow-right-alt2"></span>','url' => '#'],
                    ['id' => 'dxim_carousel_next_btn_'.$slider_id]
                );
                $html .= '</div>';
            }

        // close wrapper
        $html .= '</div>';

        // responsive arguments
        $responsive = null;
        $z = 1;
        foreach($items['breakpoints'] as $resolution => $count) {
            if($z != 1) $responsive .= ',';
            $responsive .= '{
	            breakpoint : '.$resolution.',
	            settings : { slidesToShow : '.$count.', slidesToScroll : '.$count.' }
	        }';
            $z++;
        }

        $dots = $show_nav ? 'true' : 'false';

        // JS
        $script = "<script>";

            if(!wp_doing_ajax()) $script .= "document.addEventListener('DOMContentLoaded', function(){ ";

            $script .= "jQuery( '#slider_".$slider_id."' ).slick({
                slidesToShow: ".$items['slides'].",
                slidesToScroll: ".$items['slides'].",
                infinite: true,
                dots: ".$dots.",
                appendDots : '#dxim_carousel_wrapper_".$slider_id."',
                prevArrow : '#dxim_carousel_prev_btn_".$slider_id."',
                nextArrow : '#dxim_carousel_next_btn_".$slider_id."',
                responsive: [".$responsive."],
                adaptiveHeight: true,
                autoplay : ".$args['autoplay'].",
                autoplaySpeed : ".$args['speed']."
            });";

            if(!wp_doing_ajax()) $script .= "}, false);";

        $script .= "</script>";


        $html .= $script;

        return $html;
    }


    /**
     * @param $args
     * @return string
     */
    public static function textsearch($args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['placeholder' => null, 'target' => null, 'type' => null];
        $args = array_merge($defaults, $args);

        if(empty($args['type'])) return "unknown search type.";

        $create_results_div = false;
        if(empty($args['target'])) {
            $create_results_div = true;
            $args['target'] = "wpcs_search_results_default_wrapper";
        }

        $params = $args;
        unset($params['placeholder'], $params['target'], $params['type']);

        $html = '<div class="wpcs_textsearch" data-target="'.$args['target'].'" data-type="'.$args['type'].'" data-params="'.esc_attr(json_encode($params)).'">';
            // close
            $html .= '<a href="#" class="wpcs_textsearch_reset_btn"><span class="dashicons dashicons-no-alt"></span></a>';
            // loading
            $html .= FilterForm::getInstance()->drawAjaxLoading(['category' => 'textSearch']);
            // text input
            $html .= '<input type="text" placeholder="'.$args['placeholder'].'" class="searchInput" />';
            // default results div
            if($create_results_div) $html .= '<div class="'.$args['target'].'"></div>';
        $html .= '</div>';

        return $html;
    }


}