<?php

namespace DXIM;

class Lead {

	protected static $instance = null;

    public static $post_type = "wpcs_lead";

    protected $lead_types = [
        'wpcs_location' => ['label' => 'Anfrage an Standort', 'type' => 'taxonomy'],
        'wpcs_offer' => ['label' => 'Anfrage Angebot', 'type' => 'post_type'],
        'wpcs_model' => ['label' => 'Anfrage Modell', 'type' => 'post_type'],
        'vehicle' => ['label' => 'Anfrage Fahrzeug', 'type' => 'post_type'],
        'general' => ['label' => 'Kontaktformular', 'type' => null],
        'wpcs_search_request' => ['label' => 'Suchauftrag erstellt', 'type' => 'post_type'],
        'vehicle_checkout' => ['label' => 'Fahrzeug - Reservierung', 'type' => 'post_type'],
    ];


	/**
	 *
	 * @return Lead
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() { }

    /**
     * @return void
     */
    public function init() {
        $this->_register_post_type();

        add_action('manage_wpcs_lead_posts_custom_column', [&$this, '_columns_content'], 10, 2);
        add_filter('manage_wpcs_lead_posts_columns', [&$this, '_columns']);
        add_action('restrict_manage_posts', [&$this, '_filter_lead_type_ui'],10,2);
        add_action('pre_get_posts',[&$this,'_filter_leads'],10,1);
    }


    /**
     * @param $type
     * @param $post_type
     * @param $object_id
     * @param $args
     * @return false|int|\WP_Error
     */
    public function create($type, $content = "", $post_type = "", $object_id = 0, $meta = []) {
        $post_id = wp_insert_post([
            'post_title' => time(),
            'post_type' => self::$post_type,
            'post_content' => $content,
            'post_status' => 'publish'
        ]);

        if(is_wp_error($post_id) || $post_id < 1) return $post_id;

        update_post_meta($post_id, "type", $type);
        update_post_meta($post_id, "post_type", $post_type);
        update_post_meta($post_id, "object_id", $object_id);

        foreach($meta as $key => $value) {
            update_post_meta($post_id, $key, $value);
        }

        return $post_id;
    }

    /**
     * @param $q
     * @return void
     */
    public function _filter_leads($q) {
        if(!$q->get('post_type') == Lead::$post_type || !is_admin()) return;
        $lead_type = $_GET['lead_type'] ?? "#";
        if($lead_type == "#") return;

        $mq = $q->get('meta_query');
        if(!is_array($mq)) $mq = [];

        if(!empty($lead_type)) {
            $mq[] = ['key' => 'type', 'value' => $lead_type];
            $q->set('meta_query', $mq);
        }

    }


    /**
     * @param $post_type
     * @return void
     */
    public function _filter_lead_type_ui($post_type) {
        if($post_type !== Lead::$post_type) return;

        $lead_type = $_GET['lead_type'] ?? "#";
        $html = '<select name="lead_type">';
            foreach([
                '#' => 'Lead Typ filtern',
                'search_request' => 'Suchauftrag',
                'checkout' => 'Reservierung',
                'contact_form_email' => 'E-Mail Anfrage',
                'contact_form_wpcs_offer' => 'Anfrage Angebot',
                'contact_form_vehicle' => 'Anfrage Fahrzeug',
            ] as $value => $label) {
                $sel = $value == $lead_type ? "selected" : "";
                $html .= '<option value="'.$value.'" '.$sel.'>'.$label.'</option>';
            }
        $html .= '</select>';
        print $html;
    }


    public function _columns($columns) {
        $columns['contact'] = "Kontakt";
        $columns['type'] = "Typ";
        $columns['post_type'] = "Verknüpfung";
        return $columns;
    }

    public function _columns_content($column, $post_id) {
        $meta = get_post_meta($post_id);
        $type = $meta['type'][0] ?? "";
        $post_type = $meta['post_type'][0] ?? "";
        $lead_type = $this->lead_types[$post_type] ?? "";

        // Kontakt
        if($column == "contact") {
            print $meta['email'][0] ?? "";
        }

        // Lead Type
        elseif($column == "type" && !empty($lead_type)) {

            if($lead_type['type'] == "post_type") {
                $link = post_type_exists($post_type) ? 'edit.php?post_type='.$post_type : "";
            }
            elseif($lead_type['type'] == "taxonomy") {
                $link = taxonomy_exists($post_type) ? 'edit-tags.php?taxonomy='.$post_type : "";
            }

            if(!empty($link)) print '<a href="'.$link.'">';
                print $lead_type['label'];
            if(!empty($link)) print "</a>";

        }

        // Objekt
        elseif($column == "post_type" && !empty($lead_type)) {
            $object_id = $meta['object_id'][0] ?? 0;

            if($object_id > 0 && $lead_type['type'] == "post_type") {
                $obj = get_post($object_id);
                if(empty($obj)) print "Objekt existiert nicht mehr";
                else print '<a href="'.get_edit_post_link($obj->ID).'">'.$obj->post_title."</a>";
            }

            elseif($object_id > 0 && $lead_type['type'] == "taxonomy") {
                $term = get_term($object_id, $post_type);
                if(empty($term)) print "Objekt existiert nicht mehr";
                else print '<a href="'.get_edit_term_link($term->term_id).'">'.$term->name."</a>";
            }
        }
    }

    public function _register_post_type() {

        $labels = array(
            'name'                  => _x( 'Leads', 'Post Type General Name', 'wpcs' ),
            'singular_name'         => _x( 'Lead', 'Post Type Singular Name', 'wpcs' ),
            'menu_name'             => __( 'Leads', 'wpcs' ),
            'name_admin_bar'        => __( 'Leads', 'wpcs' ),
            'archives'              => __( 'Leads', 'wpcs' ),
            'attributes'            => __( 'Leads Attributes', 'wpcs' ),
            'parent_item_colon'     => __( 'Parent Lead', 'wpcs' ),
            'all_items'             => __( 'Alle Leads', 'wpcs' ),
            'add_new_item'          => __( 'neuer Lead', 'wpcs' ),
            'add_new'               => __( 'anlegen', 'wpcs' ),
            'new_item'              => __( 'anlegen', 'wpcs' ),
            'edit_item'             => __( 'editieren', 'wpcs' ),
            'update_item'           => __( 'update', 'wpcs' ),
            'view_item'             => __( 'anzeigen', 'wpcs' ),
            'view_items'            => __( 'anzeigen', 'wpcs' ),
            'search_items'          => __( 'suchen', 'wpcs' ),
            'not_found'             => __( 'nicht gefunden', 'wpcs' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'wpcs' ),
            'featured_image'        => __( 'Featured Image', 'wpcs' ),
            'set_featured_image'    => __( 'Set featured image', 'wpcs' ),
            'remove_featured_image' => __( 'Remove featured image', 'wpcs' ),
            'use_featured_image'    => __( 'Use as featured image', 'wpcs' ),
            'insert_into_item'      => __( 'Insert into item', 'wpcs' ),
            'uploaded_to_this_item' => __( 'Uploaded to this item', 'wpcs' ),
            'items_list'            => __( 'Items list', 'wpcs' ),
            'items_list_navigation' => __( 'Items list navigation', 'wpcs' ),
            'filter_items_list'     => __( 'Filter items list', 'wpcs' ),
        );

        $args = array(
            'label'                 => __( 'Leads', 'wpcs' ),
            'description'           => __( 'Leads', 'wpcs' ),
            'labels'                => $labels,
            'supports'              => array( 'title', 'editor' ),
            'hierarchical'          => false,
            'public'                => false,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'menu_position'         => 5,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => false,
            'can_export'            => false,
            'has_archive'           => false,
            'exclude_from_search'   => true,
            'publicly_queryable'    => false,
            'capability_type'       => 'page',
        );

        register_post_type( self::$post_type, $args );

    }
}