<?php

namespace DXIM;

class Park {


    protected static $instance = null;


    /**
     *
     * @return Park
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Compare constructor.
     */
    public function __construct() {
        add_action('wp_ajax_wpcs_park', array($this,'_ajax'));
        add_action('wp_ajax_nopriv_wpcs_park', array($this,'_ajax'));
    }

    /**
     *
     */
    public function _ajax() {
        $response = [
            'action' => $_POST['action']
        ];

        // check vehicle id
        $vehicle_id = intval($_POST['vehicle_id']);
        if($vehicle_id < 1) {
            $response['error'] = 1;
            $response['msg'] = 'No vehicle id';
        }

        // remove from list
        elseif($_POST['todo'] == 'removeFromList') {
            $this->removeFromList($vehicle_id);
        }

        // add to list
        elseif($_POST['todo'] == 'addToList') {
            $this->addToList($vehicle_id);
        }

        $count = $this->getCount();
        $response['count'] = $count;

        $list = $this->getVehiclesListHtml();
        $response['list'] = $list;

        // return JSON
        header('Content-Type: application/json');
        die(json_encode($response));

    }

    /**
     * @return int
     */
    public function getCount() {
        return count($this->getVehiclesParked(false));
    }

    /**
     * @return string
     */
    public function drawCount() {
        return '<span class="wpcs_count_vehicles_parked">'.$this->getCount().'</span>';
    }

    /**
     * @return array
     */
    public function getVehiclesParked($initData = true) {
        $compare_array = Session::val('park');
        if(!is_array($compare_array)) $compare_array = [];
        $ids = array_keys($compare_array);
        if(empty($ids)) return [];

        $q = new \WP_Query([
            'post_type' => 'vehicle', 'post__in' => $ids, 'posts_per_page' => -1
        ]);

        $vehicles = [];

        if($q->post_count > 0) {
            foreach($q->posts as $tmp) {
                if($initData) $tmp = VehicleData::initDataObject($tmp);
                $vehicles[] = $tmp;
            }
        }

        return $vehicles;
    }

    /**
     * @return string
     */
    public function drawVehiclesParked() {

        $html = '<div id="wpcs_vehicle_parked">';

            $html .= '<div class="vehicles_wrapper">';
                $html .= $this->getVehiclesListHtml();
            $html .= '</div>';

            $html .= "<script>if(typeof wpcs_park !== 'undefined') { wpcs_park._setList(); }</script>";

        $html .= '</div>';

        return $html;

    }

    private function getVehiclesListHtml() {

        $vehicles = $this->getVehiclesParked();

        if(empty($vehicles)) {
            return "<h3>Keine Fahrzeuge geparkt</h3>";
        }

        $html = null;

        foreach($vehicles as $vehicle) {
            $html .= '<div class="vehicle" data-vehicle-id="'.$vehicle->ID.'">';
                $html .= FilterForm::getInstance()->drawAjaxLoading();
                $vehicle_html = CustomTemplate::getInstance()->get_template_content('park/loop', ['vehicle' => $vehicle]);

                $html .= apply_filters(DXIM_FILTER_PARKED_VEHICLE,$vehicle_html, $vehicle);
            $html .= '</div>';
        }

        return $html;
    }


    /**
     * @param $vehicle_id
     */
    public function addToList($vehicle_id) {
        $array = Session::val('park');
        $array[$vehicle_id] = true;
        Session::set('park',$array);
    }

    /**
     * @param $vehicle_id
     */
    public function removeFromList($vehicle_id) {
        $array = Session::val('park');
        unset($array[$vehicle_id]);
        Session::set('park',$array);
    }

    /**
     * @param $vehicle_id
     * @return bool
     */
    public function is_in_list($vehicle_id) {
        $list = Session::val('park');
        return isset($list[$vehicle_id]) ? true : false;
    }


}