<?php

namespace DXIM;

class Session {

    // instance
    protected static $instance = null;

    // data to store in session
    protected $data = [];

    // key in session
    protected $name = 'wpcs';

    // user ident key
    protected $ident = null;


    /**
     *
     * @return Session
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }


    /**
     * Session constructor.
     */
    public function __construct() {
        $this->_refresh();
    }

    /**
     *
     */
    private function _refresh() {
        if(!isset($_COOKIE['wpcs_ident'])) {
            $id = uniqid();
            #setcookie('wpcs_ident',$id,time()+60*60*24*30,"/",$_SERVER['HTTP_HOST']);
            $request_scheme = $_SERVER['REQUEST_SCHEME'] ?? "";
            $https = $_SERVER['HTTPS'] ?? "";

            $args = [
                'expires' => time()+60*60*24*30,
                'path' => '/',
                'domain' => $_SERVER['HTTP_HOST'],
            ];

            if($request_scheme == "https" || $https == "on") {
                $args['secure'] = true;
                $args['samesite'] = 'None';
            }

            setcookie('wpcs_ident',$id,$args);
        }
        else $id = $_COOKIE['wpcs_ident'];

        $this->ident = $id;
        $this->data = get_transient($this->name."_".$id);
    }

    /**
     * returns user session ident key
     * @return null
     */
    public static function getIdent() {
        return self::getInstance()->ident;
    }

    /**
     * @param $key
     * @return mixed
     */
    public static function val($key) {
        return isset(self::getInstance()->data[$key]) ? self::getInstance()->data[$key] : null;
    }

    /**
     * @return mixed
     */
    public static function get() {
        return self::getInstance()->data;
    }

    /**
     * @param $key
     * @param $val
     */
    public static function set($key, $val) {
        #print "set ".$key." => ".$val;
        $s = Session::getInstance();
        if(!is_array($s->data)) $s->data = [];
        $s->data[$key] = $val;
        $s->_save();
    }

    /**
     *
     */
    private function _save() {
        #$id = $_COOKIE['wpcs_ident'] ?? null;
        $id = $this->ident;
        if(is_null($id)) return;
        set_transient($this->name."_".$id,$this->data,86400); // speicher 1 Tag
    }


}