<?php

namespace DXIM;

use DXIM\theme\Job;
use DXIM\theme\JobSchema;
use DXIM\theme\LocationSchema;
use DXIM\theme\Offer;

class StructuredData {

	protected static $instance = null;

    protected $data = [];

	/**
	 *
	 * @return StructuredData
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() { }

    /**
     * @return void
     */
    public function init() {

        $options = Admin::getInstance()->getOptions();
        $schema = $options['seo']['schema'] ?? "";
        if(!$schema) return;

        add_action('wp', function() {

            /**
             * Schema for Car
             */
            if(Conf::getInstance()->isVehicleSinglePage()) {
                $vehicle = dxim_get_vehicle();
                $vs = new VehicleSchema($vehicle);
                $this->data = apply_filters('dxim_filter_schema_vehicle',$vs->create(), $vehicle);
            }

            /**
             * Schema for Job
             */
            if(is_singular(Job::$post_type)) {
                global $post;
                $js = new JobSchema($post);
                $this->data = apply_filters('dxim_filter_schema_'.Job::$post_type,$js->create(), $post);
            }

            /**
             * Location
             */
            if(is_tax(\DXIM\theme\Location::$taxonomy)) {
                $term = get_queried_object();
                $ls = new LocationSchema($term);
                $this->data = $ls->create();
            }

            /**
             * add to header
             */
            if(!empty($this->data)) {

                // add schema
                add_action('wp_head', [&$this,'_print_json_ld'],9);
            }
        });


    }

    /**
     * @param $data
     * @return string
     */
    public function _print_json_ld() {
        $html = '<script type="application/ld+json" class="wpcarsync-schema">';
        $html .= json_encode($this->data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        $html .= '</script>';

        print $html;
    }

}