<?php

/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 12/09/16
 * Time: 12:30
 */

namespace DXIM;


use DXIM\theme\Job;
use DXIM\theme\ModelData;
use DXIM\theme\Offer;
use Elementor\User;

class Template {

    protected static $instance = null;

    protected static $num_vehicles_rendered = 0;
    #protected static $num_vehicles_on_current_page = 0;


    private $footer_scripts = [];

    /**
     *
     * @return Template
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Template constructor.
     */
    public function __construct() {

        add_action('wp_enqueue_scripts', [&$this, '_enqueue'], 100);

        add_action('template_redirect', array(&$this, '_template_redirect'));

        add_filter('query_vars', array(&$this, 'add_query_vars_filter'));

        // plugin credits
        add_filter('the_content', [&$this, '_credits'], 10, 1);

        add_filter('post_type_link', [&$this, '_mod_vehicle_link'], 10, 4);

        add_action('wp_ajax_wpcs_fetchResults', [&$this, '_fetchResults']);
        add_action('wp_ajax_nopriv_wpcs_fetchResults', [&$this, '_fetchResults']);

        // wp head
        add_action('wp_head', function () {
            // js variables
            print "<script>\n";
                print "var wpcs_ajax_url = '" . admin_url('admin-ajax.php') . "';\n";
                print "var wpcs_slider_ranges = {};\n";
                print "var wpcs_archive_url = '" . Conf::getInstance()->getVehicleArchiveUrl() . "';\n";

                $curr_obj = get_queried_object();
                if($curr_obj instanceof \WP_Post_Type) print "var wpcs_current_url = '" . get_post_type_archive_link($curr_obj->name) . "';\n";
                else print "var wpcs_current_url = '" . get_permalink() . "';\n";

                if(WPCS_PRO) {
                    print "var wpcs_offer_archive_url = '" . get_post_type_archive_link(\DXIM\theme\Offer::$post_type) . "';\n";
                    print "var wpcs_job_archive_url = '" . get_post_type_archive_link(\DXIM\theme\Job::$post_type) . "';\n";
                }

                $ratio = Admin::getInstance()->getOptions("general_vehicle_image_ratio");
                $tmp = explode("_", $ratio);
                print "var wpcs_image_ratio = { w : " . $tmp[0] . " , h : " . $tmp[1] . " };\n";
            print "</script>\n";

            if(Conf::getInstance()->isVehicleSinglePage()) {
                $vehicle = dxim_get_vehicle();
                $images = VehicleData::getVehicleImages($vehicle);
                print '<meta property="og:image" content="'.$images['big'][0].'" />'."\n";
            }

        });

        // Permalinks
        add_filter('rewrite_rules_array', [&$this, '_set_rewrite_rules'], 10, 1);

        // wp
        add_action('wp', function () {

            $options = Admin::getInstance()->getOptions();
            $archive_template = $options['vehicle_archive_template'] ?? "wpcs";

            //
            if (Conf::getInstance()->isVehicleArchivePage()) {
                // beim besuch der archivseite muss diese als last search gespeichert werden,
                // andernfalls kommt der besucher nach dem Klick auf den Zurück Button ggf. auf die Startseite oder sonst wo hin
                VehicleFilter::getInstance()->_set_last_search(null, Conf::getInstance()->getVehicleArchiveUrl());

                // load archive template
                if(is_post_type_archive('vehicle') && $archive_template == "wpcs") {
                    add_action('template_include', [&$this, '_get_template_archive'],11,1);
                }
            }

            // print friendly version of single vehicle
            elseif (Conf::getInstance()->isVehicleSinglePage()) {
                add_action('template_include', [&$this, '_get_template_single'],11,1);
            }

        });

    }


    public function _get_template_archive($template) {
        return CustomTemplate::getInstance()->get_template_file('archive/archive_template');
    }

    public function _get_template_single($template) {
        global $wp_query;
        $print = isset($wp_query->query_vars['print']);

        $options = Admin::getInstance()->getOptions();
        $single_template = $options['vehicle_single_template'] ?? "wpcs";

        // load print template
        if ($print) {
            $conf = Admin::getInstance()->getOptions('vehicle_expose');
            if (empty($conf) || $conf == "pdf") $new_template = DXIM_PluginPath . '/tpl/single/single-vehicle-pdf.php';
            else $new_template = DXIM_PluginPath . '/tpl/single/single-vehicle-print.php';
            return $new_template;
        }

        // load single template
        elseif(is_singular('vehicle') && $single_template == "wpcs") {
            return CustomTemplate::getInstance()->get_template_file('single/single_template');
        }

        return $template;

    }

    /**
     *
     */
    public function _fetchResults() {

        $result = [
            'vehicle_list_html' => Shortcode::getInstance()->_show_vehicles($_POST['vehicle_list_data']),
        ];

        if (!$_POST['vehicle_list_data']['append']) {
            $result['count'] = VehicleFilter::getCount();
            $result['pagination'] = \DXIM\Html::pagination();
            $result['active_filters'] = VehicleFilter::getInstance()->drawActiveFilters();
        }

        header('Content-Type: application/json');
        print json_encode($result);
        exit;
    }

    /**
     * @param $url
     * @param $post
     * @param $leavename
     * @param $sample
     * @return false|string
     */
    public function _mod_vehicle_link($url, $post, $leavename, $sample) {
        if ($post->post_type != "vehicle") return $url;

        $data = [];
        $data['site_url'] = home_url();
        $data['archive_slug'] = PostType::$post_type_slug;
        $data['slug'] = $post->post_name;

        $make = get_post_meta($post->ID, 'make', true);
        $model = get_post_meta($post->ID, 'model', true);

        $model = Model::_getForUrl($model, $make);
        $make = Make::_getForUrl($make);

        if (empty($model)) $model = "model";
        $data['model'] = $model;

        if (empty($make)) $make = "marke";
        $data['make'] = $make;

        $permalink = $data['site_url'] . '/'.$data['archive_slug'].'/' . $data['make'] . "/" . $data['model'] . '/' . $data['slug'] . "/";
        return apply_filters(DXIM_FILTER_VEHICLE_PERMALINK, $permalink, $data, $post);
    }

    /**
     * scripts & styles
     */
    public function _enqueue() {

        wp_enqueue_script('jquery');

        // Slick Slider
        wp_enqueue_script('slick-slider', DXIM_PluginUrl . '/plugins/slick/slick.min.js', array('jquery'), DXIM_PluginVersion, true);
        wp_enqueue_style('slick-slider', DXIM_PluginUrl . '/plugins/slick/slick.css');

        // Slick Pips
        wp_enqueue_script('nouislider', DXIM_PluginUrl . '/plugins/nouislider/nouislider.min.js', array('jquery'), DXIM_PluginVersion, true);
        wp_enqueue_style('nouislider', DXIM_PluginUrl . '/plugins/nouislider/nouislider.min.css');

        // WPCS JS
        wp_enqueue_script('wpcsselect', DXIM_PluginUrl . '/js/wpcsselect.js', array('jquery'), DXIM_PluginVersion, true);
        wp_enqueue_script('wpcs', DXIM_PluginUrl . '/js/wpcs.js', array('wpcsselect', 'slick-slider'), DXIM_PluginVersion, true);
        wp_enqueue_script('wpcs-theme', DXIM_PluginUrl . '/js/wpcs_theme.js', array('wpcs'), DXIM_PluginVersion, true);

        // WPCS CSS
        wp_enqueue_style('wpcs-layout', DXIM_PluginUrl . '/css/layout.css', ['nouislider', 'slick-slider'], DXIM_PluginVersion);

        // Wordpress Icons
        wp_enqueue_style('dashicons');
    }


    /**
     * @param $template
     */
    public function _template_redirect() {

        // redirect vehicle-single when vehicle doesnt exist
        if (Conf::getInstance()->isVehicleSinglePage() && empty(VehicleData::getVehicle()) && !is_user_logged_in()) {
            wp_redirect(Conf::getInstance()->getVehicleArchiveUrl(), 301);
            exit;
        }
    }


    /**
     *
     */
    public function _set_rewrite_rules($rules) {
        $archive_page = Conf::getInstance()->getVehicleArchivePage();

        // neue variante => standard post type
        if(empty($archive_page)) {
            $make_archive = 'index.php?post_type=vehicle&make=$matches[1]&is_make_archive=1';
            $model_archive = 'index.php?post_type=vehicle&make=$matches[1]&model=$matches[2]&is_model_archive=1';
            $single_page = 'index.php?vehicle=$matches[3]';
            $single_print_page = 'index.php?vehicle=$matches[3]&print=1';
        }

        // alte variante über seiten
        else {
            $make_archive = 'index.php?page_id=' . Conf::getInstance()->getVehicleArchivePage() . '&make=$matches[1]&is_make_archive=1';
            $model_archive = 'index.php?page_id=' . Conf::getInstance()->getVehicleArchivePage() . '&make=$matches[1]&model=$matches[2]&is_model_archive=1';
            $single_page = 'index.php?page_id=' . Conf::getInstance()->getVehicleSinglePage() . '&vehicle_slug=$matches[3]';
            $single_print_page = 'index.php?page_id=' . Conf::getInstance()->getVehicleSinglePage() . '&vehicle_slug=$matches[3]&print=1';
        }

        $rewrite[PostType::$post_type_slug.'/([a-zA-Z0-9-_\.\s]+?)/?$'] = $make_archive;
        $rewrite[PostType::$post_type_slug.'/([a-zA-Z0-9-_\.\s]+?)/([a-zA-Z0-9-_\.\s]+?)/?$'] = $model_archive;
        $rewrite[PostType::$post_type_slug.'/([a-zA-Z0-9-_\.\s]+?)/([a-zA-Z0-9-_\.\s]+?)/([a-zA-Z0-9-_]+?)/?$'] = $single_page;
        $rewrite[PostType::$post_type_slug.'/([a-zA-Z0-9-_\.\s]+?)/([a-zA-Z0-9-_\.\s]+?)/([a-zA-Z0-9-_]+?)/print/?$'] = $single_print_page;

        $rewrite = apply_filters(DXIM_FILTER_REWRITE_RULES, $rewrite);

        $rules = $rewrite + $rules;
        #wpcs_p($rules);
        return $rules;
    }


    /**
     * @param $vars
     *
     * @return array
     */
    function add_query_vars_filter($vars) {
        $vars[] = "vehicle_slug";
        $vars[] = "make";
        $vars[] = "model";
        $vars[] = "print";
        $vars[] = "is_make_archive";
        $vars[] = "is_model_archive";
        return $vars;
    }

    /**
     *
     */
    public function enqueue_footer_scripts() {
        foreach ($this->footer_scripts as $script) {
            print $script;
        }
    }

    public static function drawArchiveDescription($args = []) {
        global $wp_query;

        if(isset($wp_query->query_vars['is_model_archive'])) {
            $model_slug = $wp_query->query_vars['model'] ?? "";
            $brand_slug = $wp_query->query_vars['make'] ?? "";

            $wpcs_make_slug = Make::_getFromUrl($wp_query->query_vars['make']);
            if(empty($wpcs_make_slug)) $wpcs_make_slug = $wp_query->query_vars['make'];

            $wpcs_model_slug = Model::_getFromUrl($wp_query->query_vars['model'], $wpcs_make_slug);
            if(empty($wpcs_model_slug)) $wpcs_model_slug = $wp_query->query_vars['model'];
        }
        elseif(isset($wp_query->query_vars['is_make_archive'])) {
            $brand_slug = $wp_query->query_vars['make'] ?? "";

            $wpcs_make_slug = Make::_getFromUrl($wp_query->query_vars['make']);
            if(empty($wpcs_make_slug)) $wpcs_make_slug = $wp_query->query_vars['make'];
        }

        else return null;

        // brand
        $bd = new \DXIM\theme\BrandData();
        $brand = $bd->getBrandBySlug($brand_slug);
        if(empty($brand)) $brand = $brand_slug;

        // model
        $model = null;
        if(!empty($model_slug)) {
            $md = new ModelData();
            $meta_query = [
                ['key' => 'make', 'value' => $wpcs_make_slug],
                ['key' => 'model', 'value' => $wpcs_model_slug],
            ];
            $models = $md->get_models(['meta_query' => $meta_query]);
            $model = $models[0] ?? "";
        }
        if(empty($model)) $model = $model_slug ?? null;

        $html = CustomTemplate::getInstance()->get_template_content('archive/archive_description',['brand' => $brand, 'model' => $model]);
        return apply_filters(DXIM_FILTER_ARCHIVE_DESCRIPTION, $html, $brand, $model);
    }


    /**
     * VehicleListe rendern
     * @param array $vehicles
     * @return null|string
     */
    public static function drawVehicleList($vehicles, $args) {
        $options = Admin::getInstance()->getOptions();
        $user_layout = Session::val('archive_layout');
        $options_layout = $options['archive_vehicle_layout'];
        $layout = $args['layout'] ?? $user_layout ?? $options_layout;

        // template name wurde als Parameter übergeben
        if (!empty($args['template'])) $template_file = $args['template'] . ".php";

        // layoutspezifisches template laden (carousel, grid, ...)
        else $template_file = $layout . ".php";

        // Hook File
        $template_file = "loop/" . $template_file;
        $template_file = apply_filters(DXIM_FILTER_SELECT_LOOP_TEMPLATE, $template_file);

        // define defaults from Settings
        $defaults = [
            'layout' => $options['archive_vehicle_layout'],
            'grid' => $options['archive_vehicle_grid'],
            'template_file' => $template_file,
            'append' => 0,
            'infinite' => 0,
        ];

        // overwrite defaults by shortcode args
        $args = array_merge($defaults, $args);
        $args['max_pages'] = VehicleFilter::getNumPages();
        if(!isset($args['count'])) $args['count'] = VehicleFilter::getCount();

        $html = null;
        $ajax = ((isset($options['ff_method']) && $options['ff_method'] == 'ajax') || isset($args['ajax'])) ? "1" : 0;
        $properties = Template::getInstance()->_save_propterties_for_ajax($args);

        // handelt es sich im WPCS_cars oder WPCS_vehicles?
        $id = null;
        if ($args['vehicle_list_type'] == "wpcs_vehicles") $id = 'id="wpcs_vehicle_archive"';
        else $ajax = 0;

        // Fahrzeuge an bestehende Liste anhängen
        if ($args['append']) {
            #self::$num_vehicles_on_current_page = count($vehicles);
            $formData = VehicleFilter::getInstance()->getFormData();
            $current_page = $formData['wpcsp'] ?? 1;

            foreach ($vehicles as $post) {
                VehicleData::getInstance()->setVehicle($post);
                $template_html = CustomTemplate::getInstance()->get_template_content($template_file);
                $vehicle_html = Template::drawVehicleOnArchive($post, $template_html);

                self::$num_vehicles_rendered++;
                if($args['vehicle_list_type'] == "wpcs_vehicles") $vehicle_html = apply_filters(DXIM_FILTER_ARCHIVE_VEHICLE_ADD, $vehicle_html, $post, self::$num_vehicles_rendered, $current_page, VehicleFilter::getPostsPerPage(), VehicleFilter::getCount());
                $html .= $vehicle_html;
            }

            wp_reset_postdata();
        }

        // gesamte Liste neu laden
        else {
            $html .= '<div ' . $id . ' class="wpcs_vehicle_list ' . $args['vehicle_list_type'] . '" data-ajax="' . $ajax . '" data-infinite="' . $args['infinite'] . '" data-amount="'.$args['count'].'">';
            $anchor = apply_filters(DXIM_FILTER_FORM_VEHICLES_ANCHOR, '<a name="wpcs_vehicles"></a>');
            $html .= $anchor;
            $html .= '<script>
                var vehicle_list_properties = ' . json_encode($properties) . ';                
            </script>';

            VehicleData::getInstance()->saveVehicle();

            // Carousel / Slider
            if ($args['layout'] == 'slider' || $args['layout'] == 'carousel') {
                $html .= VehicleHtml::_draw_vehicles_as_carousel($vehicles, $args);
            }

            // grid / Rows
            else {
                $html .= '<div class="wpcs_vehicles_wrapper wpcs_' . $args['layout'] . ' wpcs_grid_' . $args['grid'] . '">';
                #self::$num_vehicles_on_current_page = count($vehicles);
                $formData = VehicleFilter::getInstance()->getFormData();
                $current_page = $formData['wpcsp'] ?? 1;

                foreach ($vehicles as $post) {
                    VehicleData::getInstance()->setVehicle($post);
                    $template_html = CustomTemplate::getInstance()->get_template_content($template_file);
                    $vehicle_html = Template::drawVehicleOnArchive($post, $template_html);

                    self::$num_vehicles_rendered++;

                    // Hook nur in wpcs_vehicles, nicht in wpcs_cars ausführen
                    if($args['vehicle_list_type'] == "wpcs_vehicles") $vehicle_html = apply_filters(DXIM_FILTER_ARCHIVE_VEHICLE_ADD, $vehicle_html, $post, self::$num_vehicles_rendered, $current_page, VehicleFilter::getPostsPerPage(), VehicleFilter::getCount());

                    $html .= $vehicle_html;
                }
                $html .= '</div>';

                wp_reset_postdata();
            }

            $html .= '</div>';
        }

        VehicleData::getInstance()->restoreVehicle();

        // FormHiddens aufbauen, damit Vehicle Liste auch alle auf Seite stehen kann
        // für ajax paginierung etc...
        $html .= VehicleFilter::getInstance()->drawFormHiddens();

        return apply_filters(DXIM_FILTER_ARCHIVE_VEHICLE_LIST, $html);
    }


    /**
     * @param $args
     * @return mixed
     */
    private function _save_propterties_for_ajax($args) {
//        unset($args['template']);
        unset($args['template_file']);
        return $args;
    }

    /**
     * @param $vehicle
     * @param $template
     * @return string
     */
    public static function drawVehicleOnArchive($vehicle, $template) {
        $html = '<article class="vehicle-on-archive">';
        $html .= FilterForm::getInstance()->drawAjaxLoading();
        $html .= do_shortcode($template);
        $html .= '</article>';

        return apply_filters(DXIM_FILTER_ARCHIVE_VEHICLE_RENDERED, $html, $vehicle);
    }

    /**
     * Vehicle Archiv rendern
     * @return null|string
     */
    public static function drawVehicleArchive() {
        $archiveHtml = null;

        // check Layout
        $options = Admin::getInstance()->getOptions();
        $layout = $options['default_layout_archive'];
        $template_name = 'archive_' . str_replace("layout_", "", $layout);

        $archiveHtml .= '<div id="wpcs_' . $layout . '" class="archive_vehicle dxim_clearfix">';
        $archiveHtml .= CustomTemplate::getInstance()->get_template_content('archive/' . $template_name);
        $archiveHtml .= '</div>';

        return do_shortcode($archiveHtml);
    }

    /**
     * Vehicle Single rendern
     * @return null
     */
    public static function drawVehicleSingle() {
        $html = CustomTemplate::getInstance()->get_template_content('single/single');
        $html = '<article class="vehicle-on-single dxim_clearfix">' . $html . '</article>';
        return do_shortcode($html);
    }

    /**
     * Plugin Credits
     * @param $content
     * @return mixed
     */
    public function _credits($content) {
        $options = Admin::getInstance()->getOptions();
        $slug = $options['plugin_feature_page'];
        if (is_page($slug)) {
            $credits = '<div class="wpcs_plugin_credits container"><a href="' . DXIM_PluginWebAddress . '" title="Mobile.de Wordpress Plugin WPCarSync by ' . DXIM_PluginWebsiteName . '" target="_blank">Mobile.de Wordpress Plugin WPCarSync</a></div>';
            $content .= $credits;
        }

        return $content;
    }
}
