<?php


    class Updater {

        public $plugin_slug;
		public $version;
		public $cache_key;
		protected $update_folder;

		public function __construct($slug, $version, $folder = "wpcsupdate2") {

			$this->plugin_slug = $slug;
			$this->version = $version;
			$this->cache_key = $slug."_update_check";
			$this->update_folder = $folder;

            add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );
            add_filter( 'site_transient_update_plugins', array( $this, 'update' ) );
            add_filter( 'transient_update_plugins', array( $this, 'update' ) );
        }

        /**
         * @param $data
         * @return void
         */
        private function _set_cache($data) {
            if(!is_object($data)) $data = new stdClass();
            $data->last_check = time();
            update_option($this->cache_key, $data);
        }

        /**
         * @return false|mixed|null
         */
        private function _get_cache() {
            return get_option($this->cache_key);
        }


        /**
         * @param $update_plugins
         * @return mixed
         */
        function update( $update_plugins ) {

            if ( ! is_object( $update_plugins ) )
                return $update_plugins;

            if ( ! isset( $update_plugins->response ) || ! is_array( $update_plugins->response ) )
                $update_plugins->response = array();

            // hole aktuelle Server-Daten des Plugins
            $plugin_data = $this->request();

            // Daten leer (z.b. json auf server nicht gefunden oder sonst was)
            if(!isset($plugin_data->version)) return $update_plugins;

            // wenn lokale Version aktuell => return
            if(version_compare($this->version, $plugin_data->version, ">=")) {
                return $update_plugins;
            }

            //
            $res = new stdClass();
            $res->slug = $this->plugin_slug;
            $res->new_version = $plugin_data->version;
            $res->tested = $plugin_data->tested;
            $res->package = $plugin_data->download_url;
            #wpcs_p($res);

            $update_plugins->response[$this->plugin_slug.'/Plugin.php'] = $res;

            return $update_plugins;
        }

        /**
         * zieht die Daten zum aktuellen Update vom Server
         * verwendet 24h Cache lokal...
         * @return false|mixed
         */
        public function request() {

            $cache = $this->_get_cache();
            $last_check = $cache->last_check ?? 0;
            $next_check = $last_check + DAY_IN_SECONDS;
//            print "cache: ".date('d.m.Y H:i:s', $last_check)." / ";
//            print "time ".date('d.m.Y H:i:s')." / ";
//            print "next check ".date('d.m.Y H:i:s', $next_check);

            /**
             * Daten aus Chache sind noch gültig
             */
            if(time() < $next_check) {
                return $cache;
            }

            /**
             * Daten neu von Server holen
             */
            $remote = wp_remote_get(
                DXIM_PluginWebAddress.'/'.$this->update_folder.'/plugin.json',
                array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    )
                )
            );
            #print "echter Request für: ".DXIM_PluginWebAddress.'/'.$this->update_folder.'/plugin.json';

            if(is_wp_error( $remote ) || 200 !== wp_remote_retrieve_response_code( $remote ) || empty( wp_remote_retrieve_body( $remote ) )
            ) {
                $this->_set_cache(null);
                return false;
            }

            $body = wp_remote_retrieve_body( $remote );

            $result = json_decode($body);

            $this->_set_cache($result);
            return $result;
        }

        /**
         * Anzeige Infolink über Update
         * @param $res
         * @param $action
         * @param $args
         * @return false|stdClass
         */
        function info( $res, $action, $args ) {

            // do nothing if you're not getting plugin information right now
            if( 'plugin_information' !== $action ) {
                return $res;
            }

            // do nothing if it is not our plugin
            if( $this->plugin_slug !== $args->slug ) {
                return $res;
            }

            // get updates
            $remote = $this->request();

            if( ! $remote ) {
                return $res;
            }

            $res = new stdClass();

            $res->name = $remote->name;
            $res->slug = $remote->slug;
            $res->version = $remote->version;
            $res->tested = $remote->tested;
            $res->requires = $remote->requires;
            $res->author = $remote->author;
            $res->author_homepage = $remote->author_homepage;
            $res->download_link = $remote->download_url;
            $res->trunk = $remote->download_url;
            $res->last_updated = $remote->last_updated;

            $res->sections = array(
                'description' => $remote->sections->description,
                'changelog' => $remote->sections->changelog
            );

            if( ! empty( $remote->banners ) ) {
                $res->banners = array(
                    'low' => $remote->banners->low,
                    'high' => $remote->banners->high
                );
            }

            return $res;
        }

    }