<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 09:47
 */

namespace DXIM;


class Vehicle {

    protected static $instance = null;

    public $post_data = null;
    public $post_meta = null;
    public $status_msg = null;

    /**
     *
     * @return Vehicle
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }


    /**
	 * gibt alle Felder zurück, die als Fahrzeug-Eigenschaft dienen (specifics)
	 * @return array
	 */
	public static function getProperties() {
		$fields = [];
		$all_fields = VehicleFields::getFields();
		foreach($all_fields as $key => $array) {
			if(!isset($array['specific'])) continue;

			$tmp = $array['specific'];
			$tmp['label'] = $array['label'];
			$fields[$key] = $tmp;
		}

		#wpcs_p($fields);
		return $fields;
	}

	/**
	 * @return bool
	 */
	public function has_account(\WP_Post $vehicle) {
		return ($vehicle->dxim_meta['ad_account']) ? true : false;
	}


	/**
     * @param $ad_key
     * @return mixed
     */
    public function getByAdKey($ad_key, $init_meta = true, $key = "ad_key") {
        $q = new \WP_Query(array('meta_key' => $key,'meta_value' => $ad_key,'post_type'=>'vehicle', 'post_status' => 'publish'));
        if(empty($q->posts)) return [];

        $vehicle = $q->posts[0];
	    if($init_meta) $vehicle = VehicleData::initDataObject($vehicle);
        return $vehicle;
    }

    /**
     * @param $vehicle_id
     * @return mixed
     */
    public function getById($vehicle_id) {
    	if(intval($vehicle_id) < 1) return null;

        $q = new \WP_Query(array('post_type'=>'vehicle','p'=>$vehicle_id));
        $vehicle = $q->posts[0];
        $vehicle = VehicleData::initDataObject($vehicle);
        return $vehicle;
    }

    /**
     * @param $ad_key
     * @param $hash
     * @return array
     */
    public function createImportDataset($ad_key, $hash, $inventory_key = "") {
        if(empty($inventory_key)) $inventory_key = $ad_key;
        $vehicle['import_data'] = [
            'do' => null,
            'key' => $ad_key,
            'ID' => null,
            'seller_inventory_key' => $inventory_key
        ];

        $q = new \WP_Query(array('meta_key' => 'ad_key','meta_value' => $ad_key,'post_type'=>'vehicle', 'post_status' => 'publish'));

        // Fahrzeug existiert bereits
        if($q->post_count > 0) {
            $post = $q->posts[0];
            $hash_db = get_post_meta($post->ID, 'import_hash', true);
            $vehicle['import_data']['ID'] = $post->ID;

            // Update notwendig, falls $hash und hash in DB sich unterscheiden
            if($hash_db != $hash) {
                $vehicle['import_data']['do'] = "update";
                return $vehicle;
            }

            // kein Update notwendig, Fahrzeug aktuell
            else return $vehicle;

        }

        // Fahrzeug existiert noch nicht => neu anlegen
        else {
            $vehicle['import_data']['do'] = "new";
            return $vehicle;
        }

    }

    /**
     * @param $ad
     * @param \WP_Post $vehicle
     */
    public function save($item) {
    	$this->status_msg = null;
        $options = Admin::getInstance()->getOptions();
        $item['post_data']['ID'] = $item['import_data']['ID'];

		//
	    $item['post_meta']['ad_key'] = apply_filters('wpcs_modify_ad_key',$item['post_meta']['ad_key'], $item['post_meta']);

        // set vehicle Date from API
        $this->post_data = $item['post_data'];
        $this->post_meta = $item['post_meta'];

        // set post data defaults
        $this->post_data['post_status'] = 'publish';
        $this->post_data['post_type'] = 'vehicle';

        // Filter Vehicle to Import
        $filterData = ['post_data' => $this->post_data, 'post_meta' => $this->post_meta];
        $filterData = apply_filters(DXIM_FILTER_VEHICLE_BEFORE_IMPORT,$filterData);
        if(empty($filterData)) {
            unset($this->post_data);
            $this->status_msg = 'übersprungen anhand manueller Filterkriterien';
            return false;
        }
        else {
            $this->post_data = $filterData['post_data'];
            $this->post_meta = $filterData['post_meta'];
        }

        $is_update = intval($this->post_data['ID']) > 0;

        /**
         * envkv check
         */
        $e = new VehicleEmission();
        $check_envkv = intval($options['import']['check_envkv'] ?? 1);
        if($check_envkv) {
            if(!$e->is_allowed($this->post_meta)) {
                $this->status_msg = "Ignoriert aufgrund ENVKV: ".implode(",",$e->get_error());
                return false;
            }
        }

        #wpcs_p($this->post_meta);
        #wpcs_p($this->post_data);
        #exit;

        // Post anlegen / editieren
        add_filter( 'wp_insert_post_data', array(&$this,'_alter_post_modification_time'), 99, 2 );
        $post_id = wp_insert_post($this->post_data, true);
        remove_filter( 'wp_insert_post_data', array(&$this,'_alter_post_modification_time'), 99, 2 );

        if(is_wp_error($post_id)) {
            Sync::log($post_id->get_error_message());
            return false;
        }

        // save Post Meta & images
        elseif(intval($post_id) > 0) {

            /**
             * lösche alte Meta Daten
             */
            $this->_delete_vehicle_meta($post_id);

            /**
             * importiere neue Meta Daten
             */
            foreach($this->post_meta as $key => $value) {

                // lege meta feld neu an, falls ein wert gesetzt ist
                if(!empty($value)) {
                    if(is_array($value) && $key != "images") {
                        foreach($value as $tmpValue) {
                            add_post_meta($post_id,$key,$tmpValue,false);
                        }
                    }
                    else {
                        add_post_meta($post_id,$key,$value);
                    }
                }
            }

            /**
             * setze first_import_time
             */
            // zu welchem Zeitpunkt wurde dieses Fahrzeug das erste mal importiert?
            $first_import_time = get_post_meta($post_id, 'first_import_time', true);
            if(empty($first_import_time)) update_post_meta($post_id,'first_import_time',time());

            /**
             * Importiere Bilder
             */
            // Handle images
            if(isset($this->post_meta['images']['big'])) {
                $account = Account::getInstance()->get_account($this->post_meta['ad_account']);

                // Mobile.de
                if($account['type'] == "mobile.de") {

                    // Filesystem
                    if($options['api_options']['import_images'] == "all_filesystem") {
                        FilesystemImport::getInstance()->import($post_id, $this->post_meta['images']['big'], $this->post_meta['image_id']);
                    }

                    // Media
                    elseif($options['api_options']['import_images'] == 'all_media') {
                        MediaImport::getInstance()->import($this->post_meta['images']['big'], $post_id, $this->post_meta['image_id']);
					}

                }

                // CSV
                // kopieren der Bilder erfolgt bereits beim entpacken der CSV
                // trotzdem import hook feuern
                elseif($account['type'] == 'csv') {
					update_post_meta($post_id, "images_filesystem", $this->post_meta['images']['big']);
					do_action('wpcs_images_imported_filesystem',$post_id);
                }

                // Car Cloud Import
                elseif($account['type'] == 'carcloud') {

                    // Filesystem
                    if($options['carcloud']['images'] == "filesystem") {
                        FilesystemImport::getInstance()->import($post_id, $this->post_meta['images']['big'], $this->post_meta['image_id']);
					}

                    // Media
                    elseif($options['carcloud']['images'] == "media") {
                        MediaImport::getInstance()->import($this->post_meta['images']['big'], $post_id, $this->post_meta['image_id']);
                    }

                }

            }

	        do_action(DXIM_CORE_VEHICLE_SAVED,$post_id, $is_update);
        }

        else return false;

        return $post_id;
    }

    /**
     * @param $vehicle_id
     * @return void
     */
    function _delete_vehicle_meta($vehicle_id) {
        $meta = get_post_meta($vehicle_id);
        foreach($meta as $key => $value) {
            $field = VehicleFields::getField($key);
            $sync = !isset($field['ignore_sync']) && !in_array($key,["wpcs_db_images", "images_media_csp"]);
            if(substr($key,0,1) == "_" || !$sync) {
                continue;
            }
            delete_post_meta($vehicle_id, $key);
        }
    }

	/**
	 * @param $data
	 * @param $postarr
	 *
	 * @return mixed
	 */
    function _alter_post_modification_time( $data , $postarr ) {
        if (!empty($postarr['post_modified'])) {
            $data['post_modified'] = $postarr['post_modified'];
        }
        return $data;
    }


	/**
	 * lösche post thumbnail inkl. file
	 * @param $post_id
	 */
//    public function delete_vehicle_thumbnail($post_id) {
//
//    	$thumbnail_id = get_post_thumbnail_id($post_id);
//
//    	// lösche file
//    	if(wp_delete_attachment($thumbnail_id,true)) {
//
//    		// lösche DB Verweis zu post (unset post thumbnail)
//		    delete_post_thumbnail($post_id);
//	    }
//
//    }


	/**
	 * @param $post_id
	 * @param $image_url
	 */
//	private function _create_vehicle_thumbnail($post_id,$image_url) {
//
//        // lösche vorheriges
//        $this->delete_vehicle_thumbnail($post_id);
//
//        // importiere im Mediethek
//		$id = MediaImport::getInstance()->importFromUrl($image_url, $post_id);
//
//		// setze Bild als Post Thumbnail
//		set_post_thumbnail($post_id, $id);
//
//		return $id;
//	}




    /**
     * löschen eines Fahrzeugs über die WP UI triggert diese function
     * @param $post_id
     * @param $post
     * @return bool
     */
	public function _on_delete_post($post_id, $post = false) {
        if(!$post || $post->post_type != "vehicle") return false;

        // lösche nur dependencies, da hook vehicle bereits löscht
        $this->_delete_dependencies($post_id);

        do_action(DXIM_CORE_VEHICLE_DELETED,$post_id);
    }

    /**
     * programmatisches löschen eines Fahrzeugs, durch sync etc...
     * @param \WP_Post $vehicle
     */
    public function delete($vehicle_id) {

        // delete vehicle
        wp_delete_post( $vehicle_id, true );

        return true;
    }

    /**
     * @param $vehicle_id
     */
    private function _delete_dependencies($vehicle_id) {

        // post thumbnail löschen
        #$this->delete_vehicle_thumbnail($vehicle_id);

        // lösche daten der mediathek falls vorhanden
        MediaImport::getInstance()->delete_vehicle_media($vehicle_id);

        // lösche bilder aus bildpool
        FilesystemImport::getInstance()->delete_vehicle_images($vehicle_id);

        // lösche FilterDaten
        FilterTable::getInstance()->deleteFilters($vehicle_id);
    }

    /**
     * @return void
     */
    public function _reset_import_hashes() {
        global $wpdb;
        return $wpdb->query("DELETE FROM ".$wpdb->prefix."postmeta WHERE meta_key = 'import_hash';");
    }


    /**
     * @param $vehicle
     * @param $filterKey
     * @return array|void|null
     */
    public function getValue(&$vehicle, $filterKey) {
        $field = VehicleFields::getField($filterKey);
        $field_name = $field['meta_query']['key'] ?? null;
        $rawValue = $this->getRawValue($vehicle, $field_name);
        if(empty($rawValue)) return null;

        $depends_on_key = $field['data_source']['depends'] ?? null;
        $depends_on_value = null;
        if($depends_on_key !== null) {
            $depends_on_value = $this->getRawValue($vehicle, $depends_on_key);
        }

        if(is_array($rawValue)) {
            foreach($rawValue as $tmp) {
                $value[] = VehicleFilter::getInstance()->getReadableValue($filterKey, $tmp, $depends_on_value);
            }
        }
        else {
            $value = VehicleFilter::getInstance()->getReadableValue($filterKey, $rawValue, $depends_on_value);
        }

        return $value;
    }

    /**
     * @param $vehicle
     * @param $filterKey
     * @return mixed|null
     */
    public function getRawValue(&$vehicle, $field_name) {
        $value = $vehicle->dxim_meta[$field_name] ?? null;

        if(empty($value)) return null;
        elseif(is_array($value) && count($value) == 1) return $value[0];
        else return $value;
    }


    /**
     * gibt den lesbaren String eines Custom Fields zurück
     * wenn array werden alle Werte per $trenner als String zurück gegeben
     * @param $vehicle
     * @param $filterKey
     * @param $trenner
     * @return string|void|null
     */
    public function drawValue(&$vehicle, $filterKey, $trenner = ",") {
        $value = $this->getValue($vehicle, $filterKey);
        if(!is_array($value)) return $value;
        $html = null;
        $z = 0;
        foreach($value as $tmp) {
            if($z != 0) $html .= $trenner." ";
            $html .= $tmp;
            $z++;
        }
        return $html;
    }

}