<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 09:47
 */

namespace DXIM;


class VehicleCF {


	protected static $instance = null;

	/**
	 *
	 * @return VehicleCF
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 * Vehicle constructor.
	 */
	public function __construct() {
        add_action('add_meta_boxes_vehicle', [&$this, '_custom_fields_meta_box']);
        add_action('wp_ajax_wpcs_toggle_sync', [&$this, '_toggle_sync_status']);
        add_action('wp_ajax_wpcs_filter_table_save', [&$this, '_save_filter_table_data']);
    }

    /**
     * @param $vehicle
     * @return mixed
     */
    public function modify_import_values($vehicle) {

        //  set seller_inventory_key
        if(empty($vehicle['post_meta']['seller_inventory_key'])) $vehicle['post_meta']['seller_inventory_key'] = $vehicle['post_meta']['ad_key'];

        // set image_id
        if(empty($vehicle['post_meta']['image_id'])) $vehicle['post_meta']['image_id'] = $vehicle['post_meta']['ad_key'];

        // co2 label anhand emission selbst berechnen, falls nicht angegeben
        $e = new VehicleEmission();
        $vehicle['post_meta'] = $e->fill_co2_classes_for_import($vehicle['post_meta']);

        return $vehicle;
    }


    /**
     * @return void
     */
    public function _save_filter_table_data() {
        FilterTable::getInstance()->updateField($_POST['post_id'], $_POST['field'], $_POST['value']);
        update_post_meta($_POST['post_id'], $_POST['field'], $_POST['value']);
        exit;
    }


    /**
     * sync zu einem Fahrzeug aktivieern/deaktivieren
     * @return void
     */
	public function _toggle_sync_status() {
	    $mode = $_POST['mode'];
	    $vehicle_id = intval($_POST['vehicle_id']);

	    if($vehicle_id < 1) {
	        $response = [
	            'code' => 0, 'msg' => 'no vehicle id'
            ];
        }
	    else {
            if($mode == "deactivate") {
                update_post_meta($vehicle_id,'wpcs_sync_disabled',1);
                #FilterTable::getInstance()->updateField($vehicle_id, 'wpcs_sync_disabled', 1);
                $response = [
                    'code' => 1, 'msg' => 'Sync now disabled'
                ];
            } else {
                delete_post_meta($vehicle_id,'wpcs_sync_disabled');
                #FilterTable::getInstance()->updateField($vehicle_id, 'wpcs_sync_disabled', 0);
                $response = [
                    'code' => 1, 'msg' => 'Sync now enabled'
                ];
            }
        }

        header('Content-Type: application/json');
        die(json_encode($response));
    }




    /**
     * eine meta box für alle Fahrzeug Custom Fields registrieren
     * @param $post
     */
	public function _custom_fields_meta_box( $post ) {

        add_meta_box(
            'wpcs-filter-table',
            __( 'Filter - Daten' ),
            [&$this,'_render_filter_table_meta_box'],
            'vehicle',
            'normal',
            'high'
        );

        add_meta_box(
			'wpcs-custom-fields',
			__( 'Fahrzeug - Daten' ),
			[&$this,'_render_custom_fields_meta_box'],
			'vehicle',
			'normal'
		);


	}


	/**
     * Funktion zur Darstellung der Custom Field Meta Box
	 * @param $post
	 */
	public function _render_custom_fields_meta_box($post) {

		$html = '<div id="wpcs_custom_fields_meta_box">';

        $meta = get_post_meta($post->ID);
        foreach($meta as $key => $value) {
            $html .= '<div class="item">';
                $html .= '<label>'.$key.'</label><span class="value">'.implode(",",$value).'<br />';
            $html .= '</div>';
        }

		$html .= '</div>'; // wpcs_custom_fields

		print $html;
	}

    /**
     * @param $post
     * @return void
     */
    public function _render_filter_table_meta_box($post) {
		$html = '<div id="wpcs_filter_table_meta_box">';

        $data = FilterTable::getInstance()->getByVehicle($post->ID);
        $keys = array_keys($data);
        sort($keys);

        foreach($keys as $field) {
            $value = $data[$field];
            if($field == "post_id") continue;

            $html .= '<div class="item" data-field="'.$field.'">';
                $html .= '<label>'.$field.'</label>';
                $html .= '<input type="text" class="wpcs_filter_table_value" value="'.$value.'" name="'.$field.'" / >';
                $html .= '<a href="#" class="button button-primary wpcs_filter_table_save_btn">update</a>';

            $html .= '</div>';
        }

		$html .= '</div>'; // wpcs_custom_fields
		print $html;
	}


	/**
	 * liefert zu einem übergeben vehicle und key den passenden customfield wert
	 * @param $vehicle
	 * @param $key
	 * @param $value_key
	 * @return mixed
	 */
	public function getValue($vehicle, $key) {

		$field = $this->getField($key);
		$cf = $this->getFieldName($key);
        #if($key == "filiale") wpcs_p($field);

        // Sonderfall Bilder
		if($key == "image") {
			$value = VehicleData::getVehicleImages($vehicle)['big'][0];
			return $value;
		}

		elseif($key == "make") {
			$value_key = $vehicle->dxim_meta[$cf][0];
			$value = Make::_get($value_key);
		}

		elseif($key == "model") {
			$value_key = $vehicle->dxim_meta[$cf][0] ?? null;
			$value = Model::_get($value_key,$vehicle->dxim_meta['make'][0]);
		}

		elseif($key == "class") {
			$value_key = $vehicle->dxim_meta[$cf][0] ?? null;
			$value = VehicleClass::_get($value_key);
		}

		elseif($key == "category") {
			$value_key = $vehicle->dxim_meta[$cf][0] ?? null;
            $class = $vehicle->dxim_meta['class'][0] ?? null;
			$value = Category::_get($value_key,$class);
		}

		elseif($key == "location") {
            $l = new \DXIM\theme\LocationData();
            $location_id = get_post_meta($vehicle->ID, "ad_location", true);
            $location = $l->getById($location_id);
            $value = $location->name ?? null;
		}


		// CF Wert ist nur ein Array Key
		elseif(isset($field['fillable']['defaults']) && is_array($field['fillable']['defaults'])) {

            if(!isset($vehicle->dxim_meta[$cf])) $vehicle->dxim_meta[$cf] = null;

			if(is_array($vehicle->dxim_meta[$cf])) {
				$count = count($vehicle->dxim_meta[$cf]);
				foreach($vehicle->dxim_meta[$cf] as $value_key) {
					$tmp = $field['fillable']['defaults'][$value_key];
					if($count > 1) $value[] = $tmp;
					else $value = $tmp;
				}
			}

			if(empty($value)) {

				$value = isset($vehicle->dxim_meta[$cf][0]) ? $vehicle->dxim_meta[$cf][0] : null;
			}
		}

		// CF Wert ist nur ein Array Key
		elseif(isset($field['value']) && is_array($field['value'])) {

			if(isset($vehicle->dxim_meta[$cf]) && is_array($vehicle->dxim_meta[$cf])) {

				$count = count($vehicle->dxim_meta[$cf]);
				foreach($vehicle->dxim_meta[$cf] as $value_key) {
					$tmp = isset($field['value'][$value_key]) ? $field['value'][$value_key] : null;
					if($count > 1) $value[] = $tmp;
					else $value = $tmp;
				}

			}

			if(empty($value)) {
				$value = isset($vehicle->dxim_meta[$cf][0]) ? $vehicle->dxim_meta[$cf][0] : null;
			}

		}

		// CF Wert ist der finale Wert
		else {
            $value = isset($vehicle->dxim_meta[$cf][0]) ? $vehicle->dxim_meta[$cf][0] : null;
		}

        // CF Wert filter
		return apply_filters('DXIM_FILTER_VEHICLE_FIELD_'.$key,$value,$key,$vehicle);
	}


	/**
	 * @param $key
	 *
	 * @return mixed
	 */
	public function getField($key) {
		return VehicleFields::getField($key);
		#return $filter[$key] ?? null;
	}

	/**
	 * @param $key
	 *
	 * @return mixed
	 */
	public function getFieldName($key) {
		$field = $this->getField($key);
		return !empty($field) ? $field['meta_query']['key'] : $key;
	}

	/**
	 * @param $key
	 *
	 * @return mixed
	 */
	public function getFieldLabel($key) {
		$field = $this->getField($key);
		if($field) return $field['label'];
	}

    /**
     * @param $value
     * @param $default
     * @param $index
     * @return mixed|string
     */
    public static function getFromMetaArray($value, $default = "", $index = 0) {
        $val = is_array($value) ? $value[$index] : $value;
        return !empty($val) ? $val : $default;
    }


}