<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 09:47
 */

namespace DXIM;


class VehicleData {


	// current Vehicle on Single Page
	protected $vehicle = null;
	//
	protected $saved_vehicle = null;

	// wenn fields nicht leer sollen die felder in fields statt den standardfeldern angezeigt werden
	protected static $fields = [];

	//
	protected static $instance = null;

	/**
	 *
	 * @return VehicleData
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 * Vehicle constructor.
	 */
	public function __construct() {

		add_action('wp',function() {
			if(Conf::getInstance()->isVehicleSinglePage()) {
				$this->_init_vehicle();
			}
		});

		add_action('admin_init',function() {
            $this->_init_vehicle();
        });

	}

    /**
     * @param \WP_Post $vehicle
     */
	public function setVehicle(\WP_Post $vehicle) {
	    $vehicle = self::initDataObject($vehicle);
	    $this->vehicle = $vehicle;
    }

    /**
     *
     */
    public function saveVehicle() {
	    $this->saved_vehicle = $this->vehicle;
    }

    /**
     *
     */
    public function restoreVehicle() {
        $this->vehicle = $this->saved_vehicle;
    }

	/**
	 * @param $fields
	 */
	public static function setFields($fields) {
		foreach($fields as $key) {
			self::$fields[] = $key;
		}
	}

	/**
	 *
	 */
	public static function resetFields() {
		self::$fields = [];
	}

	/**
	 * Catch Vehicle Data from Url Parameter
	 */
	private function _init_vehicle() {
        $page_id = Conf::getInstance()->getVehicleSinglePage();

        // neue Version => vehicle wurde bereits durch main loop abgerufen
        if(empty($page_id)) {
            $vehicle = get_queried_object();
        }

        // alte Version => vehicle anhand URL slug mappen
        else {
            global $wp_query;
            $vehicle = null;

            // slug ermitteln
            $slug = $wp_query->query_vars['vehicle_slug'] ?? null;

            // anhand slug vehicle ermitteln
            if(!empty($slug)) $vehicle = get_page_by_path($slug,OBJECT,'vehicle');
        }

		// vehicle nicht vorhanden
		// => wenn user logged in Preview-Vehicle laden und seite trotzdem darstellen (wegen PageBuilder)
		// => wenn user nicht eingeloggt vehicle leer lassen (wird dann ins archiv umgeleitet)
		if(empty($vehicle) && is_user_logged_in()) $vehicle = $this->getPreviewVehicle();

		// set vehicle
        if(empty($vehicle)) return [];
		$this->vehicle = self::initDataObject($vehicle);

	}

	/**
	 * @return array
	 */
	public static function getVehicle() {
		return self::getInstance()->vehicle;
	}

	/**
	 * @return \WP_Post
	 */
	public function getPreviewVehicle() {
	    $vehicles = get_posts([
            'post_type' => 'vehicle', 'posts_per_page' => 1, 'orderby' => 'rand',
        ]);
		return (!empty($vehicles)) ? $vehicles[0] : [];
	}


	/**
	 * Sicherstellen dass es sich um einWP_Post Object inkl. dxim_meta custom fields handelt
	 * @param $vehicle
	 *
	 * @return mixed
	 */
    public static function initDataObject($vehicle) {

        if(!is_object($vehicle)) {
            $v = new Vehicle();
            $vehicle = $v->getById($vehicle);
        }

        if(!$vehicle instanceof \WP_Post) return $vehicle;

        if(!isset($vehicle->dxim_meta) && !empty($vehicle)) {
            $vehicle->dxim_meta = self::getVehicleMeta($vehicle->ID);
        }

        return apply_filters(DXIM_FILTER_QUERIED_VEHICLE_DATA,$vehicle);
    }

    /**
     * @param $vehicle
     * @return mixed
     */
    public static function getVehicleKey($vehicle) {
        return get_post_meta($vehicle->ID, 'seller_inventory_key', true);
    }

    /**
     * @param $vehicle
     * @return mixed
     */
    public static function getVehicleAdKey($vehicle) {
        return get_post_meta($vehicle->ID, 'ad_key', true);
    }


	/**
	 * @param \WP_Post $vehicle
	 *
	 * @return mixed
	 */
    public static function getVehicleContact(\WP_Post $vehicle) {
        $l = new \DXIM\theme\LocationData();
        $location_id = get_post_meta($vehicle->ID, "ad_location", true);
        $location = $l->getById($location_id);

        return apply_filters(DXIM_FILTER_VEHICLE_LOCATION_DATA,$location,$vehicle);
    }

    /**
     * @param \WP_Post $vehicle
     * @return array
     */
    public static function getVehicleFinancing(\WP_Post $vehicle) {
        $financing['annual_percentage_rate'] = $vehicle->dxim_meta['financing_annual_percentage_rate'][0] ?? '';
        $financing['bank'] = $vehicle->dxim_meta['financing_bank'][0] ?? '';
        $financing['closing_costs'] = $vehicle->dxim_meta['financing_closing_costs'][0] ?? '';
        $financing['final_installment'] = $vehicle->dxim_meta['financing_final_installment'][0] ?? '';
        $financing['first_installment'] = $vehicle->dxim_meta['financing_first_installment'][0] ?? '';
        $financing['gross_loan_amount'] = $vehicle->dxim_meta['financing_gross_loan_amount'][0] ?? '';
        $financing['monthly_installment'] = $vehicle->dxim_meta['financing_monthly_installment'][0] ?? '';
        $financing['net_loan_amount'] = $vehicle->dxim_meta['financing_net_loan_amount'][0] ?? '';
        $financing['nominal_interest_rate'] = $vehicle->dxim_meta['financing_nominal_interest_rate'][0] ?? '';
        $financing['payback_period'] = $vehicle->dxim_meta['financing_payback_period'][0] ?? '';
        $financing['payment_protection_insurance'] = $vehicle->dxim_meta['financing_payment_protection_insurance'][0] ?? '';
        $financing['type_of_nominal_interest_rate'] = $vehicle->dxim_meta['financing_type_of_nominal_interest_rate'][0] ?? '';

        foreach($financing as $key => $value) {
            if($key == "bank") continue;
            $value = str_replace(",",".",$value);

            #if($key == "final_installment" || $key == "first_installment" || $key == "gross_loan_amount" || $key == "net_loan_amount") $value = str_replace(".", "", $value);
            $financing[$key] = $value;
        }

        return $financing;
    }

    public static function getVehicleLeasing(\WP_Post $vehicle) {
        $leasing['annual_percentage_rate'] = $vehicle->dxim_meta['leasing_annual_percentage_rate'][0] ?? '';
        $leasing['gross_rate'] = $vehicle->dxim_meta['leasing_gross_rate'][0] ?? '';
        $leasing['down_payment'] = $vehicle->dxim_meta['leasing_down_payment'][0] ?? '';
        $leasing['final_payment'] = $vehicle->dxim_meta['leasing_final_payment'][0] ?? '';
        $leasing['term_of_contract'] = $vehicle->dxim_meta['leasing_term_of_contract'][0] ?? '';
        $leasing['rate_type'] = $vehicle->dxim_meta['leasing_rate_type'][0] ?? '';
        $leasing['mileage'] = $vehicle->dxim_meta['leasing_mileage'][0] ?? '';
        $leasing['low_mileage_compensation'] = $vehicle->dxim_meta['leasing_low_mileage_compensation'][0] ?? '';
        $leasing['extra_mileage_costs'] = $vehicle->dxim_meta['leasing_extra_mileage_costs'][0] ?? '';
        $leasing['lender'] = $vehicle->dxim_meta['leasing_lender'][0] ?? '';
        $leasing['nominal_interest_rate'] = $vehicle->dxim_meta['leasing_nominal_interest_rate'][0] ?? '';
        $leasing['total_amount'] = $vehicle->dxim_meta['leasing_total_amount'][0] ?? '';
        $leasing['net_loan_amount'] = $vehicle->dxim_meta['leasing_net_loan_amount'][0] ?? '';
        $leasing['destination_charges'] = $vehicle->dxim_meta['leasing_destination_charges'][0] ?? '';

        foreach($leasing as $key => $value) {
            if($key == "lender") continue;
            if(empty($value)) continue;

            $value = str_replace(",", ".", $value);

            if($key == "low_mileage_compensation" || $key == "extra_mileage_costs") $value /= 100;
            #if($key == "down_payment" || $key == "final_payment" || $key == "total_amount" || $key == "net_loan_amount" || $key == "mileage") $value = str_replace(".", "", $value);
            $leasing[$key] = $value;
        }

        return $leasing;
    }

    /**
     * @param \WP_Post $vehicle
     * @return mixed|void
     */
	public static function getVehiclePostContent(\WP_Post $vehicle) {
		$content = $vehicle->post_content;

		$content = strip_tags($content);
		$content = str_replace("\\",'<br />',$content);
		$content = preg_replace('/(\*\*)(.*?)(\*\*)/', "<strong>$2</strong>", $content);
		$content = str_replace("*",'-',$content);
        $content = str_replace("---",'----',$content);
        $content = str_replace("-----",'----',$content);
		$content = str_replace("----",'<hr />',$content);
		$content = nl2br($content);

        return apply_filters(DXIM_FILTER_VEHICLE_CONTENT_DATA,$content,$vehicle);
	}

    /**
     * @param \WP_Post $vehicle
     * @return array|mixed
     */
	public static function getVehicleHighlights(\WP_Post $vehicle) {
	    return isset($vehicle->dxim_meta['highlight']) ? $vehicle->dxim_meta['highlight'] : [];
    }


    /**
     * @param \WP_Post $vehicle
     * @return array
     */
    public static function getVehicleTitle(\WP_Post $vehicle) {

        $defaults = ['model' => null, 'model_description' => null, 'make' => null,];
        $vehicle->dxim_meta = array_merge($defaults,$vehicle->dxim_meta);
        $description = $vehicle->dxim_meta['model_description'][0] ?? null;
        $description_clean = $description;
        $model = null;

        if(isset($vehicle->dxim_meta['model'])) {
            $model = Model::_get($vehicle->dxim_meta['model'][0],$vehicle->dxim_meta['make'][0]);
            $length_model = strlen($model);
            $pos_model = strpos($description_clean, $model);
            if($pos_model !== false) {
                $pos_model = 0;
                $offset = $length_model + $pos_model;
                if($offset > strlen($description_clean)) $offset = 0;

                $pos_space = strpos($description_clean, " ", $offset);

                $model = substr($description_clean,$pos_model,$pos_space - $pos_model);
                $description_clean = trim(substr_replace($description_clean,"",$pos_model, $pos_space - $pos_model));
            } else $model = null;


        }

        $make = Make::_get($vehicle->dxim_meta['make'][0]);
        $title = [
            'make' => $make,
            'model' => $model,
            'model_description' => $description,
            'model_description_clean' => trim($description_clean)
        ];

        return apply_filters(DXIM_FILTER_VEHICLE_TITLE_DATA,$title,$vehicle);
    }


    /**
     * @param $vehicle_id
     * @return array
     */
    public static function getVehicleMeta($vehicle_id) {
        $meta = get_post_custom($vehicle_id);
        return $meta;
    }

    /**
     * @param $cf
     * $_20 = 800 width
     * $_24 = 298 width
     * $_27 = 640 width // von mobile.de max. geliefert
     * $_57 = full width
     * @return mixed
     */
    public static function getVehicleImages($vehicle, $args = []) {
		if(is_int($vehicle)) $vehicle = get_post($vehicle);
        $vehicle = self::initDataObject($vehicle);
        $options = Admin::getInstance()->getOptions();
        $defaults = ['image_type' => $options['archive_image_type'] ?? "default"];
        $args = array_merge($defaults, $args);
        $image_config = Account::getInstance()->getImageConfig($vehicle->dxim_meta['ad_account'][0]);

        /**
         * Bilder aus Dateisystem laden
         */
        if($image_config == "filesystem") {
            $files = self::getVehicleImagesFilesystem($vehicle->ID, $args['image_type']);
            $images = ['big' => $files, 'large' => $files, 'small' => $files];
        }

        /**
         * Bildverweise laden
         */
        elseif($image_config == "links") {
            $images = self::getVehicleImageLinks($vehicle);
        }

        /**
         * Bilder aus Mediathek laden
         */
        elseif($image_config == "media") {
            $images = self::getVehicleImagesMedia($vehicle, $args['image_type']);
        }


        if(empty($images['big'])) {
            $placeholder = Admin::getInstance()->getOptions()['placeholder_image'];
            if(empty($placeholder)) $placeholder = DXIM_PluginUrl.'/img/wpcarsync_no_image.gif';
            $images['small'][] = $images['large'][] = $images['big'][] = $placeholder;
        }

        $image_string = Admin::getInstance()->getOptions('general_vehicle_image_usage') ?? null;
        $image_usage = [];
        if(!empty($image_string)) $image_usage = explode(",",$image_string);

        /**
         * Bildnummern auswerten
         */
        $z = 0;
        $new_images = [];
        foreach($image_usage as $nr) {
            $index = $nr - 1;
            if(!isset($images['big'][$index])) continue;
            $new_images['big'][$z] = $images['big'][$index];
            $new_images['large'][$z] = $images['large'][$index];
            $new_images['small'][$z] = $images['small'][$index];
            $z++;
        }
        if(!empty($new_images)) $images = $new_images;

        return apply_filters(DXIM_FILTER_VEHICLE_IMAGES_DATA,$images,$vehicle);
    }




        /**
	 * @param \WP_Post $vehicle
	 *
	 * @return mixed
	 */
    private static function getVehicleImageLinks(\WP_Post $vehicle) {

        $images_serialized = $vehicle->dxim_meta['images'][0] ?? null;
	    $images = unserialize($images_serialized);

        if(!empty($images['big'])) {
            if(empty($images['large'])) $images['large'] = $images['big'];
            if(empty($images['small'])) $images['small'] = $images['large'];
        }

        return $images;
    }

    /**
     * @param \WP_POST $vehicle
     * @return mixed
     */
    private static function getVehicleImagesMedia(\WP_POST $vehicle, $image_type) {
        if($image_type == "csp" || $image_type == "csp_crop") {
            $ids = get_post_meta($vehicle->ID,"images_media_csp",true);
        }

        if(empty($ids)) {
            $image_type = "default";
            $ids = get_post_meta($vehicle->ID,"wpcs_db_images",true);
        }

        if(!is_array($ids)) $ids = [];
        foreach($ids as $attachment_id) {
            $big = wp_get_attachment_image_src($attachment_id,'full')[0];
            $ext = substr($big,-3);
            if(($ext == "png" && $image_type != "csp_crop") || ($ext == "jpg" && $image_type == "csp_crop")) continue;

            $images['big'][] = $big;
            $images['large'][] = wp_get_attachment_image_src($attachment_id,'large')[0];
            $images['small'][] = wp_get_attachment_image_src($attachment_id,'medium')[0];
        }

        return $images;
    }

    /**
     * @param $vehicle_id
     * @param $image_type
     * @return array|false|string[]
     */
    private static function getVehicleImagesFilesystem($vehicle_id, $image_type) {
        $image_name = get_post_meta($vehicle_id, "image_id", true);
        if (empty($image_name)) return [];

        // CarShots
        if($image_type == "csp_crop" || $image_type == "csp") {
            $files = CarShotsPRO::getInstance()->getImages($vehicle_id, $image_type);
        }

        // Standard
        if(empty($files)) {
            $file_names = get_post_meta($vehicle_id, "images_filesystem", true);
            // bildnamen werden beim import in db vermerkt
            $files = [];
	        if(!empty($file_names)) {
				foreach($file_names as $file_name) {
					$file = trailingslashit(FilesystemImport::getInstance()->get_uploads_dir()).$file_name;
					if(!is_file($file)) continue;
					$files[] = trailingslashit(FilesystemImport::getInstance()->get_uploads_url()).$file_name;
				}
            }
            // fallback alte version, dateisystem nach bildnamen durchsuchen
            else $files = FilesystemImport::getInstance()->getImagesByName($image_name);
        }

        return $files;
    }

    /**
     * @param \WP_Post $vehicle
     * @return mixed
     */
    public static function getVehicleEmission(\WP_Post $vehicle) {
        $e = new VehicleEmission();
        $emission = $e->get_vehicle_emission($vehicle->dxim_meta);
        return apply_filters(DXIM_FILTER_VEHICLE_EMISSION_DATA,$emission,$vehicle);
    }


    /**
     * @param $price
     * @return string
     */
    private static function _calculatePrice($price, $args) {
        $marge_min = $args['marge_min'] ?? "";
        $marge_max = $args['marge_max'] ?? "";
        $marge_percentage = $args['marge_percentage'] ?? "";

        if($marge_min === "") $marge_min = 0;
        if($marge_max === "") $marge_max = 0;
        if($marge_percentage === "") $marge_percentage = 0;

        $formatted = str_replace(',', '.', $price);
        $interest = round((float)$formatted * ((float)$marge_percentage / 100),2);

        // add margin result to price
        if ($interest > $marge_max) {
            // if margin result > margin_max return price + margin_max
            return (float)$formatted + $marge_max;
        } else if ($interest < $marge_min) {
            // if margin result < margin_min return price + margin_min
            return (float)$formatted + $marge_min;
            // else return price + margin result
        } else {
            return (float)$formatted + $interest;
        }
    }


    /**
     * @param \WP_Post $vehicle
     * @return array
     */
    public static function getVehiclePrice(\WP_Post $vehicle, $args = []) {
	    $data = [];
        $options = Admin::getInstance()->getOptions();
        $price_display = $options['vehicle_price_display'] ?? "brutto";
        $rate_display = $options['vehicle_price_rate'] ?? "both";
        $price_src = $options['vehicle_price_src'] ?? "consumer_price";

        $defaults = apply_filters(DXIM_FILTER_PRICE_ARGS_DEFAULTS, ['price_field' => $price_src, 'marge_min' => 0, 'marge_max' => 0, 'marge_percent' => 0]);

        $args = array_merge($defaults, $args);
        $cf = $vehicle->dxim_meta;

	    if(empty($price_display)) $price_display = 'brutto';

        // price
        $price_data = $cf[$args['price_field']][0] ?? 0;
        $price = floatval($price_data);
        if(empty($price)) {
            $price_data = $cf['consumer_price'][0] ?? 0;
            $price = floatval($price_data);
        }
        if(empty($price)) return null;

        $price = self::_calculatePrice($price, $args);

        $price = apply_filters(DXIM_FILTER_PRICE_SOURCE,$price,$vehicle);
        $decimal = intval($options['vehicle_price_decimal']);
        $decimals = ($decimal > 0) ? 2 : 0;

        // vat
        $cf_vat_rate = isset($cf['vat_rate'][0]) ? $cf['vat_rate'][0] : null;
        $vat_rate = floatval($cf_vat_rate);
	    $vatable = ($cf['vatable'][0] == 'true') ? 1 : 0;
        $mwst = ($vat_rate > 0) ? (100 * $vat_rate) : null;

        $data['vat_rate'] = $vat_rate;
	    $data['vatable'] = $vatable;
	    $data['price_type'] = isset($cf['price_type'][0]) ? $cf['price_type'][0] : null;
	    $data['price_display'] = $price_display;
	    $data['price_rating'] = isset($cf['price_rating'][0]) ? $cf['price_rating'][0] : null;
        $data['rate_display'] = $rate_display;
	    $data['financing'] = VehicleData::getVehicleFinancing($vehicle);
	    $data['leasing'] = VehicleData::getVehicleLeasing($vehicle);
	    $data['vehicle_price_display'] = $price_display;

        /**
         * MwSt.
         */
	    if($vatable) $data['vat_msg'] = apply_filters(DXIM_FILTER_VEHICLE_PRICE_VAT_MSG, $mwst.'% MwSt.');
        else $data['vat_msg'] = apply_filters(DXIM_FILTER_VEHICLE_PRICE_NO_VAT_MSG,null);



        $price_netto = ($mwst !== null) ? round($price / (100+$mwst) * 100,2) : $price;
	    $data['price'] = $price;
	    $data['price_netto'] = $price_netto;

        //
	    if($price > 0) {

            $data['display_price_brutto'] = number_format($price,$decimals,",",".").' &euro;';

	    	if($vatable) {
			    $data['display_price_netto'] = number_format($price_netto,$decimals,",",".").' &euro;';
		    }
            else $data['display_price_netto'] = $data['display_price_brutto'];
		}

        return apply_filters(DXIM_FILTER_VEHICLE_PRICE_DATA,$data,$vehicle);
    }

    /**
     * @param \WP_Post $vehicle
     * @return array
     */

    public static function getVehicleFeatures(\WP_Post $vehicle, $args = []) {
        $vehicle = self::initDataObject($vehicle);
        $vehicle_features = [];

        $layout = Admin::getInstance()->getOptions('single_vehicle_features_layout') ?? null;
        $layout = $args['layout'] ?? $layout;
        $features = VehicleFields::getFeatures();

        foreach($vehicle->dxim_meta as $key => $arr) {
            $filter_key = strtolower(str_replace("feature_","",$key));
            $feature = $features[$filter_key] ?? null;
            if(empty($feature)) continue;

            $cat = $feature['feature']['cat'] ?? 'other';
            foreach($arr as $value) {
                $value = VehicleFilter::getInstance()->getReadableValue($filter_key, $value);

                // Get Filters Grouped by Category
                if($layout == "grouped") {
                    $vehicle_features[$cat][$filter_key][] = $value;
                }
                // get all Filters ungrouped
                else {
                    $vehicle_features[$filter_key][] = $value;
                }
            }
        }

        return apply_filters(DXIM_FILTER_VEHICLE_FEATURES_DATA,$vehicle_features,$vehicle);
    }


    /**
     * @param \WP_Post $vehicle
     * @return array|\WP_Error
     */
    public static function getVehicleFields(\WP_Post $vehicle, $page = 'archive') {

        // $page Parameter kann auch array mit Fields Angaben sein
        $restore_fields = false;
        if(is_array($page) && !empty($page)) {
            self::setFields($page);
            $restore_fields = true;
        }
        elseif($page != "archive") $page = 'single';

        if(!empty(self::$fields)) $fields = self::$fields;
        else $fields = Admin::getInstance()->getOptions('active_fields_'.$page);

        $specifics = [];
        $vcf = VehicleCF::getInstance();

        if(empty($fields)) {
            return new \WP_Error('no_fields_configured','Bitte konfigurieren Sie die Felder unter Einstellungen -> WPCarSync');
        }
        else {
            foreach($fields as $key) {
                $value = $vcf->getValue($vehicle, $key);
				$label = $vcf->getFieldLabel($key);
                $cf = $vcf->getFieldName($key);
                $raw = $vehicle->dxim_meta[$cf][0] ?? null;
                $tmp = [
                    'key' => $key,
                    'label' => $label,
                    'value_raw' => $raw,
	                'display_value' => $value
                ];

	            $specifics[$key] = $tmp;
            }
        }

        #wpcs_p($specifics);

        if($restore_fields) self::resetFields();

        return apply_filters(DXIM_FILTER_VEHICLE_SPECIFICS_DATA,$specifics,$vehicle);
    }


}