<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 10/01/16
 * Time: 12:37
 */

namespace DXIM;


use DXIM\theme\LocationData;

class VehicleFields {

    protected static $instance = null;

    /**
     * @var array
     */
    private $fields = [

        // Hersteller
        'make' => array(
            // Bezeichnung des Feldes
            'label' => 'Hersteller',

            // Darstellung des Feldes im Suchformular
            'type' => 'select',

            // Custom Field Settings (Feldname, Suchparameter)
            'meta_query' => array('key' => 'make'),

            'data_source' => [
                'callback' => [Make::class, '_get'],
            ],

            // beindet sich das Feld standardmäßig im Suchformular und an welcher Position
            'default_form_field_column' => 1,

            // ist das feld eine Fahrzeug-Property
            // wenn ja kann es in Archive- und Single Seiten, sowie Shortcodes als Feld angezeigt werden
            'specific' => [
                // standard-feld für archive
                'default_archive' => false,

                // standard feld für single
                'default_single' => false,
            ],

            // gehört zur Kategorie "basic" (weitere Kategorien: feature, specific)
            'basic' => [],

            'fillable' => [
                //
                'type' => 'select'
            ],

        ),
        // Modell
        'model' => array(
            'label' => 'Modell',
            'type' => 'select',
            'meta_query' => array('key' => 'model'),
            'data_source' => [
                'callback' => [Model::class, '_get'],
                'depends' => 'make'
            ],
            //'elementArgs' => array('class' => 'dxim_ff_model'),
            'default_form_field_column' => 1,
            'global_query_filter' => 1,
            'specific' => [
                'default_archive' => false, 'default_single' => false,
            ],
            'basic' => [],
            'fillable' => [
                //
                'type' => 'select'
            ],
        ),
        // Preis min
        'model_description' => array(
            'label' => 'Modelbezeichnung',
            'type' => 'text',
            'meta_query' => array('key' => 'model_description', 'compare' => 'LIKE'),
            'basic' => []
        ),
        // Klasse
        'class' => array(
            'label' => 'Typ',
            'type' => 'select',
            'multiple' => false,
            'meta_query' => array('key' => 'class'),
            'data_source' => [
                'callback' => [VehicleClass::class, '_get'],
            ],
            'default_form_field_column' => 1,
            'global_query_filter' => 1,
            'specific' => [
                'default_archive' => true, 'default_single' => true
            ],
            'basic' => [],
            'fillable' => [
                //
                'type' => 'select'
            ],
        ),
        // Kategorie
        'category' => array(
            'label' => 'Kategorie',
            'type' => 'select',
            'meta_query' => array('key' => 'category'),
            'data_source' => [
                'callback' => [Category::class, '_get'],
                'depends' => 'class',
                'depends_default' => 'Car',
                'filter' => ['make', 'model', 'class']
            ],
            'global_query_filter' => 1,
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'basic' => [],
            'fillable' => [
                //
                'type' => 'select'
            ],
        ),
        // Account
        'location' => array(
            'label' => 'Standort',
            'type' => 'select',
            'meta_query' => array('key' => 'ad_location'),
            'global_query_filter' => 1,
            'specific' => [
                'default_archive' => false, 'default_single' => false,
            ],
            'basic' => [],
            'fillable' => [
                //
                'type' => 'select'
            ],
        ),
        // HU
        'general_inspection' => array(
            'label' => 'HU',
            'cache' => 1,
            'meta_query' => array('key' => 'specific_general_inspection'),
            'specific' => [
                'default_archive' => false, 'default_single' => true,
            ],
            'type' => 'range',
            'fillable' => [
                //
                'type' => 'text',
                'format' => 'date',
                'description' => 'yyyy-mm'
            ],
            'filterform' => false,
        ),

        // delivery date
        'delivery_date' => array(
            'label' => 'Lieferdatum',
            'meta_query' => array('key' => 'specific_delivery_date'),
            'specific' => [
                'default_archive' => false, 'default_single' => true,
            ],
            'field_type' => "date",

        ),

        // delivery date
        'delivery_period' => array(
            'label' => 'Lieferfrist',
            'meta_query' => array('key' => 'specific_delivery_period'),
            'specific' => [
                'default_archive' => false, 'default_single' => true,
            ],
            'field_type' => "date",

        ),

        // Seller Inventory Key
        'seller_inventory_key' => array(
            'label' => 'interne Nummer',
            'type' => 'text',
            'meta_query' => array('key' => 'seller_inventory_key', 'compare' => 'LIKE'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'basic' => []
        ),
        // Hubraum
        'cubic_capacity' => array(
            'label' => 'Hubraum',
            'cache' => 1,
            'meta_query' => array('key' => 'specific_cubic_capacity'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'type' => 'range',
            'field_type' => "int",
            'unit' => 'cm<sup>3</sup>',
            'html_formatting' => true,
        ),
        // Hubkraft
        'lifting_capacity' => array(
            'label' => 'Hubkraft',
            'meta_query' => array('key' => 'specific_lifting_capacity'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'field_type' => "int",

        ),
        // Hubhöhe
        'lifting_height' => array(
            'label' => 'Hubhöhe',
            'meta_query' => array('key' => 'specific_lifting_height'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'field_type' => "int",
        ),
        // Aufbauhöhe
        'installation_height' => array(
            'label' => 'Aufbauhöhe',
            'meta_query' => array('key' => 'specific_installation_height'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'field_type' => "int",
        ),
        // Preis
        'price' => array(
            'label' => 'Preis',
            'cache' => 1,
            'type' => 'range',
            'filter_conf' => [
                'steps' => [500],
            ],
            'meta_query' => array('key' => 'consumer_price'),
            'basic' => [],
            'unit' => '&euro;',
            'html_formatting' => true,
            'filterform' => true,
            'field_type' => "int",
        ),
        // Preis Type
        'price_type' => array(
            'label' => 'Preisart',
            'meta_query' => array('key' => 'price_type'),
            'basic' => [],
            'fillable' => [
                //
                'type' => 'select',
            ],
            'filterform' => false
        ),
        // Preis Rating
        'price_rating' => array(
            'label' => 'Preisbewertung',
            'type' => 'select',
            'meta_query' => array('key' => 'price_rating'),
            'basic' => [],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => [
                    'HIGH_PRICE' => 'hoher Preis',
                    'INCREASED_PRICE' => 'erhöhter Preis',
                    'REASONABLE_PRICE' => 'fairer Preis',
                    'GOOD_PRICE' => 'guter Preis',
                    'VERY_GOOD_PRICE' => 'sehr guter Preis'
                ]
            ],
            'specific' => [],
            'field_type' => "enum",
        ),
        'vat_rate' => array(
            'label' => 'Mehrwertsteuer',
            'meta_query' => array('key' => 'vat_rate'),
            'basic' => [],
            'field_type' => "decimal",
        ),
        // MwSt. ausweisbar
        'vatable' => array(
            'label' => 'MwSt. ausweisbar',
            'type' => 'select',
            'meta_query' => array('key' => 'vatable'),
            'basic' => [],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['false' => 'nein', 'true' => 'ja']
            ],
            //'field_type' => "bool",
        ),
        // Preis min
        'price_min' => array(
            'label' => 'Preis min.',
            'type' => 'slider',
            'filter_conf' => array(),
            'meta_query' => array('key' => 'consumer_price', 'compare' => '>=', 'type' => 'DECIMAL'),
            'field_type' => "int",
            'unit' => '&euro;',
            'html_formatting' => true
        ),
        // Preis max
        'price_max' => array(
            'label' => 'Preis max.',
            'type' => 'slider',
            'filter_conf' => array(),
            'meta_query' => array('key' => 'consumer_price', 'compare' => '<=', 'type' => 'DECIMAL'),
            'default_form_field_column' => 1,
            'field_type' => "int",
            'unit' => '&euro;',
            'html_formatting' => true
        ),
        // Power
        'power' => array(
            'label' => 'Leistung',
            'meta_query' => array('key' => 'specific_power'),
            'cache' => true,
            'specific' => [
                'default_archive' => true, 'default_single' => true
            ],
            'fillable' => [
                'description' => 'in kW'
            ],
            'filterform' => true,
            'type' => 'range',
            'html_formatting' => true,
            'unit'=> 'kW',
            'field_type' => "int",
            'filter_conf' => [
                'step' => 5,
            ],
        ),
        // Power min
        'power_min' => array(
            'label' => 'Leistung min.',
            'type' => 'slider',
            'field_type' => "int",
            'unit'=> 'kW',
            'html_formatting' => true,
            'filter_conf' => array(
                'step' => 5,
            ),
            'meta_query' => array('key' => 'specific_power', 'compare' => '>=', 'type' => 'DECIMAL'),
        ),
        // Power max
        'power_max' => array(
            'label' => 'Leistung max.',
            'type' => 'slider',
            'field_type' => "int",
            'unit' => 'kW',
            'html_formatting' => true,
            'filter_conf' => array(
                'step' => 5,
            ),
            'meta_query' => array('key' => 'specific_power', 'compare' => '<=', 'type' => 'DECIMAL'),
        ),
        // Power
        'power_ps' => array(
            'label' => 'Leistung',
            'meta_query' => array('key' => 'specific_power_ps'),
            'cache' => true,
            'specific' => [],
            'fillable' => [
                'description' => 'in PS'
            ],
            'filterform' => true,
            'type' => 'range',
            'unit' => 'PS',
            'html_formatting' => true,
            'field_type' => "int",
            'filter_conf' => [
                'step' => 5,
            ],
        ),
        // Power min
        'power_ps_min' => array(
            'label' => 'Leistung min.',
            'type' => 'slider',
            'field_type' => "int",
            'filter_conf' => array(
                'step' => 5,
            ),
            'unit' => 'PS',
            'html_formatting' => true,
            'meta_query' => array('key' => 'specific_power_ps', 'compare' => '>=', 'type' => 'DECIMAL'),
        ),
        // Power max
        'power_ps_max' => array(
            'label' => 'Leistung max.',
            'type' => 'slider',
            'field_type' => "int",
            'html_formatting' => true,
            'unit' => 'PS',
            'filter_conf' => array(
                'step' => 5,
            ),
            'meta_query' => array('key' => 'specific_power_ps', 'compare' => '<=', 'type' => 'DECIMAL'),
        ),
        // Kilometer
        'mileage' => array(
            'label' => 'Kilometerstand',
            'meta_query' => array('key' => 'specific_mileage'),
            'cache' => true,
            'specific' => [
                'default_archive' => true, 'default_single' => true
            ],
            'filterform' => true,
            'type' => 'range',
            'field_type' => "int",
            'unit' => 'km',
            'html_formatting' => true,
            'filter_conf' => [
                'step' => 1000
            ],
        ),
        // Kilometer min
        'mileage_min' => array(
            'label' => 'Kilometerstand min.',
            'filterform' => true,
            'type' => 'slider',
            'unit' => 'km',
            'html_formatting' => true,
            'field_type' => "int",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_mileage', 'compare' => '>=', 'type' => 'DECIMAL'),
        ),
        // Kilometer max
        'mileage_max' => array(
            'label' => 'Kilometerstand max.',
            'filterform' => true,
            'type' => 'slider',
            'unit' => 'km',
            'html_formatting' => true,
            'field_type' => "int",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_mileage', 'compare' => '<=', 'type' => 'DECIMAL'),
            'default_form_field_column' => 1
        ),
        // Betriebsstunden
        'operating_hours' => array(
            'label' => 'Betriebsstunden',
            'meta_query' => array('key' => 'specific_operating_hours'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'type' => 'range',
            'field_type' => "int",
        ),
        // Betriebsstunden min
        'operating_hours_min' => array(
            'label' => 'Betriebsstunden min.',
            'type' => 'select',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_operating_hours', 'compare' => '>=', 'type' => 'DECIMAL'),
        ),
        // Betriebsstunden max
        'operating_hours_max' => array(
            'label' => 'Betriebsstunden max.',
            'type' => 'select',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_operating_hours', 'compare' => '<=', 'type' => 'DECIMAL'),
        ),
        // Kraftstoff
        'fuel' => array(
            'label' => 'Kraftstoff',
            'initBy' => 'db',
            'type' => 'select',
            'field_type' => "enum",
            'meta_query' => array('key' => 'specific_fuel'),
            'default_form_field_column' => 2,
            'specific' => [
                'default_archive' => false, 'default_single' => true
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => array(
                    'PETROL' => 'Benzin', 'DIESEL' => 'Diesel', 'LPG' => 'Autogas (LPG)', 'CNG' => 'Erdgas (CNG)', 'GAS' => 'Gas', 'ELECTRICITY' => 'Elektro', 'HYBRID' => 'Hybrid (Benzin/Elektro)',
                    'HYDROGENIUM' => 'Wasserstoff', 'ETHANOL' => 'Ethanol', 'HYBRID_DIESEL' => 'Hybrid (Diesel / Elektro)', 'HYBRID_ALL' => 'Hybrid', 'OTHER' => 'Andere'
                )
            ],
        ),
        // num seats
        'num_seats' => array(
            'label' => 'Anzahl Sitze',
            'type' => 'select',
            'field_type' => "int",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_num_seats'),
            'specific' => [
                'default_archive' => false, 'default_single' => true
            ],
            'fillable' => [
                //
                'type' => 'text',
            ],
        ),
        // door count
        'door_count' => array(
            'label' => 'Anzahl Türen',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_door_count'),
            'specific' => [
                'default_archive' => false, 'default_single' => true
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['TWO_OR_THREE' => '2/3', 'FOUR_OR_FIVE' => '4/5', 'SIX_OR_SEVEN' => '6/7']
            ],
        ),
        // emission class
        'emission_class' => array(
            'label' => 'Emissionsklasse',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_emission_class'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => [
                    'EURO1' => 'Euro1', 'EURO2' => 'Euro2', 'EURO3' => 'Euro3', 'EURO4' => 'Euro4', 'EURO5' => 'Euro5', 'EURO6' => 'Euro6',
                    'EURO6C' => 'Euro6c', 'EURO6D_TEMP' => 'Euro6d-TEMP', 'EURO6D' => 'Euro6d', 'EURO6E' => 'Euro6e', 'EURO7' => 'Euro7'
                ]
            ],
        ),

        // emission Sticker
        'emission_sticker' => array(
            'label' => 'Umweltplakette',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_emission_sticker'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => [
                    'EMISSIONSSTICKER_GREEN' => '4 (Grün)', 'EMISSIONSSTICKER_YELLOW' => '3 (Gelb)', 'EMISSIONSSTICKER_RED' => '2 (Rot)', 'EMISSIONSSTICKER_NONE' => '1 (Keine)'
                ]
            ],
        ),


        // Energieeffizienzklasse
//        'emission_energy_efficiency_class' => array(
//            'label' => 'Energieeffizienz',
//            'type' => 'select',
//            'field_type' => "enum",
//            'initBy' => 'db',
//            'meta_query' => array('key' => 'specific_emission_energy_efficiency_class'),
//            'specific' => [
//                'default_archive' => false, 'default_single' => false
//            ],
//            'fillable' => [
//                //
//                'type' => 'select',
//                'defaults' => ['A+++' => 'A+++', 'A++' => 'A++', 'A+' => 'A+', 'A' => 'A', 'B' => 'B', 'C' => 'C', 'D' => 'D', 'E' => 'E', 'F' => 'F', 'G' => 'G']
//            ],
//        ),
//
//        // co2
//        'emission_co2' => array(
//            'label' => 'NEFZ Co2 Ausstoss',
//            'meta_query' => array('key' => 'specific_emission_co2'),
//            'specific' => [
//                'default_archive' => false, 'default_single' => false
//            ],
//            'field_type' => "decimal",
//            'filterform' => false,
//        ),
//
//        // inner
//        'emission_inner' => array(
//            'label' => 'NEFZ innerorts',
//            'meta_query' => array('key' => 'specific_emission_inner'),
//            'specific' => [
//                'default_archive' => false, 'default_single' => false
//            ],
//            'field_type' => "decimal",
//            'filterform' => false
//        ),
//
//        // outer
//        'emission_outer' => array(
//            'label' => 'NEFZ außerorts',
//            'meta_query' => array('key' => 'specific_emission_outer'),
//            'specific' => [],
//            'field_type' => "decimal",
//            'filterform' => false
//        ),
//
//        // combined
//        'emission_combined' => array(
//            'label' => 'NEFZ Kraftstoffverbrauch (komb.) / l',
//            'meta_query' => array('key' => 'specific_emission_combined'),
//            'specific' => [],
//            'field_type' => "decimal",
//            'filterform' => false,
//        ),
//
//        // combined
//        'emission_power_combined' => array(
//            'label' => 'NEFZ Stromverbrauch (komb.) / l',
//            'meta_query' => array('key' => 'specific_emission_power_combined'),
//            'specific' => [],
//            'field_type' => "decimal",
//            'filterform' => false,
//        ),
//
//        // combined
//        'electric_range' => array(
//            'label' => 'Elektrische Reichweite',
//            'meta_query' => array('key' => 'specific_electric_range'),
//            'specific' => [],
//            'field_type' => "decimal",
//            'unit' => 'km',
//            'html_formatting' => true,
//            'filterform' => false,
//        ),

        // combined
        'envkv_fuel_type' => array(
            'label' => 'Benzinsorte',
            'meta_query' => array('key' => 'specific_envkv_fuel_type'),
            'specific' => [],
        ),


        // condition
        'condition' => array(
            'label' => 'Zustand',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_condition'),
            'global_query_filter' => 1,
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['NEW' => 'Neuwagen', 'USED' => 'Gebrauchtwagen']
            ],
        ),
        // usage type
        'usage_type' => array(
            'label' => 'Gebrauchtzustand',
            'type' => 'select',
            'initBy' => 'db',
            'field_type' => "enum",
            'meta_query' => array('key' => 'specific_usage_type'),
            'global_query_filter' => 1,
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['DEMONSTRATION' => 'Vorführwagen', 'PRE_REGISTRATION' => 'Tageszulassung', 'EMPLOYEES_CAR' => 'Jahreswagen', 'CLASSIC' => 'Oldtimer']
            ],
        ),

        // Fahrzeug beschaedigt
        'damage_unrepaired' => array(
            'label' => 'Fahrzeug beschädigt',
            'type' => 'select',
            'field_type' => "enum",
            'meta_query' => array('key' => 'specific_damage_unrepaired'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['false' => 'Nein', 'true' => 'Ja']
            ],
        ),
        // unfallfahrzeug
        'accident_damaged' => array(
            'label' => 'Unfallfahrzeug',
            'type' => 'select',
            'field_type' => "enum",
            'meta_query' => array('key' => 'specific_accident_damaged'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['false' => 'Nein', 'true' => 'Ja']
            ],
        ),

        // fahrtauglich
        'roadworthy' => array(
            'label' => 'fahrtauglich',
            'type' => 'select',
            'field_type' => "enum",
            'meta_query' => array('key' => 'specific_roadworthy'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                'type' => 'select',
                'defaults' => ['false' => 'Nein', 'true' => 'Ja']
            ],
        ),

        // Schaltung
        'gearbox' => array(
            'label' => 'Getriebe',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_gearbox'),
            'default_form_field_column' => 2,
            'specific' => [
                'default_archive' => false, 'default_single' => true
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['MANUAL_GEAR' => 'Schaltgetriebe', 'AUTOMATIC_GEAR' => 'Automatik', 'SEMIAUTOMATIC_GEAR' => 'Halbautomatik']
            ],
        ),
        // Erstzulassung
        'first_registration' => array(
            'label' => 'Erstzulassung',
            'meta_query' => array('key' => 'specific_first_registration'),
            'specific' => [
                'default_archive' => true, 'default_single' => true,
            ],
            'fillable' => [
                //
                'type' => 'text',
                'format' => 'date',
                'description' => 'yyyy-mm'
            ],
            'type' => 'range',
            'field_type' => "date",
        ),
        // Erstzulassung ab
        'first_registration_min' => array(
            'label' => 'Erstzulassung ab',
            'type' => 'slider',
            'field_type' => "date",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_first_registration', 'compare' => '>=', 'type' => 'DATE'),
            'default_form_field_column' => 2
        ),
        // Erstzulassung bis
        'first_registration_max' => array(
            'label' => 'Erstzulassung bis',
            'type' => 'slider',
            'field_type' => "date",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_first_registration', 'compare' => '<=', 'type' => 'DATE'),
        ),
        // Baujahr
        'construction_year' => array(
            'label' => 'Baujahr',
            'meta_query' => array('key' => 'specific_construction_year'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'add_values' => true,
            ],
            'type' => 'range',
            'field_type' => "date",
        ),

        // Baujahr ab
        'construction_year_min' => array(
            'label' => 'Baujahr ab',
            'type' => 'slider',
            'field_type' => "date",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_construction_year', 'compare' => '>=', 'type' => 'DECIMAL'),
        ),

        // Baujahr bis
        'construction_year_max' => array(
            'label' => 'Baujahr bis',
            'type' => 'slider',
            'field_type' => "date",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_construction_year', 'compare' => '<=', 'type' => 'DECIMAL'),
        ),

        // Farbe außen
        'exterior_color' => array(
            'label' => 'Farbe',
            'type' => 'color',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_exterior_color'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'add_values' => true,
                'defaults' => [
                    'BLACK' => 'Schwarz', 'GREY' => 'Grau', 'BEIGE' => 'Beige', 'BROWN' => 'Braun',
                    'RED' => 'Rot', 'GREEN' => 'Grün', 'BLUE' => 'Blau', 'PURPLE' => 'Lila', 'GOLD' => 'Gold',
                    'WHITE' => 'Weiß', 'ORANGE' => 'Orange', 'SILVER' => 'Silber', 'YELLOW' => 'Gelb',
                ]
            ],
        ),

        'manufacturer_color' => array(
            'label' => 'Hersteller Farbe',
            'type' => 'select',
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_manufacturer_color'),
            'specific' => [
                'default_archive' => false, 'default_single' => true
            ],
            'fillable' => [
                //
                'type' => 'select',
                'add_values' => true,
            ],
        ),

        // Farbe innen
        'interior_color' => array(
            'label' => 'Farbe Innenraum',
            'initBy' => 'db',
            'type' => 'select',
            'field_type' => "enum",
            'meta_query' => array('key' => 'specific_interior_color'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'BLACK' => 'Schwarz', 'GREY' => 'Grau', 'BEIGE' => 'Beige', 'BROWN' => 'Braun', 'WHITE' => 'Weiß', 'OTHER_INTERIOR_COLOR' => 'Andere',
                ]
            ]
        ),

        // Airbags
        'airbags' => array(
            'label' => 'Airbags',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_airbags'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['DRIVER_AIRBAG' => 'Fahrer-Airbag', 'FRONT_AIRBAGS' => 'Front-Airbags', 'FRONT_AND_SIDE_AIRBAGS' => 'Front- und Seiten-Airbags', 'FRONT_AND_SIDE_AND_MORE_AIRBAGS' => 'Front-, Seiten- und weitere Airbags',]
            ],
        ),

        // Battery
        'battery' => array(
            'label' => 'Batterie',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_battery'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
                'defaults' => ['BATTERY_RENTED' => 'enthalten/Miete', 'BATTERY_PURCHASED' => 'inklusive', 'BATTERY_MISSING' => 'ohne Batterie',]
            ],
        ),

        // Battery Capacity
        'battery_capacity' => array(
            'label' => 'Batteriekapazität',
            'type' => 'range',
            'field_type' => "int",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_battery_capacity'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'select',
            ],
            'unit' => 'kWh',
            'html_formatting' => true,
        ),

        // Kapaziät min
        'battery_capacity_min' => array(
            'label' => 'Batteriekapazität min',
            'type' => 'slider',
            'unit' => 'kWh',
            'html_formatting' => true,
            'field_type' => "int",
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_battery_capacity', 'compare' => '>=', 'type' => 'DECIMAL'),
        ),

        // Kapaziät max
        'battery_capacity_max' => array(
            'label' => 'Batteriekapazität max',
            'type' => 'slider',
            'field_type' => "int",
            'unit' => 'kWh',
            'html_formatting' => true,
            'filter_conf' => [],
            'meta_query' => array('key' => 'specific_battery_capacity', 'compare' => '<=', 'type' => 'DECIMAL'),
        ),

        // Anzahl Vorbesitzer
        'number_owners' => array(
            'label' => 'Anzahl Vorbesitzer',
            'type' => 'select',
            'field_type' => "int",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_number_owners'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ],
            'fillable' => [
                //
                'type' => 'text',
            ],
        ),

        // Klima
        'climatisation' => array(
            'label' => 'Klimaanlage',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_climatisation'),
            'feature' => [
                'cat' => 'comfort'
            ],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'NO_CLIMATISATION' => 'Keine Klimaanlage',
                    'MANUAL_CLIMATISATION' => 'Klimaanlage',
                    'AUTOMATIC_CLIMATISATION' => 'Klimaautomatik',
                    'AUTOMATIC_CLIMATISATION_2_ZONES' => '2-Zonen-Klimaautomatik',
                    'AUTOMATIC_CLIMATISATION_3_ZONES' => '3-Zonen-Klimaautomatik',
                    'AUTOMATIC_CLIMATISATION_4_ZONES' => '4-Zonen-Klimaautomatik',
                ]
            ]
        ),
        // Innenbereich
        'interior_type' => array(
            'label' => 'Innenbereich',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_interior_type'),
            'specific' => [
                'default_archive' => false, 'default_single' => true
            ],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'LEATHER' => 'Vollleder',
                    'PARTIAL_LEATHER' => 'Teilleder',
                    'FABRIC' => 'Stoff',
                    'VELOUR' => 'Velours',
                    'ALCANTARA' => 'Alcantara',
                    'OTHER_INTERIOR_TYPE' => 'Andere'
                ]
            ]
        ),

        // Parkassistent
        'parking_assistants' => array(
            'label' => 'Parkassistent',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_parking_assistants'),
            'feature' => [
                'cat' => 'assistance'
            ],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'FRONT_SENSORS' => 'Parksensoren vorne',
                    'REAR_SENSORS' => 'Parksensoren hinten',
                    'REAR_VIEW_CAM' => 'Kamera',
                    'AUTOMATIC_PARKING' => 'Selbstlenkende Systeme',
                    'CAM_360_DEGREES' => '360° Kamera'
                ]
            ]
        ),

        // Speed Control
        'speed_control' => array(
            'label' => 'Geschwindigkeitsregelanlage',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_speed_control'),
            'feature' => ['cat' => 'assistance'],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'CRUISE_CONTROL' => 'Tempomat',
                    'ADAPTIVE_CRUISE_CONTROL' => 'Abstandstempomat',
                ]
            ]
        ),

        // radio
        'radio' => array(
            'label' => 'Radio',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_radio'),
            'feature' => ['cat' => 'multimedia'],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'TUNER' => 'Tuner/Radio',
                    'DAB_RADIO' => 'Radio DAB',
                ]
            ]
        ),

        // Anhängerkupplung
        'trailer_coupling_type' => array(
            'label' => 'Anhängerkupplung',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_trailer_coupling_type'),
            'feature' => ['cat' => 'exterior'],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'TRAILER_COUPLING_FIX' => 'Anhängerkupplung fest',
                    'TRAILER_COUPLING_SWIVELING' => 'Anhängerkupplung schwenkbar',
                    'TRAILER_COUPLING_DETACHABLE' => 'Anhängerkupplung abnehmbar',
                ]
            ]
        ),

        // Tagfahrlicht
        'daytime_running_lamps' => array(
            'label' => 'Tagfahrlicht',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_daytime_running_lamps'),
            'feature' => ['cat' => 'exterior'],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'DAYTIME_RUNNING_LIGHTS' => 'Tagfahrlicht',
                    'LED_RUNNING_LIGHTS' => 'LED-Tagfahrlicht',
                ]
            ]
        ),

        // Scheinwerfer
        'headlight_type' => array(
            'label' => 'Scheinwerfer',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_headlight_type'),
            'feature' => ['cat' => 'exterior'],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'XENON_HEADLIGHTS' => 'Xenonscheinwerfer',
                    'BI_XENON_HEADLIGHTS' => 'Bi-Xenon Scheinwerfer',
                    'LED_HEADLIGHTS' => 'LED-Scheinwerfer',
                    'LASER_HEADLIGHTS' => 'Laserlicht',
                ]
            ]
        ),

        // Kurvenlicht
        'bending_lights_type' => array(
            'label' => 'Kurvenlicht',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_bending_lights_type'),
            'feature' => ['cat' => 'exterior'],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'BENDING_LIGHTS' => 'Kurvenlicht',
                    'ADAPTIVE_BENDING_LIGHTS' => 'adaptives Kurvenlicht',
                ]
            ]
        ),

        // Kurvenlicht
        'sliding_door_type' => array(
            'label' => 'Schiebetür',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'specific_sliding_door_type'),
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'SLIDING_DOOR_RIGHT' => 'Schiebetür rechts',
                    'SLIDING_DOOR_LEFT' => 'Schiebetür links',
                    'SLIDING_DOOR_BOTH_SIDED' => 'Schiebetür beidseitig',
                ]
            ]
        ),


        // Pannenhilfe
        'breakdown_service' => array(
            'label' => 'Pannenhilfe',
            'type' => 'select',
            'field_type' => "enum",
            'initBy' => 'db',
            'meta_query' => array('key' => 'feature_breakdown_service'),
            'feature' => ['cat' => 'savety'],
            'fillable' => [
                'type' => 'select',
                'defaults' => [
                    'SPARE_WHEEL' => 'Reserverad',
                    'EMERGENCY_WHEEL' => 'Notrad',
                    'REPAIR_KIT' => 'Pannenkit',
                ]
            ]
        ),

        /**
         * WOHNMOBILE
         */
        'dimension_length' => array(
            'label' => 'Länge',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_dimension_length'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'dimension_height' => array(
            'label' => 'Höhe',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_dimension_height'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'dimension_width' => array(
            'label' => 'Breite',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_dimension_width'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'number_of_bunks' => array(
            'label' => 'Schlafplätze',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_number_of_bunks'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'axles' => array(
            'label' => 'Achsen',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_axles'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'licensed_weight' => array(
            'label' => 'Zulässiges Gesamtgewicht',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_licensed_weight'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),

        /**
         * Anhänger
         */
        'loading_space_width' => array(
            'label' => 'Laderaumbreite',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_loading_space_width'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'loading_space_height' => array(
            'label' => 'Laderaumhöhe',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_loading_space_height'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'loading_space_length' => array(
            'label' => 'Laderaumlänge',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_loading_space_length'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),
        'load_capacity' => array(
            'label' => 'Nutzlast',
            'type' => 'text',
            'field_type' => "int",
            'meta_query' => array('key' => 'specific_load_capacity'),
            'specific' => [
                'default_archive' => false, 'default_single' => false
            ]
        ),

        /**
         * Highlight
         */
        'highlight' => array(
            'label' => 'Highlights',
            'type' => 'select',
            'meta_query' => array('key' => 'highlight'),
        ),

        // Bild ID
        'image_id' => [
            'label' => 'Bild ID',
            'meta_query' => ['key' => 'image_id'],
        ],

        // AD Key
        'ad_key' => [
            'label' => 'Ad Key',
            'meta_query' => ['key' => 'ad_key'],
        ],

        // Modelbasis
        'modelbase' => [
            'label' => 'Modellbasis'
        ],

        // Fahrzeugbeschreibung
        'vehicle_description' => [
            'label' => 'Fahrzeugbeschreibung',
            'filterform' => false,

        ],

        // VIN
        'identification_number' => [
            'label' => 'Fahrgestellnummer',
            'meta_query' => array('key' => 'specific_identification_number'),
            'specific' => [],
        ],

        // Händlerpreis
        'dealer_price' => [
            'label' => 'Händlerpreis',
            'field_type' => "decimal",
        ],

        // Bauhöhe
        'installation_height' => [
            'label' => 'Bauhöhe',
            'meta_query' => array('key' => 'specific_installation_height'),
            'specific' => [],
            'field_type' => "int",
        ],

        // Hydraulikanlage
        'hydraulic_installation' => [
            'label' => 'Hydraulikanlage',
            'meta_query' => array('key' => 'specific_hydraulic_installation'),
            'specific' => []
        ],

        // Radformel
        'wheel_formula' => [
            'label' => 'Radformel',
            'meta_query' => array('key' => 'specific_wheel_formula'),
            'specific' => []
        ],

        // Schwackecode
        'schwacke_code' => [
            'label' => 'Schwacke Code',
            'meta_query' => array('key' => 'specific_schwacke_code'),
            'specific' => []
        ],

        // HSN
        'hsn' => [
            'label' => 'HSN',
            'meta_query' => array('key' => 'hsn'),
        ],

        // TSN
        'tsn' => [
            'label' => 'TSN',
            'meta_query' => array('key' => 'tsn'),
        ],

        //Laderaum Europalette
        'europallet_storage_spaces' => [
            'label' => 'Laderaum Europalette',
            'meta_query' => array('key' => 'specific_europallet_storage_spaces'),
            'specific' => []
        ],

        // Ladevolumen Europalette
        'europallet_shipping_volume' => [
            'label' => 'Ladevolumen Europalette',
            'meta_query' => array('key' => 'specific_europallet_shipping_volume'),
            'specific' => []
        ],

        // effektiver Jahreszins
        'financing_annual_percentage_rate' => [
            'label' => 'Finanzierung effektiver Jahreszins',
            'meta_query' => array('key' => 'financing_annual_percentage_rate'),
            'field_type' => "int",
        ],

        // Anzahlung
        'financing_first_installment' => [
            'label' => 'Finanzierung Anzahlung',
            'meta_query' => array('key' => 'financing_first_installment'),
            'field_type' => "int",
        ],

        // monatliche Rate
        'financing_monthly_installment' => [
            'label' => 'Finanzierung Rate',
            'meta_query' => array('key' => 'financing_monthly_installment'),
            'field_type' => "int",
        ],

        // Schlussrate
        'financing_final_installment' => [
            'label' => 'Finanzierung Schlussrate',
            'meta_query' => array('key' => 'financing_final_installment'),
            'field_type' => "int",
        ],

        // Laufzeit
        'financing_payback_period' => [
            'label' => 'Finanzierung Laufzeit',
            'meta_query' => array('key' => 'financing_payback_period'),
            'field_type' => "int",
        ],

        // Bruttokreditbetrag
        'financing_gross_loan_amount' => [
            'label' => 'Finanzierung Bruttokreditbetrag',
            'meta_query' => array('key' => 'financing_gross_loan_amount'),
            'field_type' => "decimal",
        ],

        // Abschlussgebühren
        'financing_closing_costs' => [
            'label' => 'Finanzierung Abschlussgebühren',
            'meta_query' => array('key' => 'financing_closing_costs'),
            'field_type' => "decimal",
        ],

        // Ratenabsicherung
        'financing_payment_protection_insurance' => [
            'label' => 'Finanzierung Ratenabsicherung',
            'meta_query' => array('key' => 'financing_payment_protection_insurance'),
        ],

        // Nettokreditbetrag
        'financing_net_loan_amount' => [
            'label' => 'Finanzierung Nettokreditbetrag',
            'meta_query' => array('key' => 'financing_net_loan_amount'),
            'field_type' => "decimal",
        ],

        // Anbieterbank
        'financing_bank' => [
            'label' => 'Finanzierung Anbieterbank',
            'meta_query' => array('key' => 'financing_bank'),
        ],

        // Sollzinssatz
        'financing_nominal_interest_rate' => [
            'label' => 'Finanzierung Sollzinssatz',
            'meta_query' => array('key' => 'financing_nominal_interest_rate'),
            'field_type' => "decimal",
        ],

        // Sollzinssatz
        'financing_type_of_nominal_interest_rate' => [
            'label' => 'Finanzierung Art des Soll-Zinssatzes',
            'meta_query' => array('key' => 'financing_type_of_nominal_interest_rate'),
        ],

        // Sollzinssatz
        'financing_mileage' => [
            'label' => 'Finanzierung Laufleistung',
            'meta_query' => array('key' => 'financing_mileage'),
            'field_type' => "int",
        ],

        // Landesversion
        'country_version' => [
            'label' => 'Landesversion',
            'meta_query' => array('key' => 'specific_country_version'),
            'specific' => [],
            'type' => 'select'
        ],

        // Landesversion
        'video_url' => [
            'label' => 'Video Url',
            'meta_query' => array('key' => 'specific_video_url'),
            'specific' => []
        ],

        // Produktionsdatum
        'first_models_production_date' => [
            'label' => 'Produktionsdatum',
            'meta_query' => array('key' => 'specific_first_models_production_date'),
            'specific' => [],
            'field_type' => "date",
        ],

        // WLTP Verbrauch kombiniert
        'wltp_consumption_fuel_combined' => [
            'label' => 'Kraftstoffverbrauch (kombiniert)', 'unit' => 'fuel',
            'meta_query' => array('key' => 'specific_wltp_consumption_fuel_combined'),
            'field_type' => "decimal",
        ],

        // WLTP Co2 kombiniert
        'wltp_co2_emission_combined' => [
            'label' => 'CO<sub>2</sub>-Emissionen (kombiniert)', 'unit' => 'co2',
            'meta_query' => array('key' => 'specific_wltp_co2_emission_combined'),
            'field_type' => "int",
        ],

        // WLTP StromVerbrauch kombiniert
        'wltp_consumption_power_combined' => [
            'label' => 'Stromverbrauch (kombiniert)', 'unit' => 'electricity',
            'meta_query' => array('key' => 'specific_wltp_consumption_power_combined'),
            'field_type' => "decimal",
        ],

        // WLTP StromVerbrauch kombiniert
        'wltp_consumption_power_combined_weighted' => [
            'label' => 'Stromverbrauch (gewichtet kombiniert)', 'unit' => 'electricity',
            'meta_query' => array('key' => 'specific_wltp_consumption_power_combined_weighted'),
            'field_type' => "decimal",
        ],

        // WLTP Reichweite elektrisch
        'wltp_electric_range' => [
            'label' => 'Reichweite elektrisch', 'unit' => 'km',
            'meta_query' => array('key' => 'specific_wltp_electric_range'),
            'field_type' => "int",
            'html_formatting' => true,
        ],

        // WLTP Reichweite elektrisch
        'wltp_electric_range_eaer' => [
            'label' => 'Reichweite elektrisch (EAER)', 'unit' => 'km',
            'meta_query' => array('key' => 'specific_wltp_electric_range_eaer'),
            'field_type' => "int",
            'html_formatting' => true,
        ],

        // WLTP Reichweite komb. gewichtet
        'wltp_consumption_fuel_combined_weighted' => [
            'label' => 'Kraftstoffverbrauch (gewichtet kombiniert)', 'unit' => 'fuel',
            'meta_query' => array('key' => 'specific_wltp_consumption_fuel_combined_weighted'),
            'field_type' => "decimal",
        ],

        // WLTP innerstädtisch
        'wltp_consumption_fuel_slow' => [
            'label' => 'Kraftstoffverbrauch Innenstadt', 'unit' => 'fuel',
            'meta_query' => array('key' => 'specific_wltp_consumption_fuel_slow'),
            'field_type' => "decimal",
        ],

        // WLTP Stadtrand
        'wltp_consumption_fuel_medium' => [
            'label' => 'Kraftstoffverbrauch Stadtrand',  'unit' => 'fuel',
            'meta_query' => array('key' => 'specific_wltp_consumption_fuel_medium'),
            'field_type' => "decimal",
        ],

        // WLTP innerstädtisch
        'wltp_consumption_fuel_fast' => [
            'label' => 'Kraftstoffverbrauch Landstraße', 'unit' => 'fuel',
            'meta_query' => array('key' => 'specific_wltp_consumption_fuel_fast'),
            'field_type' => "decimal",
        ],

        // WLTP Autobahn
        'wltp_consumption_fuel_very_fast' => [
            'label' => 'Kraftstoffverbrauch Autobahn', 'unit' => 'fuel',
            'meta_query' => array('key' => 'specific_wltp_consumption_fuel_very_fast'),
            'field_type' => "decimal",
        ],

        // WLTP Co2 komb. gewichtet
        'wltp_co2_emission_combined_weighted' => [
            'label' => 'CO<sub>2</sub>-Emissionen (gewichtet kombiniert)', 'unit' => 'co2',
            'meta_query' => array('key' => 'specific_wltp_co2_emission_combined_weighted'),
            'field_type' => "int",
        ],

        // WLTP Co2 Klasse
        'wltp_co2_class' => [
            'label' => 'CO<sub>2</sub>-Klasse',
            'meta_query' => array('key' => 'specific_wltp_co2_class'),
        ],

        // WLTP Co2 Klasse gewichtet kombiniert
        'wltp_co2_class_weighted_combined' => [
            'label' => 'CO<sub>2</sub>-Klasse (gewichtet kombiniert)',
            'meta_query' => array('key' => 'specific_wltp_co2_class_weighted_combined'),
        ],

      // WLTP Co2 Klasse leere Batterie
        'wltp_co2_class_low_battery' => [
            'label' => 'CO<sub>2</sub>-Klasse (leere Batterie)',
            'meta_query' => array('key' => 'specific_wltp_co2_class_low_battery'),
        ],

        // WLTP Co2 Klasse leere Batterie
        'wltp_co2_emission_low_battery' => [
            'label' => 'CO<sub>2</sub>-Emissionen (leere Batterie)', 'unit' => 'co2',
            'meta_query' => array('key' => 'specific_wltp_co2_emission_low_battery'),
            'field_type' => "int",
        ],

        // WLTP Co2 Klasse leere Batterie
        'wltp_consumption_fuel_combined_low_battery' => [
            'label' => 'Kraftstoffverbrauch (kombiniert, leere Batterie)', 'unit' => 'fuel',
            'meta_query' => array('key' => 'specific_wltp_consumption_fuel_combined_low_battery'),
            'field_type' => "decimal",
        ],

        // WLTP Co2 Klasse leere Batterie
        'wltp_consumption_power_slow' => [
            'label' => 'Stromverbrauch (Innenstadt)', 'unit' => 'electricity',
            'meta_query' => array('key' => 'specific_wltp_consumption_power_slow'),
            'field_type' => "decimal",
        ],

        // WLTP Co2 Klasse leere Batterie
        'wltp_consumption_power_medium' => [
            'label' => 'Stromverbrauch (Stadt)', 'unit' => 'electricity',
            'meta_query' => array('key' => 'specific_wltp_consumption_power_medium'),
            'field_type' => "decimal",
        ],

        // WLTP Co2 Klasse leere Batterie
        'wltp_consumption_power_fast' => [
            'label' => 'Stromverbrauch (Landstraße)', 'unit' => 'electricity',
            'meta_query' => array('key' => 'specific_wltp_consumption_power_fast'),
            'field_type' => "decimal",
        ],

        // WLTP Co2 Klasse leere Batterie
        'wltp_consumption_power_very_fast' => [
            'label' => 'Stromverbrauch (Autobahn)', 'unit' => 'electricity',
            'meta_query' => array('key' => 'specific_wltp_consumption_power_very_fast'),
            'field_type' => "decimal",
        ],

        /**
         * Leasing
         */
        // effektiver jahreszins
        'leasing_annual_percentage_rate' => [
            'label' => 'Leasing effektiver Jahreszins',
            'meta_query' => array('key' => 'leasing_annual_percentage_rate'),
            'field_type' => "decimal",
        ],

        // monatliche rate
        'leasing_gross_rate' => [
            'label' => 'Leasing Rate brutto',
            'meta_query' => array('key' => 'leasing_gross_rate'),
            'field_type' => "int",
        ],

        // anzahlung
        'leasing_down_payment' => [
            'label' => 'Leasing Anzahlung',
            'meta_query' => array('key' => 'leasing_down_payment'),
            'field_type' => "decimal",
        ],

        // Schlusszahlung
        'leasing_final_payment' => [
            'label' => 'Leasing Schlussrate',
            'meta_query' => array('key' => 'leasing_final_payment'),
            'field_type' => "decimal",
        ],

        // Laufzeit
        'leasing_term_of_contract' => [
            'label' => 'Leasing Laufzeit',
            'meta_query' => array('key' => 'leasing_term_of_contract'),
            'field_type' => "int",
        ],

        // Schlussrate

        // Zielgruppe
        'leasing_rate_type' => [
            'label' => 'Leasing Typ',
            'meta_query' => array('key' => 'leasing_rate_type'),
            'field_type' => "decimal",
        ],

        // Laufleistung
        'leasing_mileage' => [
            'label' => 'Leasing Laufleistung',
            'meta_query' => array('key' => 'leasing_mileage'),
            'field_type' => "int",
        ],

        // Laufleistung jährlich
//        'leasing_annual_mileage' => [
//            'label' => 'Leasing Laufleistung jährlich',
//            'meta_query' => array('key' => 'leasing_annual_mileage'),
//            'field_type' => "int",
//        ],

        // Minderkilometer
        'leasing_low_mileage_compensation' => [
            'label' => 'Leasing Minderkilometer',
            'meta_query' => array('key' => 'leasing_low_mileage_compensation'),
            'field_type' => "int",
        ],

        // Mehrkilometer
        'leasing_extra_mileage_costs' => [
            'label' => 'Leasing Mehrkilometer',
            'meta_query' => array('key' => 'leasing_extra_mileage_costs'),
            'field_type' => "int",
        ],

        // Anbieterbank
        'leasing_lender' => [
            'label' => 'Leasing Bank',
            'meta_query' => array('key' => 'leasing_lender'),
        ],

        // Sollzinssatz
        'leasing_nominal_interest_rate' => [
            'label' => 'Leasing Sollzinssatz',
            'meta_query' => array('key' => 'leasing_nominal_interest_rate'),
            'field_type' => "decimal",
        ],


        // Art des Sollzinssdatzes

        // Bedingungen

        // Gesamtbetrag
        'leasing_total_amount' => [
            'label' => 'Leasing Gesamtbetrag',
            'meta_query' => array('key' => 'leasing_total_amount'),
            'field_type' => "decimal",
        ],

        // Nettokreditbetrag
        'leasing_net_loan_amount' => [
            'label' => 'Leasing Nettokreditbetrag',
            'meta_query' => array('key' => 'leasing_net_loan_amount'),
            'field_type' => "decimal",
        ],

        // Zulassungskosten
        'leasing_registration_fees' => [
            'label' => 'Leasing Zulassungskosten',
            'meta_query' => array('key' => 'leasing_registration_fees'),
            'field_type' => "decimal",
        ],

        // Zulassungskosten
        'leasing_destination_charges' => [
            'label' => 'Leasing Überführungskosten',
            'meta_query' => array('key' => 'leasing_destination_charges'),
            'field_type' => "decimal",
        ],

        /** -------------------------------------
         * interne felder
         */
        // wpcs eigenes feld wann Fshrzeug zum ersten mal importiert
        // benötigt zum benachrichtigen bei suchauftrag
        'first_import_time' => [
            'meta_query' => ['compare' => '>=', 'key' => 'first_import_time'],
            'field_type' => 'int',
            'filterform' => false,
            'in_filter_table' => 1,
            'ignore_sync' => 1
        ],

        // wenn dieses feld gesetzt wird das Fahrzeug nicht mehr beim sync berücksichtigt
        'wpcs_sync_disabled' => [
            'label' => 'Sync deaktiviert',
            'meta_query' => ['compare' => '=', 'key' => 'wpcs_sync_disabled'],
            'field_type' => 'bool',
        ],

        // wpcs_rate
        'wpcs_rate' => [
            'label' => 'Rate',
            'type' => 'range',
            'filter_conf' => [
                'steps' => [10],
            ],
            'meta_query' => array('key' => ['leasing_gross_rate', 'financing_monthly_installment']),
            'basic' => [],
            'unit' => '&euro;',
            'html_formatting' => true,
            'filterform' => true,
            'field_type' => "int",
            'in_filter_table' => 1,
        ],

        'wpcs_rate_min' => array(
            'label' => 'Rate min.',
            'type' => 'slider',
            'filter_conf' => array(),
            'meta_query' => array('key' => ['leasing_gross_rate', 'financing_monthly_installment'], 'compare' => '>=', 'type' => 'DECIMAL'),
            'field_type' => "int",
            'unit' => '&euro;',
            'html_formatting' => true
        ),

        'wpcs_rate_max' => array(
            'label' => 'Rate max.',
            'type' => 'slider',
            'filter_conf' => array(),
            'meta_query' => array('key' => ['leasing_gross_rate', 'financing_monthly_installment'], 'compare' => '<=', 'type' => 'DECIMAL'),
            'field_type' => "int",
            'unit' => '&euro;',
            'html_formatting' => true
        ),

    ];


    /**
     *
     * @return VehicleFields
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     *
     */
    private function __construct() {}

    /**
     * @return void
     */
    public function init() {
        $this->_combine_fields();
    }


    /**
     * ergänzt die Features als Field
     * @return void
     */
    private function _combine_fields() {

        // features
        foreach (Features::$features as $feature) {
            $filter = [
                'label' => $feature['label'],
                'type' => 'checkbox',
                'field_type' => "bool",
                'multiple' => 0,
                'meta_query' => ['key' => 'feature_' . strtolower($feature['key'])],
                'feature' => [
                    'cat' => $feature['cat']
                ]
            ];
            $this->fields[strtolower($feature['key'])] = $filter;
        }

        // Standorte (Accounts)
        $l = new \DXIM\theme\LocationData();
        $locations = $l->getAll();
        foreach ($locations as $location) {
            $this->fields['location']['fillable']['defaults'][$location->term_id] = $location->name;
        }

    }

    /**
     * @param $field_name
     * @return bool
     */
    public static function isSpecific($field_name) {
        $fields = self::getFields();
        return isset($fields[$field_name]['specific']) && !isset($fields[$field_name]['basic']);
    }

    /**
     * @param $field_name
     * @return bool
     */
    public static function isFeature($field_name) {
        $fields = self::getFields();
        return isset($fields[$field_name]['feature']) && !isset($fields[$field_name]['basic']);
    }

    /**
     * @return null
     */
    public static function getFields() {
        return apply_filters(DXIM_FILTER_FORM_FILTERS_ARRAY,self::getInstance()->fields);
    }

    /**
     * @param $key
     * @return array|mixed
     */
    public static function getField($key) {
        $fields = self::getFields();
        $field = $fields[$key] ?? null;
        if(is_null($field)) return [];

        $field['type'] = $field['type'] ?? "text";
        $field['field_type'] = $field['field_type'] ?? "text";
        return $field;
    }

    /**
     * @param $key
     * @return bool
     */
    public static function isField($key) {
        return !(empty(self::getField($key)));
    }

    /**
     * @param $key
     * @param $settings
     * @return void
     */
    public static function setField($key, $settings) {
        self::getInstance()->fields[$key] = $settings;
    }

    /**
     * @return array
     */
    public static function getFeatures() {
        $features = [];
        foreach(self::getInstance()->fields as $filter_key => $filter) {
            if(isset($filter['feature'])) {
                $filter['feature']['cat'] = $filter['feature']['cat'] ?? 'other';
                $features[$filter_key] = $filter;
            }
        }
        return $features;
    }


    /**
     * @param $meta_key
     * @return array|mixed
     */
    public function getFieldByMetaKey($meta_key, $return = "field") {

        foreach(self::getFields() as $key => $field) {
            $dbKey = $field['meta_query']['key'] ?? null;
            if($dbKey == $meta_key) {
                if($return == "field") return $field;
                else return $key;
            }
        }

        return null;
    }
}