<?php

/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 10/01/16
 * Time: 12:37
 */

namespace DXIM;


class VehicleFilter {

    protected static $instance = null;

    // aktuell gesendete formulardaten ($_POST)
    private $formData = array();

    //
    private $formDataDefaults = array();

    // alle Fahrzeuge der aktuellen Seite
    private $vehicles = null;

    // Anzahl gefundener gefilterter Fahrzeuge insgesamt
    private $count = 0;

    // Anzahl aller gefundenen Fahrzeuge (unabhängig der pagination)
    private $num_pages = null;

    // Anzahl Fahrzeuge pro Seite
    private $posts_per_page = null;

    // ID der Seite von der die aktuellen Formulardaten stammen
    private $current_page_id = 0;

    //
    private $default_orderby = null;

    //
    private $default_order = null;

    // verhindert doppelte ausgabe der form hidden durch mehrfaches positinieren des submit buttons
    private $form_hiddens_build = false;

    // verhindert aufbau order-select, wenn orderbar verwendet und umgekehrt
    private $orderby_build = false;
    private $order_build = false;
    private $orderbar_build = false;

    /**
     * mögliche Sortierung der Suchergebnisse nach:
     * @var array
     */
    private $orderFilter = array(
        'make' => array('label' => 'Hersteller', 'key' => 'make'),
        'date' => array('label' => 'Datum', 'key' => 'post_modified', 'ignore_in_filter_table' => true, 'hide_public' => true),
        'price' => array('label' => 'Preis', 'key' => 'consumer_price', ),
        'rate' => array('label' => 'Rate', 'key' => ['financing_monthly_installment', 'leasing_gross_rate'], ),
        'mileage' => array('label' => 'Kilometer', 'key' => 'specific_mileage'),
        'first_registration' => array('label' => 'Erstzulassung', 'key' => 'specific_first_registration'),
        'seller_inventory_key' => array('label' => 'interne Nummer', 'key' => 'seller_inventory_key', 'hide_public' => true),
        'rand' => array('label' => 'Zufall', 'key' => 'RAND()', 'ignore_in_filter_table' => true, 'hide_public' => true),
    );

    /**
     * liefert alle Filter, die im Filterform verwendet werden können
     * (z.B. price_min, price_max, aber nicht price)
     */
    public function getFilterFormFilters() {
        $filters = [];
        foreach (VehicleFields::getFields() as $key => $tmp) {
            if (!isset($tmp['filterform']) || $tmp['filterform'] !== false) $filters[$key] = $tmp;
        }
        return $filters;
    }

    /**
     * @return mixed|void
     */
    public static function getFilters() {
        return VehicleFields::getFields();
    }


    /**
     * @return mixed|void
     */
    public static function getOrderFilter() {
        return apply_filters(DXIM_FILTER_FORM_ORDER_ARRAY, self::getInstance()->orderFilter);
    }

    public static function getDefaultOrderBy() {
        return self::getInstance()->default_orderby;
    }

    public static function getDefaultOrder() {
        return self::getInstance()->default_order;
    }

    /**
     * Fahrzeuge der aktuellen Seite
     * @return null
     */
    public static function getVehicles() {
        if(!is_array(self::getInstance()->vehicles)) self::getInstance()->setVehicles();
        return self::getInstance()->vehicles;
    }

    /**
     * Anzahl Fahrzeuge der aktuellen Query
     * @param $filterData
     * @return array
     */
    public static function getCount() {
        if(!is_array(self::getInstance()->vehicles)) self::getInstance()->setVehicles();
        return self::getInstance()->count;
    }

    /**
     * Anzahl Fahrzeuge der aktuellen Query
     * @param $filterData
     * @return array
     */
    public static function getPostsPerPage() {
        if(!is_array(self::getInstance()->vehicles)) self::getInstance()->setVehicles();
        return self::getInstance()->posts_per_page;
    }

    /**
     * Anzahl Fahrzeuge der aktuellen Query
     * ajax Funcktion daher die()
     */
    public function _getPreFilteredAmount() {
        print VehicleFilter::getCount();
        die();
    }

    /**
     * Anzahl Seiten der aktuellen Query
     * @return null
     */
    public static function getNumPages() {
        return self::getInstance()->num_pages;
    }

    /**
     * @return int
     */
    public function getCurrentPage() { return $this->current_page_id; }

    /**
     *
     * @return VehicleFilter
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     *
     */
    public function __construct() {

        add_action('wp', array(&$this, '_init'));
        add_action('admin_init', array(&$this, '_admin_init'));

        add_action('wp_ajax_getData', [&$this, '_getData']);
        add_action('wp_ajax_nopriv_getData', [&$this, '_getData']);

        add_action('wp_ajax_getPreFilteredAmount', [&$this, '_getPreFilteredAmount']);
        add_action('wp_ajax_nopriv_getPreFilteredAmount', [&$this, '_getPreFilteredAmount']);
    }


    /**
     *
     */
    public function _init() {
        global $post;
        $page_id = $post->ID ?? 0;

        $options = Admin::getInstance()->getOptions();
        $this->default_orderby = $options['archive_default_orderby'];
        $this->default_order = $options['archive_default_order'];

        // setze formular daten
        $formData = $_POST ?? [];
        if(empty($formData)) $formData = $_GET ?? [];
        $this->setFormData($page_id, $formData);

        self::getVehicles();
    }



    /**
     *
     */
    public function _admin_init() {

        $options = Admin::getInstance()->getOptions();
        $this->default_orderby = $options['archive_default_orderby'] ?? 'price';
        $this->default_order = $options['archive_default_order'] ?? 'asc';

        if (isset($_POST['init_ajax_data'])) {
            $this->_initVehiclesByAjaxFormData();
        }
    }

    /**
     *
     */
    private function setFormData($page_id, $formData) {
        global $wp_query;

        $this->formData = $formData;
        $this->current_page_id = $page_id;

        $options = Admin::getInstance()->getOptions();
        if (empty($this->formData['order'])) $this->formData['order'] = $options['archive_default_order'];
        if (empty($this->formData['orderby'])) $this->formData['orderby'] = $options['archive_default_orderby'];
        if (isset($this->formData['set_last_search'])) $this->_set_last_search($this->formData);

        $query_vars = [];
        // in $wp_query->query_vars kann KEY oder Label sein
        if (isset($wp_query->query_vars['make'])) {
            $make = Make::_getFromUrl($wp_query->query_vars['make']);
            $query_vars['make'] = !empty($make) ? $make : $wp_query->query_vars['make'];
        }
        if (isset($wp_query->query_vars['model'])) {
            $model = Model::_getFromUrl($wp_query->query_vars['model'], $query_vars['make']);
            $query_vars['model'] = !empty($model) ? $model : $wp_query->query_vars['model'];
        }

        // apply global filters to search form data
        $data = GlobalFilter::getInstance()->getGlobalFilters($page_id);

        // url vars hinzufügen
        if (!isset($data) || !is_array($data)) $data = [];
        $data = array_merge($query_vars, $data);

        // formdata hinzufügen
        $this->formData = array_merge($this->formData, $data);
        $this->formDataDefaults = $data;
    }


    /**
     * @param $params
     * @return void
     */
    public function _set_last_search($params = null, $force_url = null) {
        if(empty($params)) $params = $this->formData;
        unset($params['set_last_search']);

        if(!empty($params['save_user_layout'])) {
            Session::set('archive_layout',$params['save_user_layout']);
            unset($params['save_user_layout']);
        }


        $ref = $_SERVER['HTTP_REFERER'] ?? null;
        if($force_url) $ref = $force_url;
        if($ref == null) return;

        $qs = http_build_query($params);
        $info = parse_url($ref);
        $url = $info['scheme'] . '://' . $info['host'] . $info['path'];
        Session::set('last_vehicle_archive', $url . '?' . $qs);

    }

    /**
     * @return void
     */
    private function _initVehiclesByAjaxFormData() {

        // get form data froem request
        parse_str($_POST['filterFormData'], $post_data);

        $post_id = $post_data['post_id'] ?? 0;

        //
        $field = $_POST['field'] ?? null;
        if (!empty($field)) $post_data[$field] = '#';

        // set form data
        $this->setFormData($post_id, $post_data);
    }


    /**
     * liefert die Daten für die WPCSSelect Filter per ajax
     * @return void
     */
    public function _getData() {
        $load = $_POST['load'] ?? 'live';
        $filterKey = $_POST['field'] ?? null;

        if ($load == "custom") {
            $valueStr = $_POST['values'] ?? [];
            $data = $this->_getCustomData($valueStr, $filterKey);
        }
        elseif ($load == "db") $data = $this->_getDataBy('db');
        elseif ($load == "all") $data = $this->_getDataBy('all');
        else $data = $this->_getLiveData();

        // return data
        header('Content-Type: application/json');
        die(json_encode($data));
    }

    /**
     * liefert optional alle Werte der DB oder alle möglichen Werte
     * @param $typ
     * @return array|void
     */
    private function _getDataBy($typ) {
        $filterKey = $_POST['field'] ?? null;
        if ($filterKey == "") return;

        $filter = VehicleFields::getField($filterKey);
        $data = [];


        // get all values from DB
        if ($typ == "db") {
            $keys = Filter::getInstance()->getFilterValues($filterKey,[]);
            $values = $this->getFilterValuesByKeys($keys, $filterKey);
        }

        // get all possible values from callback class
        elseif ($typ == "all" && isset($filter['data_source'])) {
            $class = $filter['data_source']['callback'][0];
            $method = '_getAll';

            $depends = null;
            if (isset($filter['data_source']['depends'])) {
                parse_str($_POST['filterFormData'], $filterData);
                $depends = $filterData[$filter['data_source']['depends']] ?? $filter['data_source']['depends_default'];
            }

            $tmpData = call_user_func([$class, $method], $depends);
            $values = array_column($tmpData, 'label', 'key');
            if(empty($values)) return [];
        }

        // get all possible values from filter array
        elseif ($typ == "all" && isset($filter['fillable']['defaults'])) {
            $values = $filter['fillable']['defaults'];
        }

        // get all possible values from filter array
//        elseif ($typ == "all" && !empty($filter['value'])) {
//            $values = $filter['value'];
//        }

        // keine weitere optionen
        else $values = [];

        natsort($values);

        foreach ($values as $key => $label) {
            $data[] = ['id' => $key, 'text' => $label];
        }

        return $data;
    }

    /**
     * liefert nur die angegebenen Werte
     * @return array|void
     */
    private function _getCustomData($valueStr, $filterKey) {
        if ($filterKey == "") return;
        $values = explode(";", $valueStr);
        $dataTmp = $this->getFilterValuesByKeys($values, $filterKey);
        $data = [];
        foreach($dataTmp as $key => $label) {
            $data[] = ['id' => $key, 'text' => $label];
        }

        return $data;
    }

    /**
     * ermittelt zu einem angegebenen Array an Filter Keys die passenden Labels
     * @param $keys
     * @return array
     */
    public function getFilterValuesByKeys($keys, $filterKey, $depends_on_value = null) {
        $values = [];
        foreach ($keys as $key) {
            $label = $this->getReadableValue($filterKey, $key, $depends_on_value);
            if($label == "") $label = $key;
            $values[$key] = $label;
        }
        return $values;
    }


    /**
     * @param $filterKey
     * @param $value // kann ein KEY, Zahl oder sonstiges sein
     * @return void|null
     */
    public function getReadableValue($filterKey, $value, $depends_on_value = null) {
        $filter = VehicleFields::getField($filterKey) ?? [];
        if (empty($filter)) return null;

        // value ist bereits korrekter Wert da Ziffer
        if ($filter['type'] == "slider" || $filter['type'] == 'range') return $value;
        // Schlüssel => auflösen in callback function
        elseif(isset($filter['data_source']['callback'])) return call_user_func($filter['data_source']['callback'],$value, $depends_on_value);
        //Schlüssel => auflösen in default array deklaration
        elseif (isset($filter['fillable']['defaults'][$value])) return $filter['fillable']['defaults'][$value];
        // feature
        elseif (isset($filter['feature'])) return $filter['label'];
        // Schlüssel => $filter[value] kann gefüllt sein über einen jook, wird eigentlich nicht mehr verwendet
        elseif (isset($filter['value'][$value])) return $filter['value'][$value];
        // return default value
        else return $value;
    }



    /**
     * @param $formData
     * @param $trenner
     * @return mixed
     */
    public function formdata_to_filter_array($formData, $trenner = ",") {

        $filters = VehicleFields::getFields();
        $newFormData = [];

        foreach ($formData as $key => $value) {
            if(!isset($filters[$key])) {
                continue;
            }

            if(empty($value) || $value == "#") {
                continue;
            }

            if(!is_array($value)) {
                $tmp = explode($trenner, $value);
                if (count($tmp) > 1) $value = $tmp;
            }

            $newFormData[$key] = $value;
        }

        return apply_filters(DXIM_FILTER_VEHICLE_QUERY_FILTERS, $newFormData);
    }


    /**
     * @return array
     */
    public function _getLiveData() {
        $field = $_POST['field'];
        $formData = $this->formData;
        #wpcs_p($formData);

        // get all matching vehicles
        $values = Filter::getInstance()->getFilterValues($field, $formData);

        /**
         * Prüfen ob für das aktuelle Feld ein anderes Feld gewählt sein muss
         * z.b. Model, Category
         */
        $filter = VehicleFields::getFields();
        $data_source = $filter[$field]['data_source'] ?? [];
        $depends_on_key = $data_source['depends'] ?? null;
        $depends_on_value = null;
        if(!empty($depends_on_key)) {
            $depends_on_value = $formData[$depends_on_key] ?? $data_source['depends_default'] ?? '';
            if ($depends_on_key != "" && ($depends_on_value == '#' || empty($depends_on_value))) {
                $data = Select2Data::transformLiveData([], ['id' => '#', 'text' => 'Bitte ' . $filter[$depends_on_key]['label'] . ' wählen']);
                return $data;
            }
        }

        // get Filter Values by keys
        $data = $this->getFilterValuesByKeys($values, $field, $depends_on_value);

        // sort array
        $data = apply_filters(DXIM_FILTER_FORM_SELECT_DATA_BEFORE_SORT, $data, $field);
        natsort($data);
        $data = apply_filters(DXIM_FILTER_FORM_SELECT_DATA, $data, $field);

        $return = [];
        foreach($data as $key => $value) {
            $return[] = ['id' => $key, 'text' => $value];
        }

        return $return;
    }
    #*/



    /**
     * @return array
     */
    public function getFormData() {
        return $this->formData;
    }


    /**
     * function to get all vehicles unfiltered but ordered from DB
     * @return array
     */
    public function queryVehicles($args = [], $init = true) {
        $args = !empty($args) ? $args : $this->formData;
        $query = Filter::getInstance()->queryVehicles($args);

        // WP_Post Object soll mit Custom Fields versehen werden
        if($init) {
            $vehicles = [];
            foreach ($query['vehicles'] as $vehicle) {
                $vehicle = VehicleData::initDataObject($vehicle);
                $vehicles[] = $vehicle;
            }
            $query['vehicles'] = $vehicles;
        }

        return $query;
    }


    /**
     * @param $vehicles
     */
    public function setVehicles() {
        $query = $this->queryVehicles();

        $this->vehicles = $query['vehicles'];
        $this->count = $query['count'];
        $this->num_pages = $query['num_pages'];
        $this->posts_per_page = $query['posts_per_page'];
    }


    /**
     * Achtung! Diese Funktion ruft alle Fahrzeuge ab => schlecht
     * besser umschreiben um einen reinen COUNT() zu machen
     * @ToDo wo wird diese überhaupt verwendet?
     * @param $filterData
     * @return array
     */
    public function count($filterData = []) {
        $filterData['posts_per_page'] = -1;
        $result = $this->queryVehicles($filterData, false);
        return $result['count'];
    }



    /**
     * @return null|string
     */
    public function createOrder($args) {
        if ($this->orderbar_build) return null;

        $label = isset($args['label']) ? $args['label'] : null;
        $order = $this->formData['order'] ?? $this->default_order;

        $html = '<div class="dxim_filter order">';
        if ($label != null) $html .= '<label for="order">' . $label . '</label>';

        $handle = $order == "asc" ? "aufsteigend" : "absteigend";
        $html .= '<div class="wpcsselect dxim_order_select" name="order">';
        $html .= '<div class="wpcs_select_handle">' . $handle . '</div>';
        $html .= '<div class="wpcs_select_loading"></div>';
        $html .= '<div class="wpcs_select_arrow"></div>';
        $html .= '<input type="hidden" value="'.$order.'" class="wpcs_select_value" />';
        $html .= '<div class="data">';
        foreach (array('asc' => __('aufsteigend'), 'desc' => 'absteigend') as $key => $label) {
            $html .= '<div data-value="' . $key . '">' . $label . '</div>';
        }
        $html .= '</div>';
        $html .= '</div>';

        $html .= '</div>';

        $this->order_build = true;
        return $html;
    }

    /**
     * @return string
     */
    public function createOrderBy($args) {
        if ($this->orderbar_build) return null;

        $label = isset($args['label']) ? $args['label'] : null;
        $html = null;
        $filters = self::getOrderFilter();
        #wpcs_p($filters);
        $orderby = $this->formData['orderby'] ?? $this->default_orderby;

        $html .= '<div class="dxim_filter orderby">';
            if ($label != null) $html .= '<label for="orderby">' . $label . '</label>';
            $handle = $filters[$orderby]['label'];
            $html .= '<div class="wpcsselect dxim_orderby_select" name="orderby">';
                $html .= '<div class="wpcs_select_handle">' . $handle . '</div>';
                $html .= '<div class="wpcs_select_loading"></div>';
                $html .= '<div class="wpcs_select_arrow"></div>';
                $html .= '<input type="hidden" value="'.$orderby.'" class="wpcs_select_value" />';
                $html .= '<div class="data">';
                foreach ($filters as $key => $tmp) {
                    if(isset($tmp['hide_public'])) continue;
                    $html .= '<div data-value="' . $key . '">' . $tmp['label'] . '</div>';
                }
                $html .= '</div>';
            $html .= '</div>';
        $html .= '</div>';

        $this->orderby_build = true;

        return $html;
    }


    /**
     * @param $filterKey
     * @return null|string
     */
    public function createFilter($args) {
        $filterKey = $args['key'];
        $filter = VehicleFields::getField($filterKey);
        $filter['type'] = $filter['type'] ?? "text";
        $current_value = $this->formData[$filterKey] ?? null;

        $options = Admin::getInstance()->getOptions();
        $html = null;

        /**
         * checkbox for features
         */
        if ($filter['type'] == 'checkbox') {
            $checked = isset($this->formData[$filterKey]) ? "checked" : null;
            $html .= '<input type="checkbox" class="wpcs_checkbox" name="' . $filterKey . '" value="1" ' . $checked . ' />';
        }

        /**
         * multiple checkbox
         * zeigt alle Werte anstelle Select
         */
        elseif ($filter['type'] == 'multiple_checkbox') {
            $selectedValues = $this->formData[$filterKey] ?? [];
            $f = new FilterElementMultipleCheckbox($filterKey);
            $html .= $f->draw($selectedValues );
        }

        /**
         * radio / einfachauswahl anstelle select
         * @ToDo Styling implementieren + js submit funktion onlick
         */
        elseif ($filter['type'] == 'radio') {
            $selectedValues = $this->formData[$filterKey] ?? [];
            $f = new FilterElementRadio($filterKey);
            $html .= $f->draw($selectedValues );
        }

        /**
         * SLIDER
         */
        elseif ($filter['type'] == 'slider') {
            $f = new FilterElementSlider($filterKey);
            $html .= $f->draw($current_value);
        }

        /**
         * RANGE SLIDER
         */
        elseif ($filter['type'] == 'range') {
            $f = new FilterElementRange($filterKey);
            $filter_value_1 = $this->formData[$filterKey.'_min'] ?? 0;
            $filter_value_2 = $this->formData[$filterKey.'_max'] ?? 0;
            $html .= $f->draw($filter_value_1, $filter_value_2);
        }

        /**
         * select
         */
        elseif ($filter['type'] == 'select') {
            $f = new FilterElementSelect($filterKey);
            $filterData = $this->formdata_to_filter_array($this->formData);
            $current_value = $filterData[$filterKey] ?? null;
            $html .= $f->draw($current_value);
        }

        /**
         * color
         */
        elseif ($filter['type'] == "color") {

            $value = $this->formData[$filterKey] ?? null;
            $c = new FilterElementColor();
            $html .= $c->draw($filterKey, $value);
        }

        /**
         *
         */
        elseif ($filter['type'] == "custom") {
            $value = $this->formData[$filterKey] ?? null;
            $html = call_user_func($filter['filter_conf']['callback'], $filterKey, $value);
        }

        /**
         * input text
         */
        elseif ($filter['type'] == 'text') {
            $value = $this->formData[$filterKey] ?? null;
            $placeholder = ($options['ff_labels'] == "inside") ? $filter['label'] : null;
            $html .= '<input type="text" value="' . $value . '" placeholder="' . $placeholder . '" name="' . $filterKey . '" data-field="' . $filterKey . '" />';
        }

        return $html;
    }

    public function _get_nothing_selected_label($label, $key) {
        $options = Admin::getInstance()->getOptions();

        if ($options['ff_labels'] == 'outside') $text = $options['ff_nothing_selected_text'];
        else $text = $label;

        return apply_filters(DXIM_FILTER_WPCSSELECT_NOTHING_SELECTED_TEXT, $text, $key);
    }


    /**
     * @param $formElementsHtml
     * @return null|string
     */
    public function createFilterForm($formElementsHtml, $args) {
        if (!is_array($args)) $args = [];
        $html = "\n<!-- WPCS Filter Section -->";

        $html .= '<div class="dxim_filter_section">';
        $html .= $formElementsHtml;
        if (!isset($args['no_submit_button'])) {
            $html .= $this->drawSubmitButton($args);
        }

        $html .= '</div>';

        return $html;
    }

    /**
     * @param $args
     * @return string
     */
    public function drawSubmitButton($args = []) {
        $defaults = ['button_classes' => null, 'show_count' => 1, 'ajax' => 0];
        if (!is_array($args)) $args = [];
        $args = array_merge($defaults, $args);

        $html = "\n<!-- Filterform Submit + Hiddens -->";
        $buttonLabel = isset($args['button_label']) ? $args['button_label'] : WPCSString::getString('labels_filter_form_button');
        if ($args['show_count']) $buttonLabel .= ' <span class="amount_wrapper">(' . $this->drawSearchCount() . ')</span>';

        $redirect_url = (isset($args['redirect'])) ? $args['redirect'] : null;
        $html .= "\n" . HTML::drawButton(
            ['url' => '#', 'label' => $buttonLabel, 'classes' => 'dxim_filter_form_submit ' . $args['button_classes']],
            ['data-redirect' => $redirect_url]
        );

        return $html;
    }

    /**
     * @param $args
     * @return string
     */
    public function drawFormHiddens() {

        if (!$this->form_hiddens_build && !wp_doing_ajax()) {

            $wpcsp = $this->formData['wpcsp'] ?? 1;
            $order = $this->formData['order'] ?? self::getDefaultOrder();
            $orderby = $this->formData['orderby'] ?? self::getDefaultOrderBy();

            $html = '<div class="dxim_filter wpcs_hidden">';
            $html .= "\n" . '<input type="hidden" value="' . $this->getCurrentPage() . '" name="post_id" />';
            $html .= "\n" . '<input type="hidden" value="1" name="set_last_search" />';
            $html .= "\n" . '<input type="hidden" value="' . $wpcsp . '" id="wpcs_hidden_wpcsp" name="wpcsp" />';

            // orderby & order hiddens => werden durch klick auf orderbar gesetzt
            $html .= "\n" . '<input type="hidden" value="' .$orderby. '" name="orderby" id="dxim_orderby_hidden" />';
            $html .= "\n" . '<input type="hidden" value="' . $order. '" name="order" id="dxim_order_hidden" />';
            $html .= "</div>";

            $this->form_hiddens_build = true;

            return $html;
        }
    }

    /**
     * @return string
     */
    public function drawSearchCount() {
        return '<span class="wpcs_search_count" data-value="'.$this->count.'">' . number_format($this->count,0,",",".") . '</span>';
    }

    /**
     * @return false|string
     */
    public function drawActiveFilters() {
        $html = null;

        $html .= '<div class="wpcs_filterform_active_filters">';
        foreach ($this->formData as $key => $value) {
            $f = VehicleFields::getField($key);
            if (empty($f) || $this->formData[$key] == "#" || $this->formData[$key] == "") continue;
            $html .= '<div class="active_filter" data-key="' . $key . '" data-type="' . $f['type'] . '">';
            $html .= '<i class="dashicons dashicons-no-alt reset"></i>';
            $html .= '<span class="label">' . $f['label'] . '</span>';
            $html .= '</div>';
        }
        $html .= '</div>';
        return $html;
    }

    /**
     * @param $filter
     * @return string
     */
    public function createFilterFormElement($args) {
        $defaults = ['load' => 'live', 'values' => null];
        $args = array_merge($defaults, $args);
        $key = $args['key'] = strtolower($args['key']);
        $options = Admin::getInstance()->getOptions();

        // prüfen ob filter existiert
        $filter = VehicleFields::getField($key);
        if(empty($filter)) return "Filter " . $key . " existiert nicht";
        $filter['type'] = $filter['type'] ?? "text";

        // prüfen ob filter custom values hat
        if(isset($filter['value']) && !empty($filter['value']) && is_array($filter['value']) && !isset($args['values'])) {
            $args['load'] = "custom";
            $args['values'] = implode(";",array_keys($filter['value']));
        }

        $html = '<div class="dxim_filter label_'.$options['ff_labels'].'" data-filter-name="' . $key . '" data-filter-type="' . $filter['type'] . '" data-load="' . $args['load'] . '" data-values="' . $args['values'] . '">';

            // draw filter
            if(!Filter::getInstance()->isFilterable($key) && is_user_logged_in()) {
                #wpcs_p($filter);

                $html .= '<span class="wpcs_error">Bitte Filter registrieren</span>';
                Filter::getInstance()->registerMissingFilter($key);
            }

            // draw label
            $label = '<label for="' . $key . '">' . apply_filters(DXIM_FILTER_FILTERFORM_LABEL, $filter['label'], $key) . '</label>';

            // draw element
            $field = $this->createFilter($args);

            // draw
            if ($filter['type'] == "checkbox" && isset($filter['feature'])) $html .= $field . ' ' . $label;
            elseif ($options['ff_labels'] == 'outside') $html .= $label . " " . $field;
            else $html .= $field;

        $html .= '</div>';

        // werden einmalig aufgebaut
        $html .= $this->drawFormHiddens();

        return $html;
    }


    /**
     * @return null|string
     */
    public function drawOrderBar($args = []) {
        if ($this->order_build || $this->orderby_build) return null;

        // horizontales oder vertikales Layout
        $layout = isset($args['layout']) ? $args['layout'] : "horizontal";

        $order = $this->formData['order'] ?? self::getDefaultOrder();
        $orderby = $this->formData['orderby'] ?? self::getDefaultOrderBy();

        $html = "\n<!-- WPCS Orderbar -->";
        $html .= '<div class="dxim_ff_order_bar ' . $layout . '" data-order="' . $order. '">';

        foreach (self::getOrderFilter() as $key => $tmp) {
            $arrow = null;
            if (isset($tmp['hide_public'])) continue;

            $is_current = ($orderby == $key) ? 'current' : null;
            $html .= '<a href="#" data-orderby="' . $key . '" class="dxim_set_order ' . $key . ' ' . $is_current . '">' . $arrow . '' . $tmp['label'] . '</a>';
        }

        $html .= '</div>';

        $this->orderbar_build = true;

        if (!wp_doing_ajax()) $html .= $this->drawFormHiddens();

        return $html;
    }

    /**
     * @return null|string
     */
    public function drawFilterForm($args = []) {
        $html = null;
        $options = Admin::getInstance()->getOptions();

        // wenn Fields per Attribut übergeben wurden, werden diese verwendet,
        // alternativ werden die fields aus den Plugin - Einstellungen verwendet
        $args = $this->_parse_filter_form_args($args);

        $fields = !empty($args['columns']) ? $args['columns'] : $options['vehicle_filter'];
        $countColumns = count($fields);
        if ($countColumns > 4) $countColumns = 4;

        if($options['ff_method'] != "reload") $args['no_submit_button'] = 1;

        $form = '<div class="wpcs_filter_form_grid wpcs_columns_' . $countColumns . ' dxim_clearfix">';
        for ($a = 1; $a <= $countColumns; $a++) {
            $form .= '<div class="wpcs_filter_form_column wpcs_col_' . $a . '">';
            foreach ($fields[$a] as $filterKey) {

                // sortierung
                if ($filterKey == 'sort') {
                    $form .= dxim_createOrderby(['label' => 'Sortierung']);
                    $form .= dxim_createOrder(['label' => 'Reihenfolge']);
                }

                // submit button
                elseif ($filterKey == "submit") {
                    $form .= '<div class="dxim_filter">';
                    $form .= '<label for="form_submit" style="display:block;">&nbsp;</label>';
                    $form .= $this->drawSubmitButton($args);
                    $form .= '</div>';
                    $args['no_submit_button'] = 1;
                }

                // filter element
                else $form .= dxim_createFilterFormElement(['key' => $filterKey]);
            }
            $form .= '</div>';
        }
        $form .= '</div>'; // grid

        // add filters
        $form = apply_filters(DXIM_FILTER_FILTER_FORM, $form);

        // html
        $html .= dxim_createFilterForm($form, $args);

        return $html;
    }

    /**
     * @param $args
     *
     * @return array|void
     */
    private function _parse_filter_form_args($args) {
        #if(empty($args)) return;
        $options = Admin::getInstance()->getOptions();
        if (!isset($options['ff_labels'])) $options['ff_labels'] = 'outside';
        if (!is_array($args)) $args = [];
        $defaults = ['labels' => $options['ff_labels']];
        $args = array_merge($defaults, $args);

        $parsed = [];
        $c = 1;
        foreach ($args as $key => $value) {
            if (substr($key, 0, 6) == 'column') {
                $tmp = explode(";", $value);
                foreach ($tmp as $key => $value) {
                    if (empty($value)) unset($tmp[$key]);
                }
                $parsed['columns'][$c] = $tmp;
                $c++;
            } else {
                $parsed[$key] = $value;
            }
        }
        return $parsed;
    }
}
