<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 09:47
 */

namespace DXIM;


class VehicleHooks {


	protected $filter_base = 'DXIM_FILTER_VEHICLE_FIELD_';

	protected static $instance = null;

	/**
	 *
	 * @return VehicleData
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 * Vehicle constructor.
	 */
	public function __construct() {

		/**
		 * register all filter hooks
		 */
		add_filter($this->filter_base.'mileage',[&$this,'_filter_mileage'],10,3);
		add_filter($this->filter_base.'general_inspection',[&$this,'_filter_general_inspection'],10,3);
        add_filter($this->filter_base.'delivery_date',[&$this,'_filter_delivery_date'],10,3);
        add_filter($this->filter_base.'delivery_period',[&$this,'_filter_delivery_period'],10,3);
		add_filter($this->filter_base.'power',[&$this,'_filter_power'],10,3);
        add_filter($this->filter_base.'battery_capacity',[&$this,'_filter_battery_capacity'],10,3);
		add_filter($this->filter_base.'first_registration',[&$this,'_filter_first_registration'],10,3);
		add_filter($this->filter_base.'cubic_capacity',[&$this,'_filter_cubic_capacity'],10,3);
//		add_filter($this->filter_base.'account',[&$this,'_filter_account'],10,3);
        add_filter($this->filter_base.'manufacturer_color',[&$this,'_manufacturer_color'],10,3);

		add_filter($this->filter_base.'dimension_length',[&$this,'_filter_dimension'],10,3);
		add_filter($this->filter_base.'dimension_width',[&$this,'_filter_dimension'],10,3);
		add_filter($this->filter_base.'dimension_height',[&$this,'_filter_dimension'],10,3);
		add_filter($this->filter_base.'loading_space_height',[&$this,'_filter_dimension'],10,3);
		add_filter($this->filter_base.'loading_space_length',[&$this,'_filter_dimension'],10,3);
		add_filter($this->filter_base.'loading_space_width',[&$this,'_filter_dimension'],10,3);

		add_filter($this->filter_base.'licensed_weight',[&$this,'_filter_licensed_weight'],10,3);
		add_filter($this->filter_base.'load_capacity',[&$this,'_filter_licensed_weight'],10,3);

		add_filter(DXIM_FILTER_SPECIFICS_LABEL,[&$this,'_filter_specifics_label'],10,2);

	}

    /**
     * @param $label
     * @param $data
     * @return mixed
     */
	public function _filter_specifics_label($label, $data) {
	    if($data['key'] == "power") $label = 'Leistung';
	    return $label;
    }

	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return string
	 */
	public static function _filter_mileage($value, $key, $vehicle) {
        $value = floatval($value);
		return number_format($value,0,",",".").' km';
	}

    /**
     * @param $value
     * @param $key
     * @param $vehicle
     * @return string
     */
    public static function _manufacturer_color($value, $key, $vehicle) {
        return ucwords(strtolower($value));
    }

	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return false|null|string
	 */
	public static function _filter_general_inspection($value, $key, $vehicle) {
		if(empty($value)) return null;
		return date('m.Y',strtotime($value));
	}

    /**
     * @param $value
     * @param $key
     * @param $vehicle
     * @return false|string|null
     */
    public static function _filter_delivery_date($value, $key, $vehicle) {
        if(empty($value)) return null;
        $time = strtotime(substr($value,0,10));
        return ($time < time()) ? "sofort" : date('d.m.Y',$time);
    }

    /**
     * @param $value
     * @param $key
     * @param $vehicle
     * @return false|string|null
     */
    public static function _filter_delivery_period($value, $key, $vehicle) {
        if(empty($value)) return null;
        if($value == "7") $label = "1 Woche";
        elseif($value == "14") $label = "2 Wochen";
        elseif ($value == "42") $label = "6 Wochen";
        elseif ($value == "60") $label = "2 Monate";
        elseif ($value == "90") $label = "3 Monate";
        elseif ($value == "150") $label = "5 Monate";
        elseif ($value == "180") $label = "6 Monate";
        elseif ($value == "270") $label = "9 Monate";
        elseif ($value == "360") $label = "1 Jahr";
        else $label = $value." Tage";
        return $label." ab Bestellung";
    }

	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return string
	 */
	public static function _filter_power($value, $key, $vehicle) {
		return $value.' KW / '.round($value*1.35962).' PS';
	}

    /**
     * @param $value
     * @param $key
     * @param $vehicle
     * @return string
     */
    public static function _filter_battery_capacity($value, $key, $vehicle) {
        return $value." kWh";
    }

	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return false|string
	 */
	public static function _filter_first_registration($value, $key, $vehicle) {
        if(empty($value)) return null;
		$value = strtotime($value);
		if($value > time()) return "Neu";
		return date('m/Y',$value);
	}

	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return string
	 */
	public static function _filter_cubic_capacity($value, $key, $vehicle) {
		return $value.' ccm';
	}



	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return mixed
	 */
//	public static function _filter_account($value, $key, $vehicle) {
//		$cf = $vehicle->dxim_meta;
//		$dealer = Admin::getInstance()->getOptions('dealer');
//		return $dealer[$cf['ad_account'][0]]['company-name'];
//	}

	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return mixed
	 */
	public static function _filter_dimension($value, $key, $vehicle) {
		return number_format($value,0,",",".").' mm';
	}

	/**
	 * @param $value
	 * @param $key
	 * @param $vehicle
	 *
	 * @return mixed
	 */
	public static function _filter_licensed_weight($value, $key, $vehicle) {
		return number_format($value,0,",",".").' kg';
	}





}