<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 12/09/16
 * Time: 12:11
 */

namespace DXIM;


use DXIM\theme\LocationData;
use DXIM\theme\LocationTemplate;

class VehicleHtml {

	protected $vehicle = [];

    /**
     * -----------------------------------------------------------------------------------------------------------------
     * VEHICLE SINGLE
     */

    /**
     * @param $cf
     * @return null
     */
    public static function drawVehicleTitle(\WP_Post $vehicle, $tags = ['h1','h2']) {
	    $title = VehicleData::getVehicleTitle($vehicle);
	    $titleHtml = null;
	    $html = null;

        $html = '<div class="dxim_vehicle_title">';

	        $titleHtml .= '<'.$tags[0].'><span class="make">'.$title['make'].'</span> <span class="model">'.$title['model'].'</span></'.$tags[0].'>';
	        $titleHtml .= '<'.$tags[1].' class="model_description">'.$title['model_description_clean'].'</'.$tags[1].'>';

	        $html .= apply_filters(DXIM_FILTER_VEHICLE_TITLE_OUTPUT,$titleHtml,$vehicle);
        $html .= '</div>';

	    return apply_filters(DXIM_FILTER_VEHICLE_TITLE,$html,$title,$vehicle);
    }


    /**
     * @param $cf
     * @return null|string
     */
    public static function drawVehicleButtons(\WP_Post $vehicle,$args = []) {
        $options = Admin::getInstance()->getOptions();
        $html = null;
        $buttonHtml = null;

        $html .= '<div class="dxim_vehicle_buttons">';

            $buttonHtml .= self::drawBackToArchiveButton($args);
            $buttonHtml .= self::drawVehiclePrintButton($vehicle,$args);
            if(!isset($options['compare_disabled'])) {
                $buttonHtml .= self::drawVehicleCompareButton($vehicle,$args);
                #$buttonHtml .= self::drawVehicleComparePageButton($args);
            }

	        $html .= apply_filters(DXIM_FILTER_VEHICLE_BUTTONS_OUTPUT,$buttonHtml,$vehicle);

        $html .= '</div>';

        return apply_filters(DXIM_FILTER_VEHICLE_BUTTONS,$html,$vehicle);
    }

    /**
     *
     */
    public static function drawCompareButton($args) {
        $html = self::drawVehicleComparePageButton($args);
        return self::drawButton(['type'=>'compare_page'],$html);
    }


    /**
     * @param array $args
     * @return string|null
     */
    public static function drawVehicleParkButton($vehicle, $args  = []) {

        $class = Park::getInstance()->is_in_list($vehicle->ID) ? 'wpcs_is_parked' : null;

        $label = (isset($args['label'])) ? $args['label'] : WPCSString::getString('labels_park_vehicle_button');

        $button_classes = isset($args['button_classes']) ? $args['button_classes'] : null;
        return Html::drawButton(
            ['url'=>'#', 'classes'=>'wpcs_park_toggle_btn '.$class.' '.$button_classes, 'label' => '<span class="wpcs_icon is_parked dashicons dashicons-yes"></span> '.$label,'ajax' => 1],
            ['data-vehicle-id' => $vehicle->ID]
        );

    }

    /**
     * @param $args
     * @param $button_html
     * @return string
     */
    public static function drawButton($args, $button_html) {
        $type = $args['type'];
        $html = null;

        $html .= '<div class="dxim_vehicle_button '.$type.'">';
            $html .= $button_html;
        $html .= '</div>';

        return $html;

    }

	/**
	 * @param \WP_Post $vehicle
	 * @param array $args
	 *
	 * @return mixed|null|void
	 */
	public static function drawVehicleButton(\WP_Post $vehicle,$args = []) {
		$type = $args['type'];
		$options = Admin::getInstance()->getOptions();
		$default_options = [
		    'compare_disabled' => null,
        ];
		$options = array_merge($default_options, $options);

		$html = null;
		$buttonHtml = null;

		$html .= '<div class="dxim_vehicle_button '.$type.'">';

            if($type == 'print') $buttonHtml .= self::drawVehiclePrintButton($vehicle,$args);
            elseif($type == 'back') $buttonHtml .= self::drawBackToArchiveButton($args);
            elseif($type == 'compare' && !$options['compare_disabled']) $buttonHtml .= self::drawVehicleCompareButton($vehicle,$args);
            elseif($type == 'compare_page' && !$options['compare_disabled']) $buttonHtml .= self::drawVehicleComparePageButton($args);
            elseif($type == 'custom') $buttonHtml .= Html::drawButton(['url' => $args['url'], 'label' => $args['label'], 'classes' => $args['button_classes']]);
            elseif($type == 'park' && !isset($options['park_disabled'])) $buttonHtml .= self::drawVehicleParkButton($vehicle, $args);

            $html .= apply_filters(DXIM_FILTER_VEHICLE_BUTTON_OUTPUT,$buttonHtml,$vehicle,$type);

		$html .= '</div>';

		return apply_filters(DXIM_FILTER_VEHICLE_BUTTON,$html,$vehicle);
	}

    /**
     * @param \WP_Post $vehicle
     * @return string
     */
    public static function drawVehiclePrintButton(\WP_Post $vehicle,$args = []) {
    	if(!isset($args['button_classes'])) $args['button_classes'] = null;

    	$label = (isset($args['label'])) ? $args['label'] : WPCSString::getString('labels_single_vehicle_print');

        $url = get_permalink($vehicle->ID);
        if(substr($url,-1) != "/" ) $url .= '/';

        return Html::drawButton([
            'url' => $url.'print/',
            'label' => $label,
            'classes' => 'button_print '.$args['button_classes']
        ], [
            'target' => '_blank'
        ]);
    }

	/**
	 * @param \WP_Post $vehicle
	 * @return string
	 */
	public static function drawBackToArchiveButton($args) {
		if(!isset($args['button_classes'])) $args['button_classes'] = null;

		$label = (isset($args['label'])) ? $args['label'] : WPCSString::getString('labels_single_back_to_archive');

		$url = Html::getBackButtonUrl();

		if(empty($url)) $url = get_permalink(Conf::getInstance()->getVehicleArchivePage());

		return Html::drawButton([
			'url' => $url,
			'label' => $label,
			'classes' => $args['button_classes']
		]);
	}

    /**
     * @param \WP_Post $vehicle
     * @param string $label
     * @return string
     */
    public static function drawVehicleSingleButton(\WP_Post $vehicle, $args) {
        $url = get_permalink($vehicle->ID);
        $button_classes = isset($args['button_classes']) ? $args['button_classes'] : null;
        return Html::drawButton(['label'=>$args['label'], 'url'=>$url, 'classes'=>'wpcs_vehicle_single_btn '.$button_classes]);
    }

    /**
     * @param \WP_Post $vehicle
     * @return string
     */
    public static function drawVehicleCompareButton(\WP_Post $vehicle,$args = []) {
        $options = \DXIM\Admin::getInstance()->getOptions();
        $default_options = ['compare_disabled' => null, 'compare_page' => null];
        $options = array_merge($default_options, $options);

        $class = Compare::getInstance()->is_in_list($vehicle->ID) ? 'wpcs_in_comparison' : null;
	    $label = (isset($args['label'])) ? $args['label'] : WPCSString::getString('labels_compare_vehicle_button');
	    $button_classes = $args['button_classes'] ?? null;

        $html = '<div class="dxim_button_group">';

            $html .= Html::drawButton(
                ['url'=>'#', 'classes'=>'wpcs_compare_toggle_btn '.$class.' '.$button_classes, 'label' => '<span class="wpcs_icon in_comparison dashicons dashicons-yes"></span> '.$label,'ajax' => 1],
                ['data-vehicle-id' => $vehicle->ID, 'style' => 'margin-right:0;']
            );

            if(!$options['compare_disabled'] && intval($options['compare_page']) > 0 && !isset($args['hide_count'])) {
                $amount = count(Compare::getInstance()->getVehiclesToCompare());
                $label = '<span class="wpcs_count_vehicles_compared_wrapper">(<span class="wpcs_count_vehicles_compared">'.$amount.'</span>)</span>';
                $html .= Html::drawButton(
                    ['url'=>get_permalink($options['compare_page']), 'classes'=>'wpcs_compare_page_btn ', 'label' => $label],
                    ['title' => "Zum Fahrzeugvergleich"]
                );
            }

        $html .= '</div>';

        return $html;
    }

    /**
     * @param \WP_Post $vehicle
     * @return string
     */
    public static function drawVehicleComparePageButton($args  = []) {
        $options = \DXIM\Admin::getInstance()->getOptions();
        $default_options = [
            'compare_disabled' => null, 'compare_page' => null,
        ];
        $options = array_merge($default_options, $options);



        if(!isset($args['button_classes'])) $args['button_classes'] = null;

	    $label = (isset($args['label'])) ? $args['label'] : WPCSString::getString('labels_compare_page_button');
        if(!isset($args['hide_count'])) {
            $amount = count(Compare::getInstance()->getVehiclesToCompare());
            $label .= ' <span class="wpcs_count_vehicles_compared_wrapper">(<span class="wpcs_count_vehicles_compared">'.$amount.'</span>)</span>';
        }

        if(!$options['compare_disabled'] && intval($options['compare_page']) > 0) {
            return Html::drawButton(
                ['url'=>get_permalink($options['compare_page']), 'classes'=>'wpcs_compare_page_btn '.$args['button_classes'], 'label' => $label]
            );
        }
        return null;
    }



    /**
     * @param $cf
     * @return null|string
     */
    public static function drawVehicleContact($vehicle) {
        $html = null;
        $contactHtml = null;

        $html .= '<div class="dxim_vehicle_contact">';

            $location = VehicleData::getVehicleContact($vehicle);
            $contactHtml .= LocationTemplate::getInstance()->draw_location_address($location);

            // Öffnungszeiten
            if(WPCS_PRO) $contactHtml .= LocationTemplate::getInstance()->draw_location_opening_hours($location, ['link_days' => 1]);

            // External Info
            if(isset($contact['external'])) {
                $contactHtml .= '<p class="vehicle_is_external">'.apply_filters(DXIM_FILTER_VEHICLE_LOCATION_EXTERNAL_MSG,'Das Fahrzeug befindet sich im Außenlager.').'</p>';
            }

	        $html .= apply_filters(DXIM_FILTER_VEHICLE_CONTACT_OUTPUT,$contactHtml,$vehicle);

        $html .= '</div>';

	    return apply_filters(DXIM_FILTER_VEHICLE_CONTACT,$html,$vehicle);
    }

    /**
     * @param $vehicle
     * @return string|null
     */
    public static function drawVehicleHighlights($vehicle) {
        $html = null;
        $highlightsHtml = null;
        $highlights = VehicleData::getVehicleHighlights($vehicle);
        if(empty($highlights)) return null;

        $html .= '<div class="wpcs_vehicle_highlights">';

            foreach($highlights as $highlight) {
                $highlightsHtml .= '<div class="wpcs_highlight">'.$highlight.'</div>';
            }
            $html .= apply_filters(DXIM_FILTER_VEHICLE_HIGHLIGHTS_OUTPUT,$highlightsHtml,$highlights,$vehicle);

        $html .= '</div>';

        return apply_filters(DXIM_FILTER_VEHICLE_HIGHLIGHTS,$html,$highlights,$vehicle);
    }


    /**
     * @param $cf
     * @return null|string
     */
    public static function drawVehicleForm($vehicle) {
        if(empty($vehicle)) return null;
    	$cf = $vehicle->dxim_meta;
        #$key = VehicleData::getVehicleKey($vehicle);
        $html = null;
        #$data = ['ad_key' => $key];
        $html .= '<div class="dxim_vehicle_form">';
		    $html .= apply_filters(DXIM_FILTER_VEHICLE_FORM_OUTPUT,Form::getInstance()->contactForm($vehicle),$vehicle);
        $html .= '</div>';

	    return apply_filters(DXIM_FILTER_VEHICLE_FORM,$html,$cf);
    }

    /**
     * @param $cf
     * @return string
     */
    public static function drawVehicleSpecifics($specifics) {
        $html = null;
        $specificsHtml = null;

        $html = '<div class="dxim_vehicle_specifics">';

            // Fehler
            if(is_wp_error($specifics)) {
                $html .= '<h3>'.$specifics->get_error_message().'</h3>';
            }

            // Ausgabe
            else {
                $html .= '<div class="dxim_vehicle_specifics_list dxim_vehicle_specifics_list_single">';
                    foreach($specifics as $key => $array) {
                        if(empty($array['display_value'])) continue;
                        $tmp = Html::vehicle_fact($array);
                        if(!empty($tmp)) {
                            $specificsHtml .= '<div class="field dxim_clearfix">';
                            $specificsHtml .= $tmp;
                            $specificsHtml .= '</div>';
                        }

                    }
                    $html .= apply_filters(DXIM_FILTER_VEHICLE_SPECIFICS_OUTPUT,$specificsHtml,$specifics);
                $html .= '</div>';
            }

        $html .= '</div>';

        return apply_filters(DXIM_FILTER_VEHICLE_SPECIFICS,$html,$specifics);

    }

    /**
     * @param $cf
     * @return string
     */
    public static function drawVehicleFeatures($features, $args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['columns' => 4];
        $args = array_merge($defaults,$args);

        $layout = Admin::getInstance()->getOptions('single_vehicle_features_layout') ?? null;
        if($layout == "grouped") {
            $html = self::drawVehicleFeaturesGrouped($features,$args);
        }
        else {
            $html = self::drawVehicleFeaturesColumns($features,$args);
        }

        return apply_filters(DXIM_FILTER_VEHICLE_FEATURES,$html,$features);
    }

    /**
     * @param $features
     * @param $args
     * @return string|null
     */
    public static function drawVehicleFeaturesGrouped($features, $args) {
        $categories = Features::$featureCategories;
        $html = null;
        $columns = intval($args['columns']) > 4 ? 4 : $args['columns'];
        $group = $args['group'] ?? null;

        foreach($categories as $cat_key => $cat_name) {
            if(!empty($group) && $cat_key !== $group) continue;
            if(!isset($features[$cat_key])) continue;

            $html .= '<h3>'.$cat_name.'</h3>';
            $html .= '<ul class="dxim_feature_group columns-'.$columns.' '.$cat_key.'">';

            uasort($features[$cat_key], function($a, $b) {
                if($a[0] == $b[0]) return 0;
                return $a[0] > $b[0] ? 1 : -1;
            });

            foreach($features[$cat_key] as $feature_key => $feature_values) {
                $html .= '<li class="dxim_vehicle_feature '.$feature_key.' '.$cat_key.'">';
                $html .= '<span class="wpcs_icon dashicons dashicons-yes"></span> ';
                if(count($feature_values) > 1) {
                    $html .= implode(", ",$feature_values);
                }
                else {
                    $html .= $feature_values[0];
                }
                $html .= '</li>';
            }
            $html .= '</ul>';
        }
        return $html;
    }

    /**
     * @param $features
     * @param $args
     * @return string
     */
    public static function drawVehicleFeaturesColumns($features, $args = []) {
        uasort($features, function($a, $b) {
            if($a[0] == $b[0]) return 0;
            return $a[0] > $b[0] ? 1 : -1;
        });
    	$columns = intval($args['columns'] > 4) ? 4 : $args['columns'];
		$countFeatures = count($features);
		$featuresPerColumn = ceil($countFeatures / $columns);
		$featureKeys = array_keys($features);

		$html = null;
		$featuresHtml = null;

        $html = '<div class="dxim_vehicle_features wpcs_grid wpcs_grid_'.$columns.'">';
	        $c = 0;
		    for($a = 1; $a <= $columns; $a++) {
		    	$class = null;
		    	$featuresHtml .= '<div>';

				    $featuresHtml .= '<ul class="dxim_vehicle_feature_list">';
			        for($aa = 1; $aa <= $featuresPerColumn; $aa++, $c++) {

			            if(!isset($featureKeys[$c]) || !isset($features[$featureKeys[$c]])) continue;
			            $featuresHtml .= self::_render_single_feature($features[$featureKeys[$c]]);
				    }
				    $featuresHtml .= '</ul>';

		    	$featuresHtml .= '</div>';
		    }

		    $html .= apply_filters(DXIM_FILTER_VEHICLE_FEATURES_OUTPUT,$featuresHtml,$features);

        $html .= '</div>';

        return $html;
    }

	/**
	 * @param $feature
	 *
	 * @return string|null
	 */
    public static function _render_single_feature($feature) {
    	$html = null;
	    if(is_array($feature)) {
		    $html .= '<li><span class="wpcs_icon dashicons dashicons-yes"></span> ';
		    $z = 0;
		    foreach($feature as $tmp) {
			    if($z != 0) $html .= ', ';
			    $html .= $tmp;
			    $z++;
		    }
		    $html .= '</li>';
	    }
	    else $html .= '<li><span class="wpcs_icon dashicons dashicons-yes"></span> '.$feature.'</li>';

	    return $html;
    }

    /**
     * @param \WP_Post $vehicle
     */
    public static function drawVehicleFinancing(\WP_Post $vehicle) {
        $financing = VehicleData::getVehicleFinancing($vehicle);
        #wpcs_p($financing);
        if(empty($financing['monthly_installment'])) return null;

        $html = '<a name="financing"></a>';
        #$html .= wpcs_p($financing, true);
        $html .= '<div class="wpcs_vehicle_financing_wrapper financing">';

            $html .= '<div class="wpcs_vehicle_financing_values">';
                if(!empty($financing['monthly_installment'])) $html .= '<div class="item monthly_installment"><span class="value ">'.$financing['monthly_installment'].',- &euro;</span><span class="label">'.__('mtl. Rate','wpcs').'</span></div>';
                if(!empty($financing['payback_period'])) $html .= '<div class="item payback_period"><span class="value">'.$financing['payback_period'].' Monate</span><span class="label">'.__('Laufzeit','wpcs').'</span></div>';
                if(!empty($financing['first_installment'])) $html .= '<div class="item first_installment"><span class="value">'.number_format($financing['first_installment'],0,",",".").',- &euro;</span><span class="label">'.__('Anzahlung','wpcs').'</span></div>';
                if(!empty($financing['gross_loan_amount'])) $html .= '<div class="item gross_loan_amount"><span class="value">'.number_format($financing['gross_loan_amount'],0,",",".").',- &euro;</span><span class="label">'.__('Darlehensgesamtbetrag','wpcs').'</span></div>';
                if(!empty($financing['net_loan_amount'])) $html .= '<div class="item net_loan_amount"><span class="value">'.number_format($financing['net_loan_amount'],0,",",".").',- &euro;</span><span class="label">'.__('Nettodarlehensbetrag','wpcs').'</span></div>';
                if(!empty($financing['final_installment'])) $html .= '<div class="item final_installment"><span class="value">'.number_format($financing['final_installment'],0,",",".").',- &euro;</span><span class="label">'.__('Schlussrate','wpcs').'</span></div>';
            $html .= '</div>';

            $html .= '<dl class="wpcs_vehicle_financing_legal">';
                if(!empty($financing['closing_costs'])) $html .= '<dt>Bearbeitungsgebühr</dt><dd>'.$financing['closing_costs'].',-</dd>';
                if(!empty($financing['nominal_interest_rate'])) $html .= '<dt>Effektiver Jahreszins</dt><dd>'.$financing['nominal_interest_rate'].' %</dd>';
                if(!empty($financing['annual_percentage_rate'])) $html .= '<dt>Sollzinssatz p.A.</dt><dd>'.$financing['annual_percentage_rate'].' %</dd>';
                if(!empty($financing['bank'])) $html .= '<dt>Bank</dt><dd>'.$financing['bank'].'</dd>';
            $html .= '</dl>';

        $html .= '</div>';

        return $html;

    }

    /**
     * @param \WP_Post $vehicle
     */
    public static function drawVehicleLeasing(\WP_Post $vehicle) {
        $leasing = VehicleData::getVehicleLeasing($vehicle);
        if(empty($leasing['gross_rate'])) return null;

        $html = '<a name="leasing"></a>';
        $html .= '<div class="wpcs_vehicle_financing_wrapper leasing">';

            $html .= '<div class="wpcs_vehicle_financing_values">';
                if(!empty($leasing['gross_rate'])) $html .= '<div class="item gross_rate"><span class="value ">'.$leasing['gross_rate'].',- &euro;</span><span class="label">'.__('mtl. Rate brutto','wpcs').'</span></div>';
                if(!empty($leasing['term_of_contract'])) $html .= '<div class="item term_of_contract"><span class="value">'.$leasing['term_of_contract'].' Monate</span><span class="label">'.__('Laufzeit','wpcs').'</span></div>';
                if(!empty($leasing['down_payment'])) $html .= '<div class="item down_payment"><span class="value">'.number_format($leasing['down_payment'],0,",",".").',- &euro;</span><span class="label">'.__('Anzahlung','wpcs').'</span></div>';
                if(!empty($leasing['total_amount'])) $html .= '<div class="item total_amount"><span class="value">'.number_format($leasing['total_amount'],0,",",".").',- &euro;</span><span class="label">'.__('Gesamtbetrag','wpcs').'</span></div>';
                if(!empty($leasing['net_loan_amount'])) $html .= '<div class="item net_loan_amount"><span class="value">'.number_format($leasing['net_loan_amount'],0,",",".").',- &euro;</span><span class="label">'.__('Nettokreditbetrag','wpcs').'</span></div>';
                if(!empty($leasing['final_payment'])) $html .= '<div class="item final_payment"><span class="value">'.number_format($leasing['final_payment'],0,",", ".").',- &euro;</span><span class="label">'.__('Schlussrate','wpcs').'</span></div>';
                if(!empty($leasing['mileage'])) $html .= '<div class="item mileage"><span class="value">'.number_format($leasing['mileage'],0,",", ".").' km</span><span class="label">'.__('Laufleistung','wpcs').'</span></div>';
                #$html .= '<div class="item no_value"></div>';
                #$html .= '<div class="item no_value"></div>';
            $html .= '</div>';

            $html .= '<dl class="wpcs_vehicle_financing_legal">';
                if(!empty($leasing['low_mileage_compensation'])) $html .= '<dt>Erstattung Minderkilometer</dt><dd>'.number_format($leasing['low_mileage_compensation'],2,",",".").' &euro;</dd>';
                if(!empty($leasing['extra_mileage_costs'])) $html .= '<dt>Kosten Mehrkilometer</dt><dd>'.number_format($leasing['extra_mileage_costs'],2,",",".").' &euro;</dd>';
                if(!empty($leasing['annual_percentage_rate'])) $html .= '<dt>Effektiver Jahreszins</dt><dd>'.str_replace(".", ",", $leasing['annual_percentage_rate']).' %</dd>';
                if(!empty($leasing['nominal_interest_rate'])) $html .= '<dt>Sollzinssatz p.A.</dt><dd>'.str_replace(".", ",", $leasing['nominal_interest_rate']).' %</dd>';
                if(!empty($leasing['lender'])) $html .= '<dt>Bank</dt><dd>'.$leasing['lender'].'</dd>';
            $html .= '</dl>';

        $html .= '</div>';

        return $html;

    }


    /**
     * @return mixed|void
     */
    public static function drawVehiclePrice(\WP_Post $vehicle) {

        $html = '<div class="price_wrapper dxim_vehicle_price">';
            $html .= apply_filters(DXIM_FILTER_VEHICLE_PRICE_OUTPUT,self::getVehiclePrice($vehicle),$vehicle);
        $html .= '</div>';

        return apply_filters(DXIM_FILTER_VEHICLE_PRICE,$html,$vehicle);

    }


	/**
	 * @param \WP_Post $vehicle
	 *
	 * @return string
	 */
    public static function getVehiclePrice(\WP_Post $vehicle) {
	    $values = VehicleData::getVehiclePrice($vehicle);
        if(empty($values)) return null;

        $html = null;

	    // Preis aus Anfrage
	    if($values['price_type'] == 'ON_REQUEST') {
		    $html = '<span class="price_on_request price_big">Preis auf Anfrage</span>';
	    }

	    // keine MwSt.
	    elseif($values['vatable'] == 'false') {
		    $html  = '<span class="price_no_vat price_big">'.$values['display_price_brutto'].'</span> ';
	    }

	    // inkl. MwSt.
	    else {

		    // netto
		    if($values['price_display'] == 'netto') {
			    $html = '<span class="price_netto price_big">'.$values['display_price_netto'].'</span>';
            }

		    // brutto
		    elseif($values['price_display'] == "brutto") {
			    $html = '<span class="price_brutto price_big">'.$values['display_price_brutto'].'</span>';
                if($values['vatable']) $html .= '<span class="vat">'.$values['vat_msg'].'</span>';

            }

            // beide preise anzeigen (macht nur Sinn wenn Preis brutto)
		    elseif($values['price_display'] == "both") {
                $html = '<span class="price_big price_brutto">'.$values['display_price_brutto'].'</span>';
                if($values['vatable']) {
                    $html .= '<span class="price_netto price_small">'.$values['display_price_netto'].'<span class="netto_label"> (Netto)</span></span>';
                    $html .= '<span class="vat">'.$values['vat_msg'].'</span>';

                }
            }


	    }

	    // Finanzierung Rate
        if(!empty($values['financing']['monthly_installment']) && in_array($values['rate_display'], ['financing', 'both'])) $html .= '<div class="financing"><a class="monthly_installment" href="'.get_permalink($vehicle->ID).'#financing">Finanzierung ab '.$values['financing']['monthly_installment'].' &euro; mtl.</a></div>';

        // Leasing Rate
        if(!empty($values['leasing']['gross_rate']) && in_array($values['rate_display'], ['leasing', 'both'])) $html .= '<div class="leasing"><a class="gross_rate" href="'.get_permalink($vehicle->ID).'#leasing">Leasing ab '.$values['leasing']['gross_rate'].' &euro; mtl.</a></div>';

	    return $html;
    }

    /**
     * @param \WP_Post $vehicle
     * @param array $price_values
     * @return string
     */
    public static function drawVehiclePriceRating(\WP_Post $vehicle, $price_values = []) {

        if(empty($price_values)) $price_values = VehicleData::getVehiclePrice($vehicle);
        if(empty($price_values['price_rating']) || $price_values['price_rating'] == "NO_RATING") return null;

        $labels = VehicleFields::getFields()['price_rating']['fillable']['defaults'];

        $html = '<div class="price_rating '.strtolower($price_values['price_rating']).'">';
            $html .= '<div class="one dot"></div>';
            $html .= '<div class="two dot"></div>';
            $html .= '<div class="three dot"></div>';
            $html .= '<div class="four dot"></div>';
            $html .= '<div class="five dot"></div>';
            $html .= '<div class="label">'.$labels[$price_values['price_rating']].'</div>';
        $html .= '</div>';

        return $html;
    }


    /**
     * @param $post
     * @return null|string
     */
    public static function drawVehiclePostContent($post) {
    	$content = VehicleData::getVehiclePostContent($post);

        if(empty($content)) return null;
        $html = '<div class="dxim_vehicle_post_content">';
		    $html .= apply_filters(DXIM_FILTER_VEHICLE_POST_CONTENT_OUTPUT,$content,$post);
        $html .= '</div>';

        return apply_filters(DXIM_FILTER_VEHICLE_POST_CONTENT,$html,$post);
    }


    /**
     * @param $vehicle
     * @param array $args
     * @return mixed|string|void
     */
    public static function drawVehicleEmission($vehicle) {
        $html = null;
        $emission = VehicleData::getVehicleEmission($vehicle);

        $html .= '<div class="vehicle_single_emission_data wpcs_element_spacer wpcs_grid wpcs_grid_2">';

            // WLTP
            $html .= self::drawVehicleEmissionData($vehicle, $emission);

            // EEC Image
            $html .= self::drawVehicleEecImage($vehicle);

        $html .= '</div>';

        return apply_filters(DXIM_FILTER_VEHICLE_EMISSION,$html,$emission, $vehicle);
    }


    /**
     * @param $vehicle
     * @return string
     */
    public static function drawVehicleEmissionData(&$vehicle, $emission = []) {
        if(empty($emission)) $emission = VehicleData::getVehicleEmission($vehicle);
        $html = null;

        $html .= '<div class="wpcs_emission_data">';

            foreach(['required', 'recommended'] as $key) {
                $emission[$key] = $emission[$key] ?? [];
                foreach($emission[$key] as $field) {
                    $value = $field['value'];
                    if($value == "") continue;
                    else $value .= " ".$field['unit'];

                    $html .= '<div class="wpcs_emission_field '.$field['filterkey'].'">';
                        $html .= '<span class="label">'.$field['label'].': </span>';
                        $html .= '<span class="value">'.$value.'</span>';
                    $html .= '</div>'."\n";
                }
                $html .= '<br />';
            }

        $html .= '</div>';
        return apply_filters(DXIM_FILTER_VEHICLE_EMISSION_DATA_HTML, $html, $emission, $vehicle);
    }



    /**
     * @param $vehicle
     * @param $args
     * @return string|null
     */
    public static function drawVehicleEecImage($vehicle) {
        $fuel = $vehicle->dxim_meta['specific_fuel'][0] ?? "";
        $labels = [0 => 'A', 1 => 'B', 2 => 'C', 3 => 'D', 4 => 'E', 5 => 'F', 6 => 'G'];
        $show_labels = false;

        // Hybrid
        if(strpos($fuel,"HYBRID") !== false) {
            $show_labels = true;
            $fields = [
                ['value' => $vehicle->dxim_meta['specific_wltp_co2_class_weighted_combined'][0] ?? "", 'label' => 'gewichtet kombiniert'],
                ['value' => $vehicle->dxim_meta['specific_wltp_co2_class_low_battery'][0] ?? "", 'label' => 'bei entladener Batterie']
            ];
        }
        // alle anderen
        else $fields = [['value' => $vehicle->dxim_meta['specific_wltp_co2_class'][0] ?? "", 'label' => null]];

        $html = '<div class="wpcs_co2_class_label wpcs_grid wpcs_grid_'.(count($fields)+1).'">';

        if($show_labels) {
            $html .= '<div class="cell empty"></div>';
            foreach($fields as $field) {
                $html .= '<div class="cell field_label">'.$field['label'].'</div>';
            }
        }

        $has_value = false;

        foreach($labels as $label) {

            $html .= '<div class="cell">';
                $html .= '<div class="eec_label label_'.$label.'">'.$label.'</div>';
            $html .= '</div>';

            foreach($fields as $field) {
                $html .= '<div class="cell label">';
                    if($field['value'] == $label) {
                        $html .= '<div class="value">'.$label.'</div>';
                        $has_value = true;
                    }
                $html .= '</div>';
            }
        }

        $html .= '</div>';

        if(!$has_value) return null;

        return $html;
    }


    /**
     * @param $vehicle
     * @return null
     */
    public static function drawVehicleImage($vehicle, $args = []) {
		$defaults = ['nr' => 1, 'link' => 1];
		$args = array_merge($defaults, $args);
		#wpcS_p($args);

        $options = Admin::getInstance()->getOptions();
        $image_type = $args['image_type'] ?? $options['single_vehicle_image_type'] ?? 'default';
        $images = VehicleData::getVehicleImages($vehicle, ['image_type' => $image_type]);

        $html = null;
	    $index = $args['nr'] - 1;
		$src = $images['big'][$index] ?? "";
		$src_dir = str_replace(WPCS_UPLOADS_URL, WPCS_UPLOADS_DIR,$src);
		if(!is_file($src_dir)) return null;

        $html .= '<div id="wpcs_vehicle_image_wrapper">';
			if($args['link']) $html .= '<a href="'.$src.'">';
                $html .= '<img src="'.$src.'" alt="Fahrzeugabbildung '.$vehicle->post_title.'" />';
		    if($args['link']) $html .= '</a>';
        $html .= '</div>';

        return $html;
    }

    /**
     * @param $vehicle
     * @return string
     */
    public static function drawVehicleVideo($vehicle, $atts = []) {
        $video_url = dxim_get_vehicle_cf_value('specific_video_url',$vehicle);
        if(empty($video_url)) return null;

        $yt = isset($atts['yt']) ? $atts['yt'] : null;
        $yt_tmp = explode(";",$yt);
        $yt_defaults = ['rel' => 0];

        foreach($yt_tmp as $prop) {
            $prop_tmp = explode(":",$prop);
            if(empty($prop_tmp[0])) continue;
            $yt_defaults[$prop_tmp[0]] = $prop_tmp[1];
        }

        $yt_props = null;
        foreach($yt_defaults as $key => $value) {
            $yt_props .= ($yt_props == null) ? '?' : '&';
            $yt_props .= $key.'='.$value;
        }

        $html = null;
        $ratio = isset($atts['ratio']) ? $atts['ratio'] : "16:9";
        $ratioValues = explode(":",$ratio);
        $padding = ($ratioValues['1'] / $ratioValues[0]) * 100;

        $url = parse_url($video_url);
        if(isset($url['query'])) $key = str_replace("v=",null,$url['query']);
        else $key = str_replace("/v/",null,$url['path']);

        #$key = "9mnRfVSTvik";
        $embed_url = "https://www.youtube.com/embed/".$key.$yt_props;

        $html .= '<div id="wpcs_single_video" style="padding-bottom:'.$padding.'%;">';
            $html .= '<iframe src="'.$embed_url.'"  frameborder="0" allowfullscreen="allowfullscreen"></iframe>';
        $html .= '</div>';

        return $html;
    }

    /**
     * @param $cf
     * @return mixed|void
     */
    public static function drawVehicleImages($vehicle, $args = []) {

        $options = Admin::getInstance()->getOptions();

        $html = null;

        // images
        $image_conf = ['thumbnail' => 'big', 'image' => 'big', 'lightbox' => 'big'];
        $image_type = $args['image_type'] ?? $options['single_vehicle_image_type'] ?? 'default';
        $images = VehicleData::getVehicleImages($vehicle, ['image_type' => $image_type]);
        $image_count = count($images['big']);

        // settings
        $lightbox = ($options['single_vehicle_lightbox'] == 'on') ? true : false;
        $lightbox_class = $lightbox ? 'wpcs_lightbox_enabled' : null;
        $align = $args['align'] ?? $options['single_vehicle_image_alignment'];
        $per_slide = $args['per_slide'] ?? $options['single_vehicle_image_per_slide'] ?? 3;
        $arrowWrapper = $args['arrows'] ?? $options['single_vehicle_image_arrows'];

        if(empty($align)) $align = "horizontal";
        if(empty($per_slide)) $per_slide = 4;
        if(empty($arrowWrapper)) $arrowWrapper = 'wrapper';

        if($image_count <= 1) {
            $prevArrow = $nextArrow = null;
        }
        else {
            $prevArrow = Html::drawButton(
                ['classes' => 'dxim_control_prev_btn dxim_button','label' => '<span class="wpcs_icon dashicons dashicons-arrow-left-alt2"></span>','url' => '#'],
                ['id' => 'vehicle_single_images_prev_btn']
            );
            $nextArrow = Html::drawButton(
                ['classes' => 'dxim_control_next_btn dxim_button','label' => '<span class="wpcs_icon dashicons dashicons-arrow-right-alt2"></span>','url' => '#'],
                ['id' => 'vehicle_single_images_next_btn']
            );
        }

        $html .= '<div id="wpcs_vehicle_images_wrapper" class="'.$align.' '.$lightbox_class.' arrows_'.$arrowWrapper.'" data-align="'.$align.'" data-per-slide="'.$per_slide.'">';

                /**
                 * Single Image
                 */
                $html .= '<div id="wpcs_vehicle_single_image_wrapper">';

                    $html .= '<img src="'.$images[$image_conf['image']][0].'" id="wpcs_vehicle_single_image" />';

                    // lightbox link
                    if($lightbox) {
                        $html .= '<a href="'.$images[$image_conf['image']][0].'" class="lightbox_opener">';
                        $html .= '<span class="wpcs_icon open dashicons dashicons-search"></span>';
                        $html .= '</a>';
                    }

                    if($arrowWrapper == "image") {
                        $html .= $prevArrow.$nextArrow;
                    }

                $html .= '</div>';

                /**
                 * Thumbnails
                 */
                if($image_count > 1) {
                    $html .= '<div id="wpcs_thumbnails_wrapper">';

                    if($arrowWrapper == "thumbnails") $html .= $prevArrow;

                    $html .= '<div id="vehicle_single_thumbnails" class="thumbnail">';

                    $sources = $images[$image_conf['thumbnail']];
                    $sources_big = $images[$image_conf['lightbox']];
                    for($a = 0; $a < count($sources); $a++) {
                        $html .= '<div class="dxim_image_thumbnail">';
                        $html .= '<a href="#" class="dxim_image_thumbnail_link" data-src="'.$sources_big[$a].'">';
                        $html .= '<img src="'.$sources[$a].'" class="thumbnail" alt="Fahrzeugabbildung '.$vehicle->post_title.'" />';
                        $html .= '</a>';
                        $html .= '</div>';
                    }
                    $html .= '</div>'; // thumbnails



                    $script = '<script>';
                    $script .= 'var wpcs_lightbox_sources = '.json_encode($sources_big).';';
                    $script .= '</script>';
                    $html .= $script;

                    if($arrowWrapper == "thumbnails") $html .= $nextArrow;

                    $html .= '</div>'; // thumbnails
                }

                /**
                 * Nav Buttons
                 */
                if($arrowWrapper == "wrapper") $html .= '<div id="vehicle_single_nav_buttons">'.$prevArrow.$nextArrow.'</div>';

        $html .= '</div>'; // wrapper

        return apply_filters(DXIM_FILTER_VEHICLE_IMAGES,$html,$images);
    }


    /**
     * -----------------------------------------------------------------------------------------------------------------
     * VEHICLE ARCHIVE
     */

    public static function drawVehicleArchiveLayoutSelector() {
        $options = Admin::getInstance()->getOptions();
        $user_layout = Session::val('archive_layout');
        $options_layout = $options['archive_vehicle_layout'];
        $layout = $user_layout ?? $options_layout;

        $html = '<div id="wpcs_archive_layout_selector" class="dxim_filter">';

            // List
            $active = $layout == "rows" ? "active" : "";
            $html .= '<div class="button list_button '.$active.'" data-type="rows">';
                $html .= '<div class="list">';
                    $html .= '<div class="line"></div>';
                    $html .= '<div class="line"></div>';
                    $html .= '<div class="line"></div>';
                $html .= '</div>';
            $html .= '</div>';

            // grid
            $active = $layout == "grid" ? "active" : "";
            $html .= '<div class="button grid_button '.$active.'" data-type="grid">';
                $html .= '<div class="grid">';
                for($a = 1; $a <= 9; $a++) {
                    $html .= '<div class="line"></div>';
                }
                $html .= '</div>';
            $html .= '</div>';

            $html .= '<input type="hidden" name="save_user_layout" value="" />';
        $html .= '</div>';

        return $html;
    }


    /**
     * @param \WP_Post $vehicle
     * @return mixed|void
     */
    public static function drawVehicleArchiveImage(\WP_Post $vehicle, $args) {
        $options = Admin::getInstance()->getOptions();
        $image_type = $args['image_type'] ?? $options['archive_image_type'] ?? 'default';
        $images = VehicleData::getVehicleImages($vehicle, ['image_type' => $image_type]);

        $ratio = (!empty($options['general_vehicle_image_ratio'])) ? $options['general_vehicle_image_ratio'] : '4_3';
        $html = '<div class="dxim_grid_image ratio_'.$ratio.'">';
            $html .= '<a href="'.get_permalink($vehicle->ID).'">';
            $html .= '<img src="'.$images['large'][0].'" alt="'.$vehicle->post_title.'" />';
            $html .= '</a>';
        $html .= '</div>';

        return apply_filters(DXIM_FILTER_ARCHIVE_IMAGES,$html,$images,$vehicle);
    }

    /**
     * @param \WP_Post $vehicle
     * @return mixed|void
     */
    public static function drawVehicleArchiveTitle(\WP_Post $vehicle) {
        $title = VehicleData::getVehicleTitle($vehicle);
        $titleHtml = null;

        $html = '<div class="dxim_archive_title">';

            $html .= '<a href="'.get_permalink($vehicle->ID).'">';

                $titleHtml .= '<h2><span class="make">'.$title['make'].'</span> <span class="model">'.$title['model'].'</span></h2>';
                $titleHtml .= '<h3><span class="model_description">'.$title['model_description_clean'].'</span></h3>';

	            $html .= apply_filters(DXIM_FILTER_ARCHIVE_TITLE_OUTPUT,$titleHtml,$vehicle);

            $html .= '</a>';

        $html .= '</div>';

        return apply_filters(DXIM_FILTER_ARCHIVE_TITLE,$html,$vehicle);
    }

    /**
     * @param $vehicle
     * @return mixed|void
     */
    public static function drawVehicleArchiveSpecifics(\WP_Post $vehicle) {
        $fields = VehicleData::getVehicleFields($vehicle);
        $specificsHtml = null;

	    $specifics = '<div class="dxim_archive_specifics">';

	        if(is_wp_error($fields)) {
	            $specifics .= '<h3>'.$fields->get_error_message().'</h3>';
	        }

	        else {
	            $specifics .= '<div class="dxim_vehicle_specifics_list dxim_vehicle_specifics_list_archive dxim_clearfix">';

                    foreach($fields as $key => $array) {
                        $tmp = Html::vehicle_fact($array);
                        if(!empty($tmp)) {
                            $specificsHtml .= '<div class="field dxim_clearfix">';
                            $specificsHtml .= $tmp;
                            $specificsHtml .= '</div>';
                        }
                    }

                    $specifics .= apply_filters(DXIM_FILTER_ARCHIVE_SPECIFICS_OUTPUT,$specificsHtml,$vehicle);

	            $specifics .= '</div>';
	        }

	    $specifics .= '</div>';

        return apply_filters(DXIM_FILTER_ARCHIVE_SPECIFICS,$specifics,$fields);
    }

    /**
     * @param \WP_Post $vehicle
     */
    public static function drawVehicleArchiveEmission(\WP_Post $vehicle, $args = []) {
        if(is_array($args) && in_array("full",$args)) return self::drawVehicleEmissionData($vehicle);
        $emission = VehicleData::getVehicleEmission($vehicle);
        $emissionHtml = null;

        if(empty($emission['required']) || !is_array($emission['required'])) return null;

        $z = 0;
        foreach($emission['required'] as $field) {
            $value = $field['value'];
            if($value == "") continue;
            else $value .= " ".$field['unit'];

            if($z != 0) $emissionHtml .= "; ";
            $emissionHtml .= '<span class="wpcs_emission_field '.$field['filterkey'].'">';
                $emissionHtml .= '<span class="label">'.$field['label'].': </span>';
                $emissionHtml .= '<span class="value">'.$value.'</span>';
            $emissionHtml .= '</span>';

            $z++;
        }

        if($emissionHtml == "") return null;
        $html = '<div class="dxim_vehicle_emission vehicle-emission">';
            $html .= apply_filters(DXIM_FILTER_ARCHIVE_EMISSION_OUTPUT,$emissionHtml,$vehicle);
        $html .= '</div>';

        return apply_filters(DXIM_FILTER_ARCHIVE_EMISSION,$html,$emission, $vehicle);
    }


    /**
     * @param \WP_Post $vehicle
     * @return mixed|void
     */
    public static function drawVehicleArchiveButtons(\WP_Post $vehicle) {
        $options = Admin::getInstance()->getOptions();
        $buttonsHtml = null;

        $buttons = '<div class="button_wrapper">';

            // anzeigen
            $buttonsHtml .= self::drawVehicleSingleButton($vehicle,['label'=>WPCSString::getString('labels_archive_vehicle_button')]);

            // vergleichen
            if(!isset($options['compare_disabled'])) $buttonsHtml .= self::drawVehicleCompareButton($vehicle);

            // parken
            if(!isset($options['park_disabled'])) $buttonsHtml .= self::drawVehicleParkButton($vehicle);

	        $buttons .= apply_filters(DXIM_FILTER_ARCHIVE_BUTTONS_OUTPUT,$buttonsHtml,$vehicle);

        $buttons .= '</div>';

        return apply_filters(DXIM_FILTER_ARCHIVE_BUTTONS,$buttons,$vehicle);
    }

    /**
     * @param \WP_Post $vehicle
     * @return string
     */
    public static function drawVehicleArchivePrice(\WP_Post $vehicle) {
	    return apply_filters(DXIM_FILTER_ARCHIVE_PRICE,self::drawVehiclePrice($vehicle),$vehicle);
    }

    /**
     * @return string
     */
    public static function drawVehicleCount($args = []) {
        if(!is_array($args)) $args = VehicleFilter::getInstance()->getFormData();
        $count = \DXIM\VehicleFilter::getInstance()->count($args);
        return '<span class="vehicle_count">'.$count.'</span>';
    }



	/**
	 * Darstellung der übergebenen Fahrzeuge als Carousel
	 * @param $vehicles
	 * @return null|string
	 */
    public static function _draw_vehicles_as_carousel($vehicles, $all_args = []) {

        $slides = [];
        foreach($vehicles as $vehicle) {
            VehicleData::getInstance()->setVehicle($vehicle);
            $template_html = CustomTemplate::getInstance()->get_template_content($all_args['template_file']);

            $images = VehicleData::getVehicleImages($vehicle);
            $url = get_permalink($vehicle->ID);

            $slideData = [];
            $slideData['title'] = $vehicle->post_title;
            $slideData['images'] = $images;
            $slideData['url'] = $url;

            $item = Template::drawVehicleOnArchive($vehicle, $template_html);

            // register Filter
            $item = apply_filters(DXIM_FILTER_SHORTCODE_CAROUSEL,$item,$slideData);

            $slides[] = $item;
        }

        return Html::slider($slides, $all_args['carousel_args']);
    }



}