<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 09:47
 */

namespace DXIM;

class VehicleSchema {

    private $vehicle = null;

    public function __construct($vehicle) {
        $this->vehicle = $vehicle;
    }


    public function create() {
        $vehicle = &$this->vehicle;
        $v = new Vehicle();


        $data = [
            '@context' => "https://schema.org",
            '@type' => "Car",
            'bodyType' => $v->getRawValue($vehicle, "category"),
            "name" => $vehicle->post_title,
            "vehicleIdentificationNumber" => $v->getValue($vehicle, "identification_number"),
            "url" => get_permalink($vehicle->ID),
            "image" => $this->get_images($vehicle),
            "itemCondition" => "https://schema.org/".$this->_get_condition($v->getRawValue($vehicle, "specific_condition")),
            "offers" => [
                "availability" => "https://schema.org/".$this->_get_availability($v->getRawValue($vehicle, 'specific_delivery_date')),
                "price" => $v->getValue($vehicle, 'price'),
                "priceCurrency" => "EUR"
            ],
            "brand" => [
                "name" => $v->getValue($vehicle, "make")
            ],
            "model" => $v->getValue($vehicle, "model"),
            "vehicleConfiguration" => $v->getValue($vehicle, "model_description"),
            "vehicleModelDate" => $this->_get_model_date($v->getRawValue($vehicle, "specific_construction_year"), $v->getRawValue($vehicle, "specific_first_registration")),
            "mileageFromOdometer" => [
                "@type" => "QuantitativeValue",
                "value" => $v->getRawValue($vehicle, "specific_mileage"),
                "unitCode" => "KMT"
            ],
            "vehicleEngine" => [
                "@type" => "EngineSpecification",
                "fuelType" => $v->getRawValue($vehicle, "specific_fuel"),
            ],
            "color" => $v->getValue($vehicle, "exterior_color"),
            "vehicleInteriorColor" => $v->getValue($vehicle, "interior_color"),
            "vehicleInteriorType" => $v->getValue($vehicle, "interior_type"),
            "vehicleTransmission" => $v->getValue($vehicle, "gearbox"),
            "numberOfDoors" => $this->_get_doors($v->getRawValue($vehicle, "specific_door_count")),
            "vehicleSeatingCapacity" => $v->getValue($vehicle, "num_seats"),

        ];

        return $data;
    }

    private function get_images(&$vehicle) {
        $images_tmp = VehicleData::getVehicleImages($vehicle);
        $images = [];
        foreach($images_tmp['large'] as $img) {
            $images[] = $img;
        }
        return $images;
    }

    /**
     * @param $condition
     * @return string
     */
    private function _get_condition($condition) {
        return $condition == "NEW" ? "NewCondition" : "UsedCondition";
    }

    private function _get_doors($rawValue) {
        if($rawValue == "TWO_OR_THREE") return 3;
        if($rawValue == "FOUR_OR_FIVE") return 5;
        if($rawValue == "SIX_OR_SEVEN") return 7;
    }

    private function _get_model_date($construction_year, $first_registration) {
        if(!empty($construction_year)) return $construction_year;
        if(empty($first_registration)) return null;
        return date("Y",strtotime($first_registration));
    }

    /**
     * @param $delivery_date
     * @return string|void
     */
    private function _get_availability($delivery_date) {
        if(empty($delivery_date)) return "InStock";
        $date = strtotime($delivery_date);
        if($date > time()) return "PreSale";
        return "InStock";
    }

}