<?php

namespace DXIM;

class VehicleSearchRequest {

    protected static $instance = null;

    private $error = [];

    private $show_form_position = null;
    private $show_form_pages = [];
    private $form_done_on_current_page = false;

    /**
     *
     * @return VehicleSearchRequest
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Vehicle constructor.
     */
    public function __construct() {
        $this->_register_post_type();

        add_action('wp_ajax_wpcs_search_request', array($this, '_ajax'));
        add_action('wp_ajax_nopriv_wpcs_search_request', array($this, '_ajax'));

        add_action(DXIM_CORE_AFTER_IMPORT, [&$this, '_notify']);

        add_action('wp', function () {
            #$this->_notify();
            $this->_handle_request();
        });

        /**
         * Angebote in Börse einblenden
         */
        $options = Admin::getInstance()->getOptions();
        $position = $options['vsr_form_position'] ?? 0;
        if($position > 0) {
            $this->show_form_position = $options['vsr_form_position'];
            $this->show_form_pages = explode(",",$options['vsr_form_pages']);
            add_filter(DXIM_FILTER_ARCHIVE_VEHICLE_ADD, [&$this,'_vehicle_rendered'],10,6);
        }
    }

    /**
     * @param $html
     * @param $vehicle
     * @return mixed|string
     */
    public function _vehicle_rendered($html, $vehicle, $vehicle_nr, $page_nr, $num_vehicles_per_page, $num_vehicles_total) {
        // ermittle, an welchen Positionen das Form dargestellt werden soll
        $my_positions = [];
        foreach($this->show_form_pages as $page_tmp) {
            $pos_tmp = (($page_tmp-1) * $num_vehicles_per_page) + $this->show_form_position;
            $my_positions[] = $pos_tmp;
        }

        // ermittle aktuelle Fahrzeug Nr
        $vehicles_done = (($page_nr-1) * $num_vehicles_per_page) + $vehicle_nr;
        // ermittle letztes Fahrzeug
        $last_vehicle = $vehicles_done == $num_vehicles_total;

        // Zeige Form immer wenn
        // 1) position zutrifft
        // 2) letztes Fahrzeug aber Form noch nicht ausgegeben wurde (pos ist größer als Anzahl Fahrzeuge pro Seite)
        if(in_array($vehicle_nr, $my_positions) || ($last_vehicle && !$this->form_done_on_current_page && in_array($page_nr, $this->show_form_pages))) {
            $html .= $this->_draw_save_request(['template' => 'form_vehicle_archive']);
            $this->form_done_on_current_page = true;
        }
        return $html;
    }

    /**
     * handler für URL Aufruf
     * @return void
     */
    private function _handle_request() {

        // Umleitung E-Mail link zu Liste
        // passiert dynamisch damit Autohaus URL der Liste jederzeit ändern kann
        if (isset($_GET['wpcs_vsr_redirect'])) {
            parse_str($_SERVER['QUERY_STRING'], $args);
            $t = $args['wpcs_vsr_redirect'];
            unset($args['wpcs_vsr_redirect']);

            $qs2 = http_build_query($args);
            $qs = '?t=' . $t;
            if (!empty($qs2)) $qs .= '&' . $qs2;

            $page_id = Admin::getInstance()->getOptions('vsr_page');
            wp_redirect(get_permalink($page_id) . $qs);
            exit;
        }

        // approve transient and create search request
        elseif (isset($_GET['wpcsApproveVsr'])) {
            $t = $_GET['wpcsApproveVsr'];
            $this->_approveTransient($t);
        }
    }


    /**
     * handler für ajax requests
     * @return void
     */
    public function _ajax() {
        $response = [
            'action' => $_POST['action']
        ];

        // create transient fpr search request
        if ($_POST['todo'] == 'addSearchRequest') {
            if (!$this->_createTransient($_POST)) {
                $response['error'] = 1;
                $response['msg'] = implode("<br />", $this->error);
            } else {
                $response['status'] = 1;
                $response['msg'] = 'Vielen Dank! Um den Suchauftrag zu bestätigen, klicken Sie bitte auf den Link, den Sie soeben per E-Mail erhalten haben';
            }
        }

        // remove search request
        elseif ($_POST['todo'] == "removeSearchRequest") {
            if (!$this->_remove_search_request($_POST['hash'], $_POST['time'])) {
                $response['error'] = 1;
                $response['msg'] = $this->error;
            } else {
                $response['status'] = 1;
            }
        }

        // return JSON
        header('Content-Type: application/json');
        die(json_encode($response));
    }

    /**
     * legt einen Transient an, anhand der Benutzer per E-Mail Link also Opt-in seinen Suchauftrag generieren kann
     * @param $email
     * @param $formData
     * @return bool
     */
    private function _createTransient($data) {
        $formDataQs = $data['formData'];
        $email = $data['email'];
        $privacy = intval($data['privacy']);

        parse_str($formDataQs, $formDataArray);
        $t = uniqid();
        $filters = $this->_getFilters($formDataArray, true);
        $data = [
            'email' => $email,
            'formData' => $filters
        ];

        // Filter müssen vorhanden sein
        if (!is_array($filters)) {
            $this->error[] = "Bitte führen Sie eine Suche durch, um diese zu speichern";
        }

        // es müssen mindestens 3 Filter sein
        elseif (count($filters) < 3) {
            $this->error[] = "Bitte suchen sie mindestens nach 3 Kriterien";
        }

        // Email muss valide sein
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
            $this->error[] = "E-Mail Adresse scheint ungültig zu sein";
        }

        // Email muss valide sein
        if ($privacy < 1) {
            $this->error[] = "Bitte akzeptieren Sie die Datenschutzbestimmungen";
        }

        if (!empty($this->error)) return false;

        if (!set_transient($t, $data, 86400)) {
            $this->error[] = "Konnte Suchanfrage nicht speichern";
            return false;
        }

        $this->_send_optin_mail($email, $t);

        return true;
    }


    /**
     * @param $t
     * @return void
     */
    private function _approveTransient($t) {

        // get transient
        $data = get_transient($t);
        if (empty($data)) {
            $this->error = "Link nicht mehr gültig. Bitte generieren Sie ein neue Suchanfrage.";
            return false;
        }

        // create search request
        if (!$this->_create_search_request($data['email'], $data['formData'])) return false;

        // delete transient
        delete_transient($t);
    }

    /**
     * @param $cfData
     * @return array
     */
    private function _getFilters($cfData, $hide_empty = false) {
        $filters = [];
        foreach ($cfData as $key => $array) {
            $value = is_array($array) ? $array[0] : $array;
            if ($hide_empty && ($value == "" || $value == "#")) continue;
            if (VehicleFields::isField($key)) $filters[$key] = $value;
        }
        return $filters;
    }


    /**
     * @return void
     */
    public function _notify() {
        $toNotify = [];
        $posts = $this->getAllSearchRequests();
        $msg = null;

        foreach ($posts as $post) {
            $cf = get_post_meta($post->ID, null, true);
            $filters = $this->_getFilters($cf);
            $filters['posts_per_page'] = 5;
            $filters['first_import_time'] = $cf['last_notification_time'][0];

            $msg .=  "<strong>Suchauftrag vom " . date('d.m.Y H:i', $cf['last_notification_time'][0]) . " Uhr</strong> / letzte Benachrichtigung am: " . date("d.m.Y H:i:s", $cf['last_notification_time'][0]) . "<br />";

            $query = Filter::getInstance()->queryVehicles($filters);
            $vehiclesFiltered = $query['vehicles'];

            if (!empty($vehiclesFiltered)) {
                $data = [
                    'last_notification_time' => $cf['last_notification_time'][0],
                    'time' => $cf['time'][0],
                    'filters' => $filters,
                    'vehicles' => []
                ];

                // benachrichtige über gefundene Fahrzeuge
                $msg .= "Übersicht Auswahl Fahrzeuge:<br />";
                foreach ($vehiclesFiltered as $vehicle) {
                    $msg .= "-" . $vehicle->post_title . "<br />";
                    $data['vehicles'][] = [
                        'title' => $vehicle->post_title,
                        'ID' => $vehicle->ID,
                        'link' => get_post_permalink($vehicle->ID)
                    ];
                }

                $toNotify[$cf['email'][0]][] = $data;

                // merke Zeitpunkt der letzten Benachrichtigung
                update_post_meta($post->ID, 'last_notification_time', time());
            }
        }

        #wpcs_p($toNotify);
        #wp_die();

        $filters = VehicleFilter::getFilters();
        foreach ($toNotify as $email => $data) {
            $content = CustomTemplate::getInstance()->get_template_content('mail/search_request_new_matches');

            $msg = null;
            foreach ($data as $request) {
                $request_filter = VehicleFilter::getInstance()->formdata_to_filter_array($request['filters']);
                unset($request_filter['first_import_time']);
                $msg .= "<strong>Suchauftrag vom " . date('d.m.Y H:i', $request['time']) . " Uhr</strong><br />";
                $msg .= "<i>Suchparameter: ";

                $z = 0;
                foreach ($request_filter as $filterKey => $filterValue) {
                    if ($z != 0) $msg .= ", ";
                    if ($filterValue == "1") $msg .= $filters[$filterKey]['label'];
                    else $msg .= $filters[$filterKey]['label'] . ": " . VehicleFilter::getInstance()->getReadableValue($filterKey, $filterValue);
                    $z++;
                }
                $msg .= "</i><br />";

                $msg .= '<ul>';
                foreach ($request['vehicles'] as $vehicle) {
                    $msg .= '<li><a href="' . $vehicle['link'] . '">' . $vehicle['title'] . '</a></li>';
                }
                $msg .= '</ul><br />';
            }
            $msg .= '<hr />';
            $content = str_replace("{{search_request_list_vehicles}}", $msg, $content);

            $link = '<a href="' . $this->_getVsrUrl($this->_getHash($email)) . '">Verwalten Sie Ihre Suchaufträge.</a>';
            $content = str_replace("{{search_request_manage_requests}}", $link, $content);

            $m = new Mail();
            $m->send_mail($email, get_bloginfo('name') . ' - interessante Fahrzeuge auf Lager!', $content);
        }
    }


    /**
     * @return array|int[]|\WP_Post[]
     */
    public function getAllSearchRequests() {
        $q = new \WP_Query([
            'post_type' => 'wpcs_search_request', 'post_status' => 'publish', 'posts_per_page' => -1
        ]);
        return !empty($q->posts) ? $q->posts : [];
    }

    /**
     * @return mixed|null
     */
    public function count() {
        global $wpdb;
        $res = $wpdb->get_results('SELECT COUNT(ID) as count FROM '.$wpdb->prefix.'posts WHERE post_type = "wpcs_search_request" AND post_status = "publish";', ARRAY_A);
        return $res[0]['count'] ?? null;
    }

    /**
     * @param $hash
     * @param $time
     * @return bool
     */
    private function _remove_search_request($hash, $time) {
        $q = new \WP_Query([
            'post_type' => 'wpcs_search_request',
            'meta_query' => [
                [
                    'key' => 'hash',
                    'value' => $hash
                ],
                [
                    'key' => 'time',
                    'value' => $time
                ]
            ]
        ]);

        if ($q->post_count !== 1) {
            $this->error = "Ergebnissmenge inkorrekt";
            return false;
        }

        if (!wp_delete_post($q->posts[0]->ID, true)) {
            $this->error = "Konnte Eintrag nicht löschen";
            return false;
        }

        return true;
    }

    /**
     * @param $email
     * @param $searchData
     * @return bool
     */
    private function _create_search_request($email, $search) {
        $options = Admin::getInstance()->getOptions();

        // save post
        $time = time();
        $post_id = wp_insert_post([
            'post_title' => $email . " / " . date('d.m.Y H:i', $time),
            'post_type' => 'wpcs_search_request',
            'post_status' => 'publish'
        ]);

        // save post meta
        if ($post_id > 0) {

            // email
            add_post_meta($post_id, 'email', $email);

            // Datum
            add_post_meta($post_id, 'time', $time);

            // last notification time
            add_post_meta($post_id, 'last_notification_time', $time);

            // suchparameter werden einzeln gespeichert
            foreach ($search as $key => $value) {
                add_post_meta($post_id, $key, $value);
            }

            // hash dient dem Zugriff für den User auf seine Suchen
            $hash = $this->_getHash($email);
            add_post_meta($post_id, 'hash', $hash);

            // generiere Lead
            Lead::getInstance()->create("search_request", "", "wpcs_search_request", $post_id, ['email' => $email]);

            // mail to client
            $this->_send_request_created_mail_client($post_id);

            // mail to dealer
            if(isset($options['vsr_notify_dealer'])) {
                $this->_send_request_created_mail_dealer($post_id);
            }

            return true;
        }

        return false;
    }

    /**
     * @param $email
     * @return false|string
     */
    private function _getHash($email) {
        return hash('md5', $email . "wpcs");
    }

    /**
     * @param $hash
     * @return int[]|\WP_Post[]
     */
    private function _get_search_requests_by_hash($hash) {

        $searches = new \WP_Query([
            'post_type' => 'wpcs_search_request', 'posts_per_page' => -1,
            'meta_query' => [['key' => 'hash', 'value' => $hash]]
        ]);

        return $searches->posts;
    }


    /**
     * @param $hash
     * @return void
     */
    public function _draw_search_list($hash) {
        $searches = $this->_get_search_requests_by_hash($hash);

        $html = '<div id="wpcs_vsq_list" data-hash="' . $hash . '">';

        if (empty($searches)) {
            $html .= "Es existieren keine Suchaufträge unter dieser URL";
            $html .= '</div>'; // wpcs_vsq_list
            return $html;
        }

        $meta = get_post_meta($searches[0]->ID);
        $html .= '<h3>Zur E-Mail Adresse '.$meta['email'][0]." wurden folgende Suchaufträge gefunden</h3>";

        $filters = VehicleFields::getFields();
        foreach ($searches as $tmp) {
            $meta = get_post_meta($tmp->ID);

            $html .= '<div class="wpcs_search_request" data-time="' . $meta['time'][0] . '">';
            $html .= '<div class="delete_btn">' . FilterForm::getInstance()->drawAjaxLoading(['category' => 'searchRequest']) . '<span class="delete_icon wpcs_icon dashicons dashicons-no-alt"></span></div>';
            $html .= '<h3>Suchauftrag vom ' . date('d.m.Y H:i', $meta['time'][0]) . '</h3>';
            $last_notification = ($meta['last_notification_time'][0] == $meta['time'][0]) ? 'nie' : date('d.m.Y H:i', $meta['last_notification_time'][0]);
            $html .= '<i>zuletzt benachrichtigt am: ' . $last_notification . '</i>';
            $html .= '<p>Auftrag enhält folgende Filter:</p>';
            foreach ($meta as $key => $arr) {
                $value = $arr[0];
                if (!VehicleFields::isField($key)) continue;

                $readableValue = VehicleFilter::getInstance()->getReadableValue($key, $value);
                $label = $filters[$key]['label'];
                $display = ($label == $readableValue) ? $label : $label . " : " . $readableValue;
                $html .= '<div class="filter">' . $display . '</div>';
            }
            $html .= '</div>';
        }
        $html .= '</div>';
        $html .= "<script>if(typeof wpcs_search_request !== 'undefined') { wpcs_search_request._initList(); }</script>";

        return $html;
    }

    /**
     * @return string
     */
    public function _draw_save_request($args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['template' => 'form'];
        $args = array_merge($defaults, $args);

        $options = Admin::getInstance()->getOptions();
        $email = $options['email'] ?? null;
        $sid = uniqid();
        $html = '<div class="wpcs_save_search_request_wrapper" data-sid="' . $sid . '">';

        if(empty($email)) {
            $html .= "Bitte konfigurieren Sie die E-Mail Adresse unter Einstellungen -> WPCarSync -> Händler-Daten & Standorte.";
            $html .= '</div>'; // wpcs_save_search_request_wrapper
            return $html;
        }

        $html .= CustomTemplate::getInstance()->get_template_content('search_request/'.$args['template']);

        $html .= "<script>if(typeof wpcs_search_request !== 'undefined') { wpcs_search_request._initSearch('" . $sid . "'); }</script>";

        return $html;
    }

    /**
     * @param $hash
     * @return string
     */
    private function _getVsrUrl($hash, $params = null) {
        $url = home_url() . '?wpcs_vsr_redirect=' . $hash;
        if ($params != "") $url .= "&" . $params;
        return $url;
    }


    /**
     * @param $post_id
     * @return void
     */
    private function _send_request_created_mail_client($post_id) {
        $post = get_post($post_id);
        $subject = get_bloginfo('name') . " - Suchauftrag erstellt";

        $content = CustomTemplate::getInstance()->get_template_content('mail/search_request_success');
        $link = '<a href="' . $this->_getVsrUrl($post->hash) . '">Suchanfragen verwalten</a>';
        $content = str_replace("{{search_request_manage_requests}}", $link, $content);

        $m = new Mail();
        $m->send_mail($post->email, $subject, $content);
    }

    /**
     * @param $hash
     * @return string
     */
    private function _send_request_created_mail_dealer($post_id) {
        $meta = get_post_meta($post_id);
        $filters = VehicleFilter::getFilters();
        $html = null;

        $html .= '<h1>Neue Suchanfrage erstellt</h1>';
        $html .= "<p>E-Mail Adresse: <strong>".$meta['email'][0]."</strong></p>";

        $html .= '<p>Suchauftrag erstellt am ' . date('d.m.Y H:i', $meta['time'][0]) . '</p>';
        $html .= '<p>Auftrag enhält folgende Kriterien:</p>';

        foreach ($meta as $key => $arr) {
            $value = $arr[0];
            if (!VehicleFields::isField($key)) continue;

            $readableValue = VehicleFilter::getInstance()->getReadableValue($key, $value);
            $label = $filters[$key]['label'];
            $display = ($label == $readableValue) ? $label : $label . " : " . $readableValue;
            $html .= $display.'<br />';
        }

        // send mail
        $m = new Mail();
        $m->send_admin_mail('Neue Suchanfrage', $html);

        return $html;
    }

    /**
     * @param $email
     * @param $t
     * @return void
     */
    private function _send_optin_mail($email, $t) {
        $content = CustomTemplate::getInstance()->get_template_content('mail/search_request_optin');
        $link = '<a href="' . $this->_getVsrUrl($this->_getHash($email), 'wpcsApproveVsr=' . $t) . '">Jetzt Suchauftrag bestätigen</a>';
        $content = str_replace("{{search_request_optin_link}}", $link, $content);
        $m = new Mail();
        $m->send_mail($email, get_bloginfo('name') . ' - Bitte bestätigen Sie Ihren Suchauftrag', $content);
    }


    // Register Custom Post Type
    private function _register_post_type() {

        $labels = array(
            'name'                  => _x('Suchanfragen', 'Post Type General Name', 'wpcs'),
            'singular_name'         => _x('Suchanfrage', 'Post Type Singular Name', 'wpcs'),
            'menu_name'             => __('Suchanfragen', 'wpcs'),
            'name_admin_bar'        => __('Suchanfragen', 'wpcs'),
            'archives'              => __('Suchanfragen Archiv', 'wpcs'),
            'attributes'            => __('Suchanfragen Attribute', 'wpcs'),
            'parent_item_colon'     => __('Parent Item:', 'wpcs'),
            'all_items'             => __('Alle Suchanfragen', 'wpcs'),
            'add_new_item'          => __('neue Suchanfrage', 'wpcs'),
            'add_new'               => __('neue Suchanfrage', 'wpcs'),
            'new_item'              => __('neue Suchanfrage', 'wpcs'),
            'edit_item'             => __('Suchanfrage bearbeiten', 'wpcs'),
            'update_item'           => __('Suchanfrage bearbeiten', 'wpcs'),
            'view_item'             => __('Suchanfrage anzeigen', 'wpcs'),
            'view_items'            => __('zeige Suchanfragen', 'wpcs'),
            'search_items'          => __('suche Suchanfragen', 'wpcs'),
            'not_found'             => __('Not found', 'wpcs'),
            'not_found_in_trash'    => __('Not found in Trash', 'wpcs'),
            'featured_image'        => __('Featured Image', 'wpcs'),
            'set_featured_image'    => __('Set featured image', 'wpcs'),
            'remove_featured_image' => __('Remove featured image', 'wpcs'),
            'use_featured_image'    => __('Use as featured image', 'wpcs'),
            'insert_into_item'      => __('Insert into item', 'wpcs'),
            'uploaded_to_this_item' => __('Uploaded to this item', 'wpcs'),
            'items_list'            => __('Items list', 'wpcs'),
            'items_list_navigation' => __('Items list navigation', 'wpcs'),
            'filter_items_list'     => __('Filter items list', 'wpcs'),
        );

        $args = array(
            'label'                 => __('Suchanfrage', 'wpcs'),
            'description'           => __('Fahrzeug Suchanfragen', 'wpcs'),
            'labels'                => $labels,
            'supports'              => array('title', 'custom-fields'),
            'hierarchical'          => false,
            'public'                => false,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'menu_position'         => 5,
            'menu_icon'             => 'dashicons-search',
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => true,
            'can_export'            => false,
            'has_archive'           => false,
            'exclude_from_search'   => true,
            'publicly_queryable'    => false,
            'capability_type'       => 'page',
        );

        register_post_type('wpcs_search_request', $args);
    }
}
