<?php

namespace DXIM;

use DXIM\theme\DivisionData;
use DXIM\theme\Job;
use DXIM\theme\LocationData;

class AdminSetting {

    public static function draw_modules_modules_settings() {
        $quickaccess = Admin::getInstance()->getOptions('quickaccess');
        if(!is_array($quickaccess)) $quickaccess = [];

        $modules = Admin::getInstance()->get_modules();
        $html = '<div id="wpcs_admin_modules_wrapper">';

        foreach($modules as $module_slug => $module) {
            $inactive = $module['pro'] && !WPCS_PRO ? "inactive" : "";
            $html .= '<div class="module wpcs_admin_item '.$inactive.'">';

                $html .= '<div class="first">';
                    $html .= '<div class="icon">';
                        $html .= '<span class="dashicons '.$module['icon'].'"></span>';
                    $html .= '</div>';

                    $html .= '<div class="info">';
                        $html .= '<span class="name">'.$module['label'].'</span><br />';
                        $html .= '<span class="description">'.$module['description'].'</span>';
                    $html .= '</div>';
                $html .= '</div>';

                $html .= '<div class="options">';

                    if($module['pro'] && !WPCS_PRO) {
                        $html .= '<span class="pro_batch">PRO</span>';
                    }
                    else {

                        // link zu Settings
                        if ($module['has_settings']) $html .= '<a href="#" class="admin_menu_go_to" data-slug="' . $module_slug . '" data-section="modules">Settings <span class="dashicons dashicons-admin-generic"></span></a>';

                        // Link zum Content
                        if ($module['type'] == "post_type") $link = admin_url() . 'edit.php?post_type=' . $module['post_type'];
                        else $link = admin_url() . 'edit-tags.php?taxonomy=' . $module['taxonomy'];
                        $html .= '<a href="' . $link . '">Inhalte <span class="dashicons dashicons-edit"></span></a>';

                        // Schnellzugriff
                        $checked = isset($quickaccess[$module_slug]) ? "checked" : "";
                        $html .= '<div>in Menü <input type="checkbox" class="wpcs_checkbox" ' . $checked . ' name="dxim[quickaccess][' . $module_slug . ']" value="1" /></div>';
                    }
                $html .= '</div>'; // options

            $html .= '</div>';
        }

        $html .= '</div>';

        print $html;
    }


    /**
     * Standorte, Abteilungen und Team
     */
    public static function draw_modules_location_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        /**
         * übertragen der WPCS Standorte in neue Taxonomie
         */
        if (isset($options['locations'])) {
            foreach ($options['locations'] as $location_id => $location) {

                $args = array(
                    'hide_empty' => false, // also retrieve terms which are not used yet
                    'meta_query' => array(array('key' => 'wpcs_converted_location', 'value' => $location_id)),
                    'taxonomy'  => \DXIM\theme\Location::$taxonomy,
                );
                $term = get_terms($args)[0] ?? [];

                if (empty($term)) {
                    $new_term = wp_insert_term($location['company-name'], \DXIM\theme\Location::$taxonomy);
                    $term_id = 0;
                    if (is_array($new_term)) $term_id = $new_term['term_id'] ?? 0;
                    if ($term_id > 0) {
                        add_term_meta($term_id, "street", $location['street']);
                        add_term_meta($term_id, "zipcode", $location['zipcode']);
                        add_term_meta($term_id, "city", $location['city']);
                        add_term_meta($term_id, "email", $location['email']);
                        add_term_meta($term_id, "email_lead", $location['email_lead']);
                        add_term_meta($term_id, "phone", $location['phone']);
                        add_term_meta($term_id, "wpcs_converted_location", $location_id);

                        unset($options['locations'][$location_id]);
                    }
                }
            }
        }

        print '<h2>Standorte</h2>';


        // Slug
        print '<div class="form_item">';
            print '<label class="box">Url - Slug<br /><span class="small-text"></span></label>';
            print '<div>';
            $slug = $options['theme']['location']['slug'] ?? "standort";
            print '<input type="text" name="dxim[theme][location][slug]" value="' . $slug . '" />';
            print '</div>';
        print '</div>';

        // Template laden
        print '<div class="form_item">';
            print '<label class="box">Template Standort Einzelansicht<br /><span class="small-text">(Taxonomy Archive)</span></label>';
            print '<div>';
            $template = $options['theme']['location']['template_tax'] ?? "wpcs";
            $checked = $template == "wpcs" ? "checked" : "";
            print 'WPCarSync Standort Template <input type="radio" name="dxim[theme][location][template_tax]" value="wpcs" ' . $checked . ' />';
            $checked = $template == "theme" ? "checked" : "";
            print 'Theme Default Template <input type="radio" name="dxim[theme][location][template_tax]" value="theme" ' . $checked . ' />';
            print '</div>';
        print '</div>';

        // Darstellung Abteilung
        print '<div class="form_item">';
            print '<label class="box">Darstellung Abteilungen</label>';
            print '<select name="dxim[theme][location][division_layout]">';
            $layout = $options['theme']['location']['division_layout'] ?? null;
            foreach (['select' => 'Select', 'tabs' => 'Tabs', 'accordion' => 'Akkordeon'] as $value => $label) {
                $sel = ($value == $layout) ? "selected" : null;
                print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
            }
            print '</select>';
        print '</div>';

    }

    public static function draw_modules_team_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        $pro = !WPCS_PRO || WPCS_DEMO ? '<span class="pro">Pro</span>' : "";
        print '<h2>' . $pro . ' Team</h2>';

        if (!$l->pro) {
            print "<ul>
                <li>Stellen Sie passende Ansprechpartner zu Fahrzeugen, Angeboten, Modellen, Stellenausschreibungen, etc... dar</li>
                <li>Frei platzierbare Personen per Shortcode überall auf der Website</li>
            </ul>";
            print Admin::getInstance()->_go_pro();
        }

        // verwalten
        print '<div class="form_item">';
            print '<label class="box">Team verwalten</label>';
            $edit_link = admin_url().'/edit.php?post_type=' . \DXIM\theme\Person::$post_type;
            print '<a href="'.$edit_link.'" class="button button-primary">Team verwalten</a>';
        print '</div>';

    }

    /**
     * Angebote
     */
    public static function draw_modules_offer_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        print '<h2>Angebote</h2>';
        $display = "";
        print '<div style="display:' . $display . ';">';

        // Slug Archiv
        print '<div class="form_item">';
            print '<label class="box">Url - Slug Angebote</label>';
            print '<div>';
            $slug = $options['theme']['offer']['slug'] ?? "angebote";
            print '<input type="text" name="dxim[theme][offer][slug]" value="' . $slug . '" />';
            print '</div>';
        print '</div>';

        // Template Single
        print '<div class="form_item">';
            print '<label class="box">Template Angebot Einzelansicht</label>';
            print '<div>';
            $template = $options['theme']['offer']['template_single'] ?? 'wpcs';
            $checked = $template == "wpcs" ? "checked" : "";
            print 'WPCarSync Template <input type="radio" name="dxim[theme][offer][template_single]" value="wpcs" ' . $checked . ' />';
            $checked = $template == "theme" ? "checked" : "";
            print 'Theme Default Template <input type="radio" name="dxim[theme][offer][template_single]" value="theme" ' . $checked . ' />';
            print '</div>';
        print '</div>';

        // Template Archive
        print '<div class="form_item">';
            print '<label class="box">Template Angebot Archiv</label>';
            print '<div>';
            $template = $options['theme']['offer']['template_archive'] ?? 'wpcs';
            $checked = $template == "wpcs" ? "checked" : "";
            print 'WPCarSync Template <input type="radio" name="dxim[theme][offer][template_archive]" value="wpcs" ' . $checked . ' />';
            $checked = $template == "theme" ? "checked" : "";
            print 'Theme Default Template <input type="radio" name="dxim[theme][offer][template_archive]" value="theme" ' . $checked . ' />';
            print '</div>';
        print '</div>';

        print '<h3>Ansprechpartner</h3>';

        // Darstellung Ansprechpartner
        print '<div class="form_item">';
        print '<label class="box">Darstellung Ansprechpartner nach Standort</label>';
        print '<div>';
        print '<select name="dxim[theme][offer][team_layout]">';
        $layout = $options['theme']['offer']['team_layout'] ?? 'select';
        foreach (['all' => 'alle anzeigen', 'select' => 'Standort Auswahlbox'] as $value => $label) {
            $sel = ($value == $layout) ? "selected" : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select>';
        print '</div>';
        print '</div>';

        // Abfrage Ansprechpartner
        print '<div class="form_item">';
        print '<label class="box">Abfrage Ansprechpartner<br /><span class="small-text">Shortcode: [WPCS_person_offer]</span></label>';
        print '<ul>';
        $same_location = $options['theme']['offer']['person_query']['brand'] ?? 1;
        print '<li><label>Person muss gleicher Marke zugordnet sein wie Angebot</label><br />';
        $checked = ($same_location == 1) ? "checked" : null;
        print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][offer][person_query][brand]" value="1" />';
        $checked = ($same_location == 0) ? "checked" : null;
        print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][offer][person_query][brand]" value="0" />';
        print '</li>';

        $d = new DivisionData();
        $divisions = $d->getAll();
        print '<li><label>Person muss in folgender Abteilung sein</label><br />
                        <select name="dxim[theme][offer][person_query][division]">';
        print '<option value="0">alle Abteilungen</option>';
        $current_division = $options['theme']['offer']['person_query']['division'] ?? null;
        foreach ($divisions as $division) {
            $sel = ($current_division == $division->term_id) ? "selected" : null;
            print '<option value="' . $division->term_id . '" ' . $sel . '>' . $division->name . '</option>';
        }
        print '</select>';
        print '</li>';

        print '<li><label>Anzahl Personen anzeigen</label><br />';
        $count = $options['theme']['offer']['person_query']['count'] ?? 1;
        print '<input type="number" name="dxim[theme][offer][person_query][count]" value="' . $count . '" />';
        print '</li>';

        print '<li><label>Reihenfolge zufällig</label><br />';
        $random = $options['theme']['offer']['person_query']['random'] ?? 1;
        $checked = ($random == 1) ? "checked" : null;
        print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][offer][person_query][random]" value="1" />';
        $checked = ($random == 0) ? "checked" : null;
        print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][offer][person_query][random]" value="0" />';
        print '</li>';

        print '</ul>';
        print '</div>';

        print '<hr />';
        print '<h3>Angebote in Fahrzeugbörse platzieren</h3>';
        print '<p>Passende Angebote werden automatisch auf Basis des Models und Herstellers ausgewählt.</p>';

        // in Börse platzieren
        print '<div class="form_item">';
        print '<label class="box">Automatisch nach X. Fahrzeug einblenden<br /><span class="small-text">0 = es werden keine Angebote angezeigt</span></label>';
        print '<div>';
        $position = $options['theme']['offer']['wpcs_archive']['position'] ?? 0;
        print '<input type="number" name="dxim[theme][offer][wpcs_archive][position]" value="' . $position . '" />';
        print '</div>';
        print '</div>';

        print '<div class="form_item">';
        print '<label class="box">auf folgenden Ergebnisseiten anzeigen<br /><span class="small-text">Seitennummern angeben, Komma separiert, z.B. 1,5</span></label>';
        print '<div>';
        $pages = $options['theme']['offer']['wpcs_archive']['pages'] ?? 1;
        print '<input type="text" name="dxim[theme][offer][wpcs_archive][pages]" value="' . $pages . '" />';
        print '</div>';
        print '</div>';

        print '<div class="form_item">';
        print '<label class="box">Anzahl Angebote<br /><span class="small-text">Wenn mehr als 1 wird Slider verwendet</span></label>';
        print '<div>';
        $count = $options['theme']['offer']['wpcs_archive']['count'] ?? 1;
        print '<input type="number" name="dxim[theme][offer][wpcs_archive][count]" value="' . $count . '" />';
        print '</div>';
        print '</div>';

        print '</div>';

    }

    /**
     * @return void
     */
    public static function draw_modules_offergroup_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        print '<h2>Aktionen</h2>';

        $display = '';
        print '<div style="display:' . $display . ';">';

            // verwalten
            print '<div class="form_item">';
                print '<label class="box">Aktionen verwalten</label>';
                $edit_link = admin_url().'/edit-tags.php?taxonomy=' . \DXIM\theme\OfferGroup::$taxonomy;
                print '<a href="'.$edit_link.'" class="button button-primary">Aktionen verwalten</a>';
            print '</div>';

            print '<div class="form_item">';
                print '<label class="box">Url - Slug Aktionen</label>';
                print '<div>';
                $slug = $options['theme']['offergroup']['slug'] ?? "aktionen";
                print '<input type="text" name="dxim[theme][offergroup][slug]" value="' . $slug . '" />';
                print '</div>';
            print '</div>';

            print '<div class="form_item">';
                print '<label class="box">Template Aktion</label>';
                print '<div>';
                $template = $options['theme']['offergroup']['template_term'] ?? 'wpcs';
                $checked = $template == "wpcs" ? "checked" : "";
                print 'WPCarSync Template <input type="radio" name="dxim[theme][offergroup][template_term]" value="wpcs" ' . $checked . ' />';
                $checked = $template == "theme" ? "checked" : "";
                print 'Theme Default Template <input type="radio" name="dxim[theme][offergroup][template_term]" value="theme" ' . $checked . ' />';
                print '</div>';
            print '</div>';

        print '</div>';
    }


    /**
     * Service
     */
    public static function draw_modules_service_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        print '<h2>Service - Dienstleistungen</h2>';
        $display = "";

        print '<div style="display:' . $display . ';">';

        // Slug
        print '<div class="form_item">';
            print '<label class="box">Url - Slug Services</label>';
            print '<div>';
            $slug = $options['theme']['service']['slug'] ?? "services";
            print '<input type="text" name="dxim[theme][service][slug]" value="' . $slug . '" />';
            print '</div>';
        print '</div>';

        // Darstellung Shortcode WPCS_person
        print '<div class="form_item">';
            print '<label class="box">Template Service Einzelansicht</label>';
            print '<div>';
            $template = $options['theme']['service']['template_single'] ?? 'wpcs';
            $checked = $template == "wpcs" ? "checked" : "";
            print 'WPCarSync Template <input type="radio" name="dxim[theme][service][template_single]" value="wpcs" ' . $checked . ' />';
            $checked = $template == "theme" ? "checked" : "";
            print 'Theme Default Template <input type="radio" name="dxim[theme][service][template_single]" value="theme" ' . $checked . ' />';
            print '</div>';
        print '</div>';

        print '<h3>Ansprechpartner für Service</h3>';

        // Darstellung Ansprechpartner
        print '<div class="form_item">';
            print '<label class="box">Darstellung Ansprechpartner nach Standort</label>';
            print '<div>';
            print '<select name="dxim[theme][service][team_layout]">';
            $layout = $options['theme']['service']['team_layout'] ?? 'select';
            foreach (['all' => 'alle anzeigen', 'select' => 'Standort Auswahlbox'] as $value => $label) {
                $sel = ($value == $layout) ? "selected" : null;
                print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
            }
            print '</select>';
            print '</div>';
        print '</div>';

        // Abfrage Ansprechpartner
        print '<div class="form_item">';
            print '<label class="box">Abfrage Ansprechpartner<br /><span class="small-text">Shortcode: [WPCS_service_team]</span></label>';
            print '<ul>';

            $d = new DivisionData();
            $divisions = $d->getAll();
            print '<li><label>Person muss in folgender Abteilung sein</label><br />
                                    <select name="dxim[theme][service][person_query][division]">';
            print '<option value="0">alle Abteilungen</option>';
            $current_division = $options['theme']['service']['person_query']['division'] ?? null;
            foreach ($divisions as $division) {
                $sel = ($current_division == $division->term_id) ? "selected" : null;
                print '<option value="' . $division->term_id . '" ' . $sel . '>' . $division->name . '</option>';
            }
            print '</select>';
            print '</li>';

            print '<li><label>Anzahl Personen anzeigen</label><br />';
            $count = $options['theme']['service']['person_query']['count'] ?? 1;
            print '<input type="number" name="dxim[theme][service][person_query][count]" value="' . $count . '" />';
            print '</li>';

            print '<li><label>Reihenfolge zufällig</label><br />';
            $random = $options['theme']['service']['person_query']['random'] ?? 1;
            $checked = ($random == 1) ? "checked" : null;
            print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][service][person_query][random]" value="1" />';
            $checked = ($random == 0) ? "checked" : null;
            print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][service][person_query][random]" value="0" />';
            print '</li>';

            print '</ul>';
        print '</div>';

        print '</div>'; // wrapper

    }



    /**
     * Leads
     */
    public static function draw_modules_leads_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        print '<h2>Leads</h2>';
        $display = '';

        print '<div style="display:' . $display . ';">';

            // verwalten
            print '<div class="form_item">';
                print '<label class="box">Leads verwalten</label>';
                $edit_link = admin_url().'/edit.php?post_type=' .Lead::$post_type;
                print '<a href="'.$edit_link.'" class="button button-primary">Leads verwalten</a>';
            print '</div>';

        print '</div>'; // display wrapper

    }

    /**
     * Stellen
     */
    public static function draw_modules_jobs_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        print '<h2>Stellenagebote</h2>';
        $display = "";
        print '<div style="display:' . $display . ';">';

        // verwalten
        print '<div class="form_item">';
            print '<label class="box">Stellenangebote verwalten</label>';
            $edit_link = admin_url().'/edit.php?post_type=' .Job::$post_type;
            print '<a href="'.$edit_link.'" class="button button-primary">Stellenangebote verwalten</a>';
        print '</div>';

        // Slug
        print '<div class="form_item">';
            print '<label class="box">Url - Slug</label>';
            print '<div>';
            $slug = $options['theme']['job']['slug'] ?? "stellenangebote";
            print '<input type="text" name="dxim[theme][job][slug]" value="' . $slug . '" />';
            print '</div>';
        print '</div>';

        print '<div class="form_item">';
            print '<label class="box">Template Job Einzelansicht</label>';
            print '<div>';
            $template = $options['theme']['job']['template_single'] ?? 'wpcs';
            $checked = $template == "wpcs" ? "checked" : "";
            print 'WPCarSync Template <input type="radio" name="dxim[theme][job][template_single]" value="wpcs" ' . $checked . ' />';
            $checked = $template == "theme" ? "checked" : "";
            print 'Theme Default Template <input type="radio" name="dxim[theme][job][template_single]" value="theme" ' . $checked . ' />';
            print '</div>';
        print '</div>';

        print '<div class="form_item">';
            print '<label class="box">Template Jobs Archiv</label>';
            print '<div>';
            $template = $options['theme']['job']['template_archive'] ?? 'wpcs';
            $checked = $template == "wpcs" ? "checked" : "";
            print 'WPCarSync Template <input type="radio" name="dxim[theme][job][template_archive]" value="wpcs" ' . $checked . ' />';
            $checked = $template == "theme" ? "checked" : "";
            print 'Theme Default Template <input type="radio" name="dxim[theme][job][template_archive]" value="theme" ' . $checked . ' />';
            print '</div>';
        print '</div>';

        print '<div class="form_item">';
            print '<label class="box">Anzahl Jobs Archiv pro Seite</label>';
            print '<div>';
            $count = $options['theme']['job']['posts_per_page'] ?? 50;
            print '<input type="number" name="dxim[theme][job][posts_per_page]" value="' . $count . '" />';
            print '</div>';
            print '</div>';
        print '</div>';

    }


    /**
     * @return void Modelle
     */
    public static function draw_modules_brand_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        print '<h2>Marken</h2>';
        $display = '';
        print '<div style="display:' . $display . ';">';

            // Marken Slug
            print '<div class="form_item">';
                print '<label class="box">Url - Slug Marke</label>';
                print '<div>';
                $slug = $options['theme']['brand']['slug'] ?? "marke";
                print '<input type="text" name="dxim[theme][brand][slug]" value="' . $slug . '" />';
                print '</div>';
            print '</div>';

            // Template Marke
            print '<div class="form_item">';
                print '<label class="box">Template Marke Einzelansicht</label>';
                print '<div>';
                $template = $options['theme']['brand']['template_single'] ?? "wpcs";
                $checked = $template == "wpcs" ? "checked" : "";
                print 'WPCarSync Modell Template <input type="radio" name="dxim[theme][brand][template_single]" value="wpcs" ' . $checked . ' />';
                $checked = $template == "theme" ? "checked" : "";
                print 'Theme Default Template <input type="radio" name="dxim[theme][brand][template_single]" value="theme" ' . $checked . ' />';
                print '</div>';
            print '</div>';

            print '<div class="form_item no_border">';
                print '<label class="box">Abfrage Ansprechpartner<br /><span class="small-text"></span></label>';
                print '<ul>';
                $d = new DivisionData();
                $divisions = $d->getAll();
                print '<li><label>Person muss in folgender Abteilung sein</label><br />
                                <select name="dxim[theme][brand][person_query][division]">';
                print '<option value="0">alle Abteilungen</option>';
                $current_division = $options['theme']['brand']['person_query']['division'] ?? null;
                foreach ($divisions as $division) {
                    $sel = ($current_division == $division->term_id) ? "selected" : null;
                    print '<option value="' . $division->term_id . '" ' . $sel . '>' . $division->name . '</option>';
                }
                print '</select>';
                print '</li>';

                print '<li><label>Anzahl Personen anzeigen</label><br />';
                $count = $options['theme']['brand']['person_query']['count'] ?? 1;
                print '<input type="number" name="dxim[theme][brand][person_query][count]" value="' . $count . '" />';
                print '</li>';

                print '<li><label>Reihenfolge zufällig</label><br />';
                $random = $options['theme']['brand']['person_query']['random'] ?? 1;
                $checked = ($random == 1) ? "checked" : null;
                print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][brand][person_query][random]" value="1" />';
                $checked = ($random == 0) ? "checked" : null;
                print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][brand][person_query][random]" value="0" />';
                print '</li>';
                print '</ul>';
            print '</div>';

        print '</div>'; // display wrapper

    }

    /**
     * @return void Modelle
     */
    public static function draw_modules_model_settings() {
        $l = Admin::getInstance()->gl();
        $options = Admin::getInstance()->getOptions();

        print '<h2>Modelle</h2>';
        $display = '';
        print '<div style="display:' . $display . ';">';

            // Model Slug
            print '<div class="form_item">';
                print '<label class="box">Url - Slug Modell</label>';
                print '<div>';
                $slug = $options['theme']['model']['slug'] ?? "modell";
                print '<input type="text" name="dxim[theme][model][slug]" value="' . $slug . '" />';
                print '</div>';
            print '</div>';

            // Template Modell
            print '<div class="form_item">';
                print '<label class="box">Template Modell Einzelansicht</label>';
                print '<div>';
                $template = $options['theme']['model']['template_single'] ?? "wpcs";
                $checked = $template == "wpcs" ? "checked" : "";
                print 'WPCarSync Modell Template <input type="radio" name="dxim[theme][model][template_single]" value="wpcs" ' . $checked . ' />';
                $checked = $template == "theme" ? "checked" : "";
                print 'Theme Default Template <input type="radio" name="dxim[theme][model][template_single]" value="theme" ' . $checked . ' />';
                print '</div>';
            print '</div>';

            // Team Query Modell
            print '<div class="form_item">';
                print '<label class="box">Abfrage Ansprechpartner<br /><span class="small-text">Shortcode: [WPCS_person_model]</span></label>';
                print '<ul>';
                $same_location = $options['theme']['model']['person_query']['brand'] ?? 1;
                print '<li><label>Person muss gleicher Marke zugordnet sein wie Model</label><br />';
                $checked = ($same_location == 1) ? "checked" : null;
                print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][model][person_query][brand]" value="1" />';
                $checked = ($same_location == 0) ? "checked" : null;
                print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][model][person_query][brand]" value="0" />';
                print '</li>';

                $d = new DivisionData();
                $divisions = $d->getAll();
                print '<li><label>Person muss in folgender Abteilung sein</label><br />
                                <select name="dxim[theme][model][person_query][division]">';
                print '<option value="0">alle Abteilungen</option>';
                $current_division = $options['theme']['model']['person_query']['division'] ?? null;
                foreach ($divisions as $division) {
                    $sel = ($current_division == $division->term_id) ? "selected" : null;
                    print '<option value="' . $division->term_id . '" ' . $sel . '>' . $division->name . '</option>';
                }
                print '</select>';
                print '</li>';

                print '<li><label>Anzahl Personen anzeigen</label><br />';
                $count = $options['theme']['model']['person_query']['count'] ?? 1;
                print '<input type="number" name="dxim[theme][model][person_query][count]" value="' . $count . '" />';
                print '</li>';

                print '<li><label>Reihenfolge zufällig</label><br />';
                $random = $options['theme']['model']['person_query']['random'] ?? 1;
                $checked = ($random == 1) ? "checked" : null;
                print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][model][person_query][random]" value="1" />';
                $checked = ($random == 0) ? "checked" : null;
                print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][model][person_query][random]" value="0" />';
                print '</li>';

                print '</ul>';
            print '</div>';

        print '</div>'; // display wrapper

    }

    public static function draw_accounts_wpcs_settings() {
        $options = Admin::getInstance()->getOptions();

        print '<h2>WPCarSync Lizenz</h2>';

        // Lizenz Key
        print '<div class="form_item">';
            print '<label class="box">Lizenzschlüssel</label>';
            if (!isset($options['wpcs_key'])) $options['wpcs_key'] = null;
            print '<input type="text" class="bigger" name="dxim[wpcs_key]" value="' . $options['wpcs_key'] . '" /> ';
        print '</div>';

        // Bestell -ID
        print '<div class="form_item">';
            print '<label class="box">Bestell-ID<br /><span class="small-text">Sie finden die Bestell-ID in Ihrer Bestätigungsmail.</span></label>';
            if (!isset($options['wpcs_order_id'])) $options['wpcs_order_id'] = null;
            print '<input type="text" name="dxim[wpcs_order_id]" value="' . $options['wpcs_order_id'] . '" /> ';
        print '</div>';

        // Lizenz Speichern
        print '<div class="form_item">';
            print '<label class="box">Lizenz speichern</label>';
            print '<a href="#" class="button button-primary" id="wpcarsync_form_submit_check_license">Lizenz speichern</a> ';
        print '</div>';


        print '<table id="dxim_license_table">';

        $l = Admin::getInstance()->gl();
        $expires = strtotime($l->expires ?? "");

        print '<tr>';
        print '<th>Lizenznehmer</th>';
        print '<th>Lizenz</th>';
        print '<th>Plugin</th>';
        print '<th>Updates</th>';
        print '<th>Domains</th>';
        print '<th>Pro</th>';
        if ($l->license_version == 2) print '<th>Optionen</th>';
        print '</tr>';

        print '<tr>';

        print '<td>';
        print $l->name;
        print '</td>';

        print '<td>';
        print $l->license ? '<strong class="status_active">aktiv</strong>' : '<strong class="status_inactive">inaktiv</strong>';
        if (!empty($l->expires)) {
            print '<br />(gültig bis: ' . date('d.m.Y', $expires) . ')<br />';
        }
        if (!$l->license) print '<br />' . $l->license_msg;
        print '</td>';

        print '<td>';
        print $l->plugin ? '<strong class="status_active">aktiv</strong>' : '<strong class="status_inactive">inaktiv</strong>';
        if (!empty($l->expires && $l->license_version == 2)) {
            print '<br />(gültig bis: ' . date('d.m.Y', $expires) . ')<br />';
        }
        print '</td>';

        print '<td>';
        if (Admin::getInstance()->_updates()) {
            print '<strong class="status_active">aktiv</strong>';
            $msgTmp = !isset($l->updates_until) ? 'uneingeschränkt' : 'bis ' . date('d.m.Y', $l->updates_until);
            print '<br />(' . $msgTmp . ')';
        } else {
            print '<strong class="status_inactive">inaktiv</strong>';
            print '<br />(Für Updates bitte Lizenz erneuern)';
        }
        print '</td>';

        print '<td>';
        if (isset($l->plugin_domains) && is_array($l->plugin_domains)) {
            foreach ($l->plugin_domains as $tmpDomain) {
                print $tmpDomain . '<br />';
            }
        }
        print '</td>';

        print '<td>';
        $pro = $l->pro ?? 0;
        print $pro == 1 ? "ja" : "nein";
        print '</td>';

        if ($l->license_version == 2) {
            print '<td>';
            print '<a class="button button-secondary" href="' . $l->receipt_url . '" target="_blank">Bestellung verwalten</a><br />';
            if ($pro != 1) print '<a class="button button-secondary" href="https://www.digistore24.com/upgrade/10719-iQTJOBANSm5M/' . $options['wpcs_order_id'] . '" target="_blank">Upgrade zu Pro</a><br />';

            print '</td>';
        }

        print '</tr>';
        print '</table>';

        if (empty($options['wpcs_key']) && $l->plugin) {
            print '<p>Demo Version läuft noch bis zum ' . date('d.m.Y', $l->expire) . '<br /><strong><a href="' . DXIM_PluginWebAddress . '" target="_blank">Lizenz erwerben</a></strong></p>';
        }
        // Lizenz und Plugin abgelaufen, Version 2
        elseif (!$l->plugin && !$l->license) {
            print '<p><strong><a href="' . DXIM_PluginWebAddress . '" target="_blank">Lizenz erwerben</a></strong></p>';
        }

        // Lizenz abgelaufen, Plugin funktioniert weiterhin, Version 1
        elseif (!$l->license && $l->plugin && $l->license_version == 1) {
            print "<p><strong>Lizenz abgelaufen am " . date('d.m.Y', $expires) . '.</strong><br />
                        Das Plugin kann wie gewohnt weiter genutzt werden, Updates wurden deaktiviert.<br />
                        Profitieren Sie jetzt von neuen Funktionen und Support! <a href="' . DXIM_PluginWebAddress . '" target="_blank">Jetzt Lizenz verlängern!</a></p>';
        }

    }

    public static function draw_accounts_mobile_settings() {
        print "<h2>Mobile.de Importer</h2>";

        $a = Account::getInstance();
        $ld = new LocationData();
        $locations = $ld->getTree();
        $options = Admin::getInstance()->getOptions();

        if (!empty($locations)) {

            print "<h3>Mobile.de Account hinzufügen</h3>";

            // Username
            print '<div class="form_item">';
            print '<label class="box">Benutzername<br /><span class="small-text">Bitte verwenden Sie die Zugangsdaten der <strong><a href="' . DXIM_PluginWebAddress . '/zugangsdaten-funktionieren-nicht-the-requested-url-returned-error-401-unauthorized/" target="_blank">Inseratseinbindung</a></strong></span></label>';
            print '<div>';
            print '<input type="text" name="dxim[new_user]" /> ';
            print '</div>';
            print '</div>';

            // PAsswort
            print '<div class="form_item">';
            print '<label class="box">Passwort</label>';
            print '<div>';
            print '<input type="text" name="dxim[new_user_pass]" />';
            print '</div>';
            print '</div>';

            // Standort
            print '<div class="form_item">';
            print '<label class="box">Standort</label>';
            print '<div>';
            print self::_location_select($locations, 'dxim[new_user_location]');
            print '</div>';
            print '</div>';
        } else {
            print '<p>';
            print '<strong>Bitte legen Sie zuerst einen Standort an und speichern Sie die Einstellungen.</strong><br />';
            print '</p>';
        }

        $accounts = $a->get_accounts('mobile.de');
        if (!empty($accounts)) {

            print '<h3>aktive Mobile.de Accounts</h3>';
            print '<ul id="dxim_mobile_accounts">';
            foreach ($accounts as $account_id => $account) {
                print '<li>';
                print '<div class="top_row">';

                // USername
                print '<div class="username account_name">
                                Account-ID: ' . $account_id . '<input type="hidden" name="dxim[accounts][' . $account_id . '][user]" value="' . $account['user'] . '" />
                            </div>';

                print '<div class="buttons">';
                // edit
                print '<a href="javascript:jQuery(\'#' . $account_id . '_edit_section\').toggle();" class="button"><span class="dashicons dashicons-edit"></span></a>';

                // sync
                $cMsg = 'Diesen Account jetzt synchronisieren?';
                print '<a href="javascript:wpcs_confirm_action(\'' . Admin::getInstance()->_getAdminPage() . '&wpcs_import=1&account_id=' . $account_id . '&section_slug=accounts&slug=mobile\',\'' . $cMsg . '\');" class="button"><span class="dashicons dashicons-update"></span></a>';

                // DELETE
                $cMsg = 'Achtung!\nBeim entfernen eines Accounts werden alle dazugehörigen Fahrzeuge ebenfalls entfernt.';
                print '<a class="button" href="javascript:wpcs_confirm_action(\'' . $_SERVER['PHP_SELF'] . '?page=' . Admin::$pageSlug . '&deleteAccount=' . $account_id . '&section_slug=accounts&slug=mobile\',\'' . $cMsg . '\');"><span class="dashicons dashicons-no-alt"></span></a>';
                print '</div>'; // buttons

                print '</div>'; // top_row

                print '<div id="' . $account_id . '_edit_section" class="edit_section">';

                // USername
                print '<div class="password">';
                print '<label>Passwort</label>';
                print '<input type="text" name="dxim[accounts][' . $account_id . '][password]" value="' . $account['password'] . '" />';
                print '</div>';

                // Standort
                print '<div class="location">';
                print "<label>Standort</label>";
                print self::_location_select($locations, 'dxim[accounts][' . $account_id . '][location]', $account['location']);
                print '</div>';

                print '</div>'; // edit section

                print '<input type="hidden" name="dxim[accounts][' . $account_id . '][type]" value="mobile.de" />';
                print '</li>';
            }
            print '</ul>';
        }



        print '<h3>Optionen für Mobile.de Importer</h3><br />';

        print '<div class="form_item">';
        print '<label class="box">Fahrzeugbeschreibung</label>';
        print '<div>';
        print '<select name="dxim[api_options][description]" >';
        foreach (['enriched' => 'HTML', 'plain' => 'reiner Text'] as $value => $label) {
            $sel = ($options['api_options']['description'] == $value) ? "selected" : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select>';
        print '</div>';
        print '</div>';


        if (!isset($options['api_options']['import_images'])) $options['api_options']['import_images'] = 'none';
        print '<div class="form_item">';
        print '<label class="box">Bild-Import<br /><span class="small-text">Wie sollen die Bilder importiert werden?</span></label>';
        print '<div class="options">';
        $checked = ($options['api_options']['import_images'] == 'none') ? 'checked' : null;
        print '<input type="radio" checked name="dxim[api_options][import_images]" value="none" ' . $checked . '> Bildverweise importieren<br />';

        #$checked = ($options['api_options']['import_images'] == 'post_thumbnail') ? 'checked' : null;
        #print '<input type="radio" name="dxim[api_options][import_images]" value="post_thumbnail" ' . $checked . '> Bildverweise & Post Thumbnail importieren<br />';

        $checked = ($options['api_options']['import_images'] == 'all_media') ? 'checked' : null;
        print '<input type="radio" name="dxim[api_options][import_images]" value="all_media" ' . $checked . '> in Mediathek importieren<br />';

        $checked = ($options['api_options']['import_images'] == 'all_filesystem') ? 'checked' : null;
        print '<input type="radio" name="dxim[api_options][import_images]" value="all_filesystem" ' . $checked . '> ins Dateisystem importieren<br />';
        print '</div>';
        print '</div>';
    }

    /**
     * @return void
     */
    public static function draw_accounts_csv_settings() {
        $options = Admin::getInstance()->getOptions();
        $l = Admin::getInstance()->gl();

        print '<h2>CSV Importer verwalten</h2>';

        if ($l->pro) {

            print '<h3>Neuen CSV Importer anlegen</h3>';
            $dir = wp_upload_dir();

            print '<div class="form_item">';
                print '<label class="box">Titel des CSV Importers</label>';
                print '<div>';
                print '<input type="text" style="min-width:50%;" name="dxim[new_csv_name]" /> ';
                print '</div>';
            print '</div>';

            print '<div class="form_item">';
                print '<label class="box">Vollständiger Pfad zur CSV-Datei</label>';
                print '<div>';
                print '<input type="text" class="bigger" name="dxim[new_csv_path]" /><br />';
                print '<span class="small-text">Tipp! Pfad zum Uploadverzeichnis lautet: ' . $dir['basedir'] . '</span>';
                print '</div>';
            print '</div>';

            print '<div class="form_item">';
                print '<label class="box"">Vollständiger Pfad zur Config-Datei</label>';
                print '<div>';
                print '<input type="text" class="bigger" id="wpcs_new_csv_config_path" name="dxim[new_csv_config_path]" /><br />';
                print '<span class="small-text">Tipp! Pfad zum Uploadverzeichnis lautet: ' . $dir['basedir'] . '</span><br />';
                print '<a href="#" id="wpcs_add_csv_extended_config_file_btn" data-path="' . DXIM_PluginPath . '/classes/import/csv/csv_extended_mapping.json">Mobile.de CSV Extended Format verwenden</a><br />';
                print '</div>';
            print '</div>';

            print '<div class="form_item">';
                print '<label class="box">Name der Zip - Datei inkl. Endung, z.b. Fahrzeuge.zip</label>';
                print '<div>';
                print '<input type="text" class="bigger" name="dxim[new_csv_zip_name]" /><br />';
                print '<span class="small-text">Falls angegeben, wird diese Zip-Datei, sollte sie vorhanden sein, entpackt<br />
                            Die Zip - Datei muss im gleichen Ordner liegen wie oben im CSV-Pfad angegeben wurde</span><br />';
                print '</div>';
            print '</div>';

            // eineutiger username, identifiziert den account
            print '<input type="hidden" name="dxim[new_csv_user]" value="' . uniqid() . '" /> ';
        } else {
            print "<strong>Verwenden von CSV Accounts nur mit Pro Lizenz Möglich</strong>";
        }

        $accounts = Account::getInstance()->get_accounts('csv');
        if (!empty($accounts)) {

            print '<h3>aktive CSV Accounts</h3>';
            print '<ul id="dxim_csv_accounts">';
            foreach ($accounts as $account_id => $account) {
                /**
                 * standort mapping an neue Taxonomie anpassen
                 * - für jeden csv account mapping laden
                 * - für jedem alten wpcs_standort prüfen, ob dieser im mapping ist
                 * - => falls ja durch das neue pendant der taxonomie ersetzen
                 * - dieser Part läuft nur einmalig, nach dem speichern der einstellungen sind keine alten locations mehr vorhanden
                 */
                if (isset($options['locations'])) {
                    $mapping = get_option("wpcs_csv_locations_" . $account_id);
                    foreach ($options['locations'] as $location_id => $location) {

                        foreach ($mapping as $key => $old_location) {
                            if ($old_location == $location_id) {
                                $args = array(
                                    'hide_empty' => false, 'meta_query' => array(array('key' => 'wpcs_converted_location', 'value' => $location_id)), 'taxonomy'  => \DXIM\theme\Location::$taxonomy,
                                );
                                $term = get_terms($args)[0] ?? [];
                                $mapping[$key] = $term->term_id;
                            }
                        }
                    }
                    update_option("wpcs_csv_locations_" . $account_id, $mapping);
                }

                print '<li>';

                print '<div class="top_row">';
                    print "<strong>" . $account['name'] . " (Account-ID: " . $account_id . ")</strong>";

                    print '<div class="buttons">';
                        print '<a href="javascript:jQuery(\'#' . $account_id . '_edit_section\').toggle();" class="button"><span class="dashicons dashicons-edit"></span></a>';
                        print '<a href="' . Admin::getInstance()->_getAdminPage() . '&wpcs_settings_page=csv_config&account_config=' . $account_id . '" class="button"><span class="dashicons dashicons-admin-generic"></span></a>';
                        $cMsg = "Diesen Account jetzt synchronisieren?";
                        print '<a href="javascript:wpcs_confirm_action(\'' . Admin::getInstance()->_getAdminPage() . '&wpcs_import=1&account_id=' . $account_id . '&section_slug=accounts&slug=csv\',\'' . $cMsg . '\');" class="button"><span class="dashicons dashicons-update"></span></a>';
                        $cMsg = 'Achtung!\nBeim entfernen eines Accounts werden alle dazugehörigen Fahrzeuge ebenfalls entfernt.';
                        print '<a href="javascript:wpcs_confirm_action(\'' . $_SERVER['PHP_SELF'] . '?page=' . Admin::$pageSlug . '&deleteAccount=' . $account_id . '&section_slug=accounts&slug=csv\',\'' . $cMsg . '\');" class="button"><span class="dashicons dashicons-no-alt"></a>';
                    print '</div>'; // buttons

                print '</div>'; // top_row

                $mapped = get_option('wpcs_csv_locations_' . $account_id);
                if (empty($mapped)) {
                    print '<p style="background:red; color:white; padding:5px;">Bitte klicken sie auf Account Einstellungen und konfigurieren Sie die Standortzuweisungen.</p>';
                }

                print '<div class="edit_section" id="' . $account_id . '_edit_section">';

                    print '<div><label>Pfad zu CSV-Datei</label>';
                    print '<input type="text" name="dxim[accounts][' . $account_id . '][path_csv]" value="' . stripslashes($account['path_csv']) . '" /></div>';

                    print '<div><label>Config-Pfad</label>';
                    print '<input type="text" name="dxim[accounts][' . $account_id . '][path_config]" value="' . stripslashes($account['path_config']) . '" /></div>';

                    print '<div><label>Zip - Name</label>';
                    print '<input type="text" name="dxim[accounts][' . $account_id . '][zip_name]" value="' . stripslashes($account['zip_name']) . '" /></div>';

                print '</div>'; // edit seection

                print '<input type="hidden" name="dxim[accounts][' . $account_id . '][user]" value="' . $account['user'] . '" /> ';
                print '<input type="hidden" name="dxim[accounts][' . $account_id . '][type]" value="csv" />';
                print '<input type="hidden" name="dxim[accounts][' . $account_id . '][name]" value="' . $account['name'] . '" />';

                print '</li>';
            }
            print '</ul>';
        }

        if (!empty($accounts) || $l->pro) {
            print '<h3>Einstellungen Bild - Import für CSV Accounts</h3>';

            $v = new Vehicle();
            print '<p>Alle Bilder werden ins Dateisystem importiert.</p>';

            print '<p>';
            print '<strong>Bild - Quellordner (Optional*)</strong><br />';
            print '<span class="small-text">Sollten Ihre Fahrzeugbilder nicht im gleichen Verzeichnis wie die CSV Datei liegen,<br />
                    geben Sie bitte hier den kompletten relativen Pfad an,<br />
                    z.B. /pfad/zu/meinem/Bildordner</span><br />';
            $import_folder = $options['csv_import_options']['import_folder'] ?? null;
            print '<input type="text" class="bigger" name="dxim[csv_import_options][import_folder]" value="' . stripslashes($import_folder) . '" /><br />';
            print '</p>';
        }

    }


    public static function draw_accounts_carshotspro_settings() {
        $cspApiInfo = CarShotsPROApi::getInstance()->getApiInfo();
        $options = Admin::getInstance()->getOptions();

        print '<h2>CarShotsPRO Account</h2>';
        print '<p>Importieren Sie Fahrzeug - Freisteller direkt aus CarShotsPRO<br /><a href="' . CARSHOTSPRO_URL . '" target="_blank">Mehr über CarShotsPRO</a></p>';


        print '<h3 class="margin">Neuen CarShotsPRO - Account verknüpfen</h3>';
        if (isset(Admin::getInstance()->error_on_save['csp_new_app_token'])) {
            print '<div class="wpcs_error">' . Admin::getInstance()->error_on_save['csp_new_app_token'] . '</div>';
        }

        print '<div class="form_item no_border">';
        print '<label class="box">App Token<br /><span class="small-text">Den Token generieren Sie unter <a href="' . CARSHOTSPRO_APP . '/benutzer" target="_blank">' . CARSHOTSPRO_APP . '/benutzer</a></span></label>';
        $token = $options['carshotspro']['new_app_token'] ?? null;
        print '<div>';
        print '<input type="text" name="dxim[carshotspro][new_app_token]" value="' . $token . '" />';
        print '<a href="#" class="button button-primary" id="wpcarsync_form_submit_csp_token">verbinden</a> ';
        print '</div>';
        print '</div>';

        if (!empty($cspApiInfo)) {
            print '<h3 class="margin">Aktive CarShotsPRO Accounts</h3>';
            print '<ul id="dxim_csp_accounts">';
            $msg = 'Achtung!\nMöchten Sie den Account wirklich entfernen?';
            foreach ($cspApiInfo as $token => $info) {
                print "<li>";
                print '<div class="btn"><a href="javascript:wpcs_confirm_action(\'' . $_SERVER['PHP_SELF'] . '?page=' . Admin::$pageSlug . '&deleteCspAccount=' . $token . '&section_slug=accounts&slug=carshotspro\',\'' . $msg . '\');" class="delete_csp_app_token" data-token="' . $token . '"><span class="dashicons dashicons-no-alt"></span></a></div>';
                print '<div class="token">App Token: <br /><strong>' . $token . '</strong></div>';
                print '<div class="account">angemeldet als: <strong>' . $info['name'] . '</strong><br >Credits: <strong>' . $info['credits'] . '</strong></div>';
                print "</li>";
            }
            print '</ul>';
        }

        print '<h3 class="margin">Konfiguration</h3>';
        print "<p>Geben Sie nun an, wie IHre einzelnen Fahrzeuge durch CarShotsPRO verarbeitet werden sollen.</p>";
        print '<ol>';
        print '<li><strong>CarShotsPRO Account</strong><br />Durch welchen CarShotsPRO Account sollen die Bilder bearbeitet werden. (Leer lassen, wenn Sie keine Bearbeitung wünschen)</li>';
        print '<li><strong>Beim Import freistellen?</strong><br />Sollen alle Fahrzeuge automatisch beim Import freigestellt werden?<br />
            Einzelne Fahrzeuge können alternativ nach dem Import manuell ausgewählt werden.</li>';
        print '<li><strong>Winkel Zuordnung</strong><br />Tragen Sie die Bildnummer ein, die den benannten Winkel darstellt.<br />
            Leer lassen, um diesen Winkel zu ignorieren und nicht freizustellen.</li>';
        print '</ol>';

        print '<h4 class="margin">Legen Sie die Konfiguration nun für jeden der folgenden Accounts an:</h4>';
        $angles = CarShotsPRO::getInstance()->getAngles();
        $accounts = Account::getInstance()->get_accounts();
        if (!empty($accounts)) {
            foreach ($accounts as $account) {
                if ($account['type'] == "delete") continue;
                if ($account['type'] == "carcloud" && empty($options['carcloud']['api_key'])) continue;

                $field_name = 'dxim[carshotspro][mapping][' . $account['user'] . ']';
                $account_name = $account['name'] ?? $account['user'];
                $mapping = $options['carshotspro']['mapping'][$account['user']] ?? null;
                print '<div class="form_item">';
                print '<label class="box">Account: <strong>' . $account_name . '</strong><br /><span class="small-text">Typ: ' . $account['type'] . '<br />ID: ' . $account['user'] . '</span></label>';
                print '<div>';

                // CSP Account
                print '<p>CarShotsPRO Account zuweisen<br />';
                print '<select name="' . $field_name . '[csp_account]">';
                print '<option value="#">keine Verknüpfung mit CarShotsPro</option>';
                $csp_account = $mapping['csp_account'] ?? "";
                foreach ($cspApiInfo as $token => $info) {
                    $sel = $token == $csp_account ? "selected" : "";
                    print '<option value="' . $token . '" ' . $sel . '>' . $info['name'] . '</option>';
                }
                print '</select>';
                print '</p>';

                // beim Import freistellen
                $checked = isset($mapping['create_job_on_import']) ? "checked" : "";
                print '<p><input type="checkbox" class="wpcs_checkbox" ' . $checked . ' name="' . $field_name . '[create_job_on_import]" value="1" /> Beim Import freistellen?';
                print '</p>';

                print 'Bild - Winkel - Zuordnung';
                print '<div class="wpcs_admin_csp_angles" style="display:grid; grid-template-columns: 1fr 1fr 1fr; grid-gap:5px;">';
                foreach ($angles as $slug => $angle) {
                    $value = $mapping['angle'][$slug] ?? "";
                    print '<div><input type="number" style="min-width:60px; max-width:60px;" value="' . $value . '" name="' . $field_name . '[angle][' . $slug . ']" /> ' . $angle['label'] . '</div>';
                }
                print '</div>';

                print '</div>';
                print '</div>';
            }
        } else {
            print "Bitte legen Sie zuerst die Import - Accounts (mobile.de oder CSV) an.";
        }

    }

    /**
     * PAYPAL Account
     */
    public static function draw_accounts_paypal_settings() {
        $options = Admin::getInstance()->getOptions();

        print '<h2>Paypal IPN</h2>';
        print '<p>Sie haben die Möglichkeit, Benachrichtigungen über PayPal Zahlungen zu erhalten.</p>';

        // Mode
        $payment = $options['paypal']['mode'] ?? "#";
        print '<div class="form_item">';
            print '<label class="box">Benachrichtigung über Paypal Zahlungen erhalten</label>';
            print '<div>';
            foreach (['#' => 'deaktiviert', 'live' => 'Paypal Live', 'sandbox' => 'Paypal Sandbox'] as $value => $label) {
                $checked = ($payment == $value) ? "checked" : "";
                print $label . ' <input type="radio" name="dxim[paypal][mode]" value="' . $value . '" ' . $checked . ' />&nbsp;&nbsp;&nbsp;';
            }
            print '</div>';
        print '</div>';

        // Live Client ID
        $lcid = $options['paypal']['live_client_id'] ?? "";
        print '<div class="form_item">';
            print '<label class="box">Client ID Live</label>';
            print '<div>';
            print '<input type="text" name="dxim[paypal][live_client_id]" value="' . $lcid . '" /><br />';
            print '<a href="https://www.wpcarsync.de/dokumentation/paypal-integration/" target="_blank">Was ist das?</a>';
            print '</div>';
        print '</div>';

        // Sandbox Client ID
        $scid = $options['paypal']['sandbox_client_id'] ?? "";
        print '<div class="form_item">';
            print '<label class="box">Client ID Sandbox</label>';
            print '<div>';
            print '<input type="text" name="dxim[paypal][sandbox_client_id]" value="' . $scid . '" /><br />';
            print '<a href="https://www.wpcarsync.de/dokumentation/paypal-integration/" target="_blank">Was ist das?</a>';
            print '</div>';
        print '</div>';

        // IPN URL
        print '<div class="form_item">';
            print '<label class="box">Url für Zahlungsbenachrichtigungen (IPN)</label>';
            print '<strong>';
            print trailingslashit(home_url()) . '?wpcs_paypal_ipn_handler=1';
            print '</strong>';
        print '</div>';
    }

    /**
     * @return void
     */
    public static function draw_accounts_carcloud_settings() {
        $options = Admin::getInstance()->getOptions();
        $ld = new LocationData();
        $locations = $ld->getTree();

        print '<h2>Car-Cloud.de Account</h2>';
        print '<p>Vergrößern Sie Ihre Fahrzeugbestände mit ausgewählten Fahrzeugen direkt aus der Cloud<br />
            <a href="' . CARCLOUD_URL . '" target="_blank">Mehr über CarCloud</a></p>';

        print '<div class="form_item">';
            print '<label class="box">Api Key<br /><span class="small-text">Befindet sich in ihrem <a href="' . CARCLOUD_APP . '" target="_blank">CarCloud Account</a></span></label>';
            $api_key = $options['carcloud']['api_key'] ?? null;
            print '<input type="text" name="dxim[carcloud][api_key]" value="' . $api_key . '" />';
        print '</div>';

        print '<div class="form_item">';
            print '<label class="box">Standort<br /><span class="small-text">Welchem Standort sollen die Fahrzeuge zugewiesen werden</span></label>';
            $current_val = $options['carcloud']['location_mapping'] ?? null;
            print self::_location_select($locations, 'dxim[carcloud][location_mapping]', $current_val);
        print '</div>';

        $options['carcloud']['margin'] = $options['carcloud']['margin'] ?? null;
        $options['carcloud']['margin_min'] = $options['carcloud']['margin_min'] ?? null;
        $options['carcloud']['margin_max'] = $options['carcloud']['margin_max'] ?? null;

        $margin = !empty($options['carcloud']['margin']) ? $options['carcloud']['margin'] : 5;
        $margin_min = !empty($options['carcloud']['margin_min']) ? $options['carcloud']['margin_min'] : 300;
        $margin_max = !empty($options['carcloud']['margin_max']) ? $options['carcloud']['margin_max'] : 600;

        print '<div class="form_item">';
            print '<label class="box">Marge<br /><span class="small-text">Ihre Marge pro Fahrzeug</span></label>';
            print 'Marge in % <input type="text" style="width:70px; min-width:70px; margin-right:20px;" name="dxim[carcloud][margin]" value="' . $margin . '" />';
            print '€ min <input type="text" style="width:70px; min-width:70px; margin-right:20px;" name="dxim[carcloud][margin_min]" value="' . $margin_min . '" />';
            print '€ max <input type="text" style="width:70px; min-width:70px;" name="dxim[carcloud][margin_max]" value="' . $margin_max . '" />';
        print '</div>';

        $options['carcloud']['images'] = $options['carcloud']['images'] ?? "links";
        print '<div class="form_item">';
            print '<label class="box">Bilder<br /><span class="small-text">Wie sollen die Bilder importiert werden?</span></label>';
            print '<div class="options">';
            $checked = $options['carcloud']['images'] == "links" ? "checked" : null;
            print '<input type="radio" name="dxim[carcloud][images]" value="links" ' . $checked . ' /> Bildverweise importieren<br />';

            $checked = $options['carcloud']['images'] == "media" ? "checked" : null;
            print '<input type="radio" name="dxim[carcloud][images]" value="media" ' . $checked . ' /> in Mediathek importieren<br />';

            $checked = $options['carcloud']['images'] == "filesystem" ? "checked" : null;
            print '<input type="radio" name="dxim[carcloud][images]" value="filesystem" ' . $checked . ' /> ins Dateisystem importieren<br />';
            print '</div>';
        print '</div>';

        print '<a href="' . Admin::getInstance()->_getAdminPage() . '&wpcs_import=1&account_id=carcloud&section_slug=accounts&slug=carcloud" class="button">Diesen Account jetzt synchronisieren</a>';
    }

    public static function draw_admin_admin_settings() {
        $options = Admin::getInstance()->getOptions();

        // enable debug Mode
        $checked = (isset($options['debug'])) ? 'checked' : null;
        $content = '<input type="checkbox" class="wpcs_checkbox" value="1" name="dxim[debug]" ' . $checked . ' /> aktivieren';
        print self::_draw_option("Debug Modus", $content, "Aktivieren bei Sync Problemen");

        // Import Log
        if (!empty($options['lastSyncTime'])) {
            $content = '<a class="button" href="#" id="toggle_wpcs_import_log">Import-Log anzeigen</a>';
            $content .= '<div id="wpcs_import_log">';
            $content .= '<a href="#" id="wpcs_ext_log_btn">Erweiterte Ansicht</a>';
            $import_id = Import::getInstance()->get_current_import()['import_id'] ?? null;
            $logging = file_get_contents(WPCS_UPLOADS_DIR . '/imports/' . $import_id . '/import.log');
            $content .= nl2br($logging);
            $content .= '</div>';
        } else $content = "Keine Daten zum letzten Import vorhanden";
        print self::_draw_option("Import Log", $content, "Details zum letzten Import");

        // Import starten
        print self::_draw_option("Import starten", '<a class="button" href="' . Admin::getInstance()->_getAdminPage() . '&wpcs_import=1&section_slug=admin&slug=admin" onClick="wpcs_settings_page._showLoading(\'Starte Import... einen Moment bitte\');">Import starten</a>', "Startet die Synchronisierung der Fahrzeuge");

        $cMsg = 'Achtung!\nDadurch werden beim nächsten Import alle Fahrzeuge geupdated';
        $content = '<a class="button" href="javascript:wpcs_confirm_action(\'' . $_SERVER['PHP_SELF'] . '?page=' . Admin::$pageSlug . '&wpcs_admin_reset_import_hashes=1&section_slug=admin&slug=admin\',\'' . $cMsg . '\');">Import jetzt zurücksetzen</a>';
        print self::_draw_option("Import zurücksetzen", $content, "Erzwingt einen vollständig neuen Import");

        $cMsg = 'Dies kann einen Moment dauern!';
        $content = '<a class="button" href="javascript:wpcs_confirm_action(\'' . $_SERVER['PHP_SELF'] . '?page=' . Admin::$pageSlug . '&resetFilter=1&section_slug=admin&slug=admin\',\'' . $cMsg . '\');">Filtertabelle jetzt zurücksetzen</a>';
        print self::_draw_option("Filtertabelle neu aufbauen", $content, "Tun Sie dies, wenn Sie Fehler beim Filtern der Fahrzeuge bemerken");

        $cMsg = 'Bereinigung jetzt starten?';
        $content = '<a class="button" href="javascript:wpcs_confirm_action(\'' . $_SERVER['PHP_SELF'] . '?page=' . Admin::$pageSlug . '&wpcs_admin_cleanup_filesystem=1&section_slug=admin&slug=admin\',\'' . $cMsg . '\');">jetzt bereinigen</a>';
        print self::_draw_option('Bilderorder bereinigen', $content);

        // Settings exportieren
        print self::_draw_option("Settings exportieren/importieren",'<a class="button" href="' . Admin::getInstance()->_getAdminPage() . '&wpcs_settings_page=import_export&import_settings=1">Settings importieren/exportieren</a>');

        // Plugin zurück setzen
        print self::_draw_option("Plugin zurücksetzen",'<a class="button" style="float:right;" onclick="if(!confirm(\'Wirklich zurücksetzen?\')) return false;" href="' . Admin::getInstance()->_getAdminPage() . '&reset=1&section_slug=admin&slug=admin">Plugin zurücksetzen</a>');
    }

    /**
     * Allgemeine Einstellungen
     */
    public static function draw_settings_general_settings() {
        $options = Admin::getInstance()->getOptions();
        print '<h2>Allgemeine Einstellungen</h2>';

        // Autohaus Logo Wert bleibt erhalten, kann aber nicht mehr verändert werden,
        // da logo durch Standort Logos abgelöst wurde
        $url = $options['autohaus_logo'] ?? null;
        if (!empty($url)) {
            print '<input type="hidden" name="dxim[autohaus_logo]" value="' . $url . '" />';
        }

        print '<div class="form_item">';
            $email = isset($options['email']) && !empty($options['email']) ? $options['email'] : get_option('admin_email');
            print '<label class="box">zentrale E-Mail Adresse<br /><span class="small-text">Nicht standortspezifische E-Mails empfangen, Standard Absender</span></label> <input type="text" name="dxim[email]" value="' . $email . '" />';
        print '</div>';

        print '<h3>Import</h3>';
        print '<div class="form_item">';
            print '<label class="box">Automatischer Import</label>';
            if (!isset($options['cron'])) $options['cron'] = 'daily';

            print '<div>';
            foreach (array('#' => 'aus/manuell', 'daily' => 'täglich', '6_hours' => 'alle 6 Stunden', 'hourly' => 'stündlich') as $value => $label) {
                #print '<label for="dxim[cron]">' . $label . '</label> ';
                $checked = ($options['cron'] == $value) ? 'checked' : null;
                print '<input type="radio" class="wpcs_sync_option wpcs_checkbox" name="dxim[cron]" value="' . $value . '" ' . $checked . ' /> '.$label."<br />";
            }

            $display_cron = ($options['cron'] == "#") ? "block" : 'none';
            print '<div class="sync_cron_notice" style="display:' . $display_cron . ';">';
            print "<p><strong>Hinweis</strong></p>";
            print "Um den Import immer zu einer festen Uhrzeit zu starten, wählen sie die Option <strong>aus/cron</strong><br />";
            print "Richten sie auf Ihrem Server einen cronjob ein und rufen sie die URL <strong>" . trailingslashit(home_url()) . "?wpcs_import=1</strong> zur gewünschten Uhrzeit auf.";
            print '</div>';


            print "<script>jQuery(function(){
                            jQuery('.wpcs_sync_option').click(function() {
                                if(jQuery(this).val() == '#') jQuery('.sync_cron_notice').show();
                                else jQuery('.sync_cron_notice').hide();
                            });
                        });</script>";
            print '</div>';
        print '</div>';


        $check_envkv = $options['import']['check_envkv'] ?? 1;
        print '<div class="form_item">';
            print '<label class="box">Envkv Pflichtfelder beim Import prüfen<br /><span class="small-text">Fahrzeuge EZ älter als 8 Monate oder mehr als 1000km werden immer importiert</span></label>';
            print '<div>';
                $checked = boolval($check_envkv) ? "checked" : "";
                print '<input type="radio" name="dxim[import][check_envkv]" '.$checked.' class="wpcs_checkbox" value="1" /> fehlerhafte Fahrzeuge nicht importieren <strong>(empfohlen)</strong><br />';
                $checked = !boolval($check_envkv) ? "checked" : "";
                print '<input type="radio" name="dxim[import][check_envkv]" '.$checked.' class="wpcs_checkbox" value="0" /> alle Fahrzeuge importieren';

                print '<p class="wpcs_info"><strong>Info!</strong> Für bereits importierte Fahrzeuge wird der Filter nur wirksam, wenn sie unter Admin -> "Import zurücksetzen" klicken</p>';
            print '</div>';
        print '</div>';

        print '<h3>Preise</h3>';

        print '<div class="form_item">';
            print '<label class="box">Brutto / Netto</label>';
            print '<select name="dxim[vehicle_price_display]">';
            foreach (array('brutto' => 'Brutto', 'netto' => 'Netto', 'both' => 'Brutto & Netto') as $value => $label) {
                $sel = ($options['vehicle_price_display'] == $value) ? 'selected' : null;
                print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
            }
            print '</select> ';
        print '</div>';

        print '<div class="form_item">';
            print '<label class="box">Rate darstellen</label>';
            print '<select name="dxim[vehicle_price_rate]">';
            foreach (array('both' => 'Finanzierung & Leasing', 'financing' => 'nur Finanzierungsrate', 'leasing' => 'nur Leasingrate', 'none' => 'keine Raten anzeigen') as $value => $label) {
                $sel = ($options['vehicle_price_rate'] == $value) ? 'selected' : null;
                print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
            }
            print '</select> ';
        print '</div>';

        $options['vehicle_price_src'] = $options['vehicle_price_src'] ?? "consumer_price";
        print '<div class="form_item">';
            print '<label class="box">Preis Quelle</label><span class="small-text"></span>';
            print '<select name="dxim[vehicle_price_src]">';
            foreach (array('consumer_price' => 'Endkundenpreis', 'dealer_price' => 'Händlerpreis (Fallback Endkundenpreis)') as $value => $label) {
                $sel = ($options['vehicle_price_src'] == $value) ? 'selected' : null;
                print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
            }
            print '</select> ';
        print '</div>';

        print '<div class="form_item">';
            print '<label class="box">Kommastellen anzeigen</label>';
            print '<select name="dxim[vehicle_price_decimal]">';
            foreach (array('0' => 'nein', '1' => 'ja') as $value => $label) {
                $sel = ($options['vehicle_price_decimal'] == $value) ? 'selected' : null;
                print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
            }
            print '</select>';
        print '</div>';

        print "<h3>SEO</h3>";

        // Archive Texte
        print '<div class="form_item">';
        print '<label class="box">Marken- & Modelltexte in Börse anzeigen</label>';
        $archive_description = $options['seo']['vehicle_archive_description'] ?? 0;
        $checked = $archive_description > 0 ? "checked" : "";
        print 'Ja <input ' . $checked . ' class="wpcs_checkbox" type="radio" value="1" name="dxim[seo][vehicle_archive_description]" />&nbsp;&nbsp;&nbsp;';
        $checked = $archive_description < 1 ? "checked" : "";
        print 'Nein <input ' . $checked . ' class="wpcs_checkbox" type="radio" value="0" name="dxim[seo][vehicle_archive_description]" />';
        print '</div>';

        // Google Schema
        print '<div class="form_item">';
        print '<label class="box">Schema.org Auszeichnungen aktivieren</label>';
        $schema = $options['seo']['schema'] ?? 0;
        $checked = $schema > 0 ? "checked" : "";
        print 'Ja <input ' . $checked . ' class="wpcs_checkbox" type="radio" value="1" name="dxim[seo][schema]" />&nbsp;&nbsp;&nbsp;';
        $checked = $schema < 1 ? "checked" : "";
        print 'Nein <input ' . $checked . ' class="wpcs_checkbox" type="radio" value="0" name="dxim[seo][schema]" />';
        print '</div>';


        print '<h3>Emissionsangaben</h3>';
        print '<div class="form_item">';
        print '<label class="box">Anzeige Fahrzeug-Einzelansicht</label>';
        print '<select name="dxim[vehicle_emission_display]">';
        foreach (array('all' => 'alle Angaben zeigen', 'available' => 'nur vorhandene Werte anzeigen') as $value => $label) {
            $sel = ($options['vehicle_emission_display'] == $value) ? 'selected' : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select> ';
        print '</div>';


        print '<h3>Fahrzeug - Bilder</h3>';

        print '<div class="form_item">';
        print '<label class="box">Fahrzeugbild Seitenverhältnis</label>';
        print '<select name="dxim[general_vehicle_image_ratio]">';
        foreach (array('16_9' => '16:9', '4_3' => '4:3') as $value => $label) {
            $sel = ($options['general_vehicle_image_ratio'] == $value) ? 'selected' : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select>';
        print '</div>';


        print '<div class="form_item">';
        $image_usage = $options['general_vehicle_image_usage'] ?? null;
        print '<label class="box">Bildreihenfolge</label>';
        print '<div>';
        print '<input type="text" value="' . $image_usage . '" name="dxim[general_vehicle_image_usage]" /><br />';
        print '<span>Geben Sie die Bildnummern per Komma separiert in der Reihenfolge an, in der die Fahrzeugbilder dargestellt werden sollen.<br />
                Die erste Bildnummer zählt dabei für alle Vorschaubilder, lassen Sie einzelne Bildnummer weg um diese Bilder nicht anzuzeigen<br />
                z.B: "2,4,6" stellt das 2., 4. und 6. Bild des Fahrzeuges dar.<br />
                Leer lassen um alle Bilder in Standard - Reihenfolge zu zeigen</span><br />';
        print '</div>';
        print '</div>';

        print '<h3>Fahrzeug - Exposé</h3>';
        if (!isset($options['vehicle_expose'])) $options['vehicle_expose'] = "pdf";

        print '<div class="form_item">';
        print '<label class="box">PDF - Export</label>';
        print '<div>';
        $checked = ($options['vehicle_expose'] == "pdf" || empty($options['vehicle_expose'])) ? "checked" : null;
        print 'PDF - Export <input class="wpcs_checkbox" type="radio" name="dxim[vehicle_expose]" value="pdf" ' . $checked . ' />&nbsp;&nbsp;&nbsp;';

        $checked = ($options['vehicle_expose'] == "print") ? "checked" : null;
        print 'Druckoptimierte Webseite <input class="wpcs_checkbox" type="radio" name="dxim[vehicle_expose]" value="print" ' . $checked . ' />';
        print '</div>';
        print '</div>';

        print '<div class="form_item">';
        print '<label class="box">Bilder einbetten<br /><span class="small-text">Alle Bilder benötigen mehr Ram auf dem Server</span></label>';
        print '<div>';
        $expose_images = $options['vehicle_expose_images'] ?? 'single';
        $checked = $expose_images == "single" ? "checked" : null;
        print 'nur Hauptbild <input class="wpcs_checkbox" type="radio" name="dxim[vehicle_expose_images]" value="single" ' . $checked . ' />&nbsp;&nbsp;&nbsp;';
        $checked = $expose_images == "all" ? "checked" : null;
        print 'alle Bilder einbetten <input class="wpcs_checkbox" type="radio" name="dxim[vehicle_expose_images]" value="all" ' . $checked . ' />';
        print '</div>';
        print '</div>';


        print '<h3>globale Filter aktivieren</h3>';
        print '<div class="form_item">';
        print '<label class="box">Post Type<br /><span class="small-text">Aktivieren Sie die Beitragstypen, bei denen Sie die globalen Filter benötigen.</span></label>';
        print '<div>';
        if (!isset($options['general_global_filter_activation'])) {
            $options['general_global_filter_activation'][] = 'page';
        }

        foreach (get_post_types([], 'objects') as $pt) {
            print '<div class="wpcs_admin_post_type_item">';
            $checked = in_array($pt->name, $options['general_global_filter_activation']) ? 'checked' : null;
            print '<input type="checkbox" value="' . $pt->name . '" ' . $checked . ' name="dxim[general_global_filter_activation][]">';
            print '<label for="general_global_filter_activation">' . $pt->label . '</label>';
            print '</div>';
        }
        print '</div>';
        print '</div>';

        // CREDITS
        print '<div class="form_item">';
        print '<label class="box">Plugin Credits<br /><span class="small-text">Helfen Sie uns mit einem Backlink</span></label>';
        print '<div>';
        print '<p>Wenn Ihnen das Plugin gefällt, würden wir uns wahnsinnig auf einen Link zu unserer Seite freuen.<br />
                Je mehr Menschen das Plugin nutzen, desto mehr Anregungen und Wünsche können wir sammeln und das Plugin stetig verbessern.</p>';

        print '<p>Setzen Sie doch z.B. den folgenden Link in Ihr Impressum: <strong><br />&lt;a href="' . DXIM_PluginWebAddress . '" title="Mobile.de Wordpress Plugin WPCarSync by ' . DXIM_PluginWebsiteName . '" target="_blank"&gt;Mobile.de Wordpress Plugin WPCarSync&lt;/a&gt;</strong></p>';

        if (empty($options['plugin_feature_page'])) $options['plugin_feature_page'] = 'impressum';
        print "Plugin Hinweis automatisch aktivieren auf Seite:<br />";
        print '<select name="dxim[plugin_feature_page]">';
        print '<option value="#">keine Seite gewählt</option>';
        foreach (get_pages() as $tmp) {

            $sel = ($tmp->post_name == $options['plugin_feature_page']) ? 'selected' : null;
            print '<option value="' . $tmp->post_name . '" ' . $sel . '>' . $tmp->post_title . '</option>';
        }
        print '</select>';

        print '</div>';
        print '</div>';

    }

    /**
     * @return void
     */
    public static function draw_settings_vehicle_archive_settings() {
        $options = Admin::getInstance()->getOptions();

        print '<h2>Fahrzeug - Übersicht</h2>';
        print '<h3>Optionen zu Template & Darstellung</h3>';

        $current_archive_page = $options['vehicle_archive_page'] ?? 0;
        $current_archive_page = intval($current_archive_page);
        if ($current_archive_page > 0) {
            print '<div class="form_item">';
            print '<label class="box">Seite für Fahrzeug-Archiv<br /><span class="small-text">Diese Variante wird nicht mehr empfohlen</span></label>';
            print '<div>';
            $li = '<select name="dxim[vehicle_archive_page]">';
            $li .= '<option value="#">keine Seite gewählt</option>';
            if (!isset($options['vehicle_archive_page'])) $options['vehicle_archive_page'] = null;
            foreach (get_pages() as $tmp) {
                $sel = ($tmp->ID == $options['vehicle_archive_page']) ? 'selected' : null;
                $li .= '<option value="' . $tmp->ID . '" ' . $sel . '>' . $tmp->post_title . '</option>';
            }
            $li .= '</select>';
            print $li;
            print '</div>';
            print '</div>';
        } else {
            print '<div class="form_item">';
            print '<label class="box">Fahrzeug-Archiv URL</label>';
            print '<div>';
            $slug = $options['vehicle_post_type_slug'] ?? "fahrzeugarchiv";
            print '<input type="text" name="dxim[vehicle_post_type_slug]" value="' . $slug . '" /><br />';
            print '</div>';
            print '</div>';

            // Template Marke
            print '<div class="form_item">';
            print '<label class="box">Template Fahrzeug-Archiv</label>';
            print '<div>';
            $template = $options['vehicle_archive_template'] ?? "wpcs";
            $checked = $template == "wpcs" ? "checked" : "";
            print 'WPCarSync Archiv Template <input type="radio" name="dxim[vehicle_archive_template]" value="wpcs" ' . $checked . ' />';
            $checked = $template == "theme" ? "checked" : "";
            print 'Theme Default Template <input type="radio" name="dxim[vehicle_archive_template]" value="theme" ' . $checked . ' />';
            print '</div>';
            print '</div>';
        }

        // Layout Archiv
        print '<div class="form_item">';
        print '<label class="box">Layout Archiv-Seite</label>';
        print '<div>';
        print '<select name="dxim[default_layout_archive]">';
        foreach (array('layout_1' => 'Suchformular rechts (Template: archive_1)', 'layout_2' => 'volle Breite, Suchformular oben (Template archive_2)') as $value => $label) {
            $sel = ($value == $options['default_layout_archive']) ? 'selected' : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select>';
        print '</div>';
        print '</div>';


        // Posts per page
        print '<div class="form_item">';
        print '<label class="box">Anzahl Fahrzeuge pro Seite</label>';
        print '<div>';
        $posts_per_page = @intval($options['posts_per_page']);
        if ($posts_per_page < 1) $posts_per_page = 20;
        print '<input type="text" name="dxim[posts_per_page]" value="' . $posts_per_page . '" /><br />';
        print '</div>';
        print '</div>';

        // Seiten
        print '<div class="form_item">';
        print '<label class="box">Navigation durch Archiv-Seiten<br /><span class="small-text">Wie möchten Sie die Suchergebnissseiten blättern?</span></label>';
        print '<div>';
        print '<select name="dxim[archive_page_navigation]">';
        if (!isset($options['archive_page_navigation'])) $options['archive_page_navigation'] = "pagination";
        foreach (array('pagination' => 'Seitennummerierung', 'button' => 'Button', 'infinite_scroll' => 'Infinite Scroll') as $value => $label) {
            $sel = ($value == $options['archive_page_navigation']) ? 'selected' : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select>';
        print '</div>';
        print '</div>';

        // Darstellung Archiv
        if (empty($options['archive_vehicle_layout'])) $options['archive_vehicle_layout'] = 'grid';

        print '<div class="form_item">';
        print '<label class="box">Standarddarstellung der Fahrzeuge</label>';
        print '<div>';
        print '<select name="dxim[archive_vehicle_layout]" id="archive_vehicle_layout_select">';
        foreach (array('rows' => 'Zeilen', 'grid' => 'Raster') as $value => $label) {
            $sel = ($value == $options['archive_vehicle_layout']) ? 'selected' : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select>';

        if (empty($options['archive_vehicle_grid'])) $options['archive_vehicle_grid'] = 3;
        $grid_display = ($options['archive_vehicle_layout'] == "grid") ? 'inline-block' : 'none';

        print ' <label style="display:' . $grid_display . '" for="archive_vehicle_grid" id="archive_vehicle_grid_label">Anzahl Spalten</label> 
                    <select style="display:' . $grid_display . '" id="archive_vehicle_grid_select" name="dxim[archive_vehicle_grid]">';

        foreach (array(1, 2, 3, 4) as $value) {
            $sel = ($value == $options['archive_vehicle_grid']) ? 'selected' : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $value . '</option>';
        }
        print '</select>';

        print "<script>jQuery(function() {
                        jQuery('#archive_vehicle_layout_select').change(function(){
                            if(jQuery(this).val() == 'grid') {
                                jQuery('#archive_vehicle_grid_label').show();
                                jQuery('#archive_vehicle_grid_select').show();
                            }
                            else {
                                jQuery('#archive_vehicle_grid_label').hide();
                                jQuery('#archive_vehicle_grid_select').hide();
                            }
                        });	
                    });</script>";
        print '</div>';
        print '</div>';

        // Bildquelle
        print '<div class="form_item">';
        $image_type = $options['archive_image_type'] ?? null;
        print '<label class="box">Bildquelle<br /><span class="small-text">Welcher Bilder sollen standardmäßig angezeigt werden?</span></label>';
        print '<div>';
        print '<select name="dxim[archive_image_type]">';
        foreach (['default' => "Standardbilder", 'csp' => 'CarShotsPRO Bilder (falls vorhanden)', 'csp_crop' => 'CarShotsPRO Freisteller (falls vorhanden)'] as $value => $label) {
            $sel = $image_type == $value ? " selected" : null;
            print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
        }
        print '</select>';
        print '</div>';
        print '</div>';

        // Sortierung
        print '<div class="form_item">';
        print '<label class="box">Standard-Sortierung der Fahrzeuge auf Archivseite</label>';
        print '<div>';
        $z = 0;
        // order by
        $options['archive_default_orderby'] = $options['archive_default_orderby'] ?? "price";
        foreach (VehicleFilter::getOrderFilter() as $key => $tmp) {
            if ($z != 0) print '&nbsp;&nbsp;&nbsp;';
            $checked = ($options['archive_default_orderby'] == $key) ? 'checked' : null;
            print '<input type="radio" name="dxim[archive_default_orderby]" value="' . $key . '" ' . $checked . '> ' . $tmp['label'] . ' ';
            $z++;
        }

        print '<br />';

        // order
        $z = 0;
        $options['archive_default_order'] = $options['archive_default_order'] ?? "asc";
        foreach (array('asc' => 'aufsteigend', 'desc' => 'absteigend') as $key => $label) {
            if ($z != 0) print '&nbsp;&nbsp;&nbsp;';
            $checked = ($options['archive_default_order'] == $key) ? 'checked' : null;
            print '<input type="radio" name="dxim[archive_default_order]" value="' . $key . '" ' . $checked . '> ' . $label;
            $z++;
        }
        print '</div>';
        print '</div>';


        // Platzhalter Bild
        print '<div class="form_item">';
        print '<label class="box">Platzhalterbild wenn Fahrzeugbild fehlt</label>';
        print '<div>';
        $placeholder_image = $preview_src = $options['placeholder_image'] ?? null;
        if (empty($preview_src)) $preview_src = DXIM_PluginUrl . '/img/wpcarsync_no_image.gif';
        print '<img class="placeholder_image" src="' . $preview_src . '" /><br />';
        print 'Eigene Bild-Url angeben: (optional)<br />';
        print '<input type="text" class="bigger" name="dxim[placeholder_image]" value="' . $placeholder_image . '" /><br />';
        print '</div>';
        print '</div>';


        // Archive Fields
        print '<h3>Definition der Felder für Fahrzeug-Übersicht</h3>';
        print '<p><strong>An dieser Stelle definieren Sie, welche Felder am Fahrzeug angezeugt werden sollen.</strong><br />
        Suchen Sie dazu nach einem Feldnamen und klicken sie auf ein Feld aus der Ergebnisliste, um es hinzuzufügen. Passen sie die Reihenfolge der Felder nach Belieben an.</p>';

        // get all fields
        $default_fields = Vehicle::getProperties();
        $active_fields_archive_db = $options['active_fields_archive'] ?? [];

        // db fields
        $active_fields_archive = [];
        if (!empty($active_fields_archive_db)) {
            foreach ($active_fields_archive_db as $key) {
                $active_fields_archive[$key] = $default_fields[$key]['label'];
            }
        }
        // default fields
        else {
            foreach ($default_fields as $key => $tmp) {
                $default_archive = $tmp['default_archive'] ?? 0;
                if (empty($options['active_fields_archive']) && $default_archive) $active_fields_archive[$key] = $default_fields[$key]['label'];
            }
        }

        print '<div id="archive_fields_wrapper">';

            print '<div class="wpcs_admin_field_search" data-target="#active_fields_archive">';
                print '<input type="text" />';
                print FilterForm::getInstance()->drawAjaxLoading();
                print '<ul></ul>';
            print '</div>';

            print '<div class="active_fields_wrapper">';
                print '<h4>aktive Felder</h4>';
                print '<ul id="active_fields_archive" class="wpcarsync_connected_fields wpcarsync_connected_fields_archive">';
                foreach ($active_fields_archive as $key => $label) {
                    print '<li data-key="' . $key . '" data-value="' . $label . '"><span class="dashicons dashicons-no-alt"></span> ' . $label . '</li>';
                }
                print '</ul>';
            print '</div>';

            print '<div class="active_fields_archive_form_values_wrapper"></div>';
        print '</div>';

    }

    public static function draw_settings_vehicle_single_settings() {
        $options = Admin::getInstance()->getOptions();

        print '<h2>Fahrzeug - Einzelansicht</h2>';

        // Seitentemplate Single
        $single_page = $options['vehicle_single_page'] ?? 0;
        $single_page = intval($single_page);
        if ($single_page > 0) {
            print '<div class="form_item">';
                print '<label class="box">Seite für Fahrzeug-Einzelansicht<br /><span class="small-text">Diese Variante wird nicht mehr empfohlen</span></label>';
                print '<div>';
                    print '<select name="dxim[vehicle_single_page]">';
                    print '<option value="#">keine Seite gewählt</option>';
                    if (!isset($options['vehicle_single_page'])) $options['vehicle_single_page'] = null;
                    foreach (get_pages() as $tmp) {
                        $sel = ($tmp->ID == $options['vehicle_single_page']) ? 'selected' : null;
                        print '<option value="' . $tmp->ID . '" ' . $sel . '>' . $tmp->post_title . '</option>';
                    }
                    print '</select>';
                print '</div>';
            print '</div>';
        } else {
            // Template Marke
            print '<div class="form_item">';
                print '<label class="box">Template Fahrzeug-Einzelansicht</label>';
                print '<div>';
                    $template = $options['vehicle_single_template'] ?? "wpcs";
                    $checked = $template == "wpcs" ? "checked" : "";
                    print 'WPCarSync Single Template <input type="radio" name="dxim[vehicle_single_template]" value="wpcs" ' . $checked . ' />';
                    $checked = $template == "theme" ? "checked" : "";
                    print 'Theme Default Template <input type="radio" name="dxim[vehicle_single_template]" value="theme" ' . $checked . ' />';
                print '</div>';
            print '</div>';
        }

        print '<h3>Fahrzeug - Bildergalerie</h3>';

        // Lightbox
        print '<div class="form_item">';
            print '<label class="box">Lightbox aktivieren<br /><span class="small-text">Erlaubt dem Besucher die Fahrzeugbilder in der Einzelansicht zu vergrößern</span></label>';
            print '<div>';
                print '<select name="dxim[single_vehicle_lightbox]">';
                foreach (['on' => 'aktiviert', 'off' => 'deaktiviert'] as $value => $label) {
                    $sel = ($options['single_vehicle_lightbox'] == $value) ? "selected" : null;
                    print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
                }
                print '</select>';
            print '</div>';
        print '</div>';


        // Ausrichtung
        print '<div class="form_item">';
            print '<label class="box">Ausrichtung der Fahrzeug-Thumbnails<br /><span class="small-text">Horizontal = Horizontaler Slider unterhalb des Fahrzeugbildes, Vertikal = Vertikaler Slider neben dem Fahrzeugbild</span></label>';
            print '<div>';
                print '<select name="dxim[single_vehicle_image_alignment]">';
                $selected = $options['single_vehicle_image_alignment'] ?? "horizontal";
                foreach (['horizontal' => 'horizontal', 'vertical' => 'vertikal'] as $value => $label) {
                    $sel = ($selected == $value) ? "selected" : null;
                    print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
                }
                print '</select>';
            print '</div>';
        print '</div>';

        // Bildquelle
        print '<div class="form_item">';
            $image_type = $options['single_vehicle_image_type'] ?? null;
            print '<label class="box">Bildquelle<br /><span class="small-text">Welcher Bilder sollen standardmäßig angezeigt werden?</span></label>';
            print '<div>';
                print '<select name="dxim[single_vehicle_image_type]">';
                foreach (['default' => "Standardbilder", 'csp' => 'CarShotsPRO Bilder (falls vorhanden)', 'csp_crop' => 'CarShotsPRO Freisteller (falls vorhanden)'] as $value => $label) {
                    $sel = $image_type == $value ? " selected" : null;
                    print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
                }
                print '</select>';
            print '</div>';
        print '</div>';


        // Bilder pro Slide
        $selected = $options['single_vehicle_image_per_slide'] ?? 4;
        print '<div class="form_item">';
            print '<label class="box">Anzahl Thumbnails pro Slide</label>';
            print '<div>';
                print '<input type="number" name="dxim[single_vehicle_image_per_slide]" value="' . $selected . '">';
            print '</div>';
        print '</div>';

        // Ausrichtung
        print '<div class="form_item">';
            print '<label class="box">Position der Navigationspfeile</label>';
            print '<div>';
                print '<select name="dxim[single_vehicle_image_arrows]">';
                $selected = $options['single_vehicle_image_arrows'] ?? "wrapper";
                foreach (['image' => 'im Bild-Container', 'thumbnails' => 'im Thumbnail-Container', 'wrapper' => 'im Haupt-Container'] as $value => $label) {
                    $sel = ($selected == $value) ? "selected" : null;
                    print '<option value="' . $value . '" ' . $sel . '>' . $label . '</option>';
                }
                print '</select>';
            print '</div>';
        print '</div>';

        // Features
        print '<div class="form_item">';
            print '<label class="box">Darstellung Sonderausstattung</label>';
            print '<div>';
                $value = $options['single_vehicle_features_layout'] ?? "columns";
                $checked = ($value == "grouped") ? "checked" : null;
                print '<input type="radio" ' . $checked . ' name="dxim[single_vehicle_features_layout]" value="grouped" /> gruppiert in Kategorien<br />';
                $checked = ($value == "columns") ? "checked" : null;
                print '<input type="radio" ' . $checked . ' name="dxim[single_vehicle_features_layout]" value="columns" /> sortiert in mehreren Spalten';
            print '</div>';
        print '</div>';

        $tmp = WPCS_DEMO ? "(PRO)" : "";
        print '<h3>Ansprechpartner ' . $tmp . '</h3>';
        print '<div class="form_item">';
        print '<label class="box">Darstellung Ansprechpartner<br /><span class="small-text">Shortcode: [WPCS_persons_vehicle]</span></label>';

        if (!WPCS_PRO) :
            print '<div>Pro Feature</div>';

        else :
            print '<ul>';

            $same_location = $options['theme']['vehicle']['person_query']['location'] ?? 1;
            print '<li><label>nur Personen vom gleichen Standort wie Fahrzeug anzeigen</label><br />';
            $checked = ($same_location == 1) ? "checked" : null;
            print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][vehicle][person_query][location]" value="1" />';
            $checked = ($same_location == 0) ? "checked" : null;
            print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][vehicle][person_query][location]" value="0" />';
            print '</li>';

            $d = new DivisionData();
            $divisions = $d->getAll();
            print '<li><label>Person muss in folgender Abteilung sein</label><br />
                <select name="dxim[theme][vehicle][person_query][division]">';
            print '<option value="0">alle Abteilungen</option>';
            $current_division = $options['theme']['vehicle']['person_query']['division'] ?? null;
            foreach ($divisions as $division) {
                $sel = ($current_division == $division->term_id) ? "selected" : null;
                print '<option value="' . $division->term_id . '" ' . $sel . '>' . $division->name . '</option>';
            }
            print '</select>';
            print '</li>';

            print '<li><label>Anzahl Personen anzeigen</label><br />';
            $count = $options['theme']['vehicle']['person_query']['count'] ?? 1;
            print '<input type="number" name="dxim[theme][vehicle][person_query][count]" value="' . $count . '" />';
            print '</li>';

            print '<li><label>Reihenfolge zufällig</label><br />';
            $random = $options['theme']['vehicle']['person_query']['random'] ?? 1;
            $checked = ($random == 1) ? "checked" : null;
            print 'Ja <input type="radio" ' . $checked . ' name="dxim[theme][vehicle][person_query][random]" value="1" />';
            $checked = ($random == 0) ? "checked" : null;
            print 'Nein <input type="radio" ' . $checked . ' name="dxim[theme][vehicle][person_query][random]" value="0" />';
            print '</li>';
            print '</ul>';

        endif;

        print '</div>';


        // Darstellung der Felder im Template
        print '<h3>Ausgabe der Fahrzeugdaten definieren</h3>';
        print '<div id="admin_fields_wrapper">';
            print '<p><strong>Welche Fahrzeugdaten möchsten Sie in der Fahrzeug-Einzelansicht anzeigen?</strong><br />
                    Ziehen Sie die Felder, welche sie im Template anzeigen wollen, aus der Liste der verfügbaren inaktiven Felder (orange)<br />
                    in die Liste der aktiven Felder (grün) und passen sie die Reihenfolge entsprechend an.</p>';

            // ---------------------------------------------------------

            // get all fields
            $default_fields = Vehicle::getProperties();
            $active_fields_db = $options['active_fields_single'] ?? [];

            // db fields
            $active_fields = [];
            if (!empty($active_fields_db)) {
                foreach ($active_fields_db as $key) {
                    $active_fields[$key] = $default_fields[$key]['label'];
                }
            }
            // default fields
            else {
                foreach ($default_fields as $key => $tmp) {
                    $is_default = $tmp['default_single'] ?? 0;
                    if (empty($active_fields_db) && $is_default) $active_fields[$key] = $default_fields[$key]['label'];
                }
            }

            print '<div id="single_fields_wrapper">';

                print '<div class="wpcs_admin_field_search" data-target="#active_fields_single">';
                    print '<input type="text" />';
                    print FilterForm::getInstance()->drawAjaxLoading();
                    print '<ul></ul>';
                print '</div>';

                print '<div class="active_fields_wrapper">';
                    print '<h4>aktive Felder</h4>';
                    print '<ul id="active_fields_single" class="wpcarsync_connected_fields wpcarsync_connected_fields_archive">';
                    foreach ($active_fields as $key => $label) {
                        print '<li data-key="' . $key . '" data-value="' . $label . '"><span class="dashicons dashicons-no-alt"></span> ' . $label . '</li>';
                    }
                    print '</ul>';
                print '</div>';

                print '<div class="active_fields_single_form_values_wrapper"></div>';

            print '</div>';

        print '</div>'; // admin fields wrapper

    }

    /**
     * Suchaufträge
     */
    public static function draw_settings_searchrequests_settings() {
        $options = Admin::getInstance()->getOptions();

        print '<h2>Suchaufträge</h2>';

        print '<p>Findet ein Kunde nicht das gewünschte Fahrzeug, kann er einen Suchauftrag speichern.<br />
            Sobald das Fahrzeug nach einem Import verügbar ist, wird der Kunde vom System per E-Mail benachrichtigt</p>';

        print '<h4>Konfiguration</h4>';
        print '<div class="form_item">';
        $checked = isset($options['vsr_notify_dealer']) ? 'checked' : null;
        print '<label class="box">Autohaus Benachrichtigen<br /><span class="small-text">E-Mail über erstellten Suchauftrag an Autohaus senden</span></label> <input type="checkbox" name="dxim[vsr_notify_dealer]" ' . $checked . ' value="1" />';
        print '</div>';

        print '<h4>1) Suchanfrage speichern</h4>';
        print '<ul>';
        print '<li>Verwenden Sie den Shortcode <strong>[WPCS_vsr_save]</strong> auf einer Seite, auf der sich ein Filterformular befindet, um dem Besucher die Möglichkeit zu geben, die aktuelle Suche zu speichern</li>';
        print '</ul>';

        print '<h4>2) Suchanfrage verwalten</h4>';
        print '<ol>';
        print '<li>Legen Sie eine neue Seite an, auf der der Besucher seine Aufträge einsehen und verwalten kann.</li>';
        print '<li>Fügen Sie der neu angelegten Seite den Shortcode [WPCS_vsr_list] hinzu.</li>';
        $li = "<li>Geben Sie hier die angelegte Seite an, der Besucher erhält dann per E-Mail Zugriff auf diese Seite.<br />";
        $li .= '<select name="dxim[vsr_page]">';
        $li .= '<option value="#">keine Seite gewählt</option>';
        if (!isset($options['vsr_page'])) $options['vsr_page'] = null;
        foreach (get_pages() as $tmp) {
            $sel = ($tmp->ID == $options['vsr_page']) ? 'selected' : null;
            $li .= '<option value="' . $tmp->ID . '" ' . $sel . '>' . $tmp->post_name . '</option>';
        }
        $li .= '</select>';
        $li .= '</li>';
        print $li;
        print '</ol>';

        $datenschutz_default = "Sie erhalten einen Link an die angegebene E-Mail Adresse, über den Sie alle hinterlegten Suchaufträge zu Ihrer E-Mail einsehen und jederzeit vollständig löschen können. ";
        $datenschutz_default .= "Ihre Daten verbleiben so lange in unserer Datenbank, bis Sie die Suchaufträge löschen. ";
        $datenschutz_default .= "Ihre Daten dienen lediglich dazu, Sie bei passenden Fahrzeugen zu benachrichtigen und werden nicht anderweitig verwendet.";

        // $datenschutz = (!empty($options['vsr_privacy_text'])) ? $options['vsr_privacy_text'] : $datenschutz_default;
        $datenschutz = ($options['vsr_privacy_text'] != $datenschutz_default || $options['vsr_privacy_text'] = "") ? $options['vsr_privacy_text'] : $datenschutz_default;

        print '<h4>3) Datenschutz</h4>';
        print '<p>Geben Sie hier an, was genau mit den Daten passiert, die der Benutzer über die Suchaufträge erfasst.<br />';
        print 'Dieser Text wird unter dem Formular angezeigt, wenn auf "Datenschutzerklärung" geklickt wird.</p>';
        print '<textarea name="dxim[vsr_privacy_text]" style="min-width:50%;">' . $datenschutz . '</textarea>';

        $li = "<p>Oder verlinken Sie alternativ auf Ihre Datenschutzerklärung.";
        $li .= '<select name="dxim[vsr_privacy_page]">';
        $li .= '<option value="#">keine Seite gewählt</option>';
        if (!isset($options['vsr_privacy_page'])) $options['vsr_privacy_page'] = null;
        foreach (get_pages() as $tmp) {
            $sel = ($tmp->ID == $options['vsr_privacy_page']) ? 'selected' : null;
            $li .= '<option value="' . $tmp->ID . '" ' . $sel . '>' . $tmp->post_name . '</option>';
        }
        $li .= '</select></p>';

        print $li;

        print '<hr />';
        print '<h3>Suchauftrag automatisch in Fahrzeugbörse platzieren</h3>';
        print '<p>Zeigen Sie das Suchauftrag-Formular automatisch innerhalb der Ergebnisliste der Fahrzeugbörse an</p>';

        // in Börse platzieren
        print '<div class="form_item">';
        print '<label class="box">Automatisch nach X. Fahrzeug einblenden<br /><span class="small-text">0 = keine Anzeige</span></label>';
        print '<div>';
        $position = $options['vsr_form_position'] ?? 0;
        print '<input type="number" name="dxim[vsr_form_position]" value="' . $position . '" />';
        print '</div>';
        print '</div>';

        print '<div class="form_item">';
        print '<label class="box">auf folgenden Ergebnisseiten anzeigen<br /><span class="small-text">Seitennummern angeben, Komma separiert, z.B. 1,5</span></label>';
        print '<div>';
        $pages = $options['vsr_form_pages'] ?? 1;
        print '<input type="text" name="dxim[vsr_form_pages]" value="' . $pages . '" />';
        print '</div>';
        print '</div>';

    }

    /**
     * Filterform Settings
     */
    public static function draw_settings_filterform_settings() {
        $options = Admin::getInstance()->getOptions();
        print '<h2>Filterformular</h2>';

        //
        if (!isset($options['ff_method'])) $options['ff_method'] = "reload";
        $checked = ($options['ff_method'] == "reload") ? 'checked' : null;
        $content = '<label for="ff_method"><input type="radio" name="dxim[ff_method]" value="reload" ' . $checked . ' /> Absenden Button</label><br />';
        $checked = ($options['ff_method'] == "ajax") ? 'checked' : null;
        $content .= '<label for="ff_labels"><input type="radio" name="dxim[ff_method]" value="ajax" ' . $checked . ' /> Ajax</label> ';
        print self::_draw_option("Methode zum Versenden des Formulars", $content);

        if (!isset($options['ff_labels'])) $options['ff_labels'] = "inside";
        $checked = ($options['ff_labels'] == "outside") ? 'checked' : null;
        $content = '<label for="ff_labels"><input type="radio" name="dxim[ff_labels]" value="outside" ' . $checked . ' /> Feldname außen (als Label)</label><br />';
        $checked = ($options['ff_labels'] == "inside") ? 'checked' : null;
        $content .= '<label for="ff_labels"><input type="radio" name="dxim[ff_labels]" value="inside" ' . $checked . ' /> Feldname innen (als Option)</label> ';
        print self::_draw_option("Darstellung", $content);

        if (!isset($options['ff_nothing_selected_text']) || empty($options['ff_nothing_selected_text'])) $options['ff_nothing_selected_text'] = "- - -";
        $content = '<input type="text" name="dxim[ff_nothing_selected_text]" value="' . $options['ff_nothing_selected_text'] . '" />';
        print self::_draw_option('Anzeige für "nichts gewählt" ', $content);

        $filters = VehicleFields::getFields();
        // alle filter, die sich derzeit in einer Formularspalte befinden
        $enabledFilters = $options['vehicle_filter'] ?? [];

        // wenn noch keine Filter vorausgefüllt wurden, Standard Felder füllen
        if (empty($enabledFilters)) {
            foreach ($filters as $key => $filter) {
                if (!isset($filter['default_form_field_column'])) $filter['default_form_field_column'] = null;
                if (intval($filter['default_form_field_column']) > 0) {
                    $enabledFilters[$filter['default_form_field_column']][] = $key;
                }
            }
        }

        // alle aktivierten Filter
        // diese werden aber um die enabledfilter erweitert, damit diese korrekt funktionieren
        $active_filters = $options['active_filters'] ?? [];
        foreach ($enabledFilters as $colFilters) {
            foreach ($colFilters as $filterKey) {
                if (!in_array($filterKey, $active_filters)) $active_filters[] = $filterKey;
            }
        }

        // Missing Filters werden beim Besuch eines Formulares festgestellt und in DB gespeichert
        $missingFilters = Filter::getInstance()->getMissingFilters();
        if (is_array($missingFilters)) $active_filters = array_merge(array_keys($missingFilters), $active_filters);

        // doppelte Felder vermeiden
        $active_filters_tmp = array_unique($active_filters);
        $active_filters = [];
        foreach ($active_filters_tmp as $filterKey) {
            if (!isset($filters[$filterKey])) continue;
            $active_filters[$filterKey] = $filters[$filterKey]['label'];
        }
        asort($active_filters);

        /**
         * Filterdefinition
         */
        print "<h3>Filter Definition</h3>";
        print "<p>An dieser Stelle definieren sie, welche Felder sie als Filter verwenden möchten</p>";
        print "<ol>
            <li>Geben sie einen Suchbegriff in das Textfeld ein, um nach einem verfügbaren Feld zu suchen</li>
            <li>Klicken Sie auf eines der Ergebnisse, um das Feld in der Liste der aktiven Filter hinzuzugügen</li>
            <li>Nun kann dieses Feld als Filter verwendet werden, z.B. im Standardformular oder per Shortcode [WPCS_filter]</li>
            </ol>";

        print '<div id="wpcs_filter_search">';
            print '<input type="text" id="wpcs_ajax_search_filter" />';
            print FilterForm::getInstance()->drawAjaxLoading();
            print '<ul id="wpcs_ajax_search_filter_results"></ul>';
        print '</div>';

        print '<div id="wpcs_filter_definition" class="dxim_clearfix">';
            foreach ($active_filters as $filterKey => $label) {
                print '<div class="wpcs_active_filter" data-key="' . $filterKey . '" data-label="' . $label . '">';
                print '<span class="dashicons dashicons-no-alt"></span> ';
                print '<span class="label">' . $label . '</span>';
                print '<input type="hidden" name="dxim[active_filters][]" value="' . $filterKey . '" />';
                print '</div>';
            }
        print '</div>';
        print '<div class="clear" />';


        /**
         * Formularspalten
         */
        print "<h3>Standard - Formular</h3>";
        print "<ol>
            <li>Das Standardformular wird in Fahrzeug-Archiven angezeigt, welche den SHortcode [WPCS_archive] nutzen</li>
            <li>Das Standardformular kann auch einzeln mittels Shortcode [WPCS_filterform] verwendet werden</li>
            <li>Weisen Sie hier die einzelnen aktiven Filter den einzelnen Formularspalten zu</li>
            </ol>";

        for ($a = 1; $a <= 4; $a++) {
            print '<div class="dxim_filter_column_wrapper">';
            print '<strong>Spalte ' . $a . '</strong>';
            print '<ul class="dxim_filter_column" data-column-nr="' . $a . '">';
            if (!empty($enabledFilters[$a])) {
                foreach ($enabledFilters[$a] as $key) {
                    if (!isset($filters[$key])) continue;
                    print '<li class="vehicle_filter" data-filter-key="' . $key . '"><span class="dashicons dashicons-no-alt"></span> ' . $filters[$key]['label'] . '</li>';
                }
            }
            print '</ul>';
            print '</div>';
        }

        print '<div class="clear" />';
        print '<div id="vehicle_filters_to_save"></div>';
    }


    /**
     * Fahrzeug Vergleich
     */
    public static function draw_settings_vehicle_compare_settings() {
        $options = Admin::getInstance()->getOptions();

        print '<h2>Fahrzeug - Vergleich</h2>';
        print '<p><strong>Lassen Sie ihre Besucher Fahrzeuge miteinander vergleichen.</strong><br />
        Wenn der Vergleich aktiviert ist, werden automatisch Buttons bei der Fahrzeugübersicht und -einzelansicht dargestellt,
        um Fahrzeuge der Vergleichstabelle hinzuzufügen</p>';

        $checked = isset($options['compare_disabled']) ? 'checked' : null;
        $content = '<input type="checkbox" class="wpcs_checkbox" name="dxim[compare_disabled]" value="1" ' . $checked . ' />';
        print self::_draw_option("Fahrzeug-Vergleich deaktivieren", $content );

        print '<strong>Darstellen der Vergleichstabelle</strong>';
        print '<ol>';
        print '<li>Legen Sie eine neue Seite an</li>';
        print '<li>Fügen sie den Shortcode <strong>[WPCS_compare]</strong> ein</li>';
        print '<li>Verlinken sie die ihre Fahrzeugvergleich-Seite, z.B. im Hauptmenü</li>';
        $li = '<li>';
        $li .= 'Fahrzeugvergleich-Seite aus um Links in der Fahrzeugübersicht und -einzelansicht darszustellen.<br />';
        $li .= '<select name="dxim[compare_page]">';
        $li .= '<option value="#">keine Seite gewählt</option>';
        if (!isset($options['compare_page'])) $options['compare_page'] = null;
        foreach (get_pages() as $tmp) {
            $sel = ($tmp->ID == $options['compare_page']) ? 'selected' : null;
            $li .= '<option value="' . $tmp->ID . '" ' . $sel . '>' . $tmp->post_name . '</option>';
        }
        $li .= '</select>';
        $li .= '</li>';
        print $li;
        print '</ol>';

        print '<h2>Fahrzeugeparken</h2>';
        print '<p><strong>Lassen Sie ihre Besucher Fahrzeuge parken / merken.</strong></p>';

        $checked = isset($options['park_disabled']) ? 'checked' : null;
        $content = '<input type="checkbox" class="wpcs_checkbox" name="dxim[park_disabled]" value="1" ' . $checked . ' />';
        print self::_draw_option("Fahrzeug-Parken deaktivieren", $content, "Deaktiviert den Standard Button in der Fahrzeug Übersicht und Einzelansicht.");

        print '<ol>';
            print '<li><strong>[WPCS_vehicles_parked]</strong>: zeigt die Liste der geparkten Fahrzeuge an.</li>';
            print '<li><strong>[WPCS_count_vehicles_parked]</strong>: zeigt die Anzahl der derzeit geparkten Fahrzeuge an.</li>';
        print '</ol>';
    }


    /**
     * Kontaktform
     */
    public static function draw_settings_contactform_settings() {
        $options = Admin::getInstance()->getOptions();

        print '<h2>Kontaktformular</h2>';
        print '<p>Das Kontaktformular wird in der Fahrzeugeinzelansicht dargestellt und sendet E-Mails an die Adresse, die zum entsprechenden Händler-Account des Fahrzeugs unter dem Punkt Händer-Daten hinterlegt wurde.</p>';

        $content = '<ul>';
        if (!isset($options['form_captcha'])) $options['form_captcha'] = 1;
        // deaktiviert
        $checked = $options['form_captcha'] == "0" ? 'checked' : null;
        $content .= '<li>deaktiviert <input type="radio" name="dxim[form_captcha]" onClick="jQuery(\'#wpcs_captcha_ga\').hide(); jQuery(\'#wpcs_captcha_math\').hide();" value="0" ' . $checked . ' />';
        $content .= '</li>';

        // rechenaufgabe
        $checked = $options['form_captcha'] == "1" ? 'checked' : null;
        $display = $options['form_captcha'] == "1" ? 'display:block;' : 'display:none;';
        $content .= '<li>Rechenaufgabe <input type="radio" onClick="jQuery(\'#wpcs_captcha_math\').show(); jQuery(\'#wpcs_captcha_ga\').hide();" name="dxim[form_captcha]" value="1" ' . $checked . ' /><br />';
        $content .= '<div id="wpcs_captcha_math" style="' . $display . '">';
        if (empty($options['form_placeholder_text'])) $options['form_placeholder_text'] = 'Ergebnis hier eintragen';
        $content .= '<strong>Platzhalter - Text</strong> <input type="text" name="dxim[form_placeholder_text]" value="' . $options['form_placeholder_text'] . '" />';
        $content .= '</div>';
        $content .= '</li>';


        // google recaptcha v3
        $checked = $options['form_captcha'] == "2" ? 'checked' : null;
        $display = $options['form_captcha'] == "2" ? 'display:block;' : 'display:none;';
        $content .= '<li>Google reCaptcha v3 <input type="radio" onClick="jQuery(\'#wpcs_captcha_ga\').show(); jQuery(\'#wpcs_captcha_math\').hide();" name="dxim[form_captcha]" value="2" ' . $checked . ' />';
        $content .= '<div id="wpcs_captcha_ga" style="' . $display . '">';
        $content .= '<p>Geben Sie hier Ihre Google <a href="https://www.google.com/recaptcha/about/" target="_blank">reCAPTCHA Keys</a> ein.</p>';
        if (empty($options['form_ga_public_key'])) $options['form_ga_public_key'] = null;
        $content .= 'Public Key: <input type="text" name="dxim[form_ga_public_key]" value="' . $options['form_ga_public_key'] . '" /><br />';
        if (empty($options['form_ga_secret_key'])) $options['form_ga_secret_key'] = null;
        $content .= 'Private Key: <input type="text" name="dxim[form_ga_secret_key]" value="' . $options['form_ga_secret_key'] . '" /><br />';
        $checked = isset($options['form_prevent_ga_loading']) ? 'checked' : null;
        $content .= 'reCAPCTHA Api nicht laden (nur aktivieren, wenn bereits installiert) <input ' . $checked . ' type="checkbox" name="dxim[form_prevent_ga_loading]" value="1" />';
        $content .= '</div>';
        $content .= '</li>';
        $content .= '</ul>';
        print self::_draw_option("Spam - Schutz", $content);


        print "<h3>E-Mail Einstellungen</h3>";

        if (empty($options['form_response_msg'])) $options['form_response_msg'] = 'Vielen Dank, wir haben Ihre Nachricht erhalten.';
        print '<div class="form_item">';
        print '<label class="box">Danke Nachricht</label>';
        print '<input type="text" name="dxim[form_response_msg]" value="' . $options['form_response_msg'] . '" />';
        print '</div>';

        $options['form_thankyou_page'] = $options['form_thankyou_page'] ?? null;
        print '<div class="form_item">';
        print '<label class="box">Weiterleitung</label>';
        print '<select name="dxim[form_thankyou_page]">';
        print '<option value="#">keine Seite gewählt</option>';
        foreach (get_pages() as $tmp) {
            $sel = ($tmp->ID == $options['form_thankyou_page']) ? 'selected' : null;
            print '<option value="' . $tmp->ID . '" ' . $sel . '>' . $tmp->post_name . '</option>';
        }
        print '</select>';
        print '</div>';

        print '<div class="form_item">';
        print '<label class="box">Mail cc an Kunde</label>';
        $checked = isset($options['form_client_cc']) ? "checked" : null;
        print '<input type="checkbox" name="dxim[form_client_cc]" value="1" ' . $checked . ' />';
        print '</div>';


        print '<h3>Datenschutz - Einstellungen (dringend empfohlen)</h3>';
        print '<p>Wählen Sie hier Ihre Seite aus, auf der die Datenschutzbestimmungen zu finden sind.<br />
	    Sofern sie hier eine Seite definieren, muss der Benutzer bestätigen dass er die Datenschutzbestimmungen gelesen und verstanden hat.<br />
	    Ansonsten kann er das Formular nicht absenden.<br />';
        print '<select name="dxim[privacy_page]">';
        print '<option value="#">keine Seite gewählt</option>';
        foreach (get_pages() as $tmp) {
            $sel = ($tmp->ID == $options['privacy_page']) ? 'selected' : null;
            print '<option value="' . $tmp->ID . '" ' . $sel . '>' . $tmp->post_name . '</option>';
        }
        print '</select>';
        print '</p>';

        print '<p><strong>Info!</strong>
        E-Mails werden mit der Wordpress Funktion <strong>wp_mail()</strong> versendet.<br /> 
        Sollten sie Probleme beim versenden von E-Mails haben, müssen sie ggf. die SMTP Einstellungen zum Versand von E-Mails anpassen.<br />
        <a href="https://de.wordpress.org/plugins/search/smtp/" target="_blank">Dazu sind bereits sehr viele Wordpress Plugins kostenfrei erhältlich</a>.</p>';

    }


    public static function draw_settings_labels_settings() {
        /**
         * Labels
         */
        print '<h2>Beschriftungen &amp; Textausgaben</h2>';
        print '<p>Dieser Bereich bietet ihnen die Möglichkeit, Textausgaben des Plugins zu verändern.<br />
        Einige der früheren Felder können nun direkt unter dem Tab "Templates" bearbeitet werden. 
        </p>';

        print "<h3>Fahrzeug - Einzelansicht</h3>";

        // zurück zum Archiv / Suche
        print '<p>';
        print '<strong>Beschriftung Button "zurück zur Übersicht"</strong><br />';
        print '<input type="text" name="dxim[labels_single_back_to_archive]" value="' . WPCSString::getString('labels_single_back_to_archive') . '" /><br />';
        print '</p>';

        // Fahrzeug drucken
        print '<p>';
        print '<strong>Beschriftung Button "Fahrzeug drucken"</strong><br />';
        print '<input type="text" name="dxim[labels_single_vehicle_print]" value="' . WPCSString::getString('labels_single_vehicle_print') . '" /><br />';
        print '</p>';

        // Anfrage senden
        print '<p>';
        print '<strong>Beschriftung Button "Fahrzeug anfragen"</strong><br />';
        print '<input type="text" name="dxim[labels_single_contact_form_send]" value="' . WPCSString::getString('labels_single_contact_form_send') . '" /><br />';
        print '</p>';

        /**
         * Archiv
         */
        // Fahrzeug Anzeigen Button
        print "<hr />";
        print "<h3>Fahrzeug - Übersicht</h3>";

        print '<p>';
        print '<strong>Beschriftung Button "Fahrzeug anzeigen"</strong><br />';
        print '<input type="text" name="dxim[labels_archive_vehicle_button]" value="' . WPCSString::getString('labels_archive_vehicle_button') . '" /><br />';
        print '</p>';

        print '<p>';
        print '<strong>Beschriftung Button "weitere Fahrzeuge laden"</strong><br />';
        print '<input type="text" name="dxim[labels_archive_pagination_button]" value="' . WPCSString::getString('labels_archive_pagination_button') . '" /><br />';
        print '</p>';


        print "<hr />";
        print "<h3>Filter Formular</h3>";

        print '<p>';
        print '<strong>Beschriftung Button Suchen</strong><br />';
        print '<input type="text" name="dxim[labels_filter_form_button]" value="' . WPCSString::getString('labels_filter_form_button') . '" /><br />';
        print '</p>';

        print "<hr />";
        print "<h3>Fahrzeug - Vergleich</h3>";

        print '<p>';
        print '<strong>Beschriftung Button "zum Fahrzeugvergleich"</strong><br />';
        print '<input type="text" name="dxim[labels_compare_page_button]" value="' . WPCSString::getString('labels_compare_page_button') . '" />';
        print '</p>';

        print '<p>';
        print '<strong>Beschriftung Button um Fahrzeug zum vergleich hinzuzufügen</strong><br />';
        print '<input type="text" name="dxim[labels_compare_vehicle_button]" value="' . WPCSString::getString('labels_compare_vehicle_button') . '" />';
        print '</p>';

        print '<p>';
        print '<strong>Beschriftung Button um Fahrzeug zu parken</strong><br />';
        print '<input type="text" name="dxim[labels_park_vehicle_button]" value="' . WPCSString::getString('labels_park_vehicle_button') . '" />';
        print '</p>';
    }


    /**
     * @param $acc_id
     * @return string
     */
    public static function drawCSVAccountConfig($acc_id, &$options) {
        $page_slug = $_REQUEST['wpcs_settings_page'] ?? "";
        $account = Account::getInstance()->get_account($acc_id);
        $data = ['fm_index' => null, 'fm_key' => null];
        $html = null;

        if (empty($account)) $html .= "Account nicht gefunden";

        $html .= "<h1>Konfiguration für Account \"" . $account['name'] . "\"</h1>";

        /**
         * Datensatz testen
         */
        $html .= '<h3>CSV Import Testen</h3>';
        $html .= '<p>Schauen Sie sich an, wie der Datensatz eines Fahrzeuges importiert würde. Geben Sie dazu die Zeile der CSV ein, mit der sie testen möchten. (Index beginnend bei 0)</p>';

        $html .= '<div><div class="form_item">';
        $test_index = $_POST['wpcs_csv_test_index'] ?? 0;
        $html .= '<label class="box">CSV Index<br /><span class="small-text"></span></label> <input type="number" name="wpcs_csv_test_index" value="' . $test_index . '" /></p>';
        $html .= '</div></div>';
        $html .= '<input type="submit" class="button button-primary" name="wpcs_admin_csvaccount_test" value="Datensatz testen" />';

        // testen
        if (isset($_POST['wpcs_admin_csvaccount_test'])) {
            $index = intval($_POST['wpcs_csv_test_index']);
            $import_data = CSVAdapter::getInstance()->getVehicle($index, $account, true);
            if ($import_data == "0") {
                $html .= '<p class="wpcs_error">' . CSVAdapter::getInstance()->getError() . '</p>';
            } else {
                $html .= '<pre>';
                $html .= print_r($import_data, true);
                $html .= '</pre>';
            }
        }

        $init = CSVAdapter::getInstance()->initAccount($account, true);
        if (isset($init['error'])) {
            $html .= '<p class="wpcs_error">' . $init['error'] . '</p>';
            #return $html;
        }

        $html .= '<hr />';

        // save new field
        if (isset($_POST['wpcs_admin_csvaccount_save_fields'])) {
            if (!CSVAdapter::getInstance()->addNewCustomField($_POST['fm_index'], $_POST['fm_key'], $acc_id)) {
                $data['fm_index'] = $_POST['fm_index'];
                $data['fm_key'] = $_POST['fm_key'];
                $html .= "Ein Fehler trat auf";
            }
        }

        // save new field
        elseif (isset($_POST['wpcs_admin_csvaccount_map_locations'])) {
            $mapped = get_option('wpcs_csv_locations_' . $acc_id);
            $mapped[$_POST['new_location_value']] = $_POST['new_location_target'];
            update_option('wpcs_csv_locations_' . $acc_id, $mapped);
        }

        // save new field
        elseif (isset($_GET['delete_location_mapping'])) {
            $mapped = get_option('wpcs_csv_locations_' . $acc_id);
            unset($mapped[$_GET['delete_location_mapping']]);
            update_option('wpcs_csv_locations_' . $acc_id, $mapped);
        }

        // delete field
        elseif (isset($_GET['delete_index'])) {
            CSVAdapter::getInstance()->deleteCustomField($_GET['delete_index'], $acc_id);
        }

        $html .= "<h2>Standorte verknüpfen</h2>";
        $html .= "<p>Die Standortangaben der CSV Datei müssen dem passenden Standort in WPCarSync zugordnet werden.</p>";
        $mapped = get_option('wpcs_csv_locations_' . $acc_id);
        $ld = new LocationData();
        #$locations = $ld->getAll();
        $locations = $ld->getTree();

        if (!empty($mapped)) {
            $html .= '<ul class="wpcs_field_mapper_active_fields">';
            foreach ($mapped as $value => $location_id) {
                $location = $ld->getById($location_id);
                $error = empty($location) ? '<span class="wpcs_error">Bitte diesen Eintrag neu anlegen (Wert = ' . $location_id . ')</span>' : "";
                $label = $location->name ?? "";
                $html .= '<li><a href="' . Admin::getInstance()->_getAdminPage() . '&wpcs_settings_page='.$page_slug.'&account_config=' . $acc_id . '&delete_location_mapping=' . $value . '"><span class="dashicons dashicons-no-alt"></span></a> CSV-Standort: <strong>' . $value . '</strong>, WPCS-Standort: <strong>' . $label . '</strong> ' . $error . '</li>';
            }
            $html .= "</ul>";
        }

        $html .= "<p><strong>neuen Standort mappen</strong></p>";
        $html .= '<div class="wpcs_location_mapping">';
        $html .= '<input type="text" name="new_location_value" placeholder="CSV-Standort Wert" value="" />';
        $html .= self::_location_select($locations, "new_location_target");
        $html .= '</div>';


        $html .= '<input type="submit" class="button button-primary" name="wpcs_admin_csvaccount_map_locations" value="Standortzuweisungen speichern" />';

        $html .= "<hr />";

        $html .= "<h2>Eigene Felder definieren</h2>";
        $html .= '<p>Sie können hier weitere Felder aus der CSV Datei importieren, die nicht in der config - Datei definiert sind.</p>';

        /**
         * aktuelle Felder anzeigen
         */
        $fields = CSVAdapter::getInstance()->getCustomFields($acc_id);
        if (!empty($fields)) {
            $html .= '<h3>Übersicht zugewiesener Felder</h3>';
            $html .= '<ul class="wpcs_field_mapper_active_fields">';
            foreach ($fields as $index => $field) {
                $html .= '<li><a href="' . Admin::getInstance()->_getAdminPage() . '&wpcs_settings_page='.$page_slug.'&account_config=' . $acc_id . '&delete_index=' . $index . '"><span class="dashicons dashicons-no-alt"></span></a> CSV-Index: <strong>' . $index . '</strong>, Feldname: <strong>' . $field['name'] . '</strong></li>';
            }
            $html .= "</ul>";
        }

        /**
         * neues Feld anlegen
         */
        $html .= '<div class="wpcs_field_mapper_new_field">';
        $html .= '<h3>Neues Feld zuweisen</h3>';

        $html .= '<div class="form_item">';
        $html .= '<label class="box">Feld-Index<br /><span class="small-text">Position des Feldes in der CSV Datei, beginnend bei 0</span></label> <input type="number" name="fm_index" value="' . $data['fm_index'] . '" /></p>';
        $html .= '</div>';

        $html .= '<div class="form_item">';
        $html .= '<label class="box">Feld-Name<br /><span class="small-text">Name, unter dem das Feld als Custom Field zum Fahrzeug importiert werden soll</span></label> <input type="text" name="fm_key" value="' . $data['fm_key'] . '" /></p>';
        $html .= '</div>';

        $html .= '</div>';

        $html .= '<input type="submit" class="button button-primary" name="wpcs_admin_csvaccount_save_fields" value="neues Feld speichern" />';
        $html .= '<input type="hidden" name="account_config" value="' . $acc_id . '" />';


        return $html;
    }

    /**
     * @return string|void
     */
    public static function drawImportExportSettings() {

        $html = "<h1>Import / Export der WPCarSync Einstellungen</h1>";

        $options = Admin::getInstance()->getOptions();

        // die WPCS Settings
        $export['main_options'] = $options;

        // CSV Settings anfügen
        foreach ($options['accounts'] as $acc_id => $data) {
            $export['wpcs_csv_cf_' . $acc_id] = get_option('wpcs_csv_cf_' . $acc_id);
            $export['wpcs_csv_locations_' . $acc_id] = get_option('wpcs_csv_locations_' . $acc_id);
        }

        $html .= "<h3>Export</h3>";
        $html .= "<p>Kopieren Sie den folgenden Text, um ihn auf einer anderen WPCarSync Installation einzufügen</p>";
        $html .= '<div class="wpcs_export_string">' . (json_encode($export)) . '</div>';

        $html .= "<h3>Import</h3>";
        $html .= "<p>Fügen Sie hier die Settings ein, die aus einer anderen WPCarSync Installation übernommen werden sollen.<br />";
        $html .= "<strong>Achtung! Dabi werden ALLE bestehenden EInstellungen überschrieben.</strong></p>";
        $html .= '<textarea name="import_settings_string"></textarea>';

        // submit
        $html .= '<p><input type="submit" class="button button-primary" name="import_settings_start" value="Importieren" /></p>';

        return $html;
    }


    /**
     * @param $label
     * @param $content
     * @param $small_text
     * @return string
     */
    private static function _draw_option($label, $content, $small_text = "") {
        $html = '<div class="form_item">';

            // label
            $html .=  '<label class="box">';
                $html .= $label;
                if(!empty($small_text)) $html .= '<br /><span class="small-text">'.$small_text.'</span>';
            $html .= '</label>';

            // content
            $html .= '<div>'.$content.'</div>';

        $html .= '</div>'; // form_item

        return $html;
    }


    /**
     * @param $locations
     * @param $name
     * @return string
     */
    private static function _location_select($locations, $name, $selected = null) {
        $html = '<select name="' . $name . '">';
        foreach ($locations as $array) {
            if (!empty($array['childs'])) {
                $html .= '<optgroup label="' . $array['parent']->name . '">';
                foreach ($array['childs'] as $location) {
                    $sel = $location->term_id == $selected ? "selected" : "";
                    $html .= '<option value="' . $location->term_id . '" ' . $sel . '>' . $location->name . ' (ID:' . $location->term_id . ')</option>';
                }
                $html .= '</optgroup>';
            } else {
                $location = $array['parent'];
                $sel = $location->term_id == $selected ? "selected" : "";
                $html .= '<option value="' . $location->term_id . '" ' . $sel . '>' . $location->name . ' (ID:' . $location->term_id . ')</option>';
            }
        }
        $html .= '</select>';

        return $html;
    }

}