<?php

namespace DXIM;

class CarShotsPROApi {

	protected static $instance = null;
    protected $optionName = "wpcs_csp_api_info";
    protected $error = null;

	/**
	 *
	 * @return CarShotsPROApi
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

    public function getError() { return $this->error; }

    public function __construct() {

        add_action('wp_ajax_wpcs_csp_create_job',[&$this,'_ajax_create_job']);

    }


    /**
     * @return void
     */
    public function _ajax_create_job() {
        $vehicle_id = $_POST['vehicle_id'] ?? 0;
        $response = "";
        if(!empty($vehicle_id))  $response = $this->createJob($vehicle_id, true);

        header('Content-Type: application/json');
        print $response;
        exit;
    }

    /**
     * @return void
     */
    public function initApi() {
        // ist ein Api Call
        if($this->isApiCall()) {
            #ini_set('error_reporting',0);
            #ini_set('display_errors',0);
            header('Content-Type: application/json');
            $app_token = $_SERVER['HTTP_X_API_KEY'] ?? null;

            if($this->isApiCallValid($app_token)) {
                $json = file_get_contents('php://input');
                $data = json_decode($json, ARRAY_A);
                $job = $data['job'] ?? null;

                // Import Images from CarShotsPRO
                if($job == "import_images") {
                    if(CarShotsPROImport::getInstance()->importImages($data['ad_key'], $data['images'])) {
                        $result = json_encode(['code' => 1, 'msg' => 'vehicle images successfully saved']);
                        $this->setApiInfo($app_token, $data['company']);
                    } else $result = json_encode(['code' => -1, 'msg' => CarShotsPRO::getInstance()->getError()]);
                }
                else $result = json_encode(['code' => -1, 'msg' => 'nothing todo']);

            }
            else {
                $result = json_encode(['code' => -1, 'msg' => 'WPCarSync Api key not valid']);
            }

            print $result;
            exit;
        }
    }

    /**
     * @return bool
     */
    public function isApiCall() {
        return isset($_SERVER['HTTP_WPCS_API_CALL']);
    }

    /**
     * @return bool
     */
    public function isApiCallValid($app_token) {
        $apiInfo = $this->getApiInfo();
        return isset($apiInfo[$app_token]);
    }

    /**
     * @return mixed|null
     */
    public function connect($token) {
        $api_url = CARSHOTSPRO_API.'?api='.$token;
        $res = wp_remote_get($api_url.'&action=connect_wpcs&d='.home_url());

        $json = $res['body'] ?? null;
        $data = json_decode($json, ARRAY_A);

        if(!is_array($data)) {
            $this->error = "Serverantwort nicht lesbar";
            return false;
        }

        if(isset($data['error'])) {
            $this->error = $data['error']['msg'];
            return false;
        }

        $this->setApiInfo($token, $data['company']);
        return true;
    }


	/**
	 * @param $post_id
	 *
	 * @return void
	 */
    public function createJob($post_id, $force_job = false) {
        $account_id = get_post_meta($post_id, "ad_account", true);
	    $options = Admin::getInstance()->getOptions('carshotspro')['mapping'][$account_id] ?? [];

        // kein CSP Account zugewiesen => abbruch
		if(empty($options['csp_account']) || $options['csp_account'] == "#") return;

        // keine automatische verarbeitung aktiviert => abbruch
        if(!isset($options['create_job_on_import']) && !$force_job) {
            return;
        }

	    $key = apply_filters(CARSHOTSPRO_CREATE_JOB_KEY,get_post_meta($post_id, "seller_inventory_key", true), $post_id);
		$make = get_post_meta($post_id, "make", true);
	    $description = get_post_meta($post_id, "model_description", true);
		$title = $make." ".$description;
		$token = $options['csp_account'];
		$data = ['vehicle' => ['key' => $key, "title" => $title]];
        $uris = VehicleData::getVehicleImages($post_id, ['image_type' => 'default'])['big'];

        $images = [];
        $mapping = $options['angle'] ?? [];
        if(empty($mapping)) $mapping = CarShotsPRO::getInstance()->getDefaultMapping();

        foreach($mapping as $slug => $nr) {
            $index = $nr-1;
            if(isset($uris[$index])) $images[$slug] = $uris[$index];
        }

        $data['images'] = $images;
        $json = json_encode($data);

        $api_url = CARSHOTSPRO_API.'?api='.$token."&action=create_job";
        try {
            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => $api_url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_TIMEOUT => 0,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => $json,
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: text/plain'
                ),
            ));

            $response = curl_exec($curl);
            curl_close($curl);
            return $response;

        } catch (\Exception $e) {
            $this->error = $e->getMessage();
            return false;
        }

    }

    /**
     * @return false|mixed|void
     */
    public function getApiInfo() {
        $data = get_option($this->optionName);
        if(empty($data) || !is_array($data)) return [];
        return $data;
    }

    /**
     * @param $data
     * @return void
     */
    private function setApiInfo($token, $data) {
        $savedData = $this->getApiInfo();
        $savedData[$token] = $data;
        $this->saveApiInfo($savedData);
    }

    /**
     * @param $data
     * @return void
     */
    private function saveApiInfo($data) {
        update_option($this->optionName, $data);
    }

    /**
     * @param $token
     * @return void
     */
    public function deleteApiInfo($token) {
        $data = $this->getApiInfo();
        unset($data[$token]);
        $this->saveApiInfo($data);
    }

}