<?php

namespace DXIM;

class CarShotsPROImport {

	protected static $instance = null;
    protected $error = null;


    /**
	 *
	 * @return CarShotsPROImport
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	private function __construct() {

    }

    public function getError() { return $this->error; }

    /**
     * @return string
     */
    public function get_dir() {
        $dir = WPCS_UPLOADS_DIR.'/carshotspro';
        $this->make_dir($dir);
        return $dir;
    }

	public function get_url() {
        $dir = WPCS_UPLOADS_URL.'/carshotspro';
        $this->make_dir($dir);
        return $dir;
    }

    public function get_pool_dir() {
        $dir = WPCS_UPLOADS_DIR.'/carshotspro_import';
        $this->make_dir($dir);
        return $dir;
    }

    /**
     * @param $dir
     * @return bool|void
     */
    private function make_dir($dir) {
        if(is_dir($dir)) return true;
        mkdir($dir,0777, true);
    }

    /**
     * @param $ad_key
     * @param $images
     * @return bool
     */
    public function importImages($ad_key, $images) {
        $args = apply_filters(CARSHOTSPRO_FIND_VEHICLE, ['meta_key' => 'seller_inventory_key','meta_value' => $ad_key,'post_type'=>'vehicle', 'post_status' => 'publish']);
        $q = new \WP_Query($args);
        $vehicle = $q->posts[0] ?? [];
        if(empty($vehicle)) {
            return $this->importToPool($images);
        }

        else {
            return $this->importToVehicle($vehicle, $images);
        }
    }



    /**
     * Fahrzeug existiert in WPCS => Bilder zu Fahrzeug zuordnen
     * @param $vehicle
     * @param $images
     * @return void
     */
    private function importToVehicle(&$vehicle, &$images) {
        $image_id = get_post_meta($vehicle->ID, "image_id", true);
        $import_type = Account::getInstance()->getImageConfig(get_post_meta($vehicle->ID, 'ad_account', true));

        if($import_type == "media") MediaImport::getInstance()->import($images, $vehicle->ID, $image_id, "images_media_csp");
        else FilesystemImport::getInstance()->import($vehicle->ID, $images, $image_id, $this->get_dir(), "images_filesystem_csp");

        return true;

    }

    /**
     * @param $images
     * @return bool
     */
    private function importToPool(&$images) {
        $dir = $this->get_pool_dir();
        foreach($images as $url) {
            $info = pathinfo($url);
            FilesystemImport::getInstance()->importFromUrl($url, $dir, $info['basename']);
        }
        return true;
    }

    /**
     * wird getriggert nach einem Import
     * @return void
     */
    public function assignImages($vehicle_id) {
        $this->assignFromPool($vehicle_id);
        $this->assignFromFilesystem($vehicle_id);
    }


    /**
     * findet Bilder im finalen CSP Ordner und registriert diese als meta field zum fahrzeug
     * @param $vehicle_id
     * @return void
     */
    public function assignFromFilesystem($vehicle_id) {
        $image_id = get_post_meta($vehicle_id, "image_id", true);
        $images = FilesystemImport::getInstance()->getImagesByName($image_id,"carshotspro");
        if(empty($images)) return;

        $image_names = [];
        foreach($images as $img) {
            $tmp = pathinfo($img);
            if($tmp['extension'] == "jpg" || $tmp['extension'] == "png") $image_names[] = $tmp['basename'];
        }

        update_post_meta($vehicle_id, "images_filesystem_csp", $image_names);
    }


    /** ----------------------------------
     * findet Bilder im CSP Import Ordner
     * kopiert diese unter dem finalen Bildnamen in den finalen CSP Ordner
     * @param $vehicle_id
     * @return void
     */
    public function assignFromPool($vehicle_id) {
        $files = $this->getPoolImages($vehicle_id);
        if(empty($files)) return;

        $image_id = get_post_meta($vehicle_id, "image_id", true);
        FilesystemImport::getInstance()->import($vehicle_id, $files, $image_id, $this->get_dir(), "images_filesystem_csp");
    }

    /**
     * @param $vehicle_id
     * @return array|false
     */
    public function getPoolImages($vehicle_id) {
        $filename = apply_filters(CARSHOTSPRO_POOL_FILE_NAME, get_post_meta($vehicle_id, 'seller_inventory_key', true), $vehicle_id);
        $files = glob(trailingslashit($this->get_pool_dir()) . $filename . "_*.*");
        if(!is_array($files)) $files = [];
        natsort($files);
        return array_values($files);
    }

}