<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 16/02/16
 * Time: 12:33
 */

namespace DXIM;

use DXIM\theme\LocationData;

class Form {

    protected static $instance = null;

    protected $words = ['eins','zwei','drei','vier','fünf','sechs','sieben','acht','neun','zehn'];

    /**
     * @var null
     */
    public $current_context = null;

    /**
     * vorauswahl der location anhand der id => keine Standortauswahl anzeigen
     * @var int
     */
    protected $location_id = 0;

    public function __construct() {
        add_action('wp_ajax_wpcs_contactform', array($this,'_contactform_send'));
        add_action('wp_ajax_nopriv_wpcs_contactform', array($this,'_contactform_send'));
    }


    /**
     *
     * @return Form
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }


    /**
     * @return string|null
     */
    function _draw_fields() {
        $html = null;
        foreach($this->current_context['fields'] as $key => $field) {
            $html .= $this->_draw_field($key, $field);
        }
        return $html;
    }


    /**
     * @param $key
     * @return void
     */
    function _draw_field($key, $field) {
        #$field = $this->fields[$key] ?? [];
        $required = isset($field['validation']) ? " *" : null;
        $default_value = $field['value'] ?? "";

        $html = '<div class="field '.$key.'">';
            // label
            $html .= '<label for="'.$key.'" rel="'.$key.'">'.$field['label'].$required.'</label>';

            // Text
            if($field['type'] == "text") {
                $html .= '<input class="wpcs_input_text" type="text" name="'.$key.'" value="'.$default_value.'" />';
            }

            // Checkbox
            elseif($field['type'] == "checkbox") {
                $html .= '<div class="wpcs_multiple_checkbox">';
                foreach($field['values'] as $value => $label) {
                    $value = !is_int($value) ? $value : $label;
                    $checked = ($value == $default_value) ? "checked" : "";
                    $html .= '<div class="box">';
                       $html .= '<input type="checkbox" class="wpcs_checkbox" '.$checked.' name="'.$key.'[]" value="'.$value.'" /> <span class="label">'.$label.'</span>';
                    $html .= '</div>';
                }
                $html .= '</div>';
            }

            // select
            elseif($field['type'] == "select") {
                $html .= Html::select(['values' => $field['values'], 'name' => $key]);
            }

            // datetime
            elseif($field['type'] == "datetime") {
                $html .= '<input type="datetime-local" name="'.$key.'" />';
            }

            // Textarea
            elseif($field['type'] == "textarea") {
                $html .= '<textarea name="message" class="wpcs_textarea"></textarea>';
            }

        $html .= '</div>';

        return $html;
    }


    /**
     * @param $context
     * @return mixed
     */
    function _draw_context() {
        $contextHtml = $this->current_context['form_context_html'] ?? null;
        if(empty($contextHtml)) return;

        $html = '<div class="wpcs_form_context_wrapper">';
            $html .= "<label>Thema der Anfrage</label>";
            $html .= '<div class="html_wrapper">'.$contextHtml.'</div>';
        $html .= '</div>';

        return $html;
    }

    /**
     * @return string|null
     */
    function _draw_privacy() {
        $options = Admin::getInstance()->getOptions();
        $html = null;

        if(intval($options['privacy_page']) > 0) {
            $html .= '<div class="wpcs-form-privacy-wrapper">';
                $html .= '<input type="checkbox" class="wpcs_checkbox" name="privacy" value="1" />';
                $html .= '<label for="privacy" rel="privacy">';
                $privacy_link = apply_filters(DXIM_FILTER_FORM_PRIVACY_LINK,get_permalink($options['privacy_page']),$this->current_context);
                $html .= '* Ich habe die <a href="'.$privacy_link.'" target="_blank">Datenschutzbestimmungen</a> gelesen und akzeptiere diese.';
                $html .= '</label>';
            $html .= '</div>';
        }

        return $html;
    }

    /**
     * @return string|null
     */
    function _draw_captcha() {
        $options = Admin::getInstance()->getOptions();
        $html = null;

        if($options['form_captcha'] == "1" || !isset($options['form_captcha'])) {
            $html .= '<div class="wpcs-form-captcha-wrapper">';
            $word_int = rand(0,9);
            $word = $this->words[$word_int];
            $int = rand(1,10);
                $html .= '<label for="c" rel="c">'.$word.' + '.$int.' *</label>';
                $html .= '<input  class="wpcs_input_text" placeholder="'.$options['form_placeholder_text'].'" type="text" name="c" value="" />';
                $html .= '<input type="hidden" name="c_1" value="'.$word_int.'" />';
                $html .= '<input type="hidden" name="c_2" value="'.$int.'" />';
            $html .= '</div>';
        }

        // Captcha Google
        elseif($options['form_captcha'] == "2") {
            if(!isset($options['form_prevent_ga_loading'])) $html .= '<script src="https://www.google.com/recaptcha/api.js?render='.$options['form_ga_public_key'].'"></script>';
            $html .= '<script>var wpcs_ga_public_key = "'.$options['form_ga_public_key'].'";</script>';
        }

        return $html;
    }

    /**
     * @return string
     */
    public function _draw_submit() {
        $options = Admin::getInstance()->getOptions();

        $class = "wpcs_form_submit";
        if($options['form_captcha'] == "2") $class .= " ga";
        $html = Html::drawButton(
            ['url' => '#', 'label' => 'Anfrage senden', 'classes' => $class, 'ajax' => 1]
        );
        $html .= wp_nonce_field('wpcsContactForm','wpcsContactFormNonce',true,false);

        return '<div class="wpcs-form-submit-wrapper">'.$html.'</div>';
    }

    /**
     * @return string
     */
    public function _draw_location() {
        $l = new LocationData();
        $location_id = $this->current_context['location'] ?? 0;

        // standort bereits über den kontext vorgegeben
        if($location_id > 0) { return null; }

        // standort auswahl anzeigen
        // standort query kann über den kontexkt vorgegeben werden
        $location_query = $this->current_context['location_query'] ?? [];
        $locations = $l->getTree($location_query);
        $html = '<div class="wpcs-form-location-wrapper">';
            $html .= "<label>Anfrage an folgenden Standort senden</label>";
            $html .= Html::select(['values' => $l->getLocationSelectArray($locations), 'name' => 'location']);
        $html .= '</div>';

        return apply_filters(DXIM_FILTER_CONTACTFORM_LOCATION_HTML,$html);
    }

    /**
     * @param $key
     * @param $form_data
     * @return string
     */
    public function getFieldVal($form_data) {
        if(is_array($form_data)) $value = implode(", ",$form_data);
        else $value = strval($form_data);
        return $value;
    }

    /**
     *
     * @param $location_id
     * @param $type
     * @return mixed|null
     */
    private function get_email($location_id, $type = "lead") {
        $email = null;

        // get email by location
        if($location_id > 0) {
            if($type == "lead") $email = get_term_meta($location_id, "email_lead", true);
            if(empty($email)) $email = get_term_meta($location_id, "email", true);
        }

        if(empty($email)) {
            $options = Admin::getInstance()->getOptions();
            $email = $options['email'] ?? null;
        }

        return $email;
    }

    /**
     * @return null|string
     */
    public function contactForm($args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['grid' => 2, 'template' => "form", 'context' => null];
        $args = array_merge($defaults, $args);
        $options = Admin::getInstance()->getOptions();
        $context = null;

        if(!WPCS_PRO && !Conf::getInstance()->isVehicleSinglePage()) return null;

        $c = new FormContext();
        if(!empty($args['context'])) $context = $c->get_context($args['context']);
        $this->current_context = (!empty($context)) ? $context : $c->detect_context();

        $lead_email = $this->get_email($this->current_context['location']);

        $html = '<div class="wpcs_contact_form_wrapper">';

            if(empty($lead_email)) {
                $html .= '<h3>Formular-Fehler: E-Mail</h3>';
                $html .= 'Bitte hinterlegen Sie mindestens die zentrale E-Mail Adresse.';
                $html .= '</div>';
                return $html;
            }

            $redirect = (isset($options['form_thankyou_page']) && $options['form_thankyou_page'] != "#") ? get_permalink($options['form_thankyou_page']) : null;
            $html .= '<form method="post" id="wpcs_contactform_'.$this->current_context['name'].'" class="wpcs_contact_form" data-redirect="'.$redirect.'">';

                $html .= CustomTemplate::getInstance()->get_template_content('contactform/'.$args['template'], ['args' => $args]);
                $html .= '<input type="hidden" name="context_name" value="'.$this->current_context['name'].'" />';
                $html .= '<input type="hidden" name="context_obj_id" value="'.$this->current_context['id'].'" />';

            $html .= '</form>';
        $html .= '</div>';

        return $html;
    }


    /**
     * @param $post
     * @return mixed
     */
    public function _contactform_send() {
        $post = [];
        parse_str($_POST['post'], $post);

        /**
         *
         */
        if(isset($post['wpcsContactFormNonce']) && wp_verify_nonce($post['wpcsContactFormNonce'],'wpcsContactForm')) {
            $response = $this->_send_mail($post);
        }
        /**
         * unbekanntes formular
         */
        else {
            $response['status'] = "-1";
            $response['error'] = 'Request invalid';
        }

        // return JSON
        header('Content-Type: application/json');
        die(json_encode($response));
    }


    /**
     * @param $post
     * @return array
     */
    private function _validate(&$post) {

        $post_privacy = $post['privacy'] ?? null;

    	// privacy
		$options = Admin::getInstance()->getOptions();

	    if(intval($options['privacy_page']) > 0 && intval($post_privacy) < 1) {
			$error[] = 'privacy';
	    }

        foreach($this->current_context['fields'] as $key => $field) {
            if(!isset($field['validation'])) continue;

            if(!filter_var($post[$key], $field['validation']['filter'], $field['validation']['options'] ?? [])) {
                $error[] = $key;
            }
        }

        // captcha Rechnen
        if($options['form_captcha'] == "1" || !isset($options['form_captcha'])) {
            $result = $post['c_1'] + $post['c_2'] + 1;
            if(intval($post['c']) !== intval($result)) {
                $error[] = 'c';
            }
        }
        // Captcha Google
        elseif($options['form_captcha'] == "2") {
            $t = isset($_POST['token']) ? $_POST['token'] : null;
            $res = wp_remote_post('https://www.google.com/recaptcha/api/siteverify',[
                'body' => [
                    'secret' => $options['form_ga_secret_key'],
                    'response' => $t
                ]
            ]);
            $json = json_decode($res['body']);
            if(!$json->success || floatval($json->score) < 0.5) $error[] = 'c';
        }

        // fehler
        if(!empty($error)) return [
            'status' => -1,
            'error' => $error
        ];

        // ok
        return ['status' => 1];
    }


    /**
     * @param $post
     * @return array
     */
    private function _send_mail($post) {
        $response = $error = [];

        $c = new FormContext();
        $this->current_context = $c->get_context($post['context_name'], $post['context_obj_id']);
        $location_id = $this->current_context['location'] > 0 ? $this->current_context['location'] : $post['location'];

        $options = Admin::getInstance()->getOptions();
        $validate = $this->_validate($post);

        // validation error
        if($validate['status'] < 1) {
            $response['status'] = -1;
            $response['error'] = $validate['error'];
        }

        // standort muss gegeben sein
        elseif($location_id < 1) {
            $response['status'] = -1;
            $response['error'] = 'Fehler in Standortangabe';
        }

        // send Mail
        else {

            add_filter( 'wp_mail_content_type', array(&$this,'set_html_content_type'));

            /**
             * Mail an Autohaus
             */
            $to = apply_filters(DXIM_FILTER_FORM_DEALER_MAIL,$this->get_email($location_id),$this->current_context);
            $from = apply_filters(DXIM_FILTER_FORM_FROM_MAIL,$this->get_email($location_id, 'location'),$this->current_context);
            $subject = apply_filters(DXIM_FILTER_FORM_SUBJECT,$this->current_context['mail_subject'],$post,$this->current_context);

            $msg = CustomTemplate::getInstance()->get_template_content('contactform/mail_'.$this->current_context['name'], ['form_data' => $post, 'context' => $this->current_context]);
            $msg = apply_filters(DXIM_FILTER_FORM_MESSAGE,$msg,$post,$this->current_context);

            // mail an autohaus
            $attachments = apply_filters(DXIM_FILTER_FORM_ATTACHMENT,[],$post,$this->current_context);
            $headers = [];
            $headers[]   = 'Reply-To: '.$post['name'].' <'.$post['mail'].'>';

            $m = new Mail();
            $m->send_mail($to,$subject,$msg,$headers,$attachments);

            // save lead
            Lead::getInstance()->create("contact_form", $msg, $this->current_context['name'], $this->current_context['id'], ['email' => $post['mail']]);

            /**
             * Kopie an Kunde
             */
            if(isset($options['form_client_cc'])) {
                $msg_client = "<strong>Vielen Dank!</strong><br />Wir haben Ihre Nachricht erhalten.<br />";
                $msg_client .= "Unten finden Sie eine Kopie Ihrer Anfrage:<br />";
                $msg_client .= "<hr />";
                $msg_client .= $msg;

                $headers = [];
                $headers[]   = 'Reply-To: '.get_bloginfo('name').' <'.$from.'>';
                $subject_client = apply_filters(DXIM_FILTER_FORM_SUBJECT_CLIENT,$subject, $post, $this->current_context);

                $m->send_mail($post['mail'],$subject_client,$msg_client,$headers);
            }

            remove_filter( 'wp_mail_content_type', 'set_html_content_type' );

            $response['status'] = 1;
            $response_msg = (!empty($options['form_response_msg'])) ? $options['form_response_msg'] : 'Vielen Dank, wir haben Ihre Nachricht erhalten.';
            $response['msg'] = '<h3 class="wpcs_form_thank_you">'.$response_msg.'</h3>';
        }

        return $response;
    }


    /**
     * @return string
     */
    public function set_html_content_type() {
        return 'text/html';
    }


}