<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 16/02/16
 * Time: 12:33
 */

namespace DXIM;

use DXIM\theme\LocationData;
use DXIM\theme\ModelData;
use DXIM\theme\OfferData;
use DXIM\theme\ServiceData;
use WPMailSMTP\Vendor\Google\Service;

class FormContext {

    /**
     * @param $name
     * @param $obj_id
     * @return mixed|null
     */
    public function get_context($name, $obj_id = 0) {
        $method_name = "_get_".$name."_context";
        if(!method_exists($this,$method_name)) return [];
        return apply_filters(DXIM_FILTER_CONTACTFORM_CONTEXT, $this->{$method_name}($obj_id));
    }

    /**
     * @return mixed
     */
    public function detect_context() {

        if(Conf::getInstance()->isVehicleSinglePage()) {
            $vehicle = dxim_get_vehicle();
            return $this->get_context('vehicle', $vehicle->ID);
        }

        elseif(is_singular(\DXIM\theme\Model::$post_type)) {
            $model = \DXIM\theme\Model::getInstance()->getCurrent();
            return $this->get_context(\DXIM\theme\Model::$post_type, $model->ID);
        }

        elseif(is_singular(\DXIM\theme\Offer::$post_type)) {
            $offer = \DXIM\theme\Offer::getInstance()->getCurrent();
            return $this->get_context(\DXIM\theme\Offer::$post_type, $offer->ID);
        }

        elseif(is_singular(\DXIM\theme\Service::$post_type)) {
            $service = \DXIM\theme\Service::getInstance()->getCurrent();
            return $this->get_context(\DXIM\theme\Service::$post_type, $service->ID);
        }

        elseif(is_tax(\DXIM\theme\Location::$taxonomy)) {
            $term = \DXIM\theme\Location::getInstance()->get_current();
            return $this->get_context(\DXIM\theme\Location::$taxonomy, $term->term_id);
        }

        else {
            return $this->get_context('general');
        }

    }

    /**
     * @param $vehicle_id
     * @return array
     */
    private function _get_vehicle_context($vehicle_id = 0) {

        if($vehicle_id > 0) $vehicle = Vehicle::getInstance()->getById($vehicle_id);
        else $vehicle = dxim_get_vehicle();

        $v = new Vehicle();
        $src = VehicleData::getVehicleImages($vehicle)['large'][0] ?? null;
        $html = null;

        if(!empty($src)) $html .= '<div><img src="'.$src.'" /></div>';
        $html .= '<div>';
            $html .= "Hersteller: ".$v->drawValue($vehicle, 'make')."<br />";
            $html .= "Modell: ".$v->drawValue($vehicle, 'model_description')."<br />";
            $html .= "Interne Nummer: ".$v->drawValue($vehicle, 'seller_inventory_key')."<br />";
        $html .= '</div>';

        $mail_subject = get_bloginfo('name').' - Ihre Anfrage zum Fahrzeug '.$vehicle->post_title;

        return [
            'name' => 'vehicle',
            'label' => 'Fahrzeuganfrage',
            'id' => $vehicle->ID,
            'data' => $vehicle,
            'location' => $vehicle->dxim_meta['ad_location'][0],
            'fields' => [
                'name' => [
                    'label' => 'Name', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_UNSAFE_RAW,
                        'options' => ['min_range' => 1]
                    ]
                ],
                'mail' => [
                    'label' => 'E-Mail', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_VALIDATE_EMAIL,
                    ]
                ],
                'phone' => ['label' => 'Telefon', 'type' => 'text'],
                'message' => ['label' => 'Nachricht', 'type' => 'textarea'],
                'kind_of_request' => [
                    'label' => 'Interessiert an', 'type' => 'checkbox',
                    'values' => ['Fahrzeugangebot', 'Probefahrt', 'Rückruf', 'Finanzierungsangebot', 'Versicherungsangebot']
                ]
            ],
            'form_context_html' => $html,
            'mail_subject' => $mail_subject
        ];
    }

    /**
     * @param $offer_id
     * @return array
     */
    private function _get_wpcs_offer_context($offer_id = 0) {
        $offer = null;

        if($offer_id > 0) {
            $od = new \DXIM\theme\OfferData();
            $offers = $od->get_offers(['id' => $offer_id]);
            $offer = $offers[0] ?? null;
        }

        if(empty($offer)){
            $offer = \DXIM\theme\Offer::getInstance()->getCurrent();
        }

        $od = new OfferData();
        $brand = $od->get_brand($offer->ID);
        $location_query = [];
        if(!empty($brand)) $location_query['brand'] = [$brand->term_id];

        $src = get_the_post_thumbnail_url($offer->ID, 'full');
        $context_html = null;
        if(!empty($src)) $context_html .= '<div><img src="'.$src.'" /></div>';
        $context_html .= "<div><strong>Angebot</strong><br />".$offer->post_title.'</div>';

        //
        $mail_subject = get_bloginfo('name').' - Ihre Anfrage zum Angebot '.$offer->post_title;

        return [
            'name' => \DXIM\theme\Offer::$post_type,
            'label' => 'Angebotsanfrage',
            'data' => $offer,
            'id' => $offer->ID,
            'location' => 0,
            'location_query' => $location_query,
            'fields' => [
                'name' => [
                    'label' => 'Name', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_UNSAFE_RAW,
                        'options' => ['min_range' => 1]
                    ]
                ],
                'mail' => [
                    'label' => 'E-Mail', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_VALIDATE_EMAIL,
                    ]
                ],
                'phone' => ['label' => 'Telefon', 'type' => 'text'],
                'message' => ['label' => 'Nachricht', 'type' => 'textarea'],
            ],
            'form_context_html' => $context_html,
            'mail_subject' => $mail_subject
        ];
    }


    private function _get_wpcs_location_context($term_id) {

        $ld = new LocationData();
        $term = $ld->getById($term_id);

        //
        $mail_subject = get_bloginfo('name').' - Ihre Anfrage an den Standort '.$term->name;

        return [
            'name' => \DXIM\theme\Location::$taxonomy,
            'label' => 'Standortanfrage',
            'data' => $term,
            'id' => $term->term_id,
            'location' => $term->term_id,
            'location_query' => [],
            'fields' => [
                'name' => [
                    'label' => 'Name', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_UNSAFE_RAW,
                        'options' => ['min_range' => 1]
                    ]
                ],
                'mail' => [
                    'label' => 'E-Mail', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_VALIDATE_EMAIL,
                    ]
                ],
                'phone' => ['label' => 'Telefon', 'type' => 'text'],
                'message' => ['label' => 'Nachricht', 'type' => 'textarea'],
            ],
            'form_context_html' => "",
            'mail_subject' => $mail_subject
        ];
    }

    /**
     * @param $model_id
     * @return array
     */
    private function _get_wpcs_model_context($model_id = 0) {
        $model = null;

        if($model_id > 0) {
            $md = new \DXIM\theme\ModelData();
            $models = $md->get_models(['id' => $model_id]);
            $model = $models[0] ?? null;
        }

        if(empty($model)){
            $model = \DXIM\theme\Model::getInstance()->getCurrent();
        }

        $md = new ModelData();
        $brand = $md->get_brand($model->ID);
        $location_query = [];
        if(!empty($brand)) $location_query['brand'] = [$brand->term_id];

        $src = get_the_post_thumbnail_url($model->ID, 'full');
        $context_html = null;
        if(!empty($src)) $context_html .= '<div><img src="'.$src.'" /></div>';
        $context_html .= "<div><strong>Modell</strong><br />".$model->post_title.'</div>';

        $mail_subject = get_bloginfo('name').' - Ihre Anfrage zum Modell '.$model->post_title;

        return [
            'name' => \DXIM\theme\Model::$post_type,
            'label' => 'Modellanfrage',
            'data' => $model,
            'id' => $model->ID,
            'location' => 0,
            'location_query' => $location_query,
            'fields' => [
                'name' => [
                    'label' => 'Name', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_UNSAFE_RAW,
                        'options' => ['min_range' => 1]
                    ]
                ],
                'mail' => [
                    'label' => 'E-Mail', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_VALIDATE_EMAIL,
                    ]
                ],
                'phone' => ['label' => 'Telefon', 'type' => 'text'],
                'message' => ['label' => 'Nachricht', 'type' => 'textarea'],
                'kind_of_request' => [
                    'label' => 'Interessiert an', 'type' => 'checkbox',
                    'values' => ['Fahrzeugangebot', 'Probefahrt', 'Rückruf', 'Finanzierungsangebot', 'Versicherungsangebot']
                ]
            ],
            'form_context_html' => $context_html,
            'mail_subject' => $mail_subject
        ];
    }

    /**
     *
     */
    private function _get_wpcs_service_context($service_id = 0) {
        $service = null;

        if($service_id > 0) {
            $md = new \DXIM\theme\ServiceData();
            $services = $md->get_services(['id' => $service_id]);
            $service = $services[0] ?? null;
        }

        if(empty($service)){
            $model = \DXIM\theme\Service::getInstance()->getCurrent();
        }

        // limit to locations
        $sd = new ServiceData();
        $locations = $sd->get_locations($service_id);
        if(!is_array($locations)) $locations = [];
        $location_ids = [];
        foreach($locations as $location) {
            $location_ids[] = $location->term_id;
        }
        $location_query = ['include' => $location_ids];

        // Vorschau Html
        $src = get_the_post_thumbnail_url($service->ID, 'full');
        $context_html = null;
        if(!empty($src)) $context_html .= '<div><img src="'.$src.'" /></div>';
        $context_html .= "<div><strong>Service</strong><br />".$service->post_title.'</div>';

        // Betreff
        $mail_subject = get_bloginfo('name').' - Ihre Serviceanfrage für '.$service->post_title;

        return [
            'name' => \DXIM\theme\Model::$post_type,
            'label' => 'Serviceanfrage',
            'data' => $service,
            'id' => $service->ID,
            'location' => 0,
            'location_query' => $location_query,
            'fields' => [
                'name' => [
                    'label' => 'Name', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_UNSAFE_RAW,
                        'options' => ['min_range' => 1]
                    ]
                ],
                'mail' => [
                    'label' => 'E-Mail', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_VALIDATE_EMAIL,
                    ]
                ],
                'phone' => ['label' => 'Telefon', 'type' => 'text'],
                'message' => ['label' => 'Nachricht', 'type' => 'textarea'],
                'datetime' => ['label' => 'Wunschtermin & Uhrzeit', 'type' => 'datetime']
            ],
            'form_context_html' => $context_html,
            'mail_subject' => $mail_subject
        ];
    }

    /**
     * @return array
     */
    private function _get_general_context() {

        $mail_subject = get_bloginfo('name').' - Vielen Dank für Ihre Anfrage';

        return [
            'name' => 'general',
            'label' => 'Allgemeine Anfrage',
            'data' => [],
            'location' => 0,
            'id' => 0,
            'fields' => [
                'name' => [
                    'label' => 'Name', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_UNSAFE_RAW,
                        'options' => ['min_range' => 1]
                    ]
                ],
                'mail' => [
                    'label' => 'E-Mail', 'type' => 'text',
                    'validation' => [
                        'filter' => FILTER_VALIDATE_EMAIL,
                    ]
                ],
                'phone' => ['label' => 'Telefon', 'type' => 'text'],
                'message' => ['label' => 'Nachricht', 'type' => 'textarea'],
            ],
            'mail_subject' => $mail_subject
        ];
    }

}