<?php
namespace DXIM;

class Filter {

	protected static $instance = null;

    protected static $missingFiltersOptionName = "wpcs_missing_filters";

	protected $filters = [];

    protected $missingFilters = [];

    protected $columns = [];

	/**
	 *
	 * @return Filter
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {}

    /**
     * @return void
     */
    public function init() {
        $this->setFilters();
    }


    /**
     *
     */
    public function __destruct() {
        if(!empty($this->missingFilters)){
            update_option(self::$missingFiltersOptionName, $this->missingFilters);
        }
    }

    /**
     * @return array
     */
    public function getFilters() {
        return $this->filters;
    }


    /**
     * definition aller Felder, nach denen gefiltert werden kann (zum Aufbau der Tabelle)
     * nimmt alle benutzerdefinierten Felder
     * stellt sicher, dass auch immer alle OrderFilter existieren
     * @return void
     */
    public function setFilters() {
        $options = Admin::getInstance()->getOptions();

        // als filler deklarierte felder abfragen
        $filters = $options['active_filters'] ?? [];

        // im Formular verwendetet Felder ebenfalls als aktiv schalten
        $usedFilters = $options['vehicle_filter'] ?? [];
        foreach($usedFilters as $colFilters) {
            foreach($colFilters as $filterKey) {
                if(!in_array($filterKey, $filters)) $filters[] = $filterKey;
            }
        }

        // alle filter nach denen sortiert werden kann ebenfalls aktiv schalten
        foreach(VehicleFilter::getOrderFilter() as $filterKey => $tmp) {
            if(!isset($tmp['ignore_in_filter_table']) && !in_array($filterKey, $filters)) $filters[] = $filterKey;
        }

        // _min & _max felder werden an dieser stelle ignoriert, da diese keine eigenen Felder darstellen
        $filters = array_filter($filters, function($filterKey) {
            $minMax = substr($filterKey,-4);
            if($minMax == "_min" || $minMax == "_max") return false;
            return true;
        });


        // felder. die für WPCarSync spezifische interne vorgänge als Filter benötigt werden
        $forcedFields = array_keys(array_filter(VehicleFields::getFields(), function($field) {
            return (isset($field['in_filter_table']) && intval($field['in_filter_table']) > 0);
        }));


        $filters = array_merge($filters, $forcedFields);
        $this->filters = $filters;
    }


    /**
     * @param $key
     * @return int
     */
    public function isFilterable($key) {
        $field = VehicleFields::getField($key);
        $field_name = $field['meta_query']['key'];

        if(VehicleFields::isFeature($key)) {
            $db_fields = Filter::getPossibleFields($key);
            foreach($db_fields as $db_field) {
                if(!$this->_isFieldFilterable(strtolower($db_field))) return false;
            }
            return true;
        }

        else {

            // array aus mehreren feldnamen filtern
            if(is_array($field_name)) {
                foreach($field_name as $field_name_tmp) {
                    if(!$this->_isFieldFilterable($field_name_tmp)) {
                        return false;
                    }
                }
                return true;
            }

            // standard feld filtern
            else {
                return $this->_isFieldFilterable($field_name);
            }
        }

    }

    private function _isFieldFilterable($field_name) {
        #print "prüfe feld ".$field_name."<br />";
        $field_name = str_replace(['_min', '_max'], "", $field_name);
        return FilterTable::getInstance()->isColumn($field_name);
    }

    /**
     * liefert min und max Werte zu einem Filter (Range / Slider)
     * @param $cf_key
     * @return array
     */
    public function getMinMax($field_name) {
        global $wpdb;

        #$field = VehicleFields::getField($filterKey);
        #$field_name = $field['meta_query']['key'];
        #wpcs_p($field_name);
        #return;

        $min = null;
        $max = null;

        if(is_array($field_name)) {
            foreach($field_name as $field_name_tmp) {
                $res = Filter::getInstance()->getMinMax($field_name_tmp);
                if($min == null || $res['min'] < intval($min)) $min = $res['min'];
                if($max == null || $res['max'] > intval($max)) $max = $res['max'];
            }
        }

        else {
            $sql = "SELECT MIN(".$field_name.") as min, MAX(".$field_name.") as max FROM ".$wpdb->prefix.FilterTable::$table_name;
            $results = $wpdb->get_results($sql, ARRAY_N);
            $res = $results[0] ?? [0 => 0, 1 => 0];
            sort($res);
            $min = intval($res[0]);
            $max = intval($res[1]);
        }

        return ['min' => $min, 'max' => $max];
    }

    /**
     * eigentliche Abfrage der Fahrzeuge aus der Datenbank
     * @param $args
     * @param $filterData
     * @return void
     */
    public function queryVehicles($args) {
        global $wpdb;
        $options = Admin::getInstance()->getOptions();
        if(!$options) $options = [];

        /**
         * defaults
         */
        if(!isset($args['order'])) $args['order'] = $filterData['order'] ?? $options['archive_default_order'] ?? '';
        if(!isset($args['orderby'])) $args['orderby'] = $filterData['orderby'] ?? $options['archive_default_orderby'] ?? '';

        $forced = ['post_type' => 'vehicle', 'post_status' => 'publish'];
        $defaults = [ 'posts_per_page' => $options['posts_per_page'] ?? 20 ];
        $args = array_merge($defaults, $args, $forced);

        // query args werden für die WP_Query verwendet (post type etc...)
        // ansonsten keine Logik da alles per post_i_in definiert ist
        $query_args = array_merge($defaults, $forced);

        // extrahiere die eigentlich zu filternden Daten
        $filterData = VehicleFilter::getInstance()->formdata_to_filter_array($args);

        // pagination
        $args['wpcsp'] = $args['wpcsp'] ?? 1;
        $offset = ($args['wpcsp']-1) * $args['posts_per_page'];

        $sql = "
            SELECT SQL_CALC_FOUND_ROWS post_id 
            FROM ".$wpdb->prefix."wpcarsync_vehicle_filter 
            LEFT JOIN ".$wpdb->prefix."posts
            ON ".$wpdb->prefix."posts.ID = ".$wpdb->prefix.FilterTable::$table_name.".post_id
            WHERE post_status = 'publish' AND post_type = 'vehicle'";

        #wpcs_p($filterData);

        // append all field - filters
        foreach($filterData as $filterKey => $value) {
            if(!$this->isFilterable($filterKey)) continue;
            $sql .= $this->filterToSqlConditions($filterKey, $value);
        }

        // add exclude for specific IDs
        if(isset($args['exclude']) && is_array($args['exclude']) && !empty($args['exclude'])) {
            $sql .= " AND post_id NOT IN (".implode(",",$args['exclude']).")";
        }

        // add includefor specific IDs
        elseif(isset($args['include']) && is_array($args['include']) && !empty($args['include'])) {
            $sql .= " AND post_id IN (".implode(",",$args['include']).")";
        }

        // add custom condition to sql
        $sql = apply_filters(DXIM_FILTER_VEHICLE_SQL_ADD_CONDITION, $sql);

        // Order By
        $orderArray = VehicleFilter::getOrderFilter();
        $orderby = $orderArray[$args['orderby']]['key'] ?? "consumer_price";
        $order = $args['order'];
        if(is_array($orderby)) {
            $cond = $order == "asc" ? "<" : ">";
            $sqlOrder = "
            CASE
                WHEN ".$orderby[0]." ".$cond." ".$orderby[1]." THEN ".$orderby[0]."
                ELSE ".$orderby[1]."
            END ".$order;
        }
        else $sqlOrder = $orderby." ".$order;
        $sql .= " ORDER BY ".apply_filters(DXIM_FILTER_VEHICLE_QUERY_ORDERBY_SQL,$sqlOrder, $orderby, $order);

        // LIMIT
        if($args['posts_per_page'] > 0) $sql .= " LIMIT ".$offset.", ".$args['posts_per_page'];

        #print $sql;

        // get post ids
        $results = $wpdb->get_results($sql, ARRAY_A );
        $post_ids = [];
        foreach($results as $tmp) { $post_ids[] = $tmp['post_id']; }
        if(empty($post_ids)) return ['count' => 0, 'num_pages' => 0, 'vehicles' => [], 'posts_per_page' => $args['posts_per_page']];

        // get max count from last seaarch
        $found_rows = $wpdb->get_results("SELECT FOUND_ROWS() as count", ARRAY_A);
        $query['count'] = $found_rows[0]['count'];

        // num_pages
        $query['num_pages'] = ceil($query['count'] / $args['posts_per_page']);
        if($query['num_pages'] < 1) $query['num_pages'] = 1;

        //
        $query['posts_per_page'] = $args['posts_per_page'];

        // get the posts
        $query['vehicles'] = [];
        if(!empty($post_ids)) {
            $query_args['post__in'] = $post_ids;
            $query_args['posts_per_page'] = -1;
            $query_args['orderby'] = 'post__in';
            $q = new \WP_Query($query_args);
            $query['vehicles'] = $q->posts;
        }

        return apply_filters(DXIM_FILTER_QUERIED_VEHICLES,$query);
    }

    /**
     * verknüpft alle Filter aus dem Formular in den SQL, damit nur Ergebnisse gefunden werden
     * die auf alle FilterKriterien passen
     * @param $filterKey
     * @param $value
     * @param $field
     * @return string
     */
    private function filterToSqlConditions($filterKey, $value, $condition = "AND") {

        $field = VehicleFields::getField($filterKey);
        $compare = $field['meta_query']['compare'] ?? "=";
        $fieldType = $field['field_type'] ?? "text";
        $fieldName = $field['meta_query']['key'];
        $sql = !empty($condition) ? " ".$condition." " : null;

        if(is_array($value)) {
            $sql .= " ( ";
            $z = 0;
            foreach($value as $tmpVal) {
                $cond = $z == 0 ? null : "OR";
                $sql .= $this->filterToSqlConditions($filterKey, $tmpVal, $cond);
                $z++;
            }
            $sql .= " ) ";
        }
        else {

            // features => bool
            if(VehicleFields::isFeature($filterKey)) {
                $key = ($value == 1) ? $filterKey : strtolower($value);
                $sql .= $key." = 1 ";
            }
            // date
            elseif($fieldType == "date") {
                $sql .= "YEAR(".$fieldName.") ".$compare." ".$value." ";
            }
            // int / decimal
            elseif($fieldType == "int" || $field == "decimal") {
                if(is_array($fieldName)) {
                    $sql .= "(";
                    $z = 0;
                    foreach($fieldName as $fieldNameTmp) {
                        if($z != 0) $sql .= " OR ";
                        $sql .= "`".$fieldNameTmp."` ".$compare." ".$value." ";
                        $z++;
                    }
                    $sql .= ")";
                }
                else $sql .= "`".$fieldName."` ".$compare." ".$value." ";
            }
            // text LIKE Vergleich
            elseif($compare == "LIKE") {
                $sql .= "`".$fieldName."` ".$compare." '%".$value."%' ";
            }
            // text = Vergleich
            else {
                $sql .= "`".$fieldName."` ".$compare." '".$value."' ";
            }
        }

        return $sql;
    }


    /**
     * liefert alle möglichen Werte aus dem Live-Bestand der DB zu einem übergebenen FilterKey
     * @param $filterkey
     * @param $formData
     * @return array
     */
    #/*
    public function getFilterValues($filterKey, $formData) {
        if(!$this->isFilterable($filterKey)) return [];
        $filterData = VehicleFilter::getInstance()->formdata_to_filter_array($formData);
        $fields = VehicleFields::getFields();

        // Features
        if(VehicleFields::isFeature($filterKey)) {
            return $this->getFilterValuesFeature($filterKey, $filterData, $fields[$filterKey]);
        }
        // alle anderen Filter
        else {
            $field = VehicleFields::getField($filterKey);
            $field_name = $field['meta_query']['key'];
            $res = $this->queryFilterValues($field_name, $filterData, $fields[$filterKey]);
            $values = [];
            foreach($res as $array) {
                if($array[0] == "") continue;
                $values[] = $array[0];
            }
            return $values;
        }
    }


    /**
     * sucht alle verfügbaren Werte eines Filters im Live-Bestand der DB
     * @param $filterKey
     * @param $filterData
     * @param $field
     * @return array|object|\stdClass[]|null
     */
    #private function queryFilterValues($filterKey, $filterData, $field) {
    private function queryFilterValues($field_name, $filterData, $field) {
        global $wpdb;
        #$filter_name = $field['meta_query']['key'];
        $sql = "
        SELECT `".$field_name."` as field FROM ".$wpdb->prefix."wpcarsync_vehicle_filter
        LEFT JOIN ".$wpdb->prefix."posts ON ".$wpdb->prefix."wpcarsync_vehicle_filter.post_id = ".$wpdb->prefix."posts.ID
        WHERE post_status = 'publish' AND post_type = 'vehicle'";

        foreach($filterData as $key => $value) {
            if(!$this->isFilterable($key)) {
                continue;
            }
            $sql .= $this->filterToSqlConditions($key, $value);
        }

        $sql .= " GROUP BY field ";

        #print $sql;

        return $wpdb->get_results($sql, ARRAY_N );

    }

    /**
     * sucht alle verfügbaren Werte des angegebenen feature keys in der db
     * loopt durch alle möglichen Felder
     * @param $filterKey
     * @param $filterData
     * @param $field
     * @return array
     */
    private function getFilterValuesFeature($filterKey, $filterData, $field) {
        $db_fields = self::getPossibleFields($filterKey);
        $values = [];
        foreach($db_fields as $key) {
            $db_key = strtolower($key);
            $res = $this->queryFilterValues($db_key, $filterData, $field);
            foreach($res as $tmp) {
                if(in_array("1",$tmp)) $values[] = $key;
            }
        }

        #wpcs_p($values);
        return $values;
    }

    /**
     *
     * @param $key
     * @return array
     * liefert alle möglochen Datenbank felder zu einem feature Feld
     */
    public static function getPossibleFields($key) {
        $field = VehicleFields::getField($key);
        $defaults = $field['fillable']['defaults'] ?? [$key => 0];
        return array_keys($defaults);
    }

    /**
     * @param $filterKey
     * @return void
     */
    public function registerMissingFilter($filterKey) {
        $this->missingFilters[$filterKey] = 1;
    }

    /**
     * @return false|mixed|void
     */
    public function getMissingFilters() {
        return get_option(self::$missingFiltersOptionName);
    }

    /**
     * @return void
     */
    public function deleteMissingFilters() {
        delete_option(self::$missingFiltersOptionName);
    }


}