<?php


namespace DXIM;


class FilterElementColor {

    protected $values = [
        'WHITE' => ['colorcode' => '#fff', 'label' => 'Weiß'],
        'BLACK' => ['colorcode' => '#000', 'gradient' => [0 => ['#474747', 0], 1 => ['#191919', 100]], 'label' => 'Schwarz'],
        'GREY' => ['colorcode' => '#aaaaaa', 'gradient' => [0 => ['#aaa', 0], 1 => ['#555',100]], 'label' => 'Grau'],
        'SILVER' => [ 'colorcode' => '#C0C0C0', 'gradient' => [0 => ['rgba(204,204,204,1)', 35], 1 => ['rgba(236,236,236,1)',50], 2 => ['rgba(204,204,204,1)',65]], 'label' => 'Silber'],
        'GOLD' => [ 'colorcode' => '#FFD700', 'gradient' => [0 => ['#ffd905', 0], 1 => ['#fff291',50], 2 => ['#ffd905',100]], 'label' => 'Gold'],
        'YELLOW' => ['colorcode' => '#ffff88', 'gradient' => [0 => ['#ffff88', 0], 1 => ['#ffdd00',100]], 'label' => 'Gelb'],
        'ORANGE' => [ 'colorcode' => '#fab421', 'gradient' => [0 => ['#fab421', 0], 1 => ['#e3902a',100]], 'label' => 'Orange'],
        'BEIGE' => ['colorcode' => '#fceccc', 'gradient' => [0 => ['#fceccc', 0], 1 => ['#ccbb99',100]], 'label' => 'Beige'],
        'BROWN' => ['colorcode' => '#b97a57', 'gradient' => [0 => ['#b97a57', 0], 1 => ['#793a27',100]], 'label' => 'Braun'],
        'RED' => ['colorcode' => '#fc4643', 'gradient' => [0 => ['#fc4643', 0], 1 => ['#9f1429',100]], 'label' => 'Rot'],
        'PURPLE' => [ 'colorcode' => '#935d93', 'gradient' => [0 => ['#935d93', 0], 1 => ['#592472',100]], 'label' => 'Lila'],
        'BLUE' => [ 'colorcode' => '#4492d0', 'gradient' => [0 => ['#4492d0', 0], 1 => ['#204a86',100]], 'label' => 'Blau'],
        'GREEN' => [ 'colorcode' => '#c4e890', 'gradient' => [0 => ['#c4e890', 0], 1 => ['#679e1e',100]], 'label' => 'Grün'],
    ];

    /**
     * @param $values
     * @param $filterkey
     * @param $current_value
     */
    public function draw($filterkey, $current_value ) {
        if(!is_array($current_value)) $current_value = [$current_value];
        $values = Filter::getInstance()->getFilterValues('exterior_color',[]);

        $html = '<div class="wpcs_color_select">';

            foreach($this->values as $color_key => $color) {
                if(!in_array($color_key, $values)) continue;
                $css = isset($color['gradient']) ? $this->_to_css_gradient($color['gradient']) : $color['colorcode'];
                $active = in_array($color_key,$current_value) ? "active" : null;
                $html .= '<div title="'.$color['label'].'" class="color '.$active.' '.$color_key.'" data-key="'.$color_key.'" style="background:'.$css.';"><span class="dashicons dashicons-saved"></span></div>';
            }

            $html .= '<input type="hidden" class="color_value" value="'.implode(',',$current_value).'" name="'.$filterkey.'" />';

        $html .= '</div>';

        return $html;
    }

    /**
     * @param $gradient
     * @return string|null
     */
    private function _to_css_gradient($gradient) {
        $css = 'linear-gradient(120deg, ';
        $z = 0;
        foreach($gradient as $tmp) {
            if($z != 0) $css .= ", ";
            $css .= $tmp[0].' '.$tmp[1].'%';
            $z++;
        }
        $css .= ")";
        return $css;
    }

}