<?php
namespace DXIM;

class FilterTable {

    public static $table_name = "wpcarsync_vehicle_filter";

    public $columns = [];

    protected static $instance = null;

    /**
	 *
	 * @return FilterTable
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

    /**
     * @return false|void
     */
    private function getColumns() {
        global $wpdb;
        if(!FilterTable::getInstance()->tableExists()) return false;

        if(!empty($this->columns)) return $this->columns;

        $sql = " SHOW COLUMNS FROM ".$wpdb->prefix.FilterTable::$table_name;
        $res = $wpdb->get_results($sql);
        $columns = [];
        foreach($res as $field) {
            $columns[] = $field->Field;
        }

        $this->columns = $columns;
        return $this->columns;
    }


    /**
     * @param $key
     * @return bool
     */
    public function isColumn($key) {
        $columns = $this->getColumns();
        if(is_array($key)) {
            foreach($key as $field_name) {
                if(!in_array($field_name, $columns)) return false;
            }
            return true;
        }

        return in_array($key, $columns);
    }

    /**
     * @return void
     */
    public function tableExists() {
        global $wpdb;
        $query = "SHOW TABLES LIKE '".$wpdb->prefix.self::$table_name."'";
        $table = $wpdb->get_results($query, ARRAY_A);

        return !empty($table);
    }

    /**
     * @return void
     */
    public function resetFilterTable() {

        // generire Tabelle neu anhand übergebener Filter
        $this->createTable();

        // fülle die Tabelle mit values
        $this->fillTable();
    }

    /**
     * setzt filter fü+r alle Fahrzeuge der DB
     * @return void
     */
    private function fillTable() {
        global $wpdb;

        $sql = "SELECT ID FROM ".$wpdb->prefix."posts WHERE post_status = 'publish' AND post_type = 'vehicle'";
        $posts = $wpdb->get_results($sql, ARRAY_A);

        foreach($posts as $post) {
            $post_id = $post['ID'];
            $this->setFiltersForVehicle($post_id);
        }
    }

    /**
     * setzt Filter für die übergebene Fahrzeug ID
     * @param $post_id
     * @return void
     */
    public function setFiltersForVehicle($post_id) {
        global $wpdb;

        // delete existing filters fir vehicle
        $this->deleteFilters($post_id);

        $fields = VehicleFields::getFields();
        $filters = Filter::getInstance()->getFilters();

        $sql = "INSERT INTO ".$wpdb->prefix.self::$table_name." SET post_id = ".$post_id;
        $sql .= "\n";

        $fields_done = [];
        foreach($filters as $filterKey) {
            $field = $fields[$filterKey] ?? null;
            if(empty($field)) continue;
            $meta_key = $field['meta_query']['key'] ?? '';
            $field_type = $field['field_type'] ?? 'text';

            if(empty($meta_key)) continue;

            if(VehicleFields::isFeature($filterKey)) {
                $db_fields = Filter::getPossibleFields($filterKey);
                foreach($db_fields as $db_field_name) {
                    if(in_array($db_field_name, $fields_done)) continue;
                    $sql .= ", `".strtolower($db_field_name)."` = (SELECT COUNT(post_id) FROM ".$wpdb->prefix."postmeta WHERE post_id = ".$post_id." AND meta_key = '".$meta_key."' AND meta_value = '".$db_field_name."' LIMIT 1)";
                    $sql .= "\n";
                    $fields_done[] = $meta_key;
                }
            }
            elseif($field_type == "date") {
                if(in_array($meta_key, $fields_done)) continue;
                $sql .= ", `".$meta_key."` = CONCAT((SELECT meta_value FROM ".$wpdb->prefix."postmeta WHERE post_id = ".$post_id." AND meta_key = '".$meta_key."' LIMIT 1), '-01')";
                $sql .= "\n";
                $fields_done[] = $meta_key;

            }
            elseif($field_type == "bool") {
                if(in_array($meta_key, $fields_done)) continue;
                $sql .= ", `".$meta_key."` = (SELECT COUNT(meta_value) FROM ".$wpdb->prefix."postmeta WHERE post_id = ".$post_id." AND meta_key = '".$meta_key."' LIMIT 1)";
                $sql .= "\n";
                $fields_done[] = $meta_key;
            }
            else {
                if(is_array($meta_key)) {
                    foreach($meta_key as $meta_key_tmp) {
                        if(in_array($meta_key_tmp, $fields_done)) continue;
                        $sql .= ", `".$meta_key_tmp."` = (SELECT meta_value FROM ".$wpdb->prefix."postmeta WHERE post_id = ".$post_id." AND meta_key = '".$meta_key_tmp."' LIMIT 1)";
                        $sql .= "\n";
                        $fields_done[] = $meta_key_tmp;
                    }
                } else {
                    if(in_array($meta_key, $fields_done)) continue;
                    $sql .= ", `".$meta_key."` = (SELECT meta_value FROM ".$wpdb->prefix."postmeta WHERE post_id = ".$post_id." AND meta_key = '".$meta_key."' LIMIT 1)";
                    $sql .= "\n";
                    $fields_done[] = $meta_key;
                }
            }
        }

        #print $sql."\n\n";
        $wpdb->query($sql);
    }



    /**
     * generiert die Filtertabelle anhand der aktuelle aktiven Filter
     * @return bool|int|\mysqli_result|resource|null
     */
    private function createTable() {
        global $wpdb;

        $this->dropTable();

        $fields = VehicleFields::getFields();
        $filters = Filter::getInstance()->getFilters();
        $filters[] = "model";
        $fields_done = [];

        $sql = "CREATE TABLE ".$wpdb->prefix.self::$table_name." ( \n";
        $sql .= "post_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY\n";

        foreach($filters as $filterKey) {
            $field = $fields[$filterKey] ?? null;
            if(empty($field)) continue;
            $field_type = $field['field_type'] ?? "text";
            $field_name = $field['meta_query']['key'] ?? '';

            if(VehicleFields::isFeature($filterKey)) {
                $db_fields = Filter::getPossibleFields($filterKey);
                foreach($db_fields as $db_field_name) {
                    $sql .= "\n, ";
                    $sql .= "`".strtolower($db_field_name)."` BOOLEAN NOT NULL DEFAULT 0";
                }
            }
            else {
                if(empty($field_name)) continue;

                if($field_type == "text") $sqlExt = "VARCHAR(255)";
                elseif($field_type == "int") $sqlExt = "INT";
                elseif($field_type == "bool") $sqlExt = "BOOLEAN NOT NULL DEFAULT 0";
                elseif($field_type == "date") $sqlExt = "DATE";
                elseif($field_type == "enum") {
                    $defaults = $field['fillable']['defaults'] ?? [];
                    $values = array_map(function($val) { return "'".$val."'"; },array_keys($defaults));
                    $enum_values = implode(",", $values);
                    $sqlExt = "ENUM(".$enum_values.")";
                }

                if(is_array($field_name)) {
                    foreach($field_name as $field_name_tmp) {
                        if(in_array($field_name_tmp, $fields_done)) continue;
                        $sql .= "\n, `".$field_name_tmp."` ".$sqlExt;
                        $fields_done[] = $field_name_tmp;
                    }
                }

                else {
                    if(in_array($field_name, $fields_done)) continue;
                    $sql .= "\n, `".$field_name."` ".$sqlExt;
                    $fields_done[] = $field_name;
                }
            }
        }
        $sql .= ");";

        return $wpdb->query($sql);
    }

    /**
     * löscht die aktuelle Filtertabelle
     * wird nur verwendet, bevor die Tabelle neu generiert wird
     * @return bool|int|\mysqli_result|resource|null
     */
    private function dropTable() {
        global $wpdb;
        if(!$this->tableExists()) return;

        $sql = "DROP TABLE ".$wpdb->prefix.self::$table_name.";";
        return $wpdb->query($sql);
    }


    /**
     * @return bool|int|\mysqli_result|resource|null
     */
    private function truncateTable() {
        global $wpdb;
        $sql = "TRUNCATE TABLE ".$wpdb->prefix.self::$table_name;
        return $wpdb->query($sql);
    }

    /**
     * @param $post_id
     * @return bool|int|\mysqli_result|resource|null
     */
    public function deleteFilters($post_id) {
        global $wpdb;
        $sql = "DELETE FROM ".$wpdb->prefix.FilterTable::$table_name." WHERE post_id = ".$post_id;
        return $wpdb->query($sql);
    }

    /**
     * @param $vehicle_id
     * @return array|mixed
     */
    public function getByVehicle($vehicle_id) {
        global $wpdb;
        $sql = "SELECT * FROM ".$wpdb->prefix.FilterTable::$table_name." WHERE post_id = %d";
        $res = $wpdb->get_results($wpdb->prepare($sql,$vehicle_id), ARRAY_A);
        return $res[0] ?? [];
    }

    /**
     * @param $post_id
     * @param $meta_key
     * @param $meta_value
     * @return bool|int|\mysqli_result|resource|void|null
     */
    public function updateField($post_id, $meta_key, $meta_value) {
        global $wpdb;

        $db_value = !empty($meta_value) ? "'".$meta_value."'" : 'null';
        $sql = "UPDATE ".$wpdb->prefix.FilterTable::$table_name." SET `".$meta_key."` = ".$db_value." WHERE post_id = ".$post_id;
        return $wpdb->query($sql);
    }


}