<?php
namespace DXIM;

class GlobalFilter {

	protected static $instance = null;


	/**
	 *
	 * @return GlobalFilter
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	public function __construct() {

        add_action('add_meta_boxes', array(&$this, 'add_wpcs_query_meta_box'), 10, 2);
        add_action('save_post', array(&$this, 'save_wpcs_query_meta_box'));

        add_action('wp_ajax_handleGlobalFilters', [&$this, '_handleGlobalFilters']);

    }

    /**
     * @param $page_id
     * @return array|mixed|void
     */
    public function getGlobalFilters($page_id = 0) {
        $page_id = intval($page_id);
        if($page_id < 1) return [];

        $globalFilters = get_post_meta($page_id, 'wpcs_query', true);
        if(!is_array($globalFilters)) $globalFilters = [];
        return apply_filters(DXIM_FILTER_FORM_GLOBAL_FILTERS, $globalFilters);

    }

    /**
     * @param $post
     */
    public function add_wpcs_query_meta_box($post) {
        $options = Admin::getInstance()->getOptions();

        if (!isset($options['general_global_filter_activation'])) {
            $options['general_global_filter_activation'][] = 'page';
        }

        add_meta_box(
            'wpcs_query_meta_box',
            __(DXIM_PluginName . ' globale Einstellungen'),
            array(&$this, 'render_wpcs_query_meta_box'),
            $options['general_global_filter_activation'],
            'normal',
            'default'
        );
    }

    /**
     *
     */
    public function _handleGlobalFilters() {
        $data = [];

        // verfügbare filter anhanf suchstring laden
        if ($_POST['todo'] == "fetchFilters") {
            $val = $_POST['val'];
            $limit = $_POST['limit'] ?? "";

            foreach (VehicleFields::getFields() as $key => $filter) {
                $filter['label'] = $filter['label'] ?? "";
                if (stripos($filter['label'], $val) !== false) {
                    $is_allowed = $filter['filterform'] ?? true;
                    if(!$is_allowed) continue;

                    $type = null;
                    if (isset($filter['specific'])) $type = "specific";
                    if (isset($filter['feature'])) $type = "feature";
                    if (isset($filter['basic'])) $type = "basic";

                    if(!empty($limit) && $type != $limit) continue;

                    $tmp = [
                        'label' => $filter['label'],
                        'type' => $type,
                        'key' => $key
                    ];

                    $data[] = $tmp;
                }
            }
        }

        // filterelement anhand filterkey laden
        elseif ($_POST['todo'] == 'loadFilter') {
            $data['html'] = $this->_render_global_filter_element($_POST['key'], $_POST['post_id']);
        }


        // return data
        header('Content-Type: application/json');
        die(json_encode($data));
    }



    /**
     * @param $post_id
     */
    public function save_wpcs_query_meta_box($post_id) {
        if (!isset($_POST['save_wpcs_query'])) return;
        $args = [];

        if ($_POST['wpcs_global_filter_active'] == "Off") {
            // delete related fields
            $this->_delete_related_fields($post_id);
            // delete old query
            delete_post_meta($post_id, "wpcs_query");
        }
        else {
            // neue globale filter speichern
            if (is_array($_POST['wpcs_query'])) {
                // lösche derzeit aktive felder
                $this->_delete_related_fields($post_id);

                foreach ($_POST['wpcs_query'] as $key => $value) {
                    if ($value != "#" && !empty($value)) {
                        $args[$key] = $value;

                        // speichere jeden Wert einzeln für bessere sql queries
                        update_post_meta($post_id, $key, $value);
                    }
                }
            }

            // speichere alle felder einmal zusammen als query array
            update_post_meta($post_id, 'wpcs_query', $args);
        }

        // Feld existiert nur wenn globale Filter, nicht aber z.B. beim speichern eines Models
        if(isset($_POST['wpcs_global_filter_active'])) {
            update_post_meta($post_id, 'wpcs_global_filter_active', $_POST['wpcs_global_filter_active']);
        }
    }

    /**
     * @param $post_id
     * @return void
     */
    private function _delete_related_fields($post_id) {
        $wpcs_query = get_post_meta($post_id, "wpcs_query", true);

        // delete every single field
        if(is_array($wpcs_query)) {
            foreach($wpcs_query as $field_name => $value) {
                delete_post_meta($post_id, $field_name);
            }
        }

    }

    /**
     * @param $args
     * @param $post
     * @return string
     */
    public function render_global_filters($args, $post) {
        $html = null;

        $html .= '<div class="wpcs_ajax_loader"><div class="loading"></div></div>';

        $html .= '<strong>Filtername suchen</strong><br />
                <input type="text" id="wpcs_ajax_search_filter">';

        $html .= '<ul id="wpcs_ajax_search_filter_results">';
        $html .= '</ul>';

        $html .= '<div id="wpcs_ajax_search_active_filters">';
        $filters = VehicleFields::getFields();

        foreach ($args as $key => $value) {
            if (!isset($filters[$key])) continue;

            $html .= $this->_render_global_filter_element($key, $post->ID);
        }
        $html .= '</div>';

        $html .= '<input type="hidden" name="save_wpcs_query" value="1" />';

        return $html;
    }

    /**
     * @param $filterKey
     * @param $post_id
     * @return string
     */
    private function _render_global_filter_element($filterKey, $post_id) {
        $html = null;
        $filter = VehicleFields::getField($filterKey);
        $args = get_post_meta($post_id, 'wpcs_query', true);
        if (!is_array($args)) $args = [];

        if (isset($filter['data_source']) && is_array($filter['data_source'])) {
            if (!isset($filter['data_source']['depends'])) $filter['data_source']['depends'] = null;
            if (!isset($args[$filter['data_source']['depends']])) $args[$filter['data_source']['depends']] = null;

            $depends = $args[$filter['data_source']['depends']];
            if(empty($depends)) $depends = $filter['data_source']['depends_default'] ?? "";
            $value = $args[$filterKey] ?? null;
            if (!empty($value) && $value != "#") {
                $label = call_user_func($filter['data_source']['callback'], $value, $depends);
                $values[$value] = $label;
            } else {
                $values['#'] = DXIM_SELECT_NOTHING_SELECTED_TEXT;
            }
        } else {
            $values = $filter['fillable']['defaults'] ?? [];
        }

        $html .= '<div class="filter_item" data-key="' . $filterKey . '">';

        $html .= '<div class="filter_col">';

        // select
        if ($filter['type'] == "select") {

            $html .= '<label class="select" for="wpcs_query[' . $filterKey . ']">' . $filter['label'] . ' | <span class="dashicons dashicons-no-alt delete_filter"></span></label>';

            $multiple = (!isset($filter['multiple']) || intval($filter['multiple']) > 0) ? 1 : 0;
            $att_depends = isset($filter_depends) ? 'data-depends="' . $filter_depends . '"' : null;
            if (!isset($args[$filterKey])) $args[$filterKey] = null;

            $selected = explode(",", $args[$filterKey] ?? "");
            if (count($selected) > 1) $label = count($selected) . " ausgewählt";
            elseif (isset($values[$selected[0]])) $label = $values[$selected[0]];
            else $label = VehicleFilter::getInstance()->_get_nothing_selected_label($filter['label'], $filterKey);

            $html .= '<div class="wpcsselect ajax" data-field="' . $filterKey . '" data-nothing-selected="' . VehicleFilter::getInstance()->_get_nothing_selected_label($filter['label'], $filterKey) . '" data-multiple="' . $multiple . '" ' . $att_depends . ' name="wpcs_query[' . $filterKey . ']">';
            $html .= '<div class="wpcs_select_handle">' . $label . '</div>';
            $html .= '<div class="wpcs_select_loading"></div>';
            $html .= '<div class="wpcs_select_arrow"></div>';
            $html .= '<input type="hidden" id="wpcs_select_value_' . $filterKey . '" name="wpcs_query[' . $filterKey . ']" value="' . $args[$filterKey] . '" class="wpcs_select_value" />';
            $html .= '<div class="data">';
            $html .= '<div data-value="#">' . DXIM_SELECT_NOTHING_SELECTED_TEXT . '</div>';
            foreach ($values as $value => $label) {
                if (empty($value)) continue;
                if (!isset($args[$filterKey])) $args[$filterKey] = null;
                $html .= '<div data-value="' . $value . '">' . $label . '</div>';
            }
            $html .= '</div>';
            $html .= '</div>';
        }

        // feature muss nur vorhanden sein
        elseif (isset($filter['feature'])) {
            $html .= '<label class="feature" for="wpcs_query[' . $filterKey . ']">' . $filter['label'] . ' vorhanden | <span class="dashicons dashicons-no-alt delete_filter"></span></label>';
            $html .= '<input type="hidden" value="1" name="wpcs_query[' . $filterKey . ']" />';
        }

        // ansonsten freitext
        else {
            $form_element_value = $args[$filterKey] ?? null;
            $html .= '<label class="text" for="wpcs_query[' . $filterKey . ']">' . $filter['label'] . ' | <span class="dashicons dashicons-no-alt delete_filter"></span></label>';
            $html .= '<input type="text" value="' . $form_element_value. '" name="wpcs_query[' . $filterKey . ']" />';
        }

        $html .= '</div>'; // filter col

        $html .= '</div>';

        return $html;
    }



    /**
     * @param $post
     */
    public function render_wpcs_query_meta_box($post) {
        $html = null;

        $args = get_post_meta($post->ID, 'wpcs_query', true);
        if (!is_array($args)) $args = [];

        $html .= '<div class="wpcs_meta_box_wrapper">';
        $html .= '<p>Wenn diese Seite eine Fahrzeugübersicht enthält ([WPCS_archive], [WPCS_vehicles]), können Sie die Fahrzeugübersicht auf die folgenden Kriterien einschränken.<br >
		Möchten Sie z.B. eine Seite nur für Tageszulassungen, wählen sie unter Gebrauchtzustand -> Tageszulassung aus.<br />
		Diese Voreinstellungen können nicht vom Benutzer auf der Seite durch das Filterformular verändert werden.</p>';

        $html .= '<p>globale Filter für diese Seite aktivieren? ';
        $global_filter_active = get_post_meta($post->ID, 'wpcs_global_filter_active', true);

        $checked = empty($global_filter_active) || $global_filter_active == 'Off' ? 'checked="checked"' : null;
        $html .= 'Nein <input type="radio" ' . $checked . ' name="wpcs_global_filter_active" onClick="jQuery(\'.wpcs_global_filters_content\').hide();" value="Off" /> ';
        $checked = $global_filter_active == 'On' ? 'checked="checked"' : null;
        $html .= 'Ja <input type="radio" ' . $checked . ' name="wpcs_global_filter_active" onClick="jQuery(\'.wpcs_global_filters_content\').show();" value="On" />';
        $html .= '</p>';

        $display = ($global_filter_active == 'On') ? 'block' : 'none';
        $html .= '<div class="wpcs_global_filters_content" style="display:' . $display . ';">';

        /**
         * col 1
         */
        $html .= '<div class="wpcs_col wpcs_global_filters">';
        $html .= '<h3>Filter</h3>';
        $html .= $this->render_global_filters($args, $post);
        $html .= '</div>'; // col 1


        /**
         * Col 3
         */
        $html .= '<div class="wpcs_col">';

        $html .= '<h3>Sortierung</h3>';
        $html .= '<p>Geben Sie eine Sortierung vor, nach der die Fahrzeuge automatisch dargestellt werden. Leer lassen, um Standard-Reihenfolge zu verwenden.</p>';

        $html .= '<div class="filter_item">';
        $html .= '<label class="" for="wpcs_query[order]">Sortierreihenfolge</label>';

        $order_array = array('asc' => __('aufsteigend'), 'desc' => 'absteigend');
        if (!isset($args['order'])) $args['order'] = null;
        $handle = isset($order_array[$args['order']]) ? $order_array[$args['order']] : DXIM_SELECT_NOTHING_SELECTED_TEXT;
        $html .= '<div class="wpcsselect" data-field="order" name="wpcs_query[order]">';
        $html .= '<div class="wpcs_select_handle">' . $handle . '</div>';
        $html .= '<div class="wpcs_select_loading"></div>';
        $html .= '<div class="wpcs_select_arrow"></div>';
        $html .= '<input type="hidden" id="wpcs_select_value_order" name="wpcs_query[order]" value="' . $args['order'] . '" class="wpcs_select_value" />';
        $html .= '<div class="data">';
        $html .= '<div data-value="#">' . DXIM_SELECT_NOTHING_SELECTED_TEXT . '</div>';
        foreach ($order_array as $key => $label) {
            $html .= '<div data-value="' . $key . '">' . $label . '</div>';
        }
        $html .= '</div>';
        $html .= '</div>';

        $html .= '</div>';


        $html .= '<div class="filter_item">';
        $html .= '<label class="" for="wpcs_query[orderby]">Sortieren nach</label>';

        $filter_array = VehicleFilter::getOrderFilter();
        if (!isset($args['orderby'])) $args['orderby'] = null;
        $handle = !empty($args['orderby']) && isset($filter_array[$args['orderby']]) ? $filter_array[$args['orderby']]['label'] : DXIM_SELECT_NOTHING_SELECTED_TEXT;
        $html .= '<div class="wpcsselect" data-field="orderby">';
        $html .= '<div class="wpcs_select_handle">' . $handle . '</div>';
        $html .= '<div class="wpcs_select_loading"></div>';
        $html .= '<div class="wpcs_select_arrow"></div>';
        $html .= '<input type="hidden" id="wpcs_select_value_orderby" name="wpcs_query[orderby]" value="' . $args['orderby'] . '" class="wpcs_select_value" />';
        $html .= '<div class="data">';
        $html .= '<div data-value="#">' . DXIM_SELECT_NOTHING_SELECTED_TEXT . '</div>';
        foreach ($filter_array as $value => $array) {
            $html .= '<div data-value="' . $value . '">' . $array['label'] . '</div>';
        }
        $html .= '</div>';
        $html .= '</div>';
        $html .= '</div>';

        $html .= '</div>'; // col 3

        $html .= '<br class="clear" />';

        $html .= '</div>'; // wrapper

        print $html;
    }


}