<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 30.11.18
 * Time: 11:26
 */

namespace DXIM;


class FilesystemImport {


	protected static $instance = null;

	/**
	 *
	 * @return FilesystemImport
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

    /**
     * @return string
     */
    public function get_uploads_dir() {
        $dir = WPCS_UPLOADS_DIR.'/vehicles';

        if(!is_dir($dir)) mkdir($dir,0777,true);

        return $dir;
    }

	/**
	 * @return string
	 */
	public function get_import_dir() {
		$dir = apply_filters('wpcs_filesystem_import_dir', WPCS_UPLOADS_DIR.'/vehicles');

		if(!is_dir($dir)) mkdir($dir,0777,true);

		return $dir;
	}



	/**
     * @return string
     */
    public function get_uploads_url() {
        return WPCS_UPLOADS_URL.'/vehicles';
    }


    /**
     * @param $post_id
     * @param $images
     */
    public function import($post_id, $images, $image_name, $dir = null, $category = "images_filesystem") {
        if(empty($dir)) $dir = $this->get_import_dir();

        // copy local images to finale folder
        if(!preg_match('/^http:\/\/|https:\/\//', $images[0] )) {
            $files = $this->importFromLocal($images, $dir, $image_name);
        }

        // copy images from url
        else {
			$files = [];
            foreach($images as $index => $img) {
                $file_info = pathinfo($img);
                $extension = $file_info['extension'];
                $nr = $index+1;
                $filename = $image_name.'_'.$nr.'.'.$extension;
                $files[] = $this->importFromUrl($img,$dir,$filename);
            }

        }

	    update_post_meta($post_id, $category, $files);

	    do_action('wpcs_images_imported_filesystem',$post_id);

	    return $files;
    }

    /**
     * @param $images
     * @param $dir
     * @param $post_id
     * @return void
     */
    public function importFromLocal($images, $dir, $image_name) {
        $nr = 1;
		$files = [];
        foreach($images as $src_file) {
            $info = pathinfo($src_file);
			$final_file_name = $image_name.'_'.$nr.'.'.$info['extension'];
            $final_file = $dir."/".$final_file_name;
            if(is_file($src_file)) {
                rename($src_file, $final_file);
                $files[] = $final_file_name;
				$nr++;
            }
        }
		return $files;
    }

    /**
	 * @param $url
	 */
	public function importFromUrl($url, $to_dir, $filename) {
        $request = wp_remote_get($url);
		$image_data = $request['body'];
        $final_file = $to_dir."/".$filename;
        file_put_contents($final_file,$image_data);
		#print "speichere ".$url." nach ".$final_file."<br />";
		#if(is_file($final_file)) print $final_file." existiert<br />";
		return $filename;
	}

    /**
     * @param $image_name
     * @param $folder
     * @param $regExp
     * @param $return
     * @return array
     */
    public function getImagesByName($image_name, $folder = 'vehicles', $regExp = "_*.*", $return = 'url') {
        $dir = WPCS_UPLOADS_DIR.'/'.$folder;
        $url = WPCS_UPLOADS_URL."/".$folder;
        $filesTmp = glob(trailingslashit($dir) . $image_name . $regExp);

        $files = [];
        foreach($filesTmp as $file) {
            if(preg_match('/'.$image_name.'_([0-9]{1,2})\./', $file)) $files[] = $file;
        }

        if($return == "url") {
            $files = array_map(function ($n) use (&$dir,&$url) {
                return str_replace($dir, $url, $n);
            }, $files);
        }

        natsort($files);
        return array_values($files);
    }

    /**
     * @param $post_id
     * @return void
     */
    public function delete_vehicle_images($post_id) {
        $image_name = get_post_meta($post_id, "image_id", true);
        if(empty($image_name)) return;

        $files = glob(trailingslashit($this->get_uploads_dir()).$image_name."*.*");
        if(!is_array($files)) $files = [];

        $csp_files = glob(trailingslashit(CarShotsPROImport::getInstance()->get_dir()).$image_name."*.*");
        if(!is_array($csp_files)) $csp_files = [];

        $all_files = array_merge($files, $csp_files);
        foreach ($all_files as $file) {
            if(is_file($file)) unlink($file);
        }
    }

}