<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 08:35
 */

namespace DXIM;


class Import {

    protected static $instance = null;
    protected $error = null;
    protected $import_status_db_field = "wpcs_import_status";


    public function getError() { return $this->error; }

    /**
     *
     * @return Import
     */
    public static function getInstance() {

        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Sync constructor.
     */
    public function __construct() {

    }

    /**
     * @return false|string|null
     */
    public function create_new_import($account_ids = []) {
        $import_id = time();
        $a = Account::getInstance();
        $accounts = $a->get_accounts();
        $accounts_to_sync = [];

        // Import verzeichnis anlegen
        if(!$this->_create_import_dir($import_id)) {
            $this->error = "Konnte Import-Verzeichnis nicht anlegen";
        }

        // anhand des übergebenen paramteter können nur bestimmte accounts synchronisiert werden
        foreach($accounts as $id => $account) {
            if((!in_array($id, $account_ids) && !empty($account_ids)) || $id == "delete" ) continue;
            $accounts_to_sync[] = $id;
        }

        // mindestens 1 account muss zum sync vorhanden sein
        if(empty($accounts_to_sync)) {
            $this->error = "Keine Accounts zum Synchronisieren ausgewählt";
        }

        // wenn accounts übergeben wurden, farzeuge aller anderer account vor dem löschen schützen
        if(!empty($account_ids)) Delete::getInstance()->limitTo($import_id, $accounts_to_sync);

        // delete Account ergänzen damit Bereinigung läuft
        $accounts_to_sync[] = "delete";

        $import = [
            'import_id' => $import_id,
            'begin' => time(),
            'accounts' => $accounts_to_sync, // alle accounts die synchronisiert werden sollen
            'status' => [
                'index' => 0, // Das aktuelle Fahrzeug des aktiven accounts
                'account_index' => 0, // der aktive account
                'vehicles_count' => 0, // fahrzeuge insgesamt pro account
                'vehicles_done' => 0 // fahrzeuge verarbeitet pro account
            ]
        ];

        update_option($this->import_status_db_field, $import);

        return (!$this->error) ? $import_id : false;
    }

    /**
     * @param $import_id
     * @return string
     */
    public function get_import_dir($import_id = null) {
        if(is_null($import_id)) {
            $import = $this->get_current_import();
            $import_id = $import['import_id'];
        }
        return WPCS_UPLOADS_DIR.'/imports/'.$import_id;
    }

    /**
     * @param $import_id
     * @return false|string
     */
    private function _create_import_dir($import_id) {
        $dir = $this->get_import_dir($import_id);

        if(!is_dir($dir)) {
            if(!mkdir($dir,0777,true)) return false;
        }

        return $dir;
    }


    /**
     * @return false|mixed|void
     */
    public function get_current_import() {
        return get_option($this->import_status_db_field);
    }

    /**
     * @param $data
     * @return bool
     */
    public function save_current_import($data) {
        return update_option($this->import_status_db_field, $data);
    }

    /**
     * erhöt den Zähler erledigter Fahrzeuge um 1
     * @return void
     */
    public function vehicle_done() { $this->set_status("vehicles_done"); }

    /**
     * setzt die Anzahl aller Fahrzeuge zu einem Account
     * resettet die erledigten Fahrzeuge
     * @param $count
     * @return void
     */
    public function vehicles_count($count) {
        Sync::log("gefundene Datensätze: ".$count);
        $this->set_status("vehicles_count", $count);
        $this->set_status("vehicles_done", 0);
    }

    /** erhöht den aktuellen Index um 1
     * Zeiger, welches Fahrzeug als nächsten verarbeitet werden muss
     * @return void
     */
    public function next() { $this->set_status(); }

    /**
     * @return void
     */
    private function set_status($field = "index", $value = null) {
        $data = $this->get_current_import();

        if(is_null($value)) $data['status'][$field]++;
        else $data['status'][$field] = $value;

        $this->save_current_import($data);
    }

    /**
     * @return void
     */
    public function nextAccount() {
        // hole aktuellen Import
        $data = $this->get_current_import();

        // setze Fahrzeug index zurück auf 0 um beim ersten Fahrzeug des neuen Accounts zu beginnen
        $data['status']['index'] = 0;

        // nächsten account auf aktiv schalten
        $data['status']['account_index'] = intval($data['status']['account_index']) + 1;

        // reset account index => da immer accouunt am index 0 genommen wird
        #unset($data['accounts'][0]);
        #$data['accounts'] = array_values($data['accounts']);

        // speichern
        $this->save_current_import($data);
    }

    /**
     * @return void
     */
    public function cleanup() {
        $dir = WPCS_UPLOADS_DIR.'/imports';
        if(!is_dir($dir)) return;

        $keep_days = 7;
        $keep_until = time() - (86400 * $keep_days);

        foreach(scandir($dir) as $folder) {
            if($folder == "." || $folder == "..") continue;
            if($folder < $keep_until) {
                $tmpDir = $dir."/".$folder;
                $files = scandir($tmpDir);
                foreach($files as $file) {
                    if(is_file($tmpDir."/".$file)) unlink($tmpDir."/".$file);
                }
                rmdir($tmpDir);
            }
        }
        Sync::extLog("Bereinige Importverzeichnis");
    }



}