<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 30.11.18
 * Time: 11:26
 */

namespace DXIM;


class MediaImport {


	protected static $instance = null;

	/**
	 *
	 * @return MediaImport
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

    /**
     * @param $images
     * @param $post_id
     * @return void
     */
    public function import($images, $post_id, $file_base_name, $category = "wpcs_db_images") {

        // lösche vorhandene medien
        $this->delete_vehicle_media_by_category($post_id, $category);

        $z = 1;
        $db_images = [];
        foreach($images as $url) {
            $info = pathinfo($url);
            $id = $this->importFromUrl($url, $post_id, $file_base_name."_".$z.".".$info['extension']);
            $db_images[] = $id;
            $z++;
        }

        // importierte fahrzeuge merken
        update_post_meta($post_id,$category,$db_images);

	    do_action('wpcs_images_imported_media',$post_id, $category);

	    return $db_images;
    }

    /**
     * löscht alle Bilder zu einem Fahrzeug aus der Mediathek
     * z.b. wenn ein Fahrzeug gelöscht wird
     * @param $vehicle_id
     */
    public function delete_vehicle_media($vehicle_id) {
        $attachments = get_attached_media('image',$vehicle_id);
        #wpcs_p($attachments);
        if(!is_array($attachments)) return true;

        foreach($attachments as $attachment) {
            wp_delete_attachment($attachment->ID,true);
        }

        return true;
    }


    /**
     * löscht bestimmte Bilder eines Fahrzeuges aus der Mediathek,
     * z.b. wenn ein neuer Bildimport oder CarShotsPRO Import läuft
     * @param $post_id
     * @param $category
     * @return void
     */
    private function delete_vehicle_media_by_category($post_id, $category) {
        $attachments = get_post_meta($post_id,$category, true);
        if(!is_array($attachments)) return;

        foreach($attachments as $id) {
            wp_delete_attachment($id,true);
        }

    }


    /**
	 * @param $url
	 */
	public function importFromUrl($url, $post_id, $file_name = null) {
		// Need to require these files
		if ( !function_exists('media_handle_upload') ) {
			require_once(ABSPATH . "wp-admin" . '/includes/image.php');
			require_once(ABSPATH . "wp-admin" . '/includes/file.php');
			require_once(ABSPATH . "wp-admin" . '/includes/media.php');
		}

        $delete_after_import = true;

		// Download file to temp dir
        if(!preg_match('/^http:\/\/|https:\/\//', $url )) {
            $temp_file = $url;
            $delete_after_import = false;
        }
        else {
            $image_data = file_get_contents($url);
            $temp_file = sys_get_temp_dir()."/".$file_name;
            file_put_contents($temp_file,$image_data);
            #$temp_file = download_url( $url);
        }

		if ( is_file( $temp_file ) ) {

			$file_array = array();

			// Set variables for storage
			// fix file filename for query strings
			preg_match('/[^\?]+\.(jpg|jpe|jpeg|gif|png)/i', $url, $matches);
			#$file_array['name'] = basename($matches[0]);
			$file_array['name'] = $file_name;
			$file_array['tmp_name'] = $temp_file;

			// do the validation and storage stuff
			$id = media_handle_sideload( $file_array, $post_id, $file_name );

			// If error storing permanently, unlink
			if ( is_wp_error($id) ) {
				if($delete_after_import) @unlink($file_array['tmp_name']);
				return $id;
			}

			if(is_file($temp_file)) {
				if($delete_after_import) unlink($temp_file);
			}

            return $id;
		}

        return false;
	}

// 59 side

}