<?php

namespace DXIM;

class Queue {

	protected static $instance = null;

    protected $async = true;
    protected $remote_url = null;

    // verarbeitete Fahrzeuge pro prozess
    protected $vehicle_per_process = 5;
    #protected $vehicle_per_process = 1;

    // sekunden, die vergehen müssen, dass process neu angestoßen wird falls import hängt
    protected $seconds_inactive = 300;

	/**
	 *
	 * @return Queue
	 */
	public static function getInstance() {
	 	if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {
        $this->remote_url = trailingslashit(site_url()).'?wpcs_process_queue=1';
        #$this->remote_url = trailingslashit(site_url());
        if(WPCS_Debug) $this->async = false;
	}

    /**
     * hilft zu erkennen ob queue evtl. hängt
     * @return bool
     */
    private function _updateLastAction() {
        return update_option('wpcs_import_queue_last_action',time());
    }

    /**
     * @return bool
     */
    private function _delete_last_action() {
        return delete_option('wpcs_import_queue_last_action');
    }

    /**
     * @return void
     */
    private function _do_request() {
        wp_remote_get( $this->remote_url,['blocking' => false, 'timeout' => 0]);
        // Mit POST gabs bei einem Kunden Probleme, GET lief
        #wp_remote_post( $this->remote_url,['blocking' => false, 'timeout' => 0, 'body' => ['wpcs_process_queue' => 1]]);
    }



    /**
     * @return void
     */
    public function checkRunning() {
        if(isset($_POST['wpcs_process_queue'])) return;

        $time = get_option("wpcs_import_queue_last_action");
        if(empty($time)) return;

        $timeRestart = $time + $this->seconds_inactive;
        $now = time();

        if($timeRestart <= $now) {
            Sync::log("Import scheint zu hängen, neu starten...");
            $this->start();
            exit;
        }
    }


    /**
     * @return void
     */
    public function _process() {
        #$time_start = microtime(true);

        // import muss aktiv sein
        if(!Sync::getInstance()->isSyncActive()) return;

        Sync::extLog("neuer Prozess gestartet");

        // aktiver Import
        $i = new Import();

        // merke wann letzter Prozess lief
        $this->_updateLastAction();

        for($a = 1; $a <= $this->vehicle_per_process; $a++) {
            $import = $i->get_current_import();

            // ermittle zu importierendes Fahrzeug
            $vehicle = $this->getNextVehicle($import);

            // konnte es anhand des aktuellen status nicht gefunden werden, nächsten Account synchen
            if(empty($vehicle)) {
                Sync::extLog("Kein weiteres Fahrzeug gefunden");
                Import::getInstance()->nextAccount();
                $import = $i->get_current_import();
                $vehicle = $this->getNextVehicle($import);
            }

            // konnte wiederum kein Fahrzeug gefunden werden => alles erledigt
            if(empty($vehicle)) {
                $this->_after_import();
                return;
            }

            /**
             * verarbeitung
             */
            // sync deaktiviert
            $id = $vehicle['import_data']['ID'];
            $sync_disabled = intval(get_post_meta($id, "wpcs_sync_disabled", true));
            if($sync_disabled) {
                $this->touch($import['import_id'], $vehicle['import_data']['ID']);
                $key = $vehicle['import_data']['seller_inventory_key'] ?? "ID ".$vehicle['import_data']['ID'];
                Sync::Log("Sync deaktiviert für Fahrzeug ".$key);
            }

            // nicht tun, Fahrzeug ist aktuell
            elseif(empty($vehicle['import_data']['do'])) {
                $this->touch($import['import_id'], $vehicle['import_data']['ID']);
                Sync::extLog("Keine Aktion für Fahrzeug ".$vehicle['import_data']['seller_inventory_key']);
            }

            // Fahrzeug anlegen / bearbeiten
            elseif($vehicle['import_data']['do'] == "new" || $vehicle['import_data']['do'] == "update") {
                $v = new Vehicle();
                $vehicle_id = $v->save($vehicle);
                if($vehicle_id > 0) {
                    $this->touch($import['import_id'], $vehicle_id);
                    Sync::log("Verarbeite Vehicle ".$vehicle['import_data']['seller_inventory_key'].": ".$vehicle['import_data']['do']);
                } else Sync::log("Verarbeite Vehicle ".$vehicle['import_data']['seller_inventory_key'].": ".$v->status_msg);
            }

            // Fahrzeug löschen
            elseif($vehicle['import_data']['do'] == "delete") {
                $v = new Vehicle();
                $v->delete($vehicle['import_data']['ID']);
                Sync::log("Lösche Fahrzeug ID ".$vehicle['import_data']['ID']);
            }

            // Live Zähler Backend aktualisieren
            Import::getInstance()->vehicle_done();

            // Index in DB hochzählen für nächsten Durchlauf
            Import::getInstance()->next();

        }

        // nächsten Durchlauf starten
        if($this->async) {
            Sync::getInstance()->close_logfile();
            $this->_do_request();
        }
        else $this->_process();
    }

    /**
     * @return array|false|null[]|void
     */
    private function getNextVehicle(&$import) {
        $current_account = $import['status']['account_index'] ?? null;
        if($current_account === null) return [];

        $account_id = $import['accounts'][$current_account] ?? null;
        if($account_id === null) return [];

        $account = Account::getInstance()->get_account($account_id);

        $index = $import['status']['index'];
        $vehicle = null;

        // csv
        if($account['type'] == "csv") {
            if($index == 0) {
                Sync::log("Setup für CSV Account ".$account['name']);
                if(!CSVAdapter::getInstance()->setup($import['import_id'], $account)) return null;
            }
            $vehicle = CSVAdapter::getInstance()->getVehicle($index, $account);
        }

        // mobile.de
        elseif($account['type'] == "mobile.de") {
            if($index == 0) {
                Sync::log("Setup für Mobile.de Account ".$account['user']);
                if(!MOBILEAdapter::getInstance()->setup($import['import_id'], $account)) {
                    Sync::log(MOBILEAdapter::getInstance()->getError());
                    return false;
                }
            }
            $vehicle = MOBILEAdapter::getInstance()->getVehicle($index, $import['import_id'], $account);
        }

        // carcloud
        elseif($account['type'] == "carcloud") {
            if($index == 0) {
                Sync::log("Setup für CarCloud Account");
                if(!CarCloudAdapter::getInstance()->setup($import['import_id'])) return null;
            }
            $vehicle = CarCloudAdapter::getInstance()->getVehicle($index, $import['import_id']);
        }

        // Delete
        elseif($account['type'] == "delete") {
            if($index == 0) {
                Sync::log("Setup für Fahrzeugbereinigung");
                if(!Delete::getInstance()->setup($import['import_id'])) return null;
            }
            $vehicle = Delete::getInstance()->getVehicle($import['import_id']);
        }



        // es wurde kein weiteres Fahrzeug gefunden
        if($vehicle === 1) return null;

        // Ein Fehler trat beim aktuellen Datensatz auf => nächtes Fahrzeug suchen
        if($vehicle === 0) {
            // Live Zähler Backend aktualisieren
            Import::getInstance()->vehicle_done();

            // Index in DB hochzählen für nächsten Durchlauf
            Import::getInstance()->next();

            $import = Import::getInstance()->get_current_import();
            return $this->getNextVehicle($import);
        }

        return $vehicle;
    }


    /**
     * @param $import_id
     * @param $vehicle_id
     * @return void
     */
    private function touch($import_id, $vehicle_id) {
        #Sync::extLog("Touch Vehicle ID ".$vehicle_id);
        update_post_meta($vehicle_id, 'import_id', $import_id);
    }



    /**
     * @return void
     */
    public function start() {
        $this->_delete_last_action();

        if($this->async) {
            Sync::extLog("Queueverarbeitung im Hintergrund");
            Sync::getInstance()->close_logfile();
            #wp_remote_get( $this->remote_url,['blocking' => false, 'timeout' => 0]);
            $this->_do_request();
        }
        else {
            Sync::extLog("Queueverarbeitung im Hauptfenster");
	        $this->_process();
        }
    }

    /**
     * @return void
     */
    private function _after_import() {
        // lösche Datum letzter aktion
        $this->_delete_last_action();

        // Import beenden
        \DXIM\Sync::getInstance()->afterImport();
    }


}