<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 08:35
 */

namespace DXIM;


class Sync {

    protected static $instance = null;
    protected static $log_file_handle = null;

    /**
     * limitiert die anzahl abgefragter seiten
     * 0 = alle Seiten abfragen (alle Datensätze)
     * @var int
     */
    protected $limitImport = 0;
    #protected $limitImport = 1;

    /**
     * @var null
     */
    protected $syncMaxDuration = 7200;

    /**
     * @var array
     */
    protected $queue = [];

    /**
     * @var array
     */
    protected $interval = array('hourly'=>3600,'6_hours' => 21600, 'daily'=>86400);


    /**
     * enthält information was mit welchem Datensatz passiert ist (edit|new|delete|nothing)
     * @var array
     */
    public $error = null;


    /**
     *
     * @return Sync
     */
    public static function getInstance() {

        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Sync constructor.
     */
    private function __construct() {
        add_action('wp_ajax_checkSyncStatus',[&$this,'_ajaxCheckSyncStatus']);
    }

    /**
     *
     */
    function __destruct() {
        $this->close_logfile();
    }

    /**
     * @return int
     */
    public function _ajaxCheckSyncStatus() {
        $data = $this->getSyncStatus();
        header('Content-Type: application/json; charset=utf-8');
        echo json_encode($data);
        exit;
    }

    /**
     * @return array|int[]
     */
    public function getSyncStatus() {
        $options = Admin::getInstance()->getOptions();
        $lastSyncTime = $options['lastSyncTime'] ?? "";
        $sync_active = Sync::getInstance()->isSyncActive() ? 1 : 0;
        $defaults = [
            'sync_active' => $sync_active, 'lastSyncHumanTime' => date("d.m.Y H:i",intval($lastSyncTime)),
            'count' => 0, 'vehicles_left' => 0, 'account' => null
        ];
//        $defaults = [
//            'sync_active' => 1,
//            'vehicles_left' => 50,
//            'count' => 200,
//            'account' => "Test-Account",
//            'lastSyncHumanTime' => date("d.m.Y H:i",$lastSyncTime),
//        ];
        if(!$sync_active) return $defaults;

        $import = Import::getInstance()->get_current_import();
        $current_account = $import['status']['account_index'] ?? null;
        $account_id = $import['accounts'][$current_account] ?? null;
        $account = Account::getInstance()->get_account($account_id);
        $data = [
            'sync_active' => $sync_active, 'lastSyncHumanTime' => date("d.m.Y H:i",$lastSyncTime),
            'vehicles_left' => ($import['status']['vehicles_count'] - $import['status']['vehicles_done']),
            'count' => $import['status']['vehicles_count'],
            'account' => $account['name'] ?? $account['user']
        ];

        return $data;
    }

    /**
     * @return false|string
     */
    public function getNextSyncDate() {
        $options = Admin::getInstance()->getOptions();
        $interval = $this->interval[$options['cron']];
        return date('d.m.Y H:i',($options['lastSyncTime'] + $interval));
    }

    /**
     *
     */
    public function _init() {
        $wpcs_import = $_GET['wpcs_import'] ?? null;
        $wpcs_cancel_imports = isset($_GET['wpcs_cancel_import']) ? 1 : 0;

        // cancel all active imports / queues
        if($wpcs_cancel_imports) {
            $this->afterImport();
        }

        // starte import
        elseif($wpcs_import == "1") {
            $this->_syncAccounts();
        }

        // checken ob import hängt
        elseif($this->isSyncActive()) {
            Queue::getInstance()->checkRunning();
        }

        // checke ob import starten soll
        elseif($this->_checkSync()) {
            $this->_syncAccounts();
        }

    }


    /**
     *
     */
    private function _checkSync() {

        $options = Admin::getInstance()->getOptions();

        if(!isset($options['cron']) || $options['cron'] == "#" || isset($_GET['import'])) {
            return false;
        }

        $now = time();
        if(empty($options['lastSyncTime'])) $options['lastSyncTime'] = 0;
        $nextSync = intval($options['lastSyncTime'] + $this->interval[$options['cron']]);
        if($nextSync <= $now) {
            return true;
        }

        return false;
    }


    /**
     * @return bool
     */
    private function _syncAccounts() {

        wp_cache_flush();

        if($this->isSyncActive() && !WPCS_Debug) {
            error_log('sync already running...');
            return;
        }

        // sync datum merken
        $this->saveLastSyncDate();

        $import = new Import();
        $accounts = isset($_GET['account_id']) ? [$_GET['account_id']] : [];
        $import_id = $import->create_new_import($accounts);
        if(!$import_id) {
            self::log("Fehler: ".$import->getError());
            return;
        }

        // initialisiere log handle und leere log
        self::log('starte Import: '.date('d.m.Y H:i:s').', ID: '.$import_id);

        if(!empty($import_id)) {
			$this->_setSyncAsActive();

            // start queue
            $q = Queue::getInstance();
            $q->start();

        }
        else {
            $this->afterImport();
        }

    }


    /**
     * @param $msg
     */
    public static function log($msg) {
        if(!is_resource(self::$log_file_handle)) {
            $import_id = Import::getInstance()->get_current_import()['import_id'] ?? null;
            if($import_id !== null) {
                if(self::$log_file_handle = fopen(WPCS_UPLOADS_DIR.'/imports/'.$import_id.'/import.log', "a")) {
                    self::extLog("Öffne LogFile zum schreiben");
                }
            }
        }

        if(is_resource(self::$log_file_handle)) fwrite(self::$log_file_handle, $msg."\n");
    }

    /**
     * @param $msg
     * @return void
     */
    public static function extLog($msg) {
        self::log('<span class="extLog">'.$msg.'</span>');
    }

    /**
     * @return void
     */
    public function close_logfile() {
        if(is_resource(self::$log_file_handle)) {
            Sync::extLog("schließe Logfile Zugriff");
            fclose(self::$log_file_handle);
        }
    }

    /**
     *
     */
    public function afterImport() {
        $this->setSyncAsInactive();
        do_action(DXIM_CORE_AFTER_IMPORT);
    }

    /**
     *
     */
    private function _setSyncAsActive() {
        update_option('wpcs_import_active', 1);
    }

    /**
     *
     */
    public function setSyncAsInactive() {
        self::log('beende Import: '.date('d.m.Y H:i:s'));
        update_option('wpcs_import_active', 0);
    }

    /**
     * @return mixed|void
     */
    public function isSyncActive() {
        $is_active = boolval(get_option('wpcs_import_active',0));
        if(!$is_active) return false;

        $lastSyncTime = Admin::getInstance()->getOptions('lastSyncTime');
        $maxSyncTime = $lastSyncTime + $this->syncMaxDuration;
        $now = time();

        // wenn syny länger dauert als bestimmte zeit => zurücksetzem damit neuer sync starten kann
        if($now >= $maxSyncTime) {
            self::log('beende laufenden Import der um '.date('d.m.Y H:i',$lastSyncTime)." startete.");
            $this->setSyncAsInactive();
            return false;
        }

        return true;
    }

    /**
     *
     */
    public function saveLastSyncDate() {
        $options = Admin::getInstance()->getOptions();
        $options['lastSyncTime'] = time();
        Admin::getInstance()->saveOptions($options,false);
    }

}