<?php

namespace DXIM;

class CarCloudAdapter {

    protected static $instance = null;

    /**
     *
     * @return CarCloudAdapter
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }


    public function setup($import_id) {
        $options = Admin::getInstance()->getOptions();
        $api_key = $options['carcloud']['api_key'];
        $url = trailingslashit(CARCLOUD_API)."?api=".$api_key;

        $json = wp_remote_get($url, [
            'headers' => array(
                'Content-Type' => 'application/json',
                'X-Api-Key' => $api_key
            )
        ]);

        $res = json_decode($json['body'], true);
        $vehicles = $res['vehicles'] ?? null;

        // antwort kein JSON
        if(!is_array($vehicles)) {
            Sync::log("API Antwort nicht lesbar");
            return false;
        }

        // API Fehler
        elseif(isset($vehicles['error'])) {
            Sync::log($vehicles['error']);
            return false;
        }

        Import::getInstance()->vehicles_count(count($vehicles));

        if(file_put_contents(Import::getInstance()->get_import_dir($import_id)."/carcloud.json", json_encode($vehicles)) !== false) return true;
        else return false;
    }


    /**
     * @return mixed
     */
    private function getCachedVehicles($import_id) {
        $json_string = file_get_contents(Import::getInstance()->get_import_dir($import_id)."/carcloud.json");
        return json_decode($json_string, true);
    }


    /**
     * @param $key
     * @return mixed
     */
    public function getVehicle($index, $import_id) {

        // lade Fahrzeue aus lokaler Datei
        $vehicles = $this->getCachedVehicles($import_id);
        if(empty($vehicles)) {
            Sync::log("Keine Fahrzeuge vorhanden");
            return false;
        }

        $keys = array_keys($vehicles);
        $key = $keys[$index] ?? null;
        $ad_key = $key;
        if(is_null($key)) {
           return false;
        }

        #wpcs_p($vehicles[$key]);
        $data = $vehicles[$key] ?? [];
        $images = $vehicles[$key]['images'] ?? [];
        $hash = hash("md5",json_encode($data));

        /**
         * prüfe was mit Fahrzeug zu tun ist
         */
        $v = new Vehicle();
        $vehicle = $v->createImportDataset($key, $hash);

        if(empty($vehicle['import_data']['do'])) return $vehicle;

        /** ------------------------
         * post data
         */
        $pd = [];

        // title
        $pd['post_title'] = $data['make']." ".$data['model_description'];

        // description
        $pd['post_content'] = $data['vehicle_description'] ?? $pd['post_title'];
        unset($data['vehicle_description']);

        $vehicle['post_data'] = $pd;

        /** ----------------------
         * post meta
         */
        $pm = [
            'ad_key' => $key,
            'ad_account' => 'carcloud',
            'import_hash' => $hash,
            'external' => 1 // Hinweis für Standort-Angabe, dass Fahrzeug extern abgestellt ist
        ];

        /**
         * validiere alle Felder
         */
        foreach($data as $key => $value) {
            $key = str_replace(['specific_', 'feature_', 'consumer_'], '', $key);
            $key = strtolower($key);
            $meta_key = VehicleFields::getField($key)['meta_query']['key'] ?? null;
            if($meta_key == null) continue;

            $pm[$meta_key] = $value;
        }

        /**
         * Preis Marge einrechnen
         */
        $price_src = isset($pm['dealer_price']) && !empty($pm['dealer_price']) ? 'dealer_price' : 'consumer_price';
        $pm['consumer_price'] = $this->calculatePrice($pm[$price_src]);

        /**
         * mappe location
         */
        $options = Admin::getInstance()->getOptions();
        $location_id = $options['carcloud']['location_mapping'];
        $pm['ad_location'] = $location_id;

        /**
         * Images
         */

        $pm['image_id'] = $ad_key;
        foreach($images as $url) {
            $pm['images']['big'][] = $url;
        }

        $vehicle['post_meta'] = $pm;

        return $vehicle;
    }

    /**
     * @param $price
     * @return string
     */
    public function calculatePrice($price) {
        $options = Admin::getInstance()->getOptions();
        $margins['vehicle_margin'] = $options['carcloud']['margin'] ?? 5;
        $margins['margin_min'] = $options['carcloud']['margin_min'] ?? 300;
        $margins['margin_max'] = $options['carcloud']['margin_max'] ?? 600;

        $formatted = str_replace(',', '.', $price);

        $interest = round((float)$formatted * ((float)$margins['vehicle_margin'] / 100),2);

        // add margin result to price
        if ($interest > $margins['margin_max']) {
            // if margin result > margin_max return price + margin_max
            return (float)$formatted + $margins['margin_max'];
        } else if ($interest < $margins['margin_min']) {
            // if margin result < margin_min return price + margin_min
            return (float)$formatted + $margins['margin_min'];
            // else return price + margin result
        } else {
            return (float)$formatted + $interest;
        }
    }

}