<?php

namespace DXIM;

class CSVExtendedImportHooks {

    protected static $instance = null;

    /**
     *
     * @return CSVExtendedImportHooks
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    public function __construct() {
        add_filter('dxim_core_csv_importer_mobile_csv_extended',[&$this,'_filter_import_data'],10,3);
    }

    /**
     * @param $data
     * @param $csv
     * @param $config
     * @return mixed
     */
    public function _filter_import_data($data, $csv, $config) {

        // make
        $data['make'] = $data['make'] ?? null;
        $data['make'] = $this->_make($data['make']);

        // class & category
        $tmp = $data['category'];
        $data['category'] = $this->_category($tmp);
        $data['class'] = $this->_class($tmp);

        //
        $data['specific_general_inspection'] = $data['specific_general_inspection'] ?? null;
        $data['specific_general_inspection'] = $this->_convert_date($data['specific_general_inspection']);

        //
        $data['specific_wltp_electric_range_eaer'] = $data['specific_wltp_electric_range'];

        //
        $data['specific_first_registration'] = $data['specific_first_registration'] ?? null;
        $data['specific_first_registration'] = $this->_convert_date($data['specific_first_registration']);

        //
        $data['vat_rate'] = $data['vat_rate'] ?? null;
        $data['vat_rate'] = $this->_convert_vat_rate($data['vat_rate']);

        // erwartet vat_rate < 1
        $data['consumer_price'] = $data['consumer_price'] ?? null;
        $data['consumer_price'] = $this->_price($data['consumer_price'], $data['vatable'], $data['vat_rate']);

        // modelbasis
        $model_tmp = $csv[318] ?? null;
        $data['model'] = $this->_model($model_tmp, $data['make']);

        return $data;
    }

    /**
     * @param $model_tmp
     * @param $make
     * @return mixed|string|null
     */
    private function _model($model_tmp, $make) {

        $model = Model::_get($model_tmp, $make, true);
        if(empty($model)) {
            $model = Model::_getKey($model_tmp, $make);
            if(!empty($model)) return $model;
        }

        return $model_tmp;
    }


    /**
     * prüfen ob Hersteller key oder label ist und versuchen zu mappen
     */
    private function _make($make_value) {

        // prüfen ob MAKE schon ein passender Schlüssel ist
        $make = Make::_get($make_value, true);

        // falls nicht, anhand label nach Schlüssel suchen
        if(empty($make)) {
            $make = Make::_getKey($make_value);
            // wenn gefunden alten Wrt durch neuen Hessteller Schlüssel ersetzen
            if(!empty($make)) return $make;
        }

        return $make_value;
    }


    /**
     * @param $value
     * @return string
     */
    private function _class($value) {
        $tmp = explode(".",$value);
        return $tmp[0];
    }

    /**
     * @param $value
     * @return string
     */
    private function _category($value) {
        $tmp = explode(".",$value);
        return $tmp[1] ?? $value;
    }

    /**
     * @param $value
     * @return mixed|string|null
     */
    private function _convert_date($value) {
        if(empty($value)) return null;

        $tmp = explode(".",$value);
        if(count($tmp) == 2) return $tmp[1]."-".$tmp[0];
        if(count($tmp) == 3) return $tmp[2]."-".$tmp[1];

        return $value;
    }

    /**
     * Preis muss immer brutto importiert werden
     * prüfen ob bereits brutto => andernfalls umrechnen
     * @param $price
     * @param $vatable
     * @param $vat
     * @return float
     */
    private function _price($price, $vatable, $vat) {
        $vatable = $vatable == "true" ? 1 : 0;
        $vat = floatval($vat);
        $price = floatval(str_replace(",",".",$price));

        // ist bereits brutto
        if(!$vatable || $vat == 0) return $price;

        // in brutto umrechnen
        $vat_rate = ($vat * 100) + 100;
        return round(($price / 100) * $vat_rate,2);
    }

    /**
     * vat_rate muss immer kleiner 0 sein, z.b. 0.19 nicht 19
     * @param $value
     * @return float|int
     */
    private function _convert_vat_rate($value) {
        $value = intval($value);
        if($value < 1) return $value;
        return $value / 100;
    }




}