<?php
namespace DXIM;

require('PaypalIPN.php');
class Paypal {

    protected static $instance = null;

    /**
     *
     * @return Paypal
     */
    public static function getInstance() {
        if(self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    private function __construct() {
        $this->init();

        add_shortcode("WPCS_paypal_buttons", [&$this,'_shortcode_render_buttons']);

        add_action('wp_enqueue_scripts', function() {
            wp_enqueue_script('paypal-sdk','https://www.paypal.com/sdk/js?client-id='.WPCS_PAYPAL_CLIENT_ID.'&currency=EUR',[],null);
        });
    }

    /**
     * @return void
     */
    private function init() {
        $ipn_handler = $_GET['wpcs_paypal_ipn_handler'] ?? "";

        if ($ipn_handler) {

            $debug = false;
            $verified = false;
            $post_data = $_POST ?? [];

            if(empty($post_data)) {
                if($debug) {
                    $json = '{"mc_gross":"100.00","protection_eligibility":"Eligible","address_status":"confirmed","payer_id":"7R2DTMK6SLHVA","address_street":"Badensche Str. 24","payment_date":"03:19:11 Mar 12, 2024 PDT","payment_status":"Completed","charset":"windows-1252","address_zip":"10715","first_name":"John","mc_fee":"2.25","address_country_code":"DE","address_name":"John Doe","notify_version":"3.9","custom":"wpcs_checkout_payment_77942","payer_status":"verified","business":"daniel@dximage.de","address_country":"Germany","address_city":"Berlin","quantity":"1","verify_sign":"AJjgVAuJP5EQP6AyBHAETkhNLbzNAf7trNa0GoS4O26yqpdOKo9HcUpQ","payer_email":"sb-clx7h29871707@personal.example.com","txn_id":"0PR28358MG5511445","payment_type":"instant","last_name":"Doe","address_state":"Berlin","receiver_email":"daniel@dximage.de","payment_fee":"","shipping_discount":"0.00","insurance_amount":"0.00","receiver_id":"XMV588PLP884J","txn_type":"express_checkout","item_name":"","discount":"0.00","mc_currency":"EUR","item_number":"","residence_country":"DE","test_ipn":"1","shipping_method":"Default","transaction_subject":"","payment_gross":"","ipn_track_id":"f6627852d2b39"}';
                    $post_data = json_decode($json, ARRAY_A);
                    $verified = true;
                }
                else exit;
            }

            $options = Admin::getInstance()->getOptions('paypal');
            $mode = $options['mode'];

            /**
             * request bei paypal validieren
             */
            if(!$verified) {
                $ipn = new \PaypalIPN();

                // Use the sandbox endpoint during testing.
                if($mode == "sandbox") $ipn->useSandbox();

                // verwende php zertifikat
                $ipn->usePHPCerts();

                // verifizieren IPN Call
                $verified = $ipn->verifyIPN();
            }

            /**
             * request ok
             */
            if ($verified) {
                #file_put_contents(DXIM_PluginPath.'/postdata.txt', json_encode($_POST));
                $status = $post_data['payment_status'] ?? null;

                // Zahlung vollständig triggert action
                if($status == "Completed") {
                    do_action('wpcs_paypal_ipn_payment_completed', $post_data);
                }
            }

            // Reply with an empty 200 response to indicate to paypal the IPN was received correctly.
            header("HTTP/1.1 200 OK");
            exit;
        }
    }

    public function _shortcode_render_buttons($atts, $content) {
        $amount = $atts['amount'] ?? "";
        if(empty($amount)) return 'Bitte Betrag angeben (z.B. amount="99")';

        $custom = $atts['custom'] ?? "";
        if(empty($custom)) return 'Bitte geben Sie den custom Parameter an (z.B. custom="mein-wert")';

        $description = $content ?? "";
        $success = $atts['success_msg'] ?? "";

        return $this->render_buttons($amount, $custom, $description, $success);
    }

    /**
     * @param $amount
     * @param $custom
     * @return string
     */
    public function render_buttons($amount, $custom, $description = null, $success_html = null) {

        if(empty($description)) {
            $description = '<h2>Zahlung</h2><p>Bitte führen Sie nun die Zahlung von <strong>'.$amount.'&euro;</strong> durch.</p>';
        }

        if(empty($success_html)) {
            $success_html = "<h2>Vielen Dank, die Zahlung war erfolgreich</h2>";
        }

        $html = '<div id="wpcs_paypal_buttons_wrapper">';
            $html .= '<div id="wpcs_paypal_buttons_success" class="wpcs_hidden">'.$success_html.'</div>';
            $html .= '<div id="wpcs_paypal_buttons_msg">'.$description.'</div>';
            $html .= '<div id="wpcs_paypal_buttons"></div>';
            $html .= '<script>
                paypal.Buttons({
                    createOrder: function(data, actions) {
                        return actions.order.create({
                            purchase_units: [{
                                amount: {
                                    value: '.$amount.'
                                },
                                custom_id : \''.$custom.'\'
                            }]
                        })
                    },
    
                    onApprove: function(data, actions) {
                        return actions.order.capture().then(function(details) {                         
                            //alert("Transaction completed by " + details.payer.name.given_name);
                            jQuery("#wpcs_paypal_buttons_success").toggleClass("wpcs_hidden");
                            jQuery("#wpcs_paypal_buttons_msg").hide();
                            jQuery("#wpcs_paypal_buttons").hide();
                        });
                    },
                }).render("#wpcs_paypal_buttons");
            </script>';
        $html .= '</div>';

        return $html;
    }
}