<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 04.07.18
 * Time: 12:16
 */

namespace DXIM\theme;


use DXIM\Admin;
use DXIM\Conf;
use DXIM\Footnote;
use DXIM\Html;

class PTCF {

    protected static $instance = null;

    protected $fields = [];

    /**
     *
     * @return PTCF
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    private function __construct() {
    }

    /**
     * @return void
     */
    public function init() {
        $this->_set_fields();

        foreach ($this->fields as $pt_slug => $field) {
            add_action('add_meta_boxes_' . $pt_slug, [&$this, '_add_meta_boxes']);
        }

        add_action('save_post', [&$this, '_save_cf'], 10, 3);

        add_action('wp_ajax_wpcs_theme_pt_search', [&$this, '_search_pt']);
    }

    public function _search_pt() {
        $q = new \WP_Query([
            'post_type' => $_POST['pt'], 's' => $_POST['search'], 'posts_per_page' => -1
        ]);

        $res = [];
        foreach ($q->posts as $post) {
            $res[] = ['id' => $post->ID, 'title' => $post->post_title];
        }

        header("Content-type: application/json; charset=utf-8");
        print json_encode($res);
        exit;
    }

    /**
     * @return void
     */
    private function _set_fields() {
        $fields = [
            Person::$post_type => [
                ['name' => 'address', 'type' => 'tab', 'label' => 'Adressdaten', 'grid' => 2],
                ['name' => 'position', 'type' => 'text', 'label' => 'Position'],
                ['name' => 'email', 'type' => 'email', 'label' => 'E-Mail'],
                ['name' => 'phone', 'type' => 'number', 'label' => 'Telefon', 'info' => 'Nur Durchwahl angeben'],
                ['name' => 'fax', 'type' => 'number', 'label' => 'Fax', 'info' => 'Nur Durchwahl angeben'],
                ['name' => 'phone_base', 'type' => 'number', 'label' => 'Vorwahl erzwingen', 'info' => 'leer lassen, wenn Standort Vorwahl gewünscht'],
                ['name' => 'mobile', 'type' => 'number', 'label' => 'Mobilnummer'],
                ['name' => 'display', 'type' => 'tab', 'label' => 'Anzeige', 'grid' => 2],
                ['name' => 'order', 'type' => 'number', 'label' => 'Reihenfolge', 'default' => 10, 'info' => "Die Priorität, wenn mehrere Personen dargestellt werden"],
                ['name' => 'is_contact_person', 'type' => 'bool', 'label' => 'Kontaktperson', 'default' => 0, 'info' => 'Falls nein, wird Person nur auf Standortseiten ausgespielt'],
            ],
            Model::$post_type => [
                ['name' => 'general', 'type' => 'tab', 'label' => 'Allgemein', 'grid' => 2],
                ['name' => 'selection_image', 'type' => 'image', 'label' => 'Bild Modellauswahl', 'info' => "Wenn leer wird Beitragsbild genommen"],
                ['name' => 'price', 'type' => 'number', 'label' => 'Preis ab', 'info' => "Startpreis dieses Modells"],
                ['name' => 'mandatory_information', 'type' => 'textarea', 'label' => 'Pflichtext'],
                ['name' => 'wpycarsync', 'type' => 'tab', 'label' => 'WPCarSync', 'grid' => 1],
                ['name' => null, 'type' => 'callback', 'func' => [Model::getInstance(), '_draw_wpcarsync_mapping']],
            ],
            Offer::$post_type => [
                ['name' => 'general', 'type' => 'tab', 'label' => 'Allgemein', 'grid' => 1],
                ['name' => 'model', 'type' => 'post_type', 'post_type' => Model::$post_type, 'label' => 'Modell', 'info' => 'Gehört dieses Angebot zu einem bestimmten Modell?'],
                ['name' => 'excerpt', 'type' => 'text', 'label' => 'Anlesetext', 'info' => 'Textauszug wird als Vorschau verwendet'],
                ['name' => 'price', 'type' => 'number', 'label' => 'Angebotspreis'],
                ['name' => 'expire_date', 'type' => 'date', 'label' => 'Ablaufdatum', 'info' => 'falls angegeben, wird das Angebot bis zu diesem Datum (einschließlich) angezeigt'],
                ['name' => 'emission_info', 'type' => 'textarea', 'label' => 'Emissionsangaben'],
                ['name' => 'mandatory_information', 'type' => 'textarea', 'label' => 'allg. Pflichtangaben'],
                ['name' => 'footnotes', 'type' => 'post_type', 'post_type' => Footnote::$post_type, 'label' => 'allg. Fußnoten', 'multiple' => 1,
                    'info' => '<a href="'.admin_url().'/edit.php?post_type='.Footnote::$post_type.'" target="_blank">neue Fußnote anlegen</a>'],

                ['name' => 'leasing', 'type' => 'tab', 'label' => 'Leasing', 'grid' => 1],
                ['name' => 'leasing_price', 'type' => 'number', 'label' => 'Rate'],
                ['name' => 'leasing_down_payment', 'type' => 'number', 'label' => 'Sonderzahlung'],
                ['name' => 'leasing_period', 'type' => 'number', 'label' => 'Laufzeit in Monaten'],
                ['name' => 'leasing_mileage', 'type' => 'number', 'label' => 'jährliche Fahrleistung'],
                ['name' => 'leasing_mandatory_information', 'type' => 'textarea', 'label' => 'Leasing Pflichtext'],
                ['name' => 'leasing_footnotes', 'type' => 'post_type', 'post_type' => Footnote::$post_type, 'label' => 'Leasing Fußnoten', 'multiple' => 1,
                    'info' => '<a href="'.admin_url().'/edit.php?post_type='.Footnote::$post_type.'" target="_blank">neue Fußnote anlegen</a>'],
                ['name' => 'leasing_fields', 'type' => 'repeater', 'label' => 'Weitere Felder', 'fields' => [
                    ['name' => 'label', 'type' => 'text', 'label' => 'Beschriftung'],
                    ['name' => 'value', 'type' => 'text', 'label' => 'Wert'],
                    ['name' => 'footnotes', 'type' => 'post_type', 'post_type' => Footnote::$post_type, 'label' => 'Fußnoten', 'multiple' => 1],
                ]],


                ['name' => 'financing', 'type' => 'tab', 'label' => 'Finanzierung', 'grid' => 1],
                ['name' => 'financing_price', 'type' => 'number', 'label' => 'Rate'],
                ['name' => 'financing_down_payment', 'type' => 'number', 'label' => 'Anzahlung'],
                ['name' => 'financing_final_installment', 'type' => 'number', 'label' => 'Abschlussrate'],
                ['name' => 'financing_period', 'type' => 'number', 'label' => 'Laufzeit in Monaten'],
                ['name' => 'financing_mileage', 'type' => 'number', 'label' => 'jährliche Fahrleistung'],
                ['name' => 'financing_mandatory_information', 'type' => 'textarea', 'label' => 'Finanzierung Pflichtext'],
                ['name' => 'financing_footnotes', 'type' => 'post_type', 'post_type' => Footnote::$post_type, 'label' => 'Fußnoten', 'multiple' => 1,
                    'info' => '<a href="'.admin_url().'/edit.php?post_type='.Footnote::$post_type.'" target="_blank">neue Fußnote anlegen</a>'],
                ['name' => 'financing_fields', 'type' => 'repeater', 'label' => 'Weitere Felder', 'fields' => [
                    ['name' => 'label', 'type' => 'text', 'label' => 'Beschriftung'],
                    ['name' => 'value', 'type' => 'text', 'label' => 'Wert'],
                    ['name' => 'footnotes', 'type' => 'post_type', 'post_type' => Footnote::$post_type, 'label' => 'Fußnoten', 'multiple' => 1],
                ]],

            ],
            Job::$post_type => [
                ['name' => 'general', 'type' => 'tab', 'label' => 'Allgemein', 'grid' => 1],
                ['name' => 'excerpt', 'type' => 'textarea', 'label' => 'Anlesetext', 'info' => 'Textauszug wird als Vorschau verwendet'],
                ['name' => 'employmentType', 'type' => 'checkbox', 'label' => 'Anstellungsverhältnis', 'values' => Job::getInstance()->get_employment_types()],
                ['name' => 'homeoffice', 'type' => 'radio', 'label' => 'HomeOffice', 'values' => ['0' => 'kein HomeOffice', 1 => 'HomeOffice möglich', 2 => 'nur HomeOffice']]
            ],

            Service::$post_type => [
                ['name' => 'general', 'type' => 'tab', 'label' => 'Allgemein', 'grid' => 1],
                ['name' => 'service_icon', 'type' => 'image', 'label' => 'Icon Service'],
                ['name' => 'serviceType', 'type' => 'checkbox', 'label' => 'Serviceart', 'values' => Service::getInstance()->get_service_types()],
                ['name' => 'excerpt', 'type' => 'textarea', 'label' => 'Anlesetext', 'info' => 'Textauszug wird als Vorschau verwendet'],
            ]
        ];

        $this->fields = apply_filters(WPCS_THEME_CF_FIELDS, $fields);
    }

    public function _add_meta_boxes() {

        add_meta_box(
            'wpcs_theme_ptcf',
            __('WPCarSync Eigenschaften'),
            [&$this, '_draw_meta_box'],
            array_keys($this->fields)
        );
    }

    /**
     * @param $post
     * @return void
     */
    public function _save_cf($post_id, $post, $update) {
        if (!in_array($post->post_type, array_keys($this->fields)) || !isset($_POST['wpcs_save_cf'])) return;

        #wpcs_p($_POST);
        foreach ($this->fields[$post->post_type] as $field) {
            $value = $_POST[$field['name']] ?? null;

            // array werden nicht serialisiert, sondern als einzelne einträge gespeichert
            if ($field['type'] == "checkbox" || $field['type'] == "taxonomy" || $field['type'] == "post_type") {
                delete_post_meta($post_id, $field['name']);
                foreach ($value as $entry) {
                    add_post_meta($post_id, $field['name'], $entry);
                }
            }

            // Wert existiert nicht mehr oder ist leer => muss gelöscht werden
            elseif (!isset($value) || $value === "") {
                delete_post_meta($post_id, $field['name']);
            }

            // kein Array => einzelnen Wert speichern
            else {
                update_post_meta($post_id, $field['name'], $value);
            }
        }
    }

    /**
     * @param $post
     * @return void
     */
    public function _draw_meta_box($post) {
        $tabControl = [];
        $tabContent = [];
        $tab_index = 0;
        $tab_html = null;
        $current_grid = 0;

        foreach ($this->fields[$post->post_type] as $field) {

            if ($field['type'] == "tab") {
                if (!empty($tab_html)) $tabContent[$tab_index] = $tab_html . '</div>';

                $tab_index++;
                $tabControl[$tab_index] = $field['label'];
                $grid = $field['grid'] ?? 1;
                $tab_html = '<div class="wpcs_cf_tab_content wpcs_grid wpcs_grid_' . $grid . '">';
                $current_grid = $grid;
            } else {
                $value = get_post_meta($post->ID, $field['name']);
                $span = ($current_grid > 0 && $field['type'] == "repeater") ? 'style="grid-column:span ' . $current_grid . ';"' : "";
                $tab_html .= '<div class="wpcs_cf_wrapper ' . $field['type'] . '" ' . $span . '>';
                $tab_html .= self::render_input_field($field, $value);
                $tab_html .= '</div>';
            }
        }

        if (!empty($tab_html)) $tabContent[$tab_index] = $tab_html . "</div>";

        $html = '<input type="hidden" value="1" name="wpcs_save_cf" />';
        $html .= Html::tabs($tabControl, $tabContent);
        print $html;
    }

    public static function render_input_field($field, $value = null, $args = []) {

        // function
        if ($field['type'] == "callback") {
            $html = call_user_func($field['func']);
        }

        // Field
        else {
            $multiple = isset($field['multiple']) ? 1 : 0;
            $html = '<div class="wpcs_field ' . $field['type'] . '" data-name="'.$field['name'].'" data-multiple="'.$multiple.'">';
            if (!isset($args['hide_label'])) $html .= '<label for="' . $field['name'] . '">' . $field['label'] . '</label>';
            if (isset($field['info'])) $html .= '<p class="info">' . $field['info'] . '</p>';

            // Text
            if ($field['type'] == "text") {
                $value = $value[0] ?? null;
                $html .= '<input name="' . $field['name'] . '" type="text" value="' . $value . '" />';
            }

            // date
            elseif ($field['type'] == "date") {
                $value = $value[0] ?? null;
                $html .= '<input name="' . $field['name'] . '" type="date" value="' . $value . '" />';
            }

            // Textarea
            elseif ($field['type'] == "textarea") {
                $value = $value[0] ?? null;
                $html .= '<textarea name="' . $field['name'] . '" rows="5" cols="50" class="large-text">' . $value . '</textarea>';
            }

            // Email
            elseif ($field['type'] == "email") {
                $value = $value[0] ?? null;
                $html .= '<input name="' . $field['name'] . '" type="email" value="' . $value . '" />';
            }

            // Number
            elseif ($field['type'] == "number") {
                $value = $value[0] ?? $field['default'] ?? "";
                $html .= '<input name="' . $field['name'] . '" type="number" value="' . $value . '" />';
            }

            // bool
            elseif ($field['type'] == "bool") {
                $value = $value[0] ?? $field['default'] ?? 0;

                $checked = $value == "1" ? "checked" : "";
                $html .= '<input name="' . $field['name'] . '" ' . $checked . ' type="radio" class="wpcs_checkbox" value="1" /> Ja&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
                $checked = $value == "0" ? "checked" : "";
                $html .= '<input name="' . $field['name'] . '" ' . $checked . ' type="radio" class="wpcs_checkbox" value="0" /> Nein';
            } elseif ($field['type'] == "radio") {
                $value = $value[0] ?? $field['default'] ?? "";
                foreach ($field['values'] as $valueTmp => $label) {
                    $checked = $value == $valueTmp ? "checked" : "";
                    $html .= '<input name="' . $field['name'] . '" ' . $checked . ' type="radio" class="wpcs_checkbox" value="' . $valueTmp . '" /> ' . $label . '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
                }
            } elseif ($field['type'] == "checkbox") {
                if (!is_array($value)) $value = [];
                foreach ($field['values'] as $valueTmp => $label) {
                    $checked = in_array($valueTmp, $value) ? "checked" : "";
                    $html .= '<input name="' . $field['name'] . '[]" ' . $checked . ' type="checkbox" class="wpcs_checkbox" value="' . $valueTmp . '" /> ' . $label . '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
                }
            }
            // Image
            elseif ($field['type'] == "image") {
                $deleteBtnDisplay = "none";
                $imageDisplay = "none";
                $src = "";
                $value = $value[0] ?? null;
                if (!empty($value)) {
                    $src = wp_get_attachment_image_url($value, 'medium');
                    if (!empty($src)) {
                        $deleteBtnDisplay = $imageDisplay = "block";
                    }
                }

                $html .= '<img class="wpcs_cf_preview_image" src="' . $src . '" style="display:' . $imageDisplay . ';" />';
                $html .= '<a href="#" class="button wpcs_cf_upload">Bild auswählen</a>';
                $html .= '<a href="#" class="wpcs_cf_upload_reset" style="display:' . $deleteBtnDisplay . '">Bild löschen</a>';
                $html .= '<input name="' . $field['name'] . '" type="hidden" value="' . $value . '" />';
            }

            // Taxnomoy
            elseif ($field['type'] == "taxonomy") {
                // fallback für serialisierte speicherungen
                if (isset($value[0]) && is_array($value[0])) $value = $value[0] ?? [];

                if (!is_array($value)) $value = [];
                foreach (get_terms(['taxonomy' => $field['tax'], 'hide_empty' => false]) as $term) {
                    $checked = (in_array($term->term_id, $value)) ? "checked" : "";
                    $html .= '<input type="checkbox" name="' . $field['name'] . '[]" ' . $checked . ' value="' . $term->term_id . '" /> ' . $term->name . '<br />';
                }
            }

            // post type
            elseif ($field['type'] == "post_type") {
                $posts = [];
                if (!empty($value)) {
                    $q = new \WP_Query(['post__in' => $value, 'post_type' => $field['post_type']]);
                    $posts = $q->posts;
                }

                $title = $post->post_title ?? "";
                $html .= '<input type="text" placeholder="Suche nach Titel" value="' . $title . '" class="wpcs_post_type_search" data-pt="' . $field['post_type'] . '"  />';
                $html .= '<div class="results"></div>';
                $html .= '<div class="values">';
                foreach($posts as $item) {
                    $html .= '<div class="item">';
                        $html .= '<strong>'.$item->post_title.'</strong>';
                        $html .= '<a href="#"><span class="dashicons dashicons-no-alt"></span></a>';
                        $html .= '<input class="field_value" name="' . $field['name'] . '[]" type="hidden" value="' . $item->ID . '" />'; // hidden
                    $html .= '</div>';
                }
                $html .= '</div>';
            }


            // repeater
            elseif ($field['type'] == "repeater") {
                $value = $value[0] ?? "";
                if (!is_array($value)) $value = [];
                $keys = array_keys($value) ?? [];
                $uniq_keys = isset($keys[0]) ? array_keys($value[$keys[0]]) : [];
                if(empty($uniq_keys)) $uniq_keys[] = uniqid();

                $a = 0;
                foreach($uniq_keys as $key) {
                    $row_id = $key;
                    if(strlen($row_id) < 2) $row_id = uniqid(); // fallback, damit 0, 1, 2 konvertiert werden zu uniqid()
                    $html .= '<div class="row" data-row-id="'.$row_id.'" data-field-name="'.$field['name'].'">';

                        $html .= '<div class="buttons">';
                        $html .= '<a href="#" class="add_row"><span class="dashicons dashicons-plus"></span></a>';
                        $html .= '<a href="#" class="remove_row"><span class="dashicons dashicons-minus"></span></a>';
                        $html .= '</div>';

                        $html .= '<div class="fields" style="grid-template-columns: repeat(' . count($field['fields']) . ', 1fr)">';

                        foreach ($field['fields'] as $fieldTmp) {
                            $valueTmp = $value[$fieldTmp['name']][$row_id] ?? $value[$fieldTmp['name']][$a] ?? null;
                            if(!is_array($valueTmp)) $valueTmp = [$valueTmp];
                            $fieldTmp['name'] = $field['name'] . "[" . $fieldTmp['name'] . "][".$row_id."]";
                            $html .= self::render_input_field($fieldTmp, $valueTmp);
                        }

                        $html .= '</div>';

                    $html .= '</div>'; // row
                    $a++;
                }
            } else $html .= "Feld-Typ nicht bekannt: " . $field['type'];

            $html .= '</div>'; // field wrapper
        }


        return $html;
    }
}
