<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 04.07.18
 * Time: 12:16
 */

namespace DXIM\theme;

class TaxCF {

	protected static $instance = null;

    protected $fields = [];

	/**
	 *
	 * @return TaxCF
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	private function __construct() {}

    /**
     * @return void
     */
    public function init() {
        $this->_set_fields();

        foreach($this->fields as $tax => $fields) {
            add_action( $tax.'_add_form_fields', [&$this,'_add_term_fields']);
            add_action( $tax.'_edit_form_fields', [&$this,'_edit_term_fields'], 10, 2 );
            add_action( 'created_'.$tax, [&$this,'_save_fields'],10,2);
            add_action( 'edited_'.$tax, [&$this,'_save_fields'],10,2);
        }
    }

    /**
     * @return void
     */
    private function _set_fields() {
        $fields = [
            Location::$taxonomy => [
                ['name' => 'street', 'type' => 'text', 'label' => 'Straße'],
                ['name' => 'zipcode', 'type' => 'number', 'label' => 'PLZ'],
                ['name' => 'city', 'type' => 'text', 'label' => 'Stadt'],
                ['name' => 'email', 'type' => 'email', 'label' => 'E-Mail'],
                ['name' => 'email_lead', 'type' => 'email', 'label' => 'E-Mail Lead System'],
                ['name' => 'phone_base', 'type' => 'text', 'label' => 'Telefon', 'info' => 'Ohne Durchwahl angeben'],
                ['name' => 'phone', 'type' => 'text', 'label' => 'Durchwahl Zentrale'],
                ['name' => 'fax', 'type' => 'text', 'label' => 'Durchwahl Fax', 'pro' => 1],
                ['name' => 'opening_hours_sale', 'type' => 'textarea', 'label' => 'Öffnungszeiten Verkauf', 'pro' => 1, 'info' => 'Format:<br />Mo:0800-1230,1330-1800<br />Jeden Tag in einer neuen Zeile eingeben'],
                ['name' => 'opening_hours_service', 'type' => 'textarea', 'label' => 'Öffnungszeiten Service/Werkstatt', 'pro' => 1],
                ['name' => 'teaser_image', 'type' => 'image', 'label' => 'Teaser-Bild', 'pro' => 1],
                ['name' => 'logo', 'type' => 'image', 'label' => 'Standort Logo', 'pro' => 1],
                ['name' => 'page_content', 'type' => 'textarea', 'label' => 'Seiteninhalt'],
                ['name' => 'gallery', 'type' => 'repeater', 'label' => 'Gallery', 'pro' => 1, 'fields' => [
                    ['name' => 'text', 'type' => 'text', 'label' => 'Beschreibung'],
                    ['name' => 'image', 'type' => 'image', 'label' => 'Bild'],
                ]],
                ['name' => 'brands', 'type' => 'taxonomy', 'label' => 'Marken', 'tax' => Brand::$taxonomy, 'pro' => 1],
                ['name' => 'contact_person_jobs', 'type' => 'post_type', 'post_type' => Person::$post_type, 'label' => 'Ansprechpartner Jobs', 'info' => 'Wer ist an diesem Standort Ansprechpartner für Stellenausschreibungen?', 'pro' => 1 ],
            ],
            Brand::$taxonomy => [
                ['name' => 'order', 'type' => 'number', 'label' => 'Reihenfolge'],
                ['name' => 'logo', 'type' => 'image', 'label' => 'Markenlogo'],
                ['name' => 'brand_type', 'type' => 'bool', 'label' => 'nur Service'],
                ['name' => 'vehicle_archive_description', 'type' => 'textarea', 'label' => 'Text für Fahrzeug Archiv', 'info' => 'Kann optional auf der Archivseite für z.B. fahrzeugarchiv/marke/ angezeigt werden'],
            ],
            Division::$taxonomy => [
                ['name' => 'order', 'type' => 'number', 'label' => 'Reihenfolge', 'default' => 10],
            ],
            Tag::$taxonomy => [
                ['name' => 'is_job_filter', 'type' => 'bool', 'label' => 'als Filter für Jobs aktivieren'],
                ['name' => 'is_offer_filter', 'type' => 'bool', 'label' => 'als Filter für Angebote aktivieren'],
            ],
            OfferGroup::$taxonomy => [
                ['name' => 'teaser_image', 'type' => 'image', 'label' => 'Teaser-Bild'],
                ['name' => 'excerpt', 'type' => 'textarea', 'label' => 'Vorschautext'],
                ['name' => 'page_content', 'type' => 'textarea', 'label' => 'Seiteninhalt'],
            ],

        ];

        $this->fields = apply_filters(WPCS_THEME_TAX_FIELDS, $fields);
    }

    /**
     * @param $term_id
     * @return void
     */
    public function _save_fields($term_id, $taxonomy) {
        $tax = $_POST['wpcs_save_for_tax'] ?? null;
        if(empty($tax)) return;
        if(!isset($this->fields[$tax])) return;

        foreach($this->fields[$tax] as $field) {
            $value = $_POST[$field['name']];
            if($field['type'] == "text") $value = sanitize_text_field($value);

            // array werden nicht serialisiert, sondern als einzelne einträge gespeichert
            if($field['type'] == "checkbox" || $field['type'] == "taxonomy") {
                delete_term_meta($term_id, $field['name']);
                foreach($value as $entry) {
                    add_term_meta($term_id, $field['name'], $entry);
                }
            }

            // kein Array => einzelnen Wert speichern
            else {
                update_term_meta($term_id, $field['name'], $value);
            }
        }

    }

    /**
     * @return void
     */
    public function _add_term_fields($taxonomy) {
        $html = null;
        foreach($this->fields[$taxonomy] as $field) {
            if(!WPCS_PRO && isset($field['pro'])) continue;

            $html .= '<div class="form-field">';
            $html .= PTCF::render_input_field($field);
            $html .= '</div>';
        }

        //
        $html .= '<input type="hidden" name="wpcs_save_for_tax" value="'.$taxonomy.'" />';
        print $html;
    }

    public function _edit_term_fields($term, $taxonomy) {
        $html = null;
        foreach($this->fields[$taxonomy] as $field) {
            if(!WPCS_PRO && isset($field['pro'])) continue;
            $value = get_term_meta($term->term_id, $field['name']);
            #wpcs_p($value);

            $html .= '<tr class="form-field">';
            $html .= '<th><label for="'.$field['name'].'">'.$field['label'].'</label></th>';
            $html .= '<td>'.PTCF::render_input_field($field, $value, ['hide_label' => true]).'</td>';
            $html .= '</tr>';
        }

        //
        $html .= '<input type="hidden" name="wpcs_save_for_tax" value="'.$taxonomy.'" />';

        print $html;
    }


}