<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 04.07.18
 * Time: 12:16
 */

namespace DXIM\theme;


use DXIM\Admin;
use DXIM\Conf;

class Theme {

    protected static $instance = null;

    // result from ajax request
    protected $res = [];


    /**
     *
     * @return Theme
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    private function __construct() {
    }


    public function init() {
        TaxCF::getInstance()->init();
        UserCF::getInstance()->init();
        PTCF::getInstance()->init();

        Division::getInstance()->init();
        Location::getInstance()->init();
        Person::getInstance()->init();
        Brand::getInstance()->init();
        Model::getInstance()->init();
        Offer::getInstance()->init();
        Tag::getInstance()->init();
        Job::getInstance()->init();
        Service::getInstance()->init();

        add_action('admin_enqueue_scripts', [&$this, '_load_scripts']);

        add_action('wp_ajax_nopriv_wpcs_theme_load', [&$this, '_ajax_load']);
        add_action('wp_ajax_wpcs_theme_load', [&$this, '_ajax_load']);

        add_shortcode('WPCS_archive_pagination', [&$this, '_draw_pagination']);
    }

    public function _load_scripts() {
        if (!did_action('wp_enqueue_media')) {
            wp_enqueue_media();
        }
    }

    /**
     * @return void
     */
    public function _ajax_load() {
        $action = $_POST['do'];

        // LOAD MODELS BY BRAND ID
        if ($action == "load_models") {
            $brand_id = $_POST['settings']['brand_id'] ?? 0;
            $brand_id = intval($brand_id);
            if ($brand_id < 1) {
                $this->_response("Fehler in brand_id", 0);
            } else {
                $html = ModelTemplate::getInstance()->draw_selection(['brand_id' => $brand_id]);
                $this->_response($html);
            }
        } else {
            $this->_response("Aktion nicht erlaubt", 0);
        }


        header('Content-type: application/json');
        print json_encode($this->res);
        exit;
    }

    private function _response($msg, $code = 1, $data = []) {
        $this->res = ['result' => $msg, 'code' => $code, 'data' => $data];
    }

    /**
     * @return mixed|null
     */
    public function get_settings($type) {
        $settings = Admin::getInstance()->getOptions('theme');
        return $settings[$type] ?? [];
    }

    public function _draw_pagination($args) {
        if (!is_array($args)) $args = [];
        $defaults = ['prev_text' => "zurück", 'next_text' => "weiter"];
        $args = array_merge($args, $defaults);
        return get_the_posts_pagination($args);
    }
}
