<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 04.07.18
 * Time: 12:16
 */

namespace DXIM\theme;

class UserCF {

	protected static $instance = null;

    protected $fields = [];

	/**
	 *
	 * @return TaxCF
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	private function __construct() {}

    /**
     * @return void
     */
    public function init() {

        $this->_set_fields();

        add_action( 'show_user_profile', [&$this,'_profile_fields'] );
        add_action( 'edit_user_profile', [&$this,'_profile_fields'] );

        add_action( 'personal_options_update', [&$this,'_save_profile_fields'] );
        add_action( 'edit_user_profile_update', [&$this,'_save_profile_fields'] );
    }

    /**
     * @return void
     */
    private function _set_fields() {
        $fields = [];
        $this->fields = apply_filters(WPCS_THEME_USER_FIELDS, $fields);
    }

    /**
     * @param $term_id
     * @return void
     */
    public function _save_profile_fields($user_id) {

        if( ! isset( $_POST[ '_wpnonce' ] ) || ! wp_verify_nonce( $_POST[ '_wpnonce' ], 'update-user_' . $user_id ) ) {
            return;
        }

        if( ! current_user_can( 'edit_user', $user_id ) ) {
            return;
        }

        foreach($this->fields as $field) {
            if(!isset($_POST[$field['name']])) continue;
            update_user_meta( $user_id, $field['name'], $_POST[$field['name']]);
        }

    }

    /**
     * @return void
     */
    public function _profile_fields($user) {
        $html = null;
        $section_open = false;
        foreach($this->fields as $field) {

            if($field['type'] == "section") {
                if($section_open) $html .= $this->_close_section();

                $section_open = true;
                $html .= $this->_open_section($field['label']);
            }

            else {
                $value = get_user_meta($user->ID, $field['name']);
                $html .= '<tr>';
                    $html .= '<th><label for="city">'.$field['label'].'</label></th>';
                    $html .= '<td>'.PTCF::render_input_field($field, $value, ['hide_label' => 1]).'</td>';
                $html .= '</tr>';
            }

        }

        if($section_open) $html .= $this->_close_section();

        print $html;
    }

    private function _open_section($headline) {
        $html = "<h2>".$headline."</h2>";
        $html .= '<table class="form-table">';
        return $html;
    }

    private function _close_section() {
        return '</table>';
    }

}