<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;


use DXIM\Conf;
use DXIM\CustomTemplate;

class Brand {

	protected static $instance = null;

    public static $taxonomy = "wpcs_brand";

    public $filter = null;

    protected $brand = null;


	/**
	 *
	 * @return Brand
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {

	}

    /**
     * @return void
     */
    public function init() {
        if(!WPCS_PRO) return;

        $this->_register_taxonomy();

        $options = Theme::getInstance()->get_settings('brand');

        $this->filter = [];
        if(isset($_GET['filter_brand'])) $this->filter = $_GET['filter_brand'];

        add_action('wp', function() use ($options) {

            if(is_tax(self::$taxonomy)) {
                $this->brand =get_queried_object();

                // set template
                if($options['template_single'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template'],10,1);
                }
            }

        });


        BrandShortcode::getInstance()->init();
    }

    public function get_filter() { return $this->filter; }

    public function get_current() { return $this->brand; }

    public function get_brand($args) {
        $brand = null;
        if(!empty($args['id'])) {
            $ld = new BrandData();
            $brand = $ld->getById($args['id'], "term");
        } else $brand = $this->get_current();
        return $brand;
    }


    public function _get_template($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/brand/single');
    }


    function _register_taxonomy() {

        $labels = array(
            'name'                       => _x( 'Marken', 'Taxonomy General Name', 'wpcs' ),
            'singular_name'              => _x( 'Marke', 'Taxonomy Singular Name', 'wpcs' ),
            'menu_name'                  => __( 'Marken', 'wpcs' ),
            'all_items'                  => __( 'Alle Marken', 'wpcs' ),
            'parent_item'                => __( 'Parent Marken', 'wpcs' ),
            'parent_item_colon'          => __( 'Parent Marken:', 'wpcs' ),
            'new_item_name'              => __( 'neue Marke', 'wpcs' ),
            'add_new_item'               => __( 'neue Marken', 'wpcs' ),
            'edit_item'                  => __( 'Marke bearbeiten', 'wpcs' ),
            'update_item'                => __( 'Marke editieren', 'wpcs' ),
            'view_item'                  => __( 'Marke anzeigen', 'wpcs' ),
            'separate_items_with_commas' => __( 'Separate items with commas', 'wpcs' ),
            'add_or_remove_items'        => __( 'Add or remove items', 'wpcs' ),
            'choose_from_most_used'      => __( 'Choose from the most used', 'wpcs' ),
            'popular_items'              => __( 'Popular Items', 'wpcs' ),
            'search_items'               => __( 'Search Items', 'wpcs' ),
            'not_found'                  => __( 'Not Found', 'wpcs' ),
            'no_terms'                   => __( 'No items', 'wpcs' ),
            'items_list'                 => __( 'Items list', 'wpcs' ),
            'items_list_navigation'      => __( 'Items list navigation', 'wpcs' ),
        );

        $options = Theme::getInstance()->get_settings('brand');
        $rewrite = array(
            'slug'                       => $options['slug'] ?? 'marke',
            'with_front'                 => true,
            'hierarchical'               => false,
        );


        $args = array(
            'labels'                     => $labels,
            'hierarchical'               => true,
            'public'                     => true,
            'show_ui'                    => true,
            'show_admin_column'          => true,
            'show_in_nav_menus'          => true,
            'show_tagcloud'              => true,
            'rewrite'                   => $rewrite
        );
        register_taxonomy( self::$taxonomy, array( Model::$post_type, Person::$post_type, Offer::$post_type ), $args );

    }

}