<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;


use DXIM\CustomTemplate;

class BrandTemplate {

	protected static $instance = null;

	/**
	 *
	 * @return BrandTemplate
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {	}

    public function init() {
        add_shortcode("WPCS_brand_logos", [&$this,'_shortcode_draw_brand_logos']);
    }

    public function _shortcode_draw_brand_logos($args) {
        if(!is_array($args)) $args = [];
        return $this->draw_brand_logos($args);
    }

    /**
     * @param $brand
     * @param $args
     * @return false|string
     */
    public function draw_brand_team($brand, $args = []) {
        if(!is_array($args)) $args = [];
        $settings = Theme::getInstance()->get_settings('brand');
        $args = array_merge($settings, $args);

        $ld = new LocationData();
        $query_args = [];
        if(!empty($brand)) $query_args['brand'] = [$brand->term_id];
        $locations = $ld->getTree($query_args);
        #wpcs_p($locations);

        $pd = new PersonData();
        $locations_persons = [];
        foreach($locations as $array) {
            $locations_tmp = !empty($array['childs']) ? $array['childs'] : [$array['parent']];
            foreach($locations_tmp as $location) {
                $query = [
                    'location' => $location->term_id,
                    'division' => $settings['person_query']['division'],
                    'count' => $settings['person_query']['count'],
                    'random' => $settings['person_query']['random'],
                    'brand' => $brand->term_id
                ];

                $persons = $pd->getPersons($query) ?? null;
                $locations_persons[$location->term_id] = $persons;
            }

        }

        $template = $args['template'] ?? 'persons_locations';
        return CustomTemplate::getInstance()->get_template_content('theme/person/'.$template,['locations' => $locations, 'locations_persons' => $locations_persons, 'args' => $args]);
    }

    /**
     * @param $brand
     * @param $args
     * @return false|string
     */
    public function draw_brand_models($brand, $args) {
        if(!is_array($args)) $args = [];
        #$defaults = ['grid' => 3];
        #$args = array_merge($defaults, $args);
        $args['brand_id'] = $brand->term_id;
        return ModelTemplate::getInstance()->draw_selection($args);
    }

    /**
     * @param $brand
     * @param $args
     * @return false|string
     */
    public function draw_brand_content($brand, $args) {
        if(!is_array($args)) $args = [];
        $args['brand_id'] = $brand->term_id;
        return CustomTemplate::getInstance()->get_template_content('theme/brand/content', ['brand' => $brand, 'args' => $args]);
    }

    public function draw_brand_logo($brand, $args) {
        if(!is_array($args)) $args = [];
        $args['brand_id'] = $brand->term_id;
        return CustomTemplate::getInstance()->get_template_content('theme/brand/logo', ['brand' => $brand, 'args' => $args]);
    }

    public function draw_brand_offers($brand, $args) {
        if(!is_array($args)) $args = [];
        $defaults = ['layout' => 'list', 'posts_per_page' => 5];
        $args = array_merge($defaults, $args);

        $od = new OfferData();
        $offers = $od->get_offers(['brand' => $brand->slug, 'posts_per_page' => $args['posts_per_page']]);

        return OfferTemplate::getInstance()->draw_offers($offers,$args);
    }

    public function draw_brand_offers_link($brand, $args) {
        if(!is_array($args)) $args = [];
        $defaults = ['label' => 'Alle Angebote zeigen'];
        $args = array_merge($defaults, $args);

        $link = get_post_type_archive_link(Offer::$post_type);
        $html = '<a class="dxim_button dxim_button_light" href="'.$link.'?filter_brand[]='.$brand->term_id.'">'.$args['label'].'</a>';
        return $html;
    }

    /**
     * Zeigt die Marken Logos aller Marken an
     * @param $args
     * @return false|string
     */
    public function draw_brand_logos($args = []) {
        $defaults = ['align' => 'left', 'location_id' => 0, 'service' => 0];
        $args = array_merge($defaults, $args);

        $filter = [
            'service' => $args['service']
        ];

        if($args['location_id'] > 0) {
            $ids = get_term_meta($args['location_id'], "brands");
            if(isset($ids[0]) && is_array($ids[0])) $ids = $ids[0]; // fallback zur alten serialisierten version
            if(!empty($ids)) $filter['include'] = $ids;
        }

        $bd = new BrandData();
        $brands = $bd->getAll($filter);

        $template = $args['template'] ?? "logos";
        return \DXIM\CustomTemplate::getInstance()->get_template_content('theme/brand/'.$template, ['brands' => $brands, 'args' => $args]);
    }

}