<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\CustomTemplate;

class Job {

	protected static $instance = null;

    public static $post_type = "wpcs_job";

    private $employment_types = [
        'FULL_TIME' => 'Vollzeit', 'PART_TIME' => 'Teilzeit', 'EDUCATION' => 'Ausbildung', 'INTERNSHIP' => 'Praktikant'
    ];

    protected $job = null;

    protected $filter = [];


	/**
	 *
	 * @return Job
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}


    public function get_employment_types() {
        return $this->employment_types;
    }

	/**
	 *
	 */
	private function __construct() {}

    /**
     * @return void
     */
    public function init() {
        if(!WPCS_PRO) return;

        $options = Theme::getInstance()->get_settings('job');

        if(isset($_GET['filter_job_type'])) {
            $this->filter['types'] = $_GET['filter_job_type'];
        }

        add_action('wp', function() use ($options) {

            if(is_singular(Job::$post_type)) {
                global $post;
                $this->job = $post;
                if($options['template_single'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template'],10,1);
                }
            }

            if(is_post_type_archive(Job::$post_type)) {
                if($options['template_archive'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template_archive'],10,1);
                }
            }
        });

        add_filter('pre_get_posts', [&$this, '_mod_main_query']);

        JobShortcode::getInstance()->init();

        $this->_register_post_type();

    }

    /**
     * @param $query
     * @return void
     */
    public function _mod_main_query($query) {
        $pt = $query->get("post_type");
        if(is_admin() || !$query->is_main_query() || $pt != Job::$post_type) return;
        $options = Theme::getInstance()->get_settings('job');

        $posts_per_page = $options['posts_per_page'] ?? 50;
        $query->set('posts_per_page', $posts_per_page);

        $meta_query = $query->get('meta_query');
        if(!is_array($meta_query)) $meta_query = [];

        $tax_query = [];


        // filter location
        $filter_location = Location::getInstance()->get_filter();
        if(!empty($filter_location)) {
            $tax_query[] = ['taxonomy' => Location::$taxonomy, 'terms' => $filter_location];
        }

        // filter tags
        $filter_tags = Tag::getInstance()->get_filter();
        if(!empty($filter_tags)) {
            $array = [
                'relation' => 'AND'
            ];
            foreach($filter_tags as $group => $tags) {
                $array[] = ['taxonomy' => Tag::$taxonomy, 'terms' => $tags]; // OR
            }
            $tax_query[] = $array;
        }

        // type filter
        $filter_types = $this->filter['types'] ?? [];
        if(!empty($filter_types)) {
            $mq = [
                'relation' => 'OR',
            ];
            foreach($filter_types as $type) {
                $mq[] = ['key' => 'employmentType', 'value' => $type];
            }
            $meta_query[] = $mq;
        }

        $query->set('tax_query', $tax_query);
        $query->set('meta_query', $meta_query);
    }

    /**
     * @param $template
     * @return string
     */
    public function _get_template($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/job/single');
    }

    /**
     * @param $template
     * @return string
     */
    public function _get_template_archive($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/job/archive');
    }

    /**
     * @return mixed|null
     */
    public function get_current() { return $this->job; }

    public function get_filter() { return $this->filter; }


    public function _register_post_type() {

        $labels = array(
            'name'                  => _x( 'Jobs', 'Post Type General Name', 'wpcs' ),
            'singular_name'         => _x( 'Job', 'Post Type Singular Name', 'wpcs' ),
            'menu_name'             => __( 'Jobs', 'wpcs' ),
            'name_admin_bar'        => __( 'Jobs', 'wpcs' ),
            'archives'              => __( 'Jobs', 'wpcs' ),
            'attributes'            => __( 'Jobs Attributes', 'wpcs' ),
            'parent_item_colon'     => __( 'Parent Job', 'wpcs' ),
            'all_items'             => __( 'Alle Jobs', 'wpcs' ),
            'add_new_item'          => __( 'neuer Job', 'wpcs' ),
            'add_new'               => __( 'anlegen', 'wpcs' ),
            'new_item'              => __( 'anlegen', 'wpcs' ),
            'edit_item'             => __( 'editieren', 'wpcs' ),
            'update_item'           => __( 'update', 'wpcs' ),
            'view_item'             => __( 'anzeigen', 'wpcs' ),
            'view_items'            => __( 'anzeigen', 'wpcs' ),
            'search_items'          => __( 'suchen', 'wpcs' ),
            'not_found'             => __( 'nicht gefunden', 'wpcs' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'wpcs' ),
            'featured_image'        => __( 'Featured Image', 'wpcs' ),
            'set_featured_image'    => __( 'Set featured image', 'wpcs' ),
            'remove_featured_image' => __( 'Remove featured image', 'wpcs' ),
            'use_featured_image'    => __( 'Use as featured image', 'wpcs' ),
            'insert_into_item'      => __( 'Insert into item', 'wpcs' ),
            'uploaded_to_this_item' => __( 'Uploaded to this item', 'wpcs' ),
            'items_list'            => __( 'Items list', 'wpcs' ),
            'items_list_navigation' => __( 'Items list navigation', 'wpcs' ),
            'filter_items_list'     => __( 'Filter items list', 'wpcs' ),
        );

        $options = Theme::getInstance()->get_settings('job');
        $rewrite = array(
            'slug'                  => $options['slug'] ?? 'stellenangebote',
            'with_front'            => true,
            'pages'                 => true,
            'feeds'                 => true,
        );

        $args = array(
            'label'                 => __( 'Jobs', 'wpcs' ),
            'description'           => __( 'Jobs', 'wpcs' ),
            'labels'                => $labels,
            'supports'              => array( 'title', 'editor', 'thumbnail' ),
            'taxonomies'            => array( Location::$taxonomy, Tag::$taxonomy ),
            'hierarchical'          => false,
            'public'                => true,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'menu_position'         => 5,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => true,
            'can_export'            => true,
            'has_archive'           => true,
            'exclude_from_search'   => false,
            'publicly_queryable'    => true,
            'rewrite'               => $rewrite,
            'capability_type'       => 'page',
        );

        register_post_type( self::$post_type, $args );

    }

}