<?php
/**
 * Created by PhpStorm.
 * User: dxPro
 * Date: 08/01/16
 * Time: 09:47
 */

namespace DXIM\theme;

class JobSchema {

    private $job = null;

    public function __construct($job) {
        $this->job = $job;
    }

    public function create() {

        $jd = new JobData();
        $locations = $jd->get_locations($this->job->ID);

        $ld = new LocationData();
        $locations = $ld->buildTree($locations);

        $keys = array_keys($locations);
        $locations = $locations[$keys[0]];

        $homeOffice = get_post_meta($this->job->ID, "homeoffice", true);

        $data = [
            '@context' => "https://schema.org",
            '@type' => "JobPosting",
            "title" => $this->job->post_title,
            "description" => $this->job->post_content,
            "datePosted" => $this->job->post_date,
            "employmentType" => $this->get_employment_type(),
            "hiringOrganization" => $this->_get_hiring_organisation($locations['parent']),
            "jobLocation" => $this->_get_job_location($locations['childs']),
        ];

        // 100% HomeOffice Möglich
        if($homeOffice) $data["jobLocationType"] = "TELECOMMUTE";

        return $data;
    }

    /**
     * @return string[]
     */
    private function _get_hiring_organisation($location) {

        $data = [
            "@type" => "Organization",
            "name" => $location->name,
            "sameAs" => get_term_link($location->term_id),
        ];

        $logo_id = get_term_meta($location->term_id, "logo", true);
        if(!empty($logo_id)) {
            $data['logo'] = wp_get_attachment_image_url($logo_id);
        }

        return $data;

    }

    private function _get_job_location($locations) {
        $data = [];
        foreach($locations as $location) {
            $data[] = [
                "@type" => "Place",
                "address" => [
                    "@type" => "PostalAddress",
                    "streetAddress" => get_term_meta($location->term_id, "street", true),
                    "addressLocality" => get_term_meta($location->term_id, "city", true),
                    "postalCode" => get_term_meta($location->term_id, "zipcode", true),
                    "addressCountry" => "DE",
                ]
            ];
        }

        return $data;
    }

    private function get_employment_type() {
        $value = get_post_meta($this->job->ID, "employmentType", true);
        if($value == "INTERNSHIP") $value = "INTERN";
        elseif($value == "EDUCATION") $value = "FULL_TIME";

        return $value;
    }

}