<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\Admin;
use DXIM\CustomTemplate;

class Location {

	protected static $instance = null;

    public static $taxonomy = "wpcs_location";

    // WP_Term, der anhand des Slug ermittelt wurde
    protected $location = null;

    protected $filter = null;
    protected $filter_city = null;

	/**
	 *
	 * @return Location
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {

	}

    /**
     * @return mixed|null
     */
    public function get_current() { return $this->location; }

    public function get_filter() { return $this->filter; }
    public function get_city_filter() { return $this->filter_city; }

    /**
     * @return void
     */
    public function init() {
        $this->_register_taxonomy();

        $options = Theme::getInstance()->get_settings('location');

        $this->filter = [];
        if(isset($_GET['filter_location'])) $this->filter = $_GET['filter_location'];

        $this->filter_city = [];
        if(isset($_GET['filter_location_city'])) $this->filter_city = $_GET['filter_location_city'];

        add_action('wp', function() use ($options) {

            if(is_tax(self::$taxonomy)) {
                $this->location = get_queried_object();

                // set template
                if($options['template_tax'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template'],10,1);
                }

            }
        });

        LocationShortcode::getInstance()->init();
        LocationTemplate::getInstance()->init();
    }

    public function _get_template($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/location/single');
    }


    function _register_taxonomy() {

        $labels = array(
            'name'                       => _x( 'Standorte', 'Taxonomy General Name', 'wpcs' ),
            'singular_name'              => _x( 'Standort', 'Taxonomy Singular Name', 'wpcs' ),
            'menu_name'                  => __( 'Standorte', 'wpcs' ),
            'all_items'                  => __( 'Alle Standorte', 'wpcs' ),
            'parent_item'                => __( 'Parent Standort', 'wpcs' ),
            'parent_item_colon'          => __( 'Parent Standort:', 'wpcs' ),
            'new_item_name'              => __( 'neuer Standort', 'wpcs' ),
            'add_new_item'               => __( 'neuer Standort', 'wpcs' ),
            'edit_item'                  => __( 'Standort bearbeiten', 'wpcs' ),
            'update_item'                => __( 'Standort editieren', 'wpcs' ),
            'view_item'                  => __( 'Standort anzeigen', 'wpcs' ),
            'separate_items_with_commas' => __( 'Separate items with commas', 'wpcs' ),
            'add_or_remove_items'        => __( 'Add or remove items', 'wpcs' ),
            'choose_from_most_used'      => __( 'Choose from the most used', 'wpcs' ),
            'popular_items'              => __( 'Popular Items', 'wpcs' ),
            'search_items'               => __( 'Search Items', 'wpcs' ),
            'not_found'                  => __( 'Not Found', 'wpcs' ),
            'no_terms'                   => __( 'No items', 'wpcs' ),
            'items_list'                 => __( 'Items list', 'wpcs' ),
            'items_list_navigation'      => __( 'Items list navigation', 'wpcs' ),
        );

        $options = Theme::getInstance()->get_settings('location');
        $rewrite = array(
            'slug'                       => $options['slug'] ?? 'standort',
            'with_front'                 => true,
            'hierarchical'               => false,
        );

        $public = (bool)WPCS_PRO;

        $args = array(
            'labels'                     => $labels,
            'hierarchical'               => true,
            'public'                     => $public,
            'show_ui'                    => true,
            'show_admin_column'          => true,
            'show_in_nav_menus'          => true,
            'show_tagcloud'              => true,
            'rewrite'                    => $rewrite,
        );
        register_taxonomy( self::$taxonomy, array( Person::$post_type ), $args );

    }


}