<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;


class LocationData {


    /**
     * @return array
     */
    public function getAll($args = []) {
        if (!is_array($args)) $args = [];
        $defaults = ['parent' => '', 'include' => ''];
        $args = array_merge($defaults, $args);

        $forced = ['taxonomy' => Location::$taxonomy, 'hide_empty' => false];
        $query = array_merge($args, $forced);

        return get_terms($query);
    }

    /**
     * @param $args
     * @return array
     */
    public function getTree($args = []) {
        if (!is_array($args)) $args = [];

        $all_locations = $this->getFiltered($args);
        $locations = $this->buildTree($all_locations);
        return $locations;
    }

    public function buildTree($all_locations) {

        $parent_ids = [];
        $no_parents = [];
        foreach ($all_locations as $location) {
            if ($location->parent > 0) {
                $parent_ids[$location->parent] = 1;
            } else {
                $no_parents[] = $location;
            }
        }

        $locations = [];
        if (!empty($parent_ids)) {
            $parents = $this->getFiltered(['include' => array_keys($parent_ids)]);
            foreach ($parents as $parent) {
                $locations[$parent->term_id]['parent'] = $parent;
                $locations[$parent->term_id]['childs'] = $this->_get_childs($parent->term_id, $all_locations);
            }
        }

        if (!empty($no_parents)) {
            foreach ($no_parents as $term) {
                if (isset($locations[$term->term_id])) continue;
                $locations[$term->term_id]['parent'] = $term;
                $locations[$term->term_id]['childs'] = [];
            }
        }

        return $locations;
    }

    private function _get_childs($parent_id, $terms) {
        $valid = [];
        foreach ($terms as $term) {
            if ($term->parent == $parent_id) $valid[] = $term;
        }
        return $valid;
    }


    /**
     * @param $args
     * @return int[]|string|string[]|\WP_Error|\WP_Term[]
     */
    public function getFiltered($args = []) {
        if (!is_array($args)) $args = [];
        $defaults = ['city' => [], 'brand' => [], 'parent' => "", 'include' => ""];
        $args = array_merge($defaults, $args);

        $query = ['taxonomy' => Location::$taxonomy, 'hide_empty' => false, 'orderby' => 'name', 'order' => 'ASC'];
        if ($args['parent'] !== "") $query['parent'] = $args['parent'];
        if ($args['include'] !== "") $query['include'] = $args['include'];

        $meta_query = [];

        if(!empty($args['brand'])) {
            $meta_query[] = [
                'key' => 'brands', 'value' => $args['brand'], 'compare' => 'IN'
            ];
        }

        // Filtere nach Stadt direkt in meta query
        if (!empty($args['city'])) $meta_query[] = ['key' => 'city', 'value' => $args['city']];
        if (!empty($meta_query)) $query['meta_query'] = $meta_query;

        // abrufen der terms
        $terms = get_terms($query);
        return array_values($terms);
    }

    /**
     * @param $term_id
     * @return array|\WP_Error|\WP_Term
     */
    public function getById($term_id) {
        $term = get_term_by("id", $term_id, Location::$taxonomy);
        if (empty($term)) return [];
        return $term;
    }

    /**
     * @param $openingHoursText
     * @return array|array[]
     */
    public function parse_opening_hours($openingHoursText) {
        if (empty($openingHoursText)) return [];

        $days = [
            'Mo' => ['nr' => 1, 'label' => 'Montag'],
            'Di' => ['nr' => 2, 'label' => 'Dienstag'],
            'Mi' => ['nr' => 3, 'label' => 'Mittwoch'],
            'Do' => ['nr' => 4, 'label' => 'Donnerstag'],
            'Fr' => ['nr' => 5, 'label' => 'Freitag'],
            'Sa' => ['nr' => 6, 'label' => 'Samstag'],
            'So' => ['nr' => 7, 'label' => 'Sonntag'],
        ];

        // Zeilen in einzelne Tage aufteilen
        $openings = [];
        foreach (explode("\n", $openingHoursText) as $line) {
            $line_data = explode(":", $line);
            $times = explode(",", $line_data[1] ?? "");
            $times = array_map(function ($entry) {
                return explode("-", $entry);
            }, $times);
            $openings[$line_data[0]] = $times;
        }

        foreach ($days as $short => $day_data) {
            $opening_data = $openings[$short] ?? [];
            $days[$short]['times'] = $opening_data;
        }

        return $days;
    }

    public function is_location_open($days) {
        $currentDay = current_time('N');
        $currentTime = current_time('Hi');
        $is_open = 0;
        $closes_at = null;
        $opens_at = null;

        foreach ($days as $day) {
            if ($day['nr'] != $currentDay) continue;
            foreach ($day['times'] as $time) {
                if (intval($time[0]) > $currentTime && empty($opens_at)) {
                    $opens_at = ['nr' => $currentDay, 'label' => $day['label'], 'time' => $time[0]];
                }
                if (intval($time[0]) <= $currentTime && intval($time[1]) >= $currentTime) {
                    $is_open = 1;
                    $closes_at = $time[1];
                    break;
                }
            }
        }

        $z = 1;
        $check_day = $currentDay + 1;
        while (empty($opens_at)) {
            if ($check_day > 7) $check_day = 1;

            foreach ($days as $day) {
                if ($day['nr'] != $check_day) continue;
                if (isset($day['times'][0][0])) {
                    $opens_at = ['nr' => $check_day, 'label' => $day['label'], 'time' => $day['times'][0][0]];
                    break;
                }
            }

            $check_day++;
            if ($z > 5) break; // sicherheit bzgl. endlosschleife
            $z++;
        }

        return ['is_open' => $is_open, 'closes_at' => $closes_at, 'opens_at' => $opens_at, 'current_day' => $currentDay];
    }

    /**
     * @param $locationTree
     * @return array
     */
    public function getLocationSelectArray($locationTree) {
        foreach ($locationTree as $array) {
            $locations = !empty($array['childs']) ? $array['childs'] : [$array['parent']];
            foreach ($locations as $tmp) {
                $label = ($array['parent']->term_id != $tmp->term_id) ? $array['parent']->name . " " . $tmp->name : $tmp->name;
                $values[$tmp->term_id] = apply_filters('dxim_filter_location_select_name', $label, $tmp);
            }
        }
        return $values;
    }
}
