<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\CustomTemplate;

class LocationShortcode {

    protected static $instance = null;

    /**
     *
     * @return LocationShortcode
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     *
     */
    private function __construct() {
    }

    /**
     * @return void
     */
    public function init() {
        add_shortcode('WPCS_location_title', [&$this, '_draw_location_title']);
        add_shortcode('WPCS_location_description', [&$this, '_draw_location_description']);
        add_shortcode('WPCS_location_image', [&$this, '_draw_location_image']);
        add_shortcode('WPCS_location_gallery', [&$this, '_draw_location_gallery']);
        add_shortcode('WPCS_location_address', [&$this, '_draw_location_address']);
        add_shortcode('WPCS_location_team', [&$this, '_draw_location_team']);
        add_shortcode('WPCS_location_opening_hours', [&$this, '_draw_location_opening_hours']);
        add_shortcode('WPCS_location_brands', [&$this, '_draw_location_brands']);
        add_shortcode('WPCS_location_jobs', [&$this, '_draw_location_jobs']);
        add_shortcode('WPCS_location_offers', [&$this, '_draw_location_offers']);
        add_shortcode('WPCS_location_services', [&$this, '_draw_location_services']);

        add_shortcode("WPCS_location_filter", [&$this, '_draw_filter_locations']);
        add_shortcode("WPCS_location_city_filter", [&$this, '_draw_filter_location_cities']);

        add_shortcode("WPCS_locations", [&$this, '_draw_locations']);
    }

    /**
     * @param $args
     * @return array|false|mixed|\WP_Error|\WP_Term|null
     */
    private function _get_location($args) {
        $location = null;
        if (!empty($args['id'])) {
            $ld = new LocationData();
            $location = $ld->getById($args['id']);
        } else $location = Location::getInstance()->get_current();
        return $location;
    }

    public function _draw_locations($args) {
        return LocationTemplate::getInstance()->draw_locations($args);
    }

    public function _draw_location_address($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_address($location, $args);
    }

    public function _draw_location_image($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_image($location, $args);
    }

    public function _draw_location_gallery($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_gallery($location, $args);
    }

    public function _draw_location_opening_hours($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_opening_hours($location, $args);
    }

    public function _draw_location_title($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_title($location, $args);
    }

    public function _draw_location_description($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_description($location, $args);
    }

    public function _draw_location_team($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_team($location, $args);
    }

    public function _draw_location_brands($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_brands($location, $args);
    }

    public function _draw_location_jobs($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_jobs($location, $args);
    }

    public function _draw_location_offers($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_offers($location, $args);
    }

    public function _draw_location_services($args) {
        $location = $this->_get_location($args);
        return LocationTemplate::getInstance()->draw_location_services($location, $args);
    }

    /**
     * @param $args
     * @return string
     */
    public function _draw_filter_locations($args) {
        return LocationTemplate::getInstance()->draw_filter_locations($args);
    }

    /**
     * @param $args
     * @return string
     */
    public function _draw_filter_location_cities($args) {
        $ld = new LocationData();
        $locations = $ld->getAll();
        $active_filters = Location::getInstance()->get_city_filter() ?? [];

        $cities = [];
        foreach ($locations as $location) {
            $city = get_term_meta($location->term_id, 'city', true);
            if (in_array($city, $cities) || empty($city)) continue;
            $cities[] = $city;
        }

        $html = '<div class="wpcs_theme_filters location_cities">';
        foreach ($cities as $city) {
            $html .= '<div class="theme_filter location_city_filter city">';
            $checked = (in_array($city, $active_filters)) ? "checked" : "";
            $html .= '<input type="checkbox" class="wpcs_checkbox" name="filter_location_city[]" value="' . $city . '" ' . $checked . ' />';
            $html .= '<span class="label">' . $city . '</span>';
            $html .= '</div>';
        }
        $html .= '</div>';

        return $html;
    }
}
