<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\CustomTemplate;

class LocationTemplate {

    protected static $instance = null;

    /**
     *
     * @return LocationTemplate
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     *
     */
    private function __construct() {
    }

    /**
     * @return void
     */
    public function init() {
    }

    /**
     * @param $args
     * @return false|string
     */
    public function draw_locations($args = []) {
        if (!is_array($args)) $args = [];
        $defaults = ['layout' => 'grid', 'grid' => 3, 'location' => "", 'title' => ""];
        $args = array_merge($defaults, $args);
        $ld = new LocationData();
        $filter = [
            'brand' => Brand::getInstance()->get_filter(),
            'city' => Location::getInstance()->get_city_filter(),
            'parent' => $args['location']
        ];
        $locations = $ld->getTree($filter);

        $template = $args['template'] ?? $args['layout'];
        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['locations' => $locations, 'args' => $args]);
    }


    public function draw_location_image($location, $args = []) {
        $template = $args['template'] ?? 'image.php';
        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['location' => $location]);
    }

    public function draw_location_gallery($location, $args = []) {
        $template = $args['template'] ?? 'gallery.php';
        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['location' => $location]);
    }

    public function draw_location_title($location, $args = []) {
        if (!is_array($args)) $args = [];
        $defaults = ['title' => 'parent'];
        $args = array_merge($defaults, $args);

        $parent = null;
        if ($args['title'] == "parent") {
            $ld = new LocationData();
            $parent = $ld->getById($location->parent);
        }

        $template = $args['template'] ?? 'title';
        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['location' => $location, 'parent' => $parent, 'args' => $args]);
    }

    public function draw_location_description($location, $args = []) {
        $template = $args['template'] ?? 'description.php';
        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['location' => $location]);
    }

    public function draw_location_opening_hours($location, $args = []) {
        if (!is_array($args)) $args = [];
        $defaults = ['type' => 'sale', 'show_status' => 1, 'show_days' => 1, 'link_days' => 0];
        $args = array_merge($defaults, $args);

        $template = $args['template'] ?? 'opening_hours';
        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['location' => $location, 'args' => $args]);
    }

    public function draw_location_address($location, $args = []) {
        if(empty($location)) return null;

        if (!is_array($args)) $args = [];
        $defaults = ['title' => 'parent'];
        $args = array_merge($defaults, $args);

        $parent = null;
        if ($args['title'] == "parent") {
            $ld = new LocationData();
            $parent = $ld->getById($location->parent);
        }

        $template = $args['template'] ?? 'address';
        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['location' => $location, 'parent' => $parent, 'args' => $args]);
    }

    public function draw_location_brands($location, $args = []) {
        if(!is_array($args)) $args = [];
        $args['location_id'] = $location->term_id;
        return BrandTemplate::getInstance()->draw_brand_logos($args);
    }


    /**
     * @param $location
     * @param $args
     * @return false|string
     */
    public function draw_location_services($location, $args = []) {
        $sd = new ServiceData();
        $services = $sd->get_services_by_location($location->term_id, $args);
        return ServiceTemplate::getInstance()->draw_services($services, $args);
    }

    /**
     * @return false|string
     */
    public function draw_location_team($location, $args = []) {
        if (!is_array($args)) $args = [];
        $settings = Theme::getInstance()->get_settings('location');
        $args = array_merge($settings, $args);

        $d = new DivisionData();
        $divisions = $d->getAll();

        $pd = new PersonData();
        $team = [];

        foreach ($divisions as $key => $division) {
            $persons = $pd->getPersons(['division' => $division->term_id, 'location' => $location->term_id, 'is_contact' => 0]);
            if (empty($persons)) {
                unset($divisions[$key]);
                continue;
            }
            $team[$division->term_id] = $persons;
        }

        $divisions = array_values($divisions);
        if (empty($divisions) || empty($team)) return null;

        $layout = $args['division_layout'] ?? "select";
        $template = $args['template'] ?? 'division_' . $layout . '.php';

        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['divisions' => $divisions, 'team' => $team, 'args' => $args]);
    }

    /**
     *
     */
    public function draw_location_jobs($location, $args = []) {
        $jd = new JobData();
        $jobs = $jd->get_jobs(['location' => $location->term_id, 'posts_per_page' => -1]);

        return JobTemplate::getInstance()->draw_jobs($jobs, $args);
    }

    /**
     * @param $location
     * @param $args
     * @return false|string
     */
    public function draw_location_offers($location, $args = []) {
        $brand_ids = get_term_meta($location->term_id, "brands");
        $od = new OfferData();
        $offers = $od->get_offers(['brand' => $brand_ids]);

        return OfferTemplate::getInstance()->draw_offers($offers, $args);
    }

    /**
     * @param $args
     * @return false|string
     */
    public function draw_filter_locations($args) {
        if (!is_array($args)) $args = [];
        $defaults = ['layout' => 'checkbox'];
        $args = array_merge($defaults, $args);

        $ld = new LocationData();
        $locations = $ld->getTree();

        $active_filters = Location::getInstance()->get_filter() ?? [];

        $layout = $args['layout'] ?? "checkbox";
        $template = $args['template'] ?? 'filter_' . $layout . '.php';

        return CustomTemplate::getInstance()->get_template_content('theme/location/' . $template, ['locations' => $locations, 'active_filters' => $active_filters]);
    }
}
