<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;


use DXIM\CustomTemplate;
use DXIM\GlobalFilter;

class Model {

    protected static $instance = null;

    public static $post_type = "wpcs_model";

    protected $model = null;

    public $wpcs_query = "";

    /**
     *
     * @return Model
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     *
     */
    private function __construct() {
    }

    public function init() {

        if (!WPCS_PRO) return;

        $this->_register_post_type();
        $options = Theme::getInstance()->get_settings('model');

        add_action('wp', function () use ($options) {

            if (is_singular(Model::$post_type)) {
                global $post;

                // set model Object
                $this->model = $post;

                // set wpcs query params
                $this->set_query();

                // set template
                if ($options['template_single'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template'], 10, 1);
                }
            }
        });

        ModelTemplate::getInstance()->init();
        ModelShortcode::getInstance()->init();
    }


    /**
     * @return mixed|null
     */
    public function getCurrent() {
        return $this->model;
    }


    /**
     * @return void
     */
    private function set_query() {
        $this->wpcs_query = get_post_meta($this->model->ID, 'wpcs_query', true);
        if (!is_array($this->wpcs_query)) $this->wpcs_query = [];
        if (empty($this->wpcs_query)) {
            $md = new ModelData();
            $brand = $md->get_brand($this->model->ID);
            $this->wpcs_query['make'] = $brand->name;
            $this->wpcs_query['model'] = $this->model->post_title;
        }
    }


    /**
     * @param $template
     * @return string
     */
    public function _get_template($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/model/single.php');
    }

    /**
     * Backend Meta Box globale Filter zum Mapping des passenden Modells in WPCarSync
     * @return string
     */
    public function _draw_wpcarsync_mapping() {
        global $post;
        $global_filters = get_post_meta($post->ID, 'wpcs_query', true);
        if (!is_array($global_filters)) $global_filters = [];

        $html = "<h4>WPCarSync Mapping</h4>";
        $html .= '<p>Definieren Sie hier, um welches WPCarSync Modell es sich handelt.</p>';
        $html .= '<div id="wpcs_query_meta_box" class="wpcs_global_filters">';
        $html .= GlobalFilter::getInstance()->render_global_filters($global_filters, $post);
        $html .= '</div>';

        return $html;
    }


    public function _register_post_type() {

        $labels = array(
            'name'                  => _x('Modelle', 'Post Type General Name', 'wpcs'),
            'singular_name'         => _x('Modell', 'Post Type Singular Name', 'wpcs'),
            'menu_name'             => __('Modelle', 'wpcs'),
            'name_admin_bar'        => __('Modelle', 'wpcs'),
            'archives'              => __('Modelle', 'wpcs'),
            'attributes'            => __('Modell Attributes', 'wpcs'),
            'parent_item_colon'     => __('Parent Modell', 'wpcs'),
            'all_items'             => __('Alle Modelle', 'wpcs'),
            'add_new_item'          => __('neues Modell', 'wpcs'),
            'add_new'               => __('anlegen', 'wpcs'),
            'new_item'              => __('anlegen', 'wpcs'),
            'edit_item'             => __('editieren', 'wpcs'),
            'update_item'           => __('update', 'wpcs'),
            'view_item'             => __('anzeigen', 'wpcs'),
            'view_items'            => __('anzeigen', 'wpcs'),
            'search_items'          => __('suchen', 'wpcs'),
            'not_found'             => __('nicht gefunden', 'wpcs'),
            'not_found_in_trash'    => __('Not found in Trash', 'wpcs'),
            'featured_image'        => __('Featured Image', 'wpcs'),
            'set_featured_image'    => __('Set featured image', 'wpcs'),
            'remove_featured_image' => __('Remove featured image', 'wpcs'),
            'use_featured_image'    => __('Use as featured image', 'wpcs'),
            'insert_into_item'      => __('Insert into item', 'wpcs'),
            'uploaded_to_this_item' => __('Uploaded to this item', 'wpcs'),
            'items_list'            => __('Items list', 'wpcs'),
            'items_list_navigation' => __('Items list navigation', 'wpcs'),
            'filter_items_list'     => __('Filter items list', 'wpcs'),
        );

        $options = Theme::getInstance()->get_settings('model');
        $rewrite = array(
            'slug'                  => $options['slug'] ?? 'modell',
            'with_front'            => true,
            'pages'                 => true,
            'feeds'                 => true,
        );

        $args = array(
            'label'                 => __('Modelle', 'wpcs'),
            'description'           => __('Modelle', 'wpcs'),
            'labels'                => $labels,
            'supports'              => array('title', 'editor', 'excerpt', 'thumbnail', 'page-attributes'),
            'taxonomies'            => array(Brand::$taxonomy, Tag::$taxonomy),
            'hierarchical'          => false,
            'public'                => true,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'menu_position'         => 5,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => true,
            'can_export'            => true,
            'has_archive'           => false,
            'exclude_from_search'   => false,
            'publicly_queryable'    => true,
            'rewrite'               => $rewrite,
            'capability_type'       => 'page',
        );

        register_post_type(self::$post_type, $args);
    }
}
