<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;


class ModelData {
    /**
     * @param $model_id
     * @return int|\WP_Post|null
     */
    public function getById($model_id) {
        $q = new \WP_Query(['p' => $model_id, 'post_type' => Model::$post_type]);
        return $q->post_count > 0 ? $q->posts[0] : null;
    }

    /**
     * @param $post_id
     * @return false|\WP_Error|\WP_Term[]
     */
    public function get_brand($post_id) {
        $terms = get_the_terms($post_id, Brand::$taxonomy);
        return $terms[0] ?? [];
    }

    /**
     * @param $post_id
     * @return false|\WP_Error|\WP_Term[]
     */
    public function get_tags($post_id) {
        $tags = get_the_terms($post_id, Tag::$taxonomy);
        return is_array($tags) ? $tags : [];
    }


    public function get_models($args) {
        if(!is_array($args)) $args = [];
        $defaults = ['brand_id' => '', 'slug' => '', 'id' => 0, 'tags' => '', 'meta_query' => '', 'orderby' => 'menu_order', 'order' => 'ASC', 'posts_per_page' => -1, 'meta_query' => []];
        $args = array_merge($defaults, $args);
        $query = ['post_type' => Model::$post_type, 'post_status' => 'publish'];
        $query['posts_per_page'] = $args['posts_per_page'];
        $query['orderby'] = $args['orderby'];
        $query['order'] = $args['order'];

        $tax_query = [];

        // filter model id
        if(!empty($args['id'])) {
            $query['p'] = $args['id'];
        }

        // filter model slug
        elseif(!empty($args['slug'])) {
            $query['name'] = $args['slug'];
        }

        // filter brand by id
        if(!empty($args['brand_id'])) {
            $tax_query[] = [
                ['taxonomy' => Brand::$taxonomy, 'terms' => $args['brand_id']]
            ];
        }

        // Filter tags by slug
        if(!empty($args['tags'])) {
            $tags = explode(",",$args['tags']);
            $tax_query[] = [
                ['taxonomy' => Tag::$taxonomy, 'terms' => $tags, 'field' => 'slug']
            ];
        }

        $query['tax_query'] = $tax_query;

        if(!empty($args['meta_query'])) $query['meta_query'] = $args['meta_query'];

        $q = new \WP_Query($query);
        return $q->posts;

    }


}