<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;


use DXIM\CustomTemplate;
use DXIM\Html;
use DXIM\Shortcode;

class ModelTemplate {

	protected static $instance = null;


	/**
	 *
	 * @return ModelTemplate
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {}

    public function init() {}


    /**
     * @param $model
     * @param $args
     * @return false|string
     */
    public function draw_team($model, $args = []) {
        if(!is_array($args)) $args = [];
        $defaults = [];
        $settings = Theme::getInstance()->get_settings('model');
        $settings = array_merge($defaults, $settings);
        $args = array_merge($settings, $args);
        $md = new ModelData();

        // get brand
        $brand = $md->get_brand($model->ID);

        // hole alle locations
        // filtere nach Marke
        $ld = new LocationData();
        $query_args = [];
        if(!empty($brand)) $query_args['brand'] = [$brand->term_id];
        $locations = $ld->getTree($query_args);

        $pd = new PersonData();
        $locations_persons = [];
        foreach($locations as $array) {

            $locations_tmp = !empty($array['childs']) ? $array['childs'] : [$array['parent']];
            foreach($locations_tmp as $location) {
                $query = [
                    'location' => $location->term_id,
                    'division' => $settings['person_query']['division'],
                    'count' => $settings['person_query']['count'],
                    'random' => $settings['person_query']['random']
                ];
                if($settings['person_query']['brand'] == "1" && !empty($brand)) $query['brand'] = $brand->term_id;

                $persons = $pd->getPersons($query) ?? null;
                $locations_persons[$location->term_id] = $persons;
            }
        }

        $template = $args['template'] ?? 'persons_locations';
        return CustomTemplate::getInstance()->get_template_content('theme/person/'.$template,['locations' => $locations, 'locations_persons' => $locations_persons, 'args' => $args]);
    }


    /**
     * @return false|string
     */
    public function draw_title($model, $args) {
        $md = new ModelData();
        $brand = $md->get_brand($model->ID);

        $template = $args['template'] ?? 'title.php';
        return CustomTemplate::getInstance()->get_template_content('theme/model/'.$template,['model' => $model, 'brand' => $brand]);
    }

    /**
     * @param $location
     * @param $args
     * @return false|string
     */
    public function draw_image($model, $args = []) {
        $template = $args['template'] ?? 'image.php';
        return CustomTemplate::getInstance()->get_template_content('theme/model/'.$template, ['model' => $model]);
    }

    /**
     * @param $model
     * @param $args
     * @return false|string
     */
    public function draw_tags($model, $args = []) {
        $template = $args['template'] ?? 'tags.php';
        return CustomTemplate::getInstance()->get_template_content('theme/model/'.$template, ['model' => $model]);
    }

    /**
     * @param $model
     * @param $args
     * @return false|string
     */
    public function draw_mandatory_text($model, $args) {
        $template = $args['template'] ?? 'mandatory_text.php';
        return CustomTemplate::getInstance()->get_template_content('theme/model/'.$template,['model' => $model]);
    }

    /**
     * @return false|string
     */
    public function draw_selection($args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['layout' => 'grid', 'grid' => 4, 'brand_id' => 0];
        $args = array_merge($defaults, $args);

        $md = new ModelData();
        $models = $md->get_models($args);
        $template = $args['template'] ?? 'selection_'.$args['layout'].'.php';
        return CustomTemplate::getInstance()->get_template_content('theme/model/'.$template,['models' => $models, 'args' => $args]);
    }

    /**
     * @return false|string
     */
    public function draw_offers($model, $args) {
        if(!is_array($args)) $args = [];
        $defaults = ['layout' => 'list'];
        $args = array_merge($defaults, $args);
        $model_id = $model->ID;

        $od = new OfferData();
        $offers = $od->get_offers(['model' => $model_id, 'posts_per_page' => 5]);
        if(empty($offers)) return "Derzeit sind keine Angebote zu diesem Modell vorhanden";

        return OfferTemplate::getInstance()->draw_offers($offers,$args);
    }

    /**
     * @param $model
     * @param $args
     * @return mixed|null
     */
    public function draw_cars($model, $args) {
        if(!is_array($args)) $args = [];
        $defaults = ['layout' => 'list', 'posts_per_page' => 5];
        $query = Model::getInstance()->wpcs_query;
        if(!is_array($query)) $query = [];
        $args = array_merge($defaults, $query, $args);
        return Shortcode::getInstance()->_cars($args);
    }

    /**
     * @param $model
     * @param $args
     * @return string|null
     */
    public function draw_cars_link($model, $args) {
        if(!is_array($args)) $args = [];
        $defaults = ['label' => 'alle Lagerfahrzeuge zu diesem Modell anzeigen'];
        $args = array_merge($defaults, $args);

        $query = \DXIM\theme\Model::getInstance()->wpcs_query;
        if(!is_array($query) || empty($query)) return null;
        $html = '<a class="dxim_button dxim_button_light" href="'.Html::linkToArchive($query).'">'.$args['label'].'</a>';

        return $html;
    }


}