<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\CustomTemplate;
use DXIM\VehicleFilter;

class Offer {

	protected static $instance = null;

    public static $post_type = "wpcs_offer";

    protected $offer = null;

    protected $filter = null;
    protected $wpcs_archive_position = 0;
    protected $wpcs_archive_pages = [];
    protected $offer_done_on_current_page = false;

    public static $payments = [
        'price' => ['fieldname' => 'price', 'label' => 'Barpreis'],
        'leasing' => ['fieldname' => 'leasing_price', 'label' => 'Leasing'],
        'financing' => ['fieldname' => 'financing_price', 'label' => 'Finanzierung'],
    ];

	/**
	 *
	 * @return Offer
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {}

    public function init() {
        if(!WPCS_PRO) return;

        $options = Theme::getInstance()->get_settings('offer');

        $this->filter = [];
        if(isset($_GET['wpcs_brand'])) $this->filter['brands'] = $_GET['wpcs_brand'];
        if(isset($_GET['wpcs_payment'])) $this->filter['payments'] = $_GET['wpcs_payment'];

        add_action('wp', function() use ($options) {

            if(is_singular(Offer::$post_type)) {
                global $post;
                $this->offer = $post;
                if($options['template_single'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template'],10,1);
                }
            }

            if(is_post_type_archive(Offer::$post_type)) {
                if($options['template_archive'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template_archive'],10,1);
                }
            }

        });

        /**
         * Archiv Filter
         */
        add_filter('pre_get_posts', [&$this, '_filter_offers']);

        /**
         *
         */
        add_filter('the_title', [&$this, '_filter_the_title'],10,2);

        /**
         * Angebote in Börse einblenden
         */
        $position = $options['wpcs_archive']['position'] ?? 0;
        if($position > 0) {
            $this->wpcs_archive_position = $options['wpcs_archive']['position'];
            $this->wpcs_archive_pages = explode(",",$options['wpcs_archive']['pages']);
            add_filter(DXIM_FILTER_ARCHIVE_VEHICLE_ADD, [&$this,'_vehicle_rendered'],10,6);
        }

        OfferShortcode::getInstance()->init();
        OfferGroup::getInstance()->init();

        $this->_register_post_type();
    }

    /**
     * @param $title
     * @param $post_id
     * @return mixed|string
     */
    public function _filter_the_title($title, $post_id) {
        if(get_post_type($post_id) != self::$post_type || is_admin()) return $title;

        $title .= wpcs_offer_footnotes($post_id);
        return $title;
    }


    /**
     * @param $html
     * @param $vehicle
     * @return mixed|string
     */
    public function _vehicle_rendered($html, $vehicle, $vehicle_nr, $page_nr, $num_vehicles_per_page, $num_vehicles_total) {
        // ermittle, an welchen Positionen das Form dargestellt werden soll
        $my_positions = [];
        foreach($this->wpcs_archive_pages as $page_tmp) {
            $pos_tmp = (($page_tmp-1) * $num_vehicles_per_page) + $this->wpcs_archive_position;
            $my_positions[] = $pos_tmp;
        }

        // ermittle aktuelle Fahrzeug Nr
        $vehicles_done = (($page_nr-1) * $num_vehicles_per_page) + $vehicle_nr;
        // ermittle letztes Fahrzeug
        $last_vehicle = $vehicles_done == $num_vehicles_total;

        // Zeige Form immer wenn
        // 1) position zutrifft
        // 2) letztes Fahrzeug aber Form noch nicht ausgegeben wurde (pos ist größer als Anzahl Fahrzeuge pro Seite)
        #$html .= "<span>vehicle nr. ".$vehicles_done.", last_vehicle: ".$last_vehicle."</span>";
        if(in_array($vehicles_done, $my_positions) || ($last_vehicle && !$this->offer_done_on_current_page && in_array($page_nr, $this->wpcs_archive_pages))) {
            $html .= OfferTemplate::getInstance()->draw_offer_wpcs_archive($vehicle);
            $this->offer_done_on_current_page = true;
        }

        return $html;
    }

    /**
     * @param $query
     * @return void
     */
    public function _filter_offers($query) {
        $pt = $query->get("post_type");
        if(is_admin() || !$query->is_main_query() || $pt != Offer::$post_type) return $query;

        $tax_query = $query->get('tax_query');
        if(!is_array($tax_query)) $tax_query = [];

        $meta_query = [];

        // filter marken
        $filter_brands = Brand::getInstance()->get_filter();
        if(!empty($filter_brands)) {
            $tax_query[] = ['taxonomy' => Brand::$taxonomy, 'terms' => $filter_brands];
        }

        // filter tags
        $filter_tags = Tag::getInstance()->get_filter();
        if(!empty($filter_tags)) {
            $array = [
                'relation' => 'AND'
            ];
            foreach($filter_tags as $group => $tags) {
                $array[] = ['taxonomy' => Tag::$taxonomy, 'terms' => $tags]; // OR
            }
            $tax_query[] = $array;
        }

        $query->set('tax_query', $tax_query);

        // Ablaufdatum
        $meta_query[] = [
            'relation' => 'OR',
            ['key' => 'expire_date', 'compare' => 'NOT EXISTS'],
            ['key' => 'expire_date', 'value' => ''],
            ['key' => 'expire_date', 'value' => date('Y-m-d'), 'compare' => '>='],
        ];

        // filter payment method
        if(isset($this->filter['payments'])) {
            $payment = [
                'relation' => 'OR'
            ];
            foreach($this->filter['payments'] as $fieldname) {
                $payment[] = ['key' => $fieldname, 'compare' => 'EXISTS'];
            }
            $meta_query[] = $payment;
        }

        $query->set('meta_query', $meta_query);
    }

    /**
     * @return mixed|null
     */
    public function getCurrent() { return $this->offer; }

    public function get_filter() { return $this->filter; }

    /**
     * @param $template
     * @return string
     */
    public function _get_template($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/offer/single');
    }

    /**
     * @param $template
     * @return string
     */
    public function _get_template_archive($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/offer/archive');
    }

    public function _register_post_type() {

        $labels = array(
            'name'                  => _x( 'Angebote', 'Post Type General Name', 'wpcs' ),
            'singular_name'         => _x( 'Angebot', 'Post Type Singular Name', 'wpcs' ),
            'menu_name'             => __( 'Angebote', 'wpcs' ),
            'name_admin_bar'        => __( 'Angebote', 'wpcs' ),
            'archives'              => __( 'Angebote', 'wpcs' ),
            'attributes'            => __( 'Angebote Attributes', 'wpcs' ),
            'parent_item_colon'     => __( 'Parent Angebot', 'wpcs' ),
            'all_items'             => __( 'Alle Angebote', 'wpcs' ),
            'add_new_item'          => __( 'neues Angebot', 'wpcs' ),
            'add_new'               => __( 'anlegen', 'wpcs' ),
            'new_item'              => __( 'anlegen', 'wpcs' ),
            'edit_item'             => __( 'editieren', 'wpcs' ),
            'update_item'           => __( 'update', 'wpcs' ),
            'view_item'             => __( 'anzeigen', 'wpcs' ),
            'view_items'            => __( 'anzeigen', 'wpcs' ),
            'search_items'          => __( 'suchen', 'wpcs' ),
            'not_found'             => __( 'nicht gefunden', 'wpcs' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'wpcs' ),
            'featured_image'        => __( 'Featured Image', 'wpcs' ),
            'set_featured_image'    => __( 'Set featured image', 'wpcs' ),
            'remove_featured_image' => __( 'Remove featured image', 'wpcs' ),
            'use_featured_image'    => __( 'Use as featured image', 'wpcs' ),
            'insert_into_item'      => __( 'Insert into item', 'wpcs' ),
            'uploaded_to_this_item' => __( 'Uploaded to this item', 'wpcs' ),
            'items_list'            => __( 'Items list', 'wpcs' ),
            'items_list_navigation' => __( 'Items list navigation', 'wpcs' ),
            'filter_items_list'     => __( 'Filter items list', 'wpcs' ),
        );

        $options = Theme::getInstance()->get_settings('offer');

        $rewrite = array(
            'slug'                  => $options['slug'] ?? 'angebote',
            'with_front'            => true,
            'pages'                 => true,
            'feeds'                 => true,
        );
        $args = array(
            'label'                 => __( 'Angebote', 'wpcs' ),
            'description'           => __( 'Angebote', 'wpcs' ),
            'labels'                => $labels,
            'supports'              => array( 'title', 'editor', 'thumbnail' ),
            'taxonomies'            => array( Brand::$taxonomy ),
            'hierarchical'          => false,
            'public'                => true,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'menu_position'         => 5,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => true,
            'can_export'            => true,
            'has_archive'           => true,
            'exclude_from_search'   => false,
            'publicly_queryable'    => true,
            'rewrite'               => $rewrite,
            'capability_type'       => 'page',
        );

        register_post_type( self::$post_type, $args );

    }

}