<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\Footnote;

class OfferData {

    /**
     * @param $model_id
     * @return int|\WP_Post|null
     */
    public function getById($model_id) {
        $q = new \WP_Query(['p' => $model_id, 'post_type' => Offer::$post_type]);
        return $q->post_count > 0 ? $q->posts[0] : null;
    }

    /**
     * @param $post_id
     * @return false|\WP_Error|\WP_Term[]
     */
    public function get_brand($post_id) {
        $terms = get_the_terms($post_id, Brand::$taxonomy);
        return $terms[0] ?? [];
    }

    /**
     * @param $post_id
     * @return false|\WP_Error|\WP_Term[]
     */
    public function get_tags($post_id) {
        return get_the_terms($post_id, Tag::$taxonomy);
    }

    public function get_offers($args) {
        if(!is_array($args)) $args = [];
        $defaults = ['model' => 0, 'brand' => '', 'tag' => [], 'slug' => "", "id" => "", 'paypment' => '', 'posts_per_page' => 10, 'orderby' => ''];
        $args = array_merge($defaults, $args);
        $query = ['post_type' => Offer::$post_type, 'post_status' => 'publish', 'posts_per_page' => $args['posts_per_page']];
        if(!empty($args['orderby'])) $query['orderby'] = $args['orderby'];

        $tax_query = [];
        $meta_query = ['relation' => 'AND'];

        // immer ablaufdatum berücksichtigen
        $meta_query[] = [
            'relation' => 'OR',
            ['key' => 'expire_date', 'compare' => 'NOT EXISTS'],
            ['key' => 'expire_date', 'value' => date('Y-m-d'), 'compare' => '>='],
        ];

        // filter payment method
        if(isset($args['payment'])) {
            $payment = ['relation' => 'OR'];
            $payments = explode(",",$args['payment']);
            foreach($payments as $key) {
                $payment[] = ['key' => Offer::$payments[$key]['fieldname'], 'compare' => 'EXISTS'];
            }
            $meta_query[] = $payment;
        }

        // IDS
        if(!empty($args['id'])) {
            $ids = explode(",",$args['id']);
            $query['post__in'] = $ids;
        }

        // slug
        elseif(!empty($args['slug'])) {
            $slugs = explode(",",$args['slug']);
            $query['post_name__in'] = $slugs;
        }

        else {

            // Model Ids
            if(is_array($args['model'])) {
                $meta_query[] = ['key' => 'model', 'value' => $args['model'], 'compare' => 'IN'];
            }
            // 1 Model ID
            elseif($args['model'] > 0) {
                $meta_query[] = ['key' => 'model', 'value' => $args['model']];
            }

            // Marke
            if(!empty($args['brand'])) {
                if(is_array($args['brand'])) {
                    $tax_query[] = ['taxonomy' => Brand::$taxonomy, 'terms' => $args['brand']];
                }
                else {
                    $brands = explode(",", $args['brand']);
                    $tax_query[] = ['taxonomy' => Brand::$taxonomy, 'terms' => $brands, 'field' => 'slug'];
                }
            }

            // Tag
            if(!empty($args['tag']) && is_array($args['tag'])) {
                $array = ['relation' => 'AND'];
                foreach($args['tag'] as $group_name => $tags_tmp) {
                    // query tags by id
                    if(is_array($tags_tmp)) {
                        $array[] = ['taxonomy' => Tag::$taxonomy, 'terms' => $tags_tmp];
                    }
                    // query tags by slug
                    else {
                        $tags = explode(",", $tags_tmp);
                        $array[] = ['taxonomy' => Tag::$taxonomy, 'terms' => $tags, 'field' => 'slug'];
                    }
                }
                $tax_query[] = $array;
            }

            // OfferGroup
            if(!empty($args['group'])) {
                $tax_query[] = ['taxonomy' => OfferGroup::$taxonomy, 'terms' => $args['group'], 'field' => 'slug'];
            }

        }

        $query['meta_query'] = $meta_query;
        $query['tax_query'] = $tax_query;

        $q = new \WP_Query($query);

        $offers = $q->posts;
        return $offers;
    }

    /**
     * @param $offer_id
     * @return array
     */
    public function register_footnotes($offer_id) {
        $nrs = [];

        // allg. Fußnoten
        $footnotes = get_post_meta($offer_id, "footnotes");
        foreach($footnotes as $id) {
            $nrs['general'][] = Footnote::getInstance()->register_footnote($id);
        }

        // leasing Fußnoten
        $footnotes = get_post_meta($offer_id, "leasing_footnotes");
        foreach($footnotes as $id) {
            $nrs['leasing'][] = Footnote::getInstance()->register_footnote($id);
        }

        // leasing Fußnoten
        $footnotes = get_post_meta($offer_id, "financing_footnotes");
        foreach($footnotes as $id) {
            $nrs['financing'][] = Footnote::getInstance()->register_footnote($id);
        }

        // Zusatzfelder leasing Fußnoten
        $leasing_fields = get_post_meta($offer_id, "leasing_fields", true);
        $leasing_fields['footnotes'] = $leasing_fields['footnotes'] ?? [];
        foreach($leasing_fields['footnotes'] as $row_id => $values) {
            foreach($values as $footnote_id) {
                $nrs[$row_id][] = Footnote::getInstance()->register_footnote($footnote_id);
            }
        }

        // Zusatzfelder leasing Fußnoten
        $financing_fields = get_post_meta($offer_id, "financing_fields", true);
        $financing_fields['footnotes'] = $financing_fields['footnotes'] ?? [];
        foreach($financing_fields['footnotes'] as $row_id => $values) {
            foreach($values as $footnote_id) {
                $nrs[$row_id][] = Footnote::getInstance()->register_footnote($footnote_id);
            }
        }

        // Angebotsspezifische Fußnoten
        foreach(["mandatory_information", "financing_mandatory_information", "leasing_mandatory_information"] as $key) {
            $value = get_post_meta($offer_id, $key, true);
            if(empty($value)) continue;
            $nrs[$key][] = Footnote::getInstance()->register_footnote($offer_id."_".$key, ['meta_key' => $key, 'post_id' => $offer_id]);
        }

        #wpcs_p($nrs);
        return $nrs;
    }


}