<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;


use Cassandra\Custom;
use DXIM\Admin;
use DXIM\CustomTemplate;
use DXIM\Footnote;
use DXIM\Session;
use DXIM\VehicleFilter;

class OfferTemplate {

	protected static $instance = null;


	/**
	 *
	 * @return OfferTemplate
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {}

    public function init() {    }

    /**
     * @param $offers
     * @param $args
     * @return false|string
     */
    public function draw_offers($offers, $args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['layout' => 'grid', 'grid' => 3];
        $args = array_merge($defaults, $args);
        $template = $args['template'] ?? $args['layout'];

        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template, ['offers' => $offers, 'args' => $args]);
    }

    /**
     * @return void
     */
    public function draw_offers_archive($args) {
        if(!is_array($args)) $args = [];
        $defaults = ['layout' => 'grid', 'grid' => 2];
        $args = array_merge($defaults, $args);
        $template = $args['template'] ?? $args['layout'];
        $offers = [];

        while ( have_posts() ) {
            the_post();
            global $post;
            $offers[] = $post;
        }

        return \DXIM\CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template, ['offers' => $offers, 'args' => $args]);

    }


    /**
     * @param $args
     * @return false|string
     */
    public function draw_team($offer, $args = []) {
        if(!is_array($args)) $args = [];
        $defaults = [];
        $settings = Theme::getInstance()->get_settings('offer');
        $settings = array_merge($defaults, $settings);
        $args = array_merge($settings, $args);

        // get brand
        $md = new OfferData();
        $brand = $md->get_brand($offer->ID);

        $ld = new LocationData();
        $query_args = [];
        if(!empty($brand)) $query_args['brand'] = [$brand->term_id];
        $locations = $ld->getTree($query_args);

        $pd = new PersonData();
        $locations_persons = [];
        foreach($locations as $array) {
            $locations_tmp = !empty($array['childs']) ? $array['childs'] : [$array['parent']];
            foreach($locations_tmp as $location) {
                $query = [
                    'location' => $location->term_id,
                    'division' => $settings['person_query']['division'],
                    'count' => $settings['person_query']['count'],
                    'random' => $settings['person_query']['random']
                ];
                if($settings['person_query']['brand'] == "1" && isset($brand->term_id)) $query['brand'] = $brand->term_id;

                $persons = $pd->getPersons($query) ?? null;
                $locations_persons[$location->term_id] = $persons;
            }
        }

        $template = $args['template'] ?? 'persons_locations';
        return CustomTemplate::getInstance()->get_template_content('theme/person/'.$template,['locations' => $locations, 'locations_persons' => $locations_persons, 'args' => $args]);
    }

    /**
     * @param $args
     * @return false|string
     */
    public function draw_mandatory_text($offer, $args) {
        if(!is_array($args)) $args = [];

        $template = $args['template'] ?? 'mandatory_text.php';
        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template,['offer' => $offer]);
    }

    public function draw_image($offer, $args) {
        if(!is_array($args)) $args = [];

        $template = $args['template'] ?? 'image.php';
        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template,['offer' => $offer]);
    }

//    public function draw_title($offer, $args) {
//        if(!is_array($args)) $args = [];
//
//        $template = $args['template'] ?? 'title.php';
//        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template,['offer' => $offer, 'args' => $args]);
//    }

    /**
     * @param $args
     * @return false|string
     */
    public function draw_prices($offer, $args) {
        if(!is_array($args)) $args = [];

        $template = $args['template'] ?? 'prices.php';
        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template,['offer' => $offer]);
    }

    public function draw_price($offer, $args) {
        if(!is_array($args)) $args = [];

        $template = $args['template'] ?? 'price.php';
        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template,['offer' => $offer]);
    }

    public function draw_leasing($offer, $args) {
        if(!is_array($args)) $args = [];

        $template = $args['template'] ?? 'leasing.php';
        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template,['offer' => $offer]);
    }

    public function draw_financing($offer, $args) {
        if(!is_array($args)) $args = [];

        $template = $args['template'] ?? 'financing.php';
        return CustomTemplate::getInstance()->get_template_content('theme/offer/'.$template,['offer' => $offer]);
    }

    /**
     * Angebote sind einem model zugeordnet
     * anhand einer Model-ID wird das passende Angebot (oder mehrere) selektiert
     * @param $vehicle
     * @return string
     */
    public function draw_offer_wpcs_archive(&$vehicle) {
        $options = Theme::getInstance()->get_settings('offer');
        $formData = VehicleFilter::getInstance()->getFormData();
        $make = $formData['make'] ?? dxim_get_vehicle_cf_value("make", $vehicle);
        $model = $formData['model'] ?? null;

        $md = new ModelData();
        $offers = [];

        // wenn marke und model leer können keine Angebote ermittelt werden
        if(empty($make) && empty($model)) return null;

        // hole passende Modelle anhand Marke und/oder Modell
        $meta_query = [];
        if(!empty($make)) $meta_query[] = ['key' => 'make', 'value' => $make];
        if(!empty($model)) $meta_query[] = ['key' => 'model', 'value' => $model];

        $models = $md->get_models(['meta_query' => $meta_query, 'orderby' => 'rand']) ?? null;
        $models_ids = [];
        foreach($models as $model) {
            $models_ids[] = $model->ID;
        }
        if(empty($models_ids)) return null;

        // Hole zum Model passende Angebote
        $od = new OfferData();
        $offers = $od->get_offers(['model' => $models_ids, 'orderby' => 'rand', 'posts_per_page' => $options['wpcs_archive']['count']]);

        // keine Angebote gefunden
        if(empty($offers)) return null;

        // Angebote wahlweise als Slider oder Single anzeigen
        $default_layout = Admin::getInstance()->getOptions('archive_vehicle_layout');
        $user_layout = Session::val('archive_layout');
        $wpcs_archive_layout = $args['layout'] ?? $user_layout ?? $default_layout;

        if(empty($wpcs_archive_layout)) $wpcs_archive_layout = "grid";

        $slides = [];
        foreach($offers as $offer) {
            $html_offer = CustomTemplate::getInstance()->get_template_content('theme/offer/wpcs_archive_'.$wpcs_archive_layout,['offer' => $offer]);
            $slides[] = $html_offer;
        }

        $html = '<div id="wpcs_offer_on_archive" class="'.$wpcs_archive_layout.'">';
            if(count($offers) > 1) $html .= \DXIM\Html::slider($slides, ['slides' => 1]);
            else $html .= $slides[0];
        $html .= "</div>";

        return $html;
    }

    /**
     * @param $offer_id
     * @param $categories
     * @return string
     */
    public function draw_footnotes($offer_id, $categories) {
        $od = new \DXIM\theme\OfferData();
        $footnotes = $od->register_footnotes($offer_id);
        return \DXIM\Footnote::getInstance()->draw_footnote_links($footnotes,$categories);
    }
}