<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\Admin;
use DXIM\CustomTemplate;
use DXIM\VehicleFilter;

class OfferGroup {

	protected static $instance = null;

    public static $taxonomy = "wpcs_offer_group";

    private $term = null;

	/**
	 *
	 * @return OfferGroup
	 */
	public static function getInstance() {
		if(self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 *
	 */
	private function __construct() {}

    public function init() {
        if(!WPCS_PRO) return;

        $this->_register_taxonomy();
        $options = Theme::getInstance()->get_settings('offergroup');

        add_action('wp', function() use ($options) {

            if (is_tax(OfferGroup::$taxonomy)) {
                $this->term = get_queried_object();
                if ($options['template_term'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template_archive'], 10, 1);
                }
            }

        });

        OfferGroupShortcode::getInstance()->init();
    }


    /**
     * @param $template
     * @return string
     */
    public function _get_template_archive($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/offergroup/archive');
    }

    /**
     * @return mixed|null
     */
    public function get_current() { return $this->term; }

    /**
     * @return void
     */
    function _register_taxonomy() {

        $labels = array(
            'name'                       => _x( 'Aktionen', 'Taxonomy General Name', 'wpcs' ),
            'singular_name'              => _x( 'Aktion', 'Taxonomy Singular Name', 'wpcs' ),
            'menu_name'                  => __( 'Aktionen', 'wpcs' ),
            'all_items'                  => __( 'Alle Aktionen', 'wpcs' ),
            'parent_item'                => __( 'Parent Aktionen', 'wpcs' ),
            'parent_item_colon'          => __( 'Parent Aktion:', 'wpcs' ),
            'new_item_name'              => __( 'neue Aktion', 'wpcs' ),
            'add_new_item'               => __( 'neue Aktion', 'wpcs' ),
            'edit_item'                  => __( 'Aktion bearbeiten', 'wpcs' ),
            'update_item'                => __( 'Aktion editieren', 'wpcs' ),
            'view_item'                  => __( 'Aktion anzeigen', 'wpcs' ),
            'separate_items_with_commas' => __( 'Separate items with commas', 'wpcs' ),
            'add_or_remove_items'        => __( 'Add or remove items', 'wpcs' ),
            'choose_from_most_used'      => __( 'Choose from the most used', 'wpcs' ),
            'popular_items'              => __( 'Popular Items', 'wpcs' ),
            'search_items'               => __( 'Search Items', 'wpcs' ),
            'not_found'                  => __( 'Not Found', 'wpcs' ),
            'no_terms'                   => __( 'No items', 'wpcs' ),
            'items_list'                 => __( 'Items list', 'wpcs' ),
            'items_list_navigation'      => __( 'Items list navigation', 'wpcs' ),
        );

        $options = Theme::getInstance()->get_settings('offergroup');
        $slug = $options['slug'] ?? "aktionen";
        $rewrite = array(
            'slug'                       => $slug,
            'with_front'                 => true,
            'hierarchical'               => false,
        );

        $args = array(
            'labels'                     => $labels,
            'hierarchical'               => true,
            'public'                     => true,
            'show_ui'                    => true,
            'show_admin_column'          => true,
            'show_in_nav_menus'          => true,
            'show_tagcloud'              => true,
            'rewrite'                    => $rewrite,
        );

        register_taxonomy( self::$taxonomy, array( Offer::$post_type ), $args );
    }

}