<?php
/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

class PersonData {


    /**
     * @param $slug
     * @return array|\WP_Post|null
     */
    public function getPersonBySlug($slug) {
        return get_page_by_path($slug,OBJECT,'wpcs_person');
    }

    /**
     * @param $person_id
     * @return array|mixed|\WP_Term
     */
    public function get_location($person_id) {
        $terms = get_the_terms($person_id, Location::$taxonomy);
        return $terms[0] ?? [];
    }

    /**
     * @param $person_id
     * @return mixed
     */
    public function get_meta($person_id) {
        $meta = get_post_meta($person_id);

        if(empty($meta['phone_base'][0])) {
            $location = $this->get_location($person_id);
            $phone_base = get_term_meta($location->term_id, 'phone_base');
            $meta['phone_base'] = $phone_base;
        }

        return $meta;
    }

    /**
     * @param int|\WP_Post $vehicle
     * @param $args
     * @return int[]|\WP_Post[]
     */
    public function getPersons($args = []) {
        if(!is_array($args)) $args = [];
        $defaults = ['count' => -1, 'random' => 0, 'title' => null, 'slug' => "", 'id' => 0, 'location' => 0, 'division' => 0, 'brand' => 0, 'is_contact' => 1];
        $args = array_merge($defaults, $args);
        $query = ['post_type' => Person::$post_type, 'posts_per_page' => $args['count'], 'post_status' => 'publish'];
        $tax_query = [];
        $meta_query = [];

        // id
        if(!empty($args['id'])) {
            $ids = explode(",",$args['id']);
            $query['post__in'] = $ids;
            $args['is_contact'] = 0;
        }

        // slug
        elseif(!empty($args['slug'])) {
            $slugs = explode(",",$args['slug']);
            $query['post_name__in'] = $slugs;
            $args['is_contact'] = 0;
        }

        // title macht search
        elseif(!empty($args['title'])) {
            $query['s'] = $args['title'];
        }

        else {
            // location
            if(intval($args['location']) > 0) {
                $tax_query[] = ['taxonomy' => Location::$taxonomy, 'terms' => $args['location']];
            }

            // division
            if(intval($args["division"]) > 0) {
                $tax_query[] = ['taxonomy' => Division::$taxonomy, 'terms' => $args['division']];
            }

            // brand
            if(intval($args["brand"]) > 0) {
                $tax_query[] = ['taxonomy' => Brand::$taxonomy, 'terms' => $args['brand']];
            }
        }

        if($args['random'] == "1") $query['orderby'] = "rand";
        else {
            $query['orderby'] = 'meta_value_num';
            $query['order'] = 'ASC';
            $query['meta_key'] = 'order';
        }

        $query['tax_query'] = $tax_query;

        // nur Personen die Contactperson sind ausspielen
        if($args['is_contact']) {
            $meta_query[] = ['key' => 'is_contact_person', 'value' => 1];
        }

        $query['meta_query'] = $meta_query;

        $q = new \WP_Query($query);
        return $q->posts;

    }

}