<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use DXIM\CustomTemplate;
use DXIM\VehicleFilter;

class Service {

    protected static $instance = null;

    public static $post_type = "wpcs_service";

    private $service_types = [
        'FINANCIAL_SERVICES' => 'Finanzdienstleistung', 'REPAIR_SHOP' => 'Werkstatt', 'OTHERS' => 'Sonstige',
    ];

    protected $service = null;

    /**
     *
     * @return Service
     */
    public static function getInstance() {
        if (self::$instance === null) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    public function get_service_types() {
        return $this->service_types;
    }


    /**
     *
     */
    private function __construct() {
    }

    public function init() {
        if (!WPCS_PRO) return;

        $options = Theme::getInstance()->get_settings('service');

        add_action('wp', function () use ($options) {

            if (is_singular(Service::$post_type)) {
                global $post;
                $this->service = $post;
                if ($options['template_single'] == "wpcs") {
                    add_action('template_include', [&$this, '_get_template'], 10, 1);
                }
            }
        });


        add_filter('pre_get_posts', [&$this, '_mod_single_page_query']);

        add_filter('manage_wpcs_service_posts_columns', [&$this, '_manage_wpcs_service_posts_columns']);
        add_action('manage_wpcs_service_posts_custom_column', [&$this, '_manage_wpcs_service_posts_custom_column'], 10, 2);

        ServiceShortcode::getInstance()->init();

        $this->_register_post_type();
    }

    /**
     * @return mixed|null
     */
    public function getCurrent() {
        return $this->service;
    }



    public function _mod_single_page_query($query) {
        global $post;
        if (is_null($post)) return;
        $pt = $post->post_type;



        if (is_admin() || $pt != Service::$post_type) return;

        if (is_singular(Service::$post_type)) {
            global $post;
            $query->set('post__not_in', array($post->ID));
        }
    }


    /**
     * @param $template
     * @return string
     */
    public function _get_template($template) {
        return CustomTemplate::getInstance()->get_template_file('theme/service/single');
    }

    function _manage_wpcs_service_posts_columns($columns) {
        $new_columns = array_slice($columns, 0, 3, true) +
            array('wpcs_service_type' => 'Serviceart') +
            array_slice($columns, 3, null, true);
        return $new_columns;
    }

    function _manage_wpcs_service_posts_custom_column($column, $post_id) {
        if ($column == 'wpcs_service_type') {
            $tmp_service_types = get_post_meta($post_id, 'serviceType');
            $types = $this->service_types;
            $labels_array = array_map(function ($type) use ($types) {
                return isset($types[$type]) ? $types[$type] : '';
            }, $tmp_service_types);

            $labels_string = implode(', ', $labels_array);

            echo $labels_string;
        }
    }

    public function _register_post_type() {

        $labels = array(
            'name'                  => _x('Services', 'Post Type General Name', 'wpcs'),
            'singular_name'         => _x('Service', 'Post Type Singular Name', 'wpcs'),
            'menu_name'             => __('Services', 'wpcs'),
            'name_admin_bar'        => __('Services', 'wpcs'),
            'archives'              => __('Services', 'wpcs'),
            'attributes'            => __('Services Attributes', 'wpcs'),
            'parent_item_colon'     => __('Parent Service', 'wpcs'),
            'all_items'             => __('Alle Services', 'wpcs'),
            'add_new_item'          => __('neuer Service', 'wpcs'),
            'add_new'               => __('anlegen', 'wpcs'),
            'new_item'              => __('anlegen', 'wpcs'),
            'edit_item'             => __('editieren', 'wpcs'),
            'update_item'           => __('update', 'wpcs'),
            'view_item'             => __('anzeigen', 'wpcs'),
            'view_items'            => __('anzeigen', 'wpcs'),
            'search_items'          => __('suchen', 'wpcs'),
            'not_found'             => __('nicht gefunden', 'wpcs'),
            'not_found_in_trash'    => __('Not found in Trash', 'wpcs'),
            'featured_image'        => __('Featured Image', 'wpcs'),
            'set_featured_image'    => __('Set featured image', 'wpcs'),
            'remove_featured_image' => __('Remove featured image', 'wpcs'),
            'use_featured_image'    => __('Use as featured image', 'wpcs'),
            'insert_into_item'      => __('Insert into item', 'wpcs'),
            'uploaded_to_this_item' => __('Uploaded to this item', 'wpcs'),
            'items_list'            => __('Items list', 'wpcs'),
            'items_list_navigation' => __('Items list navigation', 'wpcs'),
            'filter_items_list'     => __('Filter items list', 'wpcs'),
        );

        $options = Theme::getInstance()->get_settings('service');

        $rewrite = array(
            'slug'                  => $options['slug'] ?? 'services',
            'with_front'            => true,
            'pages'                 => true,
            'feeds'                 => true,
        );

        $args = array(
            'label'                 => __('Services', 'wpcs'),
            'description'           => __('Services', 'wpcs'),
            'labels'                => $labels,
            'supports'              => array('title', 'editor', 'thumbnail'),
            'taxonomies'            => array(Location::$taxonomy),
            'hierarchical'          => false,
            'public'                => true,
            'show_ui'               => true,
            'show_in_menu'          => false,
            'menu_position'         => 5,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => true,
            'can_export'            => true,
            'has_archive'           => false,
            'exclude_from_search'   => false,
            'publicly_queryable'    => true,
            'rewrite'               => $rewrite,
            'capability_type'       => 'page',
        );

        register_post_type(self::$post_type, $args);
    }
}
