<?php

/**
 * Created by PhpStorm.
 * User: dx
 * Date: 2019-04-03
 * Time: 10:11
 */

namespace DXIM\theme;

use Locale;

class ServiceData {


    /**
     * @param $service_id
     * @return int|\WP_Post|null
     */
    public function getById($service_id) {
        $q = new \WP_Query(['p' => $service_id, 'post_type' => Service::$post_type]);
        return $q->post_count > 0 ? $q->posts[0] : null;
    }

    /**
     * @param $service_id
     * @return false|\WP_Error|\WP_Term[]
     */
    public function get_locations($service_id) {
        return get_the_terms($service_id, Location::$taxonomy);
    }


    /**
     * @param $location_id
     */
    public function get_services_by_location($location_id, $args) {
        if (!is_array($args)) $args = [];
        $defaults = ['posts_per_page' => -1, 'orderby' => ''];
        $args = array_merge($defaults, $args);

        if (!empty($args['orderby'])) $query['orderby'] = $args['orderby'];

        $meta_query = ['relation' => 'AND'];

        $query = array(
            'post_type'      => Service::$post_type,
            'posts_per_page' => $args['posts_per_page'],
            'post_status'    => 'publish',
            'tax_query'      => array(
                array(
                    'taxonomy' => Location::$taxonomy,
                    'field'    => 'term_id',
                    'terms'    => array($location_id),
                ),
            ),
        );

        if (!empty($args['service_type'])) {
            $service_types = explode(",", $args['service_type']);
            $mq = [
                'relation' => 'OR',
            ];
            foreach ($service_types as $type) {
                $mq[] = ['key' => 'serviceType', 'value' => $type];
            }
            $meta_query[] = $mq;
        }
        $query['meta_query'] = $meta_query;

        $q = new \WP_Query($query);
        return $q->posts;
    }

    public function get_services($args) {
        if (!is_array($args)) $args = [];
        $defaults = ['slug' => "", "id" => "", 'posts_per_page' => 10, 'orderby' => '', 'same_type' => 0];
        $args = array_merge($defaults, $args);
        $query = ['post_type' => Service::$post_type, 'post_status' => 'publish', 'posts_per_page' => $args['posts_per_page']];
        if (!empty($args['orderby'])) $query['orderby'] = $args['orderby'];
        $tax_query = [];
        $meta_query = ['relation' => 'AND'];

        // IDS
        if (!empty($args['id'])) {
            $ids = explode(",", $args['id']);
            $query['post__in'] = $ids;
        }

        // slug
        elseif (!empty($args['slug'])) {
            $slugs = explode(",", $args['slug']);
            $query['post_name__in'] = $slugs;
        }

        if (!empty($args['service_type'])) {
            $service_types = explode(",", $args['service_type']);
            $mq = [
                'relation' => 'OR',
            ];
            foreach ($service_types as $type) {
                $mq[] = ['key' => 'serviceType', 'value' => $type];
            }
            $meta_query[] = $mq;
        }

        if ($args['same_type'] == 1 && is_singular(Service::$post_type)) {
            $currentService = Service::getInstance()->getCurrent();
            $type = get_post_meta($currentService->ID, 'serviceType', true);
            $mq[] = ['key' => 'serviceType', 'value' => $type];
            $meta_query[] = $mq;
        }

        $query['meta_query'] = $meta_query;
        $query['tax_query'] = $tax_query;

        $q = new \WP_Query($query);
        return $q->posts;
    }
}
