var wpcs_admin_theme = null;

(function($) {

    $(function() {

        wpcs_admin_theme = new WPCSAdminTheme();

    }); // domready

    function WPCSAdminTheme() {

        var that = this;
        this.timeout_id = null;
        this.current_field = null;

        this._construct = function() {
            this.initRepeaterRow();
            this.initFields();
        }

        this.initRepeaterRow = function() {
            $('.wpcs_field.repeater').each(function() {
                let repeater = this;
                $('.row',repeater).each(function() {
                    let row = $(this);
                    that.initAddRemoveRow(row);
                });

            });
        }


        this.initAddRemoveRow = function(row) {

            $('a.add_row', row).click(function(e) {
                e.preventDefault();
                let new_row = row.clone();
                let row_id = row.data('row-id');
                let new_row_id = Date.now();

                let html = new_row.html();
                html = html.replaceAll(row_id, new_row_id);
                new_row.html(html);
                new_row.attr('data-row-id', new_row_id);

                row.after(new_row);

                that.initAddRemoveRow(new_row);

                $('.wpcs_field.image', new_row).each(function(){
                    let field = $(this);
                    that.initImageField(field);
                });

                $('.wpcs_field.post_type', new_row).each(function(){
                    let field = $(this);
                    that.initPostTypeField(field);
                });

            });


            $('a.remove_row', row).click(function(e) {
                e.preventDefault();
                row.remove();
            });
        }


        this.initPostTypeField = function(field) {

            $('.wpcs_post_type_search', field).keyup(function() {
                let v = $(this).val();
                let pt = $(this).data('pt');
                that.current_field = field;
                //$('.field_value',field).val("")

                window.clearTimeout(that.timeout_id);

                if(v == "") return;
                that.timeout_id = window.setTimeout(that.performPTSearch,500,v, pt);
            });

            $('.values .item', field).each(function() {
                that.initPTItem(this);
            });

        }

        this.performPTSearch = function(search, pt) {

            $('.results div',that.current_field).remove();
            $('.results',that.current_field).hide();
            let field_name = $(that.current_field).data('name');
            let multiple = parseInt($(that.current_field).data('multiple'));

            $.ajax({method : 'POST', url : wpcs_ajax_url,
                data : { action : 'wpcs_theme_pt_search', search : search, pt : pt },
                success : function(data) {
                    $(data).each(function() {
                        let entry = this;
                        let div = $('<div data-id="'+entry.id+'">'+entry.title+'</div>');
                        div.click(function() {

                            if(multiple < 1) {
                                $('.values div',that.current_field).remove();
                            }

                            let new_item_string = '<div class="item">'+
                                '<strong>'+$(this).html()+'</strong>'+
                                '<a href="#"><span class="dashicons dashicons-no-alt"></span></a>'+
                                '<input class="field_value" name="'+field_name+'[]" type="hidden" value="'+$(this).data('id')+'" />'+
                                '</div>';
                            let new_item = $(new_item_string);
                            $('.values',that.current_field).append(new_item);

                            that.initPTItem(new_item);
                            div.remove();

                        });

                        $('.results',that.current_field).append(div).show();
                    });
                }
            });
        }


        this.initPTItem = function(item) {
            $('a', item).click(function(e) {
                e.preventDefault();
                item.remove();
                $('.wpcs_post_type_search', that.current_field).removeAttr('disabled');
            })
        }

        this.initFields = function() {

            $('.wpcs_field.image').each(function(){
                let field = $(this);
                that.initImageField(field);
            });

            $('.wpcs_field.post_type').each(function() {
                let field = $(this);
                that.initPostTypeField(field);
            });

            /**
             * leeren des Image Uploader nach ajax request
             */
            // Leere Bild src und value nach erfolgreichem anlegen
            // nur auf dem Taxonomy screen, da dieser nicht refresht sondern per ajax speichert
            // dazu vergleichen wir einfach, ob sich an der anzahl terms etwas verändert hat
            let numberOfTags = 0;
            let newNumberOfTags = 0;

            // when there are some terms are already created
            if( ! $( '#the-list' ).children( 'tr' ).first().hasClass( 'no-items' ) ) {
                numberOfTags = $( '#the-list' ).children( 'tr' ).length;
            }

            $(document).ajaxComplete( function( event, xhr, settings ){

                newNumberOfTags = $( '#the-list' ).children('tr').length;
                if( parseInt( newNumberOfTags ) > parseInt( numberOfTags ) ) {

                    numberOfTags = newNumberOfTags;

                    $( '.wpcs_cf_preview_image' ).each( function(){
                        $(this).hide(); // hide image
                        $(this).next().next().next().val(''); // empty hidden field
                        $(this).next().next().hide(); // hide remove image button
                    });
                }

            });


        }


        this.initImageField = function(field) {

            /**
             * init image upload button
             */
            $('.wpcs_cf_upload', field).click(function( event ){
                event.preventDefault(); // prevent default link click and page refresh

                const button = $(this)
                const imageId = button.next().next().val();

                const customUploader = wp.media({
                    title: 'Insert image', // modal window title
                    library : {
                        // uploadedTo : wp.media.view.settings.post.id, // attach to the current post?
                        type : 'image'
                    },
                    button: {
                        text: 'Use this image' // button label text
                    },
                    multiple: false
                }).on( 'select', function() { // it also has "open" and "close" events
                    const attachment = customUploader.state().get( 'selection' ).first().toJSON();
                    //button.removeClass( 'button' ).html( '<img src="' + attachment.url + '">'); // add image instead of "Upload Image"
                    button.prev().attr('src', attachment.url).show();
                    button.next().show(); // show "Remove image" link
                    button.next().next().val( attachment.id ); // Populate the hidden field with image ID
                })

                // already selected images
                customUploader.on( 'open', function() {

                    if( imageId ) {
                        const selection = customUploader.state().get( 'selection' )
                        attachment = wp.media.attachment( imageId );
                        attachment.fetch();
                        selection.add( attachment ? [attachment] : [] );
                    }

                })

                customUploader.open()

            });

            /**
             * init image remove button
             */
            $( '.wpcs_cf_upload_reset', field ).click(function( event ){
                event.preventDefault();
                const button = $(this);
                button.prev().prev().hide();
                button.next().val( '' ); // emptying the hidden field
                button.hide();
            });

        }

        this._construct();
    }

})(jQuery);