(function($) {


    /** ----------------------------------------------------------------------------------
     * WPCS SELECT
     */
    $.fn.wpcsselect = function(options) {

        var defaults = {
            onRefresh: function() {},
            onSelect: function() {},
            handle_selector: '.wpcs_select_handle',
            loading_selector: '.wpcs_select_loading',
            arrow_selector: '.wpcs_select_arrow',
            value_selector: null,
            list_selector: '#wpcs_select_list',
            target: null
        };

        var settings = $.extend({}, defaults, options);

        if (settings.target !== null) {
            settings.value_selector = settings.target;
        } else {
            settings.value_selector = ".wpcs_select_value";
            //settings.value_selector = "#wpcs_select_value_" + $(this).data('field');
        }


        /**
         *
         * @param element
         * @returns {*}
         * @private
         */
        function _create(select) {
            let element = $(select);
            let ul;
            ul = $('ul' + settings.list_selector);
            if (ul.length < 1) {
                ul = $('<ul id="wpcs_select_list"></ul>');
                $('body').append(ul);
            }

            _initialize(element);

            return element;
        }

        /**
         *
         * @param element
         * @private
         */
        function _initialize(element) {

            $(settings.handle_selector + ', ' + settings.arrow_selector, element).click(function() {
                if ($(element).hasClass('open')) {
                    _close(element);
                } else {
                    _close($('div.wpcsselect.open'));
                    _open(element);
                }
            });


        }

        /**
         *
         * @param select
         * @private
         */
        function _init_options(element) {
            let multiple = element.data('multiple');

            $(settings.list_selector + ' li').click(function() {

                if (multiple > 0) {
                    $(this).toggleClass('active');
                    _set_multiple_value(element);
                } else {
                    _set_value(element, $(this).attr('data-index'));
                    _close(element);
                }

                settings.onSelect.call();
            });

        }

        /**
         *
         * @param select
         * @private
         */
        function _open(element) {
            let multiple = parseInt($(element).data('multiple'));

            $(element).addClass('open');

            _calculate_position();

            let ul = $('ul' + settings.list_selector);
            ul.addClass('open');

            if (multiple > 0) ul.addClass('multiple');
            else ul.removeClass('multiple');

            // fülle auswahlliste
            if (settings.ajax) { _createListFromAjax(element); } else { _createListFromDataDiv(element); }


            window.setTimeout(function(){
                $(document).bind('click', _close_current);
                $(window).bind('resize', _calculate_position);
            }, 10);
        }

        function _calculate_position() {
            let element = $('div.wpcsselect.open');
            let handle = $(settings.handle_selector, element);
            let handlePos = $(settings.handle_selector, element).offset();

            // positioniere auswahlliste
            let ul = $('ul' + settings.list_selector);
            ul.css({
                top: handlePos.top + handle.height(),
                left: handlePos.left,
                width: handle.outerWidth()
            });

        }

        /**
         *
         * @private
         */
        function _close_current(e) {
            if (!$(e.target).hasClass('wpcs_select_option') && !$(e.target).hasClass('wpcs_select_handle') && !$(e.target).hasClass('wpcs_select_arrow') && !$(e.target).hasClass('wpcs_select_multiple_checkbox')) {
                _close($('div.wpcsselect.open'));
            }


        }

        /**
         *
         * @param select
         * @private
         */
        function _close(element) {
            $(element).removeClass('open');
            $('ul' + settings.list_selector).removeClass('open');
            $(document).unbind('click', _close_current);
            $(window).unbind('resize', _calculate_position);
        }

        /**
         *
         * @param select
         * @param list
         * @private
         */
        function _refresh(element, list, sel) {

            // selected value kann übergeben werden,
            // alternativ wird nach aktuellem Value gesucht
            // ansonstem keine vorselektierung
            let selected = sel || null;
            let selected_value;
            let multiple = element.data('multiple');

            if (selected !== null) selected_value = selected;
            else selected_value = $(settings.value_selector, element).val() || '';

            if (multiple > 0) {
                selected_value = selected_value.split(",");
                if (!Array.isArray(selected_value)) selected_value = [];
            }

            let ul = $('ul' + settings.list_selector);
            let selected_index = 0;
            $('li', ul).remove();

            let i = 0;
            for (let [index, obj] of list) {

                if (obj.value == "#" && multiple > 0 && list.size != 1) continue;

                let active_class = '';
                if (multiple > 0 && obj.value != "#") {
                    let active = selected_value.some(function(array_element) {
                        return array_element == obj.value;
                    });
                    active_class = active > 0 ? 'active' : '';
                }

                let elementHtml = '<li class="wpcs_select_option ' + active_class + '" data-value="' + obj.value + '" data-label="' + obj.label + '" data-index="' + obj.index + '">';
                elementHtml += obj.label;
                elementHtml += '</li>';

                let option = $(elementHtml);
                if (selected_value == obj.value) {
                    selected_index = i;
                }
                ul.append(option);
                i++;
            }

            _init_options(element);

            if (multiple > 0) _set_multiple_value(element);
            else _set_value(element, selected_index);

            settings.onRefresh.call();
        }

        /**
         *
         * @param select
         * @param index
         * @private
         */
        function _set_value(element, index) {

            let li = $(settings.list_selector + ' li').eq(index);

            // unset old
            $(settings.list_selector + ' li.selected').removeClass('selected');

            // set selected
            li.addClass('selected');

            // refresh handle label
            $(settings.handle_selector, element).html(li.data('label'));

            // set value to form element
            $(settings.value_selector, element).val(li.data('value'));
        }

        /**
         *
         * @param element
         * @private
         */
        function _set_multiple_value(element) {
            let value = '';
            let count = 0;
            let labelValue = '';
            let labelTmp = '';
            let valueTmp = '';


            // build value string
            $(settings.list_selector + ' li').each(function() {
                if ($(this).hasClass('active')) {
                    labelTmp = $(this).data('label');
                    valueTmp = $(this).data('value');
                    if (value != "") value += ',';
                    value += valueTmp;
                    labelValue = labelTmp;
                    count++;
                }
            });

            // set label
            let label = '';
            if (count == 0) label = $(element).data('nothing-selected');
            else if (count == 1) label = labelValue;
            else if (count > 1) label = count + ' ausgewählt';
            $(settings.handle_selector, element).html(label);

            // set value
            $(settings.value_selector, element).val(value);
        }

        /**
         *
         * @param element
         * @private
         */
        function _createListFromDataDiv(element) {
            let list = new Map();
            $('.data div', element).each(function(i){
                let value = $(this).attr('data-value');
                let label = $(this).html();
                list.set(i, { value: value, label: label, index: i });
            });
            _refresh(element, list);
        }

        /**
         *
         * @param select
         * @private
         */
        function _createListFromAjax(element) {
            if (!settings.ajax) return;
            let ajax = Object.assign({}, settings.ajax);
            ajax.data = settings.ajax.data();

            let multiple = parseInt(element.data('multiple'));

            $(settings.loading_selector, element).show();
            $(settings.arrow_selector, element).hide();

            $('li', settings.list_selector).remove();

            $.ajax(ajax).done(function(data) {
                let list = new Map();
                let z = 0;

                if(multiple < 1) {
                    list.set(z, { value: '#', label: element.data('nothing-selected'), index: z });
                    z++;
                }

                for (let index in data) {
                    //index = parseInt(index);
                    let value = data[index]['id'];
                    let label = data[index]['text'];
                    list.set(z, { value: value, label: label, index: z });
                    z++;
                }

                $(settings.loading_selector, element).hide();
                $(settings.arrow_selector, element).show();

                _refresh(element, list);
            });

        }

        /**
         *
         */
        return this.each(function() {
            select = this;
            _create(select);
        });
    };

})(jQuery);